function [ isBRL, IsBRL, BRLniLines, BRLNLines, BRLK1, BRLK2, IsBRLK1 ] = brl_check( ptype, nPol, isEQForm, isNU, n, nOld, k, isT, isR )

% BRL_CHECK Check for BRLs
%
%   This function is used to check for BRLs in the
%   POS3POLY problem.
%
%   OUTPUTS:
%   
%   isBRL - vector for every "polynomial"; 0 - BRL, 1 - not BRL
%   IsBRL - 1 - if any BRL, 0 - no BRLs
%   BRLNILINES - nr. of lines for each BRL describing the filter
%   BRLNLINES  - total nr. of lines describing the filters in the BRLs
%   BRLK1 - vector with K2 for every pol.
%   BRLK2 - vector with K2 for every pol.
%   IsBRLK1 - if any matrix BRL (if any BRLK1>1)

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 08.02.10-10.10.11

% check for BRL
isBRL      = zeros( nPol, 1 );
% number of lines for each BRL
BRLniLines = zeros( nPol, 1 ); 
% total lines describing the filters
% in all the BRLs
BRLNLines  = 0;
% 'K1' & 'K2' field for BRL;
% init with (default) scalar case
BRLK1      = zeros( nPol, 1 );
BRLK2      = zeros( nPol, 1 );

% M holds the number of scalar coefficients for every polynomial
% N holds the number of scalar coefficients for every causal
% polynomial if we have BRL
M = zeros( nPol, 1 );
N = zeros( nPol, 1 );

Line = 0;

for i = 1 : nPol
    
    if isT( i ) && isR( i )
        % hybrid polynomial
        M( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, ptype{ i }.real );
    elseif isT( i )
        % trigonometric polynomial
        M( i ) = lenpol( nOld{ i }, k( i ), ptype{ i }.trigonometric, 0 );
    else
        % real polynomial
        M( i ) = lenpol( nOld{ i }, k( i ), 0, ptype{ i }.real );
    end
    
    % check for field 'brl'
    if isfield( ptype{ i }, 'brl' )
        
        % BRL type
        isBRL( i ) = 1;
              
        % check if 'hsize' is empty
        assert( ~isempty( ptype{ i }.brl.hsize ), 'Field ''hsize'' from field ''brl'' is empty; ppol: %d.', i );
        
        % check if 'hsize' is scalar or vector
        assert( isvector( ptype{ i }.brl.hsize ), 'Field ''hsize'' from field ''brl'' must be scalar or vector; ppol: %d.', i );
        
        % number of elements in 'hsize'
        hsizeLen = length( ptype{ i }.brl.hsize );
        assert( hsizeLen == 1 || hsizeLen == 2, 'Field ''hsize'' from field ''brl'' must be of one or two elements; ppol: %d.', i );

        % count lines for the BRLs
        if isEQForm
            % EQForm
            if isNU
                BRLniLines( i ) = length( ptype{ i }.brl.hrows );
            end
        else        
            % INEQForm
            
            % length of the causal polynomial
            N( i ) = lencpol( n{ i }, ptype{ i }.brl.hsize );
            
            % the lines that need to be cut and added afterwards
            ptype{ i }.brl.hrows = ( Line + 1 ) : ( Line + N );
            
            % advance to the next polynomial
            Line = Line + M( i ) + N( i );          
        end
        
        BRLNLines = BRLNLines + BRLniLines( i );
        
        if hsizeLen == 1
            % set  fields 'K1' & 'K2'
            BRLK1( i ) = ptype{ i }.brl.hsize;
            BRLK2( i ) = ptype{ i }.brl.hsize;
        else
            % copy fields 'K1' & 'K2' 
            BRLK1( i ) = ptype{ i }.brl.hsize( 1 );
            BRLK2( i ) = ptype{ i }.brl.hsize( 2 );
        end
    else
        Line = Line + M( i );
    end
end

% check if any BRL
IsBRL = nnz( isBRL );

% check if any matrix BRL
IsBRLK1 = cf( IsBRL, nnz( BRLK1 - 1 ), 0 );