function [ D ] = coef_var_mat_multi( D, Line, LimD, ptype, nPol, n, k, nOld, nNonz, nNonzOld, IsT, IsR, isT, isR, isC, ...
                                     TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                                     IsD, isBRL, BRLK2, ...
                                     isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPol, SBPosDegsDomPol, ...
                                     isD, degDNonz, nDomPol, ncCS, nDomPolCS, ...
                                     nNonzSi, nNonzOldSi )

% COEF_VAR_MAT_MULTI Create SeDuMi matrix for the multivariate case
%
%   This function returns the D matrix, that we use
%   to transform the user's problem into a SeDuMi
%   problem.
%
%   The problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 17.12.09-19.06.10

global FID;

% if we have pos. dom.
if IsD
    % indexes in LimD for the coefs. of the
    % SOSs for every pos. pol. from the pos. dom.
    nDomPolMax = max( nDomPol );
    dmpIdxAll  = zeros( nDomPolMax, 2 );
    dmpIdxAll( :, 1 ) = 2 * ( 1 : full( nDomPolMax ) )' + 1;
    dmpIdxAll( :, 2 ) = dmpIdxAll( :, 1 ) + 1;
end

% check for any
% hybrid polynomial
if IsT && IsR
    % there is at least one hybrid polynomial
    
    if ~IsD
        % no pos. dom.
        
        % for every polynomial
        for i = 1 : nPol
            
            t0 = cputime;
            
            % print ppol, n, k
            p3p_fprintf( FID, '%3d : %10s %3d', i, [ '[' num2str( nOld{ i } ) ']' ], k( i ) );
            % print nt, nr
            if isT( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.trigonometric );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            if isR( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.real );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            % print Qsize, isC, isD, isBRL, isSB
            p3p_fprintf( FID, ' %6d   %1d   %1d     %1d    %1d', Qsize( i ), isC( i ), 0, isBRL( i ), isSB( i ) );
            
            if isT( i ) && ~isR( i )
                % trigonometric polynomial

                if ~isSB( i )
                    [ D, Line ] = coef_var_mat_multi_tr_pol   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                isT( i ), TList, TVecUnq, QOldsize( i ), isBRL( i ), BRLK2( i ) );
                else
                    [ D, Line ] = coef_var_mat_multi_tr_pol_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                isT( i ), TList, TVecUnq, isBRL( i ), BRLK2( i ), ...
                                                                SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
                end
                
            elseif ~isT( i ) && isR( i )
                % real polynomial

                if ~isSB( i )
                    [ D, Line ] = coef_var_mat_multi_tr_pol   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                isT( i ), HList, HVecUnq, QOldsize( i ), isBRL( i ), BRLK2( i ) );
                else
                    [ D, Line ] = coef_var_mat_multi_tr_pol_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                isT( i ), HList, HVecUnq, isBRL( i ), BRLK2( i ), ...
                                                                SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
                end
            else
                % hybrid polynomial

                if ~isSB( i )
                    [ D, Line ] = coef_var_mat_multi_h_pol   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                               ptype{ i }.trigonometric, TList, HList, TVecUnq, HVecUnq, ...
                                                               isBRL( i ), BRLK2( i ) );
                else
                    [ D, Line ] = coef_var_mat_multi_h_pol_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                               ptype{ i }.trigonometric, TList, HList, TVecUnq, HVecUnq, ...
                                                               isBRL( i ), BRLK2( i ), ...
                                                               SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );    
                end
            end
            
            % print time
            t0 = cputime - t0;
            [ h, m, s ] = sec2hour( t0 );
            p3p_fprintf( FID, '  %2d:%2d:%2d\n' , h, m, s );
        end
    else
        % with pos. dom.
        
        % for every polynomial
        for i = 1 : nPol
            
            t0 = cputime;
            
            % print ppol, n, k
            p3p_fprintf( FID, '%3d : %10s %3d', i, [ '[' num2str( nOld{ i } ) ']' ], k( i ) );
            % print nt, nr
            if isT( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.trigonometric );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            if isR( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.real );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            % print Qsize, isC, isD, isBRL, isSB
            p3p_fprintf( FID, ' %6d   %1d   %1d     %1d    %1d', Qsize( i ), isC( i ), 1, isBRL( i ), isSB( i ) );
            
            if isT( i ) && ~isR( i )
                % trigonometric polynomial

                if ~isD( i )
                    % no pos. dom.
                    if ~isSB( i )
                        [ D, Line ] = coef_var_mat_multi_tr_pol   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                    isT( i ), TList, TVecUnq, QOldsize( i ), isBRL( i ), BRLK2( i ) );
                    else
                        [ D, Line ] = coef_var_mat_multi_tr_pol_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                    isT( i ), TList, TVecUnq, isBRL( i ), BRLK2( i ), ...
                                                                    SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
                    end
                else
                    % with pos. dom.
                    if ~isSB( i )
                        [ D, Line ] = coef_var_mat_multi_tr_pd  ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                  isT( i ), TList, TVecUnq, QOldsize( i ), ...
                                                                  degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                                  ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nNonzSi{ i }, ...
                                                                  nNonzOldSi{ i }, dmpIdxAll, isBRL( i ), BRLK2( i ) );
                    else
                        [ D, Line ] = coef_var_mat_multi_tr_pd_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                   isT( i ), TList, TVecUnq, ...
                                                                   degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                                   ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nDomPolCS( i ), ...
                                                                   nNonzSi{ i }, nNonzOldSi{ i }, dmpIdxAll, isBRL( i ), BRLK2( i ), ...
                                                                   SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
                    end
                end
            elseif ~isT( i ) && isR( i )
                % real polynomial

                if ~isD( i )
                    % no pos. dom.
                    if ~isSB( i )
                        [ D, Line ] = coef_var_mat_multi_tr_pol   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                    isT( i ), HList, HVecUnq, QOldsize( i ), isBRL( i ), BRLK2( i ) );
                    else
                        [ D, Line ] = coef_var_mat_multi_tr_pol_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                    isT( i ), HList, HVecUnq, isBRL( i ), BRLK2( i ), ...
                                                                    SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
                    end
                else
                    % with pos. dom.
                    if ~isSB( i )
                        [ D, Line ] = coef_var_mat_multi_tr_pd   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                   isT( i ), HList, HVecUnq, QOldsize( i ), ...
                                                                   degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                                   ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nNonzSi{ i }, ...
                                                                   nNonzOldSi{ i }, dmpIdxAll, isBRL( i ), BRLK2( i ) );
                    else
                        [ D, Line ] = coef_var_mat_multi_tr_pd_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                   isT( i ), HList, HVecUnq, ...
                                                                   degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                                   ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nNonzSi{ i }, ...
                                                                   nNonzOldSi{ i }, dmpIdxAll, isBRL( i ), BRLK2( i ), ...
                                                                   SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i }, ...
                                                                   SBDegsAddSubDomPol, SBPosDegsDomPol );
                    end
                end
            else
                % hybrid polynomial

                if ~isD( i )
                    % no pos. dom.
                    if ~isSB( i )
                        [ D, Line ] = coef_var_mat_multi_h_pol( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                ptype{ i }.trigonometric, TList, HList, TVecUnq, HVecUnq, ...
                                                                isBRL( i ), BRLK2( i ) );
                    else
                        [ D, Line ] = coef_var_mat_multi_h_pol( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                ptype{ i }.trigonometric, TList, HList, TVecUnq, HVecUnq, ...
                                                                isBRL( i ), BRLK2( i ), ...
                                                                SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
                    end
                else
                    % with pos. dom.
                    if ~isSB( i )
                        [ D, Line ] = coef_var_mat_multi_h_pd  ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                 ptype{ i }.trigonometric, TList, HList, TVecUnq, HVecUnq, ...
                                                                 degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                                 ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nNonzSi{ i }, ...
                                                                 nNonzOldSi{ i }, dmpIdxAll, isBRL( i ), BRLK2( i ) );
                    else
                        [ D, Line ] = coef_var_mat_multi_h_pd_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                  ptype{ i }.trigonometric, TList, HList, TVecUnq, HVecUnq, ...
                                                                  degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                                  ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nDomPolCS( i ), ...
                                                                  nNonzSi{ i }, nNonzOldSi{ i }, ...
                                                                  dmpIdxAll, isBRL( i ), BRLK2( i ), ...
                                                                  SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i }, ...
                                                                  SBDegsAddSubDomPol, SBPosDegsDomPol );
                    end
                end
            end
            
            % print time
            t0 = cputime - t0;
            [ h, m, s ] = sec2hour( t0 );
            p3p_fprintf( FID, '  %2d:%2d:%2d\n' , h, m, s );
        end
    end
else
    % there are no hybrid polynomials
    
    % list of matrices
    MList   = cf( IsT, TList,   HList   );
    % vector of unique values
    MVecUnq = cf( IsT, TVecUnq, HVecUnq );
    
    if ~IsD
        % no pos. dom.
        
        % for every polynomial
        for i = 1 : nPol
            
            t0 = cputime;
            
            % print ppol, n, k
            p3p_fprintf( FID, '%3d : %10s %3d', i, [ '[' num2str( nOld{ i } ) ']' ], k( i ) );
            % print nt, nr
            if isT( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.trigonometric );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            if isR( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.real );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            % print Qsize, isC, isD, isBRL, isSB
            p3p_fprintf( FID, ' %6d   %1d   %1d     %1d    %1d', Qsize( i ), isC( i ), 0, isBRL( i ), isSB( i ) );
                       
            if ~isSB( i )
                [ D, Line ] = coef_var_mat_multi_tr_pol   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                            isT( i ), MList, MVecUnq, QOldsize( i ), isBRL( i ), BRLK2( i ) );
            else
                [ D, Line ] = coef_var_mat_multi_tr_pol_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                            isT( i ), MList, MVecUnq, isBRL( i ), BRLK2( i ), ...
                                                            SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
            end

            % print time
            t0 = cputime - t0;
            [ h, m, s ] = sec2hour( t0 );
            p3p_fprintf( FID, '  %2d:%2d:%2d\n' , h, m, s );
        end
    else
        % with pos. dom.
        
        % for every polynomial
        for i = 1 : nPol
            
            t0 = cputime;
            
            % print ppol, n, k
            p3p_fprintf( FID, '%3d : %10s %3d', i, [ '[' num2str( nOld{ i } ) ']' ], k( i ) );
            % print nt, nr
            if isT( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.trigonometric );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            if isR( i )
                p3p_fprintf( FID, '  %1d', ptype{ i }.real );
            else
                p3p_fprintf( FID, '  %1d', 0 );
            end
            % print Qsize, isC, isD, isBRL, isSB
            p3p_fprintf( FID, ' %6d   %1d   %1d     %1d    %1d', Qsize( i ), isC( i ), 1, isBRL( i ), isSB( i ) );
            
            if ~isD( i )
                % no pos. dom.
                if ~isSB( i )
                    [ D, Line ] = coef_var_mat_multi_tr_pol   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                                isT( i ), MList, MVecUnq, QOldsize( i ), isBRL( i ), BRLK2( i ) );
                else
                    [ D, Line ] = coef_var_mat_multi_tr_pol_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                                isT( i ), MList, MVecUnq, isBRL( i ), BRLK2( i ), ...
                                                                SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i } );
                end
            else
                % with pos. dom.
                if ~isSB( i )
                    [ D, Line ] = coef_var_mat_multi_tr_pd   ( D, Line, LimD( i, : ), k( i ), nNonz{ i }, nNonzOld{ i }, ...
                                                               isT( i ), MList, MVecUnq, QOldsize( i ), ...
                                                               degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                               ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nNonzSi{ i }, ...
                                                               nNonzOldSi{ i }, dmpIdxAll, isBRL( i ), BRLK2( i ) );
                else
                    [ D, Line ] = coef_var_mat_multi_tr_pd_sb( D, Line, LimD( i, : ), k( i ), nNonz{ i }, ...
                                                               isT( i ), MList, MVecUnq, ...
                                                               degDNonz{ i }, ptype{ i }.dom.coef, ...
                                                               ptype{ i }.dom.nc, ncCS{ i }, nDomPol( i ), nDomPolCS( i ), ...
                                                               nNonzSi{ i }, nNonzOldSi{ i }, ...
                                                               dmpIdxAll, isBRL( i ), BRLK2( i ), ...
                                                               SBNrDegs( i ), SBDegsAddSub{ i }, SBPosDegs{ i }, ...
                                                               SBDegsAddSubDomPol, SBPosDegsDomPol );
                end
            end
            
            % print time
            t0 = cputime - t0;
            [ h, m, s ] = sec2hour( t0 );
            p3p_fprintf( FID, '  %2d:%2d:%2d\n' , h, m, s );
        end
    end
end