function [ D, Line ] = coef_var_mat_multi_h_pol_sb( D, Line, LimD, K, nzi, nt, TList, HList, TVecUnq, HVecUnq, isBRLi, BRLK2i, ...
                                                    SBNrDegsi, SBDegsAddSubi, SBPosDegsi )

% COEF_VAR_MAT_MULTI_H_POL_SB Create SeDuMi matrix for a multivariate hybrid polynomial
%
%   This function returns the D matrix, that we use
%   to transform the user's problem into a SeDuMi
%   problem.
%
%   The problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.04.09-15.09.11

% degree + 1
nzi1 = nzi + 1;

nNonzLen = length( nzi );

% for the 1st
% real variable
nt1 = nt + 1;

% find indexes in the "list"
% of indexes;
%
% trig. part
idx = sparse( nNonzLen, 1 );
for degi = 1 : nt
    idx( degi ) = find( TVecUnq == nzi( degi ) );
end
% real part
for degi = nt1 : nNonzLen
    idx( degi ) = find( HVecUnq == nzi( degi ) );
end

if ~isBRLi
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % no BRL, no need to extend the vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if K == 1
        % scalar coefficients

        % for every n-tuple
        for nLine = 1 : SBNrDegsi

            deg = SBDegsAddSubi( nLine, : );

            % init with the first degree, which is
            % certainly for a trig. variable (nt < nNonzLen)
            if deg( 1 ) >= 0
                T = TList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : )  ;
            else
                T = TList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )' ;
            end

            % the rest of the trig. variables (if any)
            for degi = 2 : nt
                if deg( degi ) >= 0
                    T = kron( TList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                else
                    T = kron( TList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                end
            end

            % the real degrees
            for degi = nt1 : nNonzLen
                T = kron( HList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
            end       

            % we add the vectorized
            % matrix from "trace"
            D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( cmatc( T, SBPosDegsi ) )';

            Line = Line + 1;
        end
    else
        % matrix coefficients

        % number of elem.
        % in matrix
        K2 = K ^ 2;
        
        % "half" of the
        % matrix
        hK = sumn( K );
        
        % for all the n-tuples
        for nLine = 1 : SBNrDegsi
            
            deg = SBDegsAddSubi( nLine, : );
            
            if iszero( deg( 1 : nt ) )
                
                % deg( nt+1 : end ) = (0,0,...,0);
                % the real part for the degree is zero =>
                % the coefficient is a symmetric matrix
                for elem = 1 : hK
                    
                    [ row, col ] = sind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ) , : );
                    else
                        T = TList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 )', : );
                    end
                    
                    % the rest of the trig. variables (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ) , T );
                        else
                            T = kron( TList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( degi ) }( deg( degi ) * nzi1( degi ) + 1 : ( deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                    end
                    
                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( kron( cmatc( T, SBPosDegsi ), E ) )';
                    
                    Line = Line + 1;
                end
            else
                % the non-symmetric matrices      
                
                for elem = 1 : K2
                    
                    [ row, col ] = ind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ) ;
                    else
                        T = TList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )';
                    end
                    
                    % the rest of the trig. variables (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                        else
                            T = kron( TList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                    end
                    
                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( kron( cmatc( T, SBPosDegsi ), E ) )';
                    
                    Line = Line + 1;
                end
            end
        end
    end
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % BRL, must extend the vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if K == 1
        % scalar coefficients

        % for every n-tuple
        for nLine = 1 : SBNrDegsi

            deg = SBDegsAddSubi( nLine, : );

            % init with the first degree, which is
            % certainly for a trig. variable (nt < nNonzLen)
            if deg( 1 ) >= 0
                T = TList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : )  ;
            else
                T = TList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )' ;
            end

            % the rest of the trig. variables (if any)
            for degi = 2 : nt
                if deg( degi ) >= 0
                    T = kron( TList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                else
                    T = kron( TList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                end
            end

            % the real degrees
            for degi = nt1 : nNonzLen
                T = kron( HList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
            end       

            % we add the extended vectorized
            % matrix from "trace"
            D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', cmatc( T, SBPosDegsi ), BRLK2i, [ 'r' 'd' ] ) )';

            Line = Line + 1;
        end
    else
        % matrix coefficients

        % number of elem.
        % in matrix
        K2 = K ^ 2;

        % "half" of the
        % matrix
        hK = sumn( K );
        
        % for all the n-tuples
        for nLine = 1 : SBNrDegsi
            
            deg = SBDegsAddSubi( nLine, : );            
            
            if iszero( deg( 1 : nt ) )
                
                % deg( nt+1 : end ) = (0,0,...,0);
                % the real part for the degree is zero =>
                % the coefficient is a symmetric matrix
                for elem = 1 : hK
                    
                    [ row, col ] = sind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ) , : );
                    else
                        T = TList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 )', : );
                    end
                    
                    % the rest of the trig. variables (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ) , T );
                        else
                            T = kron( TList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( degi ) }( deg( degi ) * nzi1( degi ) + 1 : ( deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                    end
                    
                    % we add the extended vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', kron( cmatc( T, SBPosDegsi ), E ), BRLK2i, [ 'r' 'd' ] ) )';
                    
                    Line = Line + 1;
                end
            else
                % the non-symmetric matrices 
                
                for elem = 1 : K2
                    
                    [ row, col ] = ind2sub( K, elem );
                    
                    % selector matrix
                    E = sparse( col, row, 1, K, K, 1 );
                    
                    % init with the first degree, which is
                    % certainly for a trig. variable (nt < nNonzLen)
                    if deg( 1 ) >= 0
                        T = TList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ) ;
                    else
                        T = TList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )';
                    end
                    
                    % the rest of the trig. variables (if any)
                    for degi = 2 : nt
                        if deg( degi ) >= 0
                            T = kron( TList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : )  , T );
                        else
                            T = kron( TList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )' , T );
                        end
                    end
                    
                    % the real degrees
                    for degi = nt1 : nNonzLen
                        T = kron( HList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                    end
                    
                    % we add the extended vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', kron( cmatc( T, SBPosDegsi ), E ), BRLK2i, [ 'r' 'd' ] ) )';
                    
                    Line = Line + 1;
                end
            end
        end
    end
end