function [ D, Line ] = coef_var_mat_multi_tr_pol( D, Line, LimD, K, nzi, nzoi, isT, MList, MVecUnq, QOsi, isBRLi, BRLK2i )

% COEF_VAR_MAT_MULTI_TR_POL Create SeDuMi matrix for a multivariate trig./real polynomial
%
%   This function returns the D matrix, that we use
%   to transform the user's problem into a SeDuMi
%   problem.
%
%   The problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 24.04.09-17.02.10
       
% degree + 1
nzi1 = nzi + 1;

nNonzLen = length( nzi );

if ~isBRLi
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % no BRL, no need to extend the vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if isT
        % trigonometric polynomial

        % the number of n-tuples
        NTuple = nch( nzi );
        nTuple = ntuple( nzi, NTuple, 1 );

        if K == 1
            % scalar coefficients

            % find indexes in the "list"
            % of indexes
            idx = sparse( nNonzLen, 1 );
            for degi = 1 : nNonzLen
                idx( degi ) = find( MVecUnq == nzi( degi ) );
            end

            % for every n-tuple
            for nLine = 1 : NTuple

                deg = nTuple( nLine, : );

                % init with the
                % first degree
                if deg( 1 ) >= 0
                    T = MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : );
                else
                    T = MList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )';
                end

                % the rest of the degrees
                for degi = 2 : nNonzLen
                    if deg( degi ) >= 0
                        T = kron( MList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
                    else
                        T = kron( MList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                    end
                end

                % we add the vectorized
                % matrix from "trace"
                D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( T )';

                Line = Line + 1;
            end
        else
            % matrix coefficients

            % number of elem.
            % in matrix
            K2 = K ^ 2;

            % "half" of the
            % matrix
            hK = sumn( K );

            % find indexes in the "list"
            % of indexes
            idx = sparse( nNonzLen, 1 );
            for degi = 1 : nNonzLen
                idx( degi ) = find( MVecUnq == nzi( degi ) );
            end

            % deg = (0,0,...,0);
            % the coefficient is a
            % symmetric matrix
            for elem = 1 : hK

                [ row, col ] = sind2sub( K, elem );

                % selector matrix     
                E = sparse( col, row, 1, K, K, 1 ); 

                % init with the
                % first degree
                T = kron( MList{ idx( 1 ) }(  1 : nzi1( 1 ), : ), E );

                % the rest of the degrees
                for degi = 2 : nNonzLen
                    T = kron( MList{ idx( degi ) }( 1 : nzi1( degi ), : ), T );
                end

                % we add the vectorized
                % matrix from "trace"
                D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( T )';

                Line = Line + 1;
            end

            % for the rest of the n-tuples
            for nLine = 2 : NTuple

                deg = nTuple( nLine, : );

                for elem = 1 : K2

                    [ row, col ] = ind2sub( K, elem );

                    % selector matrix     
                    E = sparse( col, row, 1, K, K, 1 ); 

                    % init with the
                    % first degree
                    if deg( 1 ) >= 0
                        T = kron( MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ),  E );
                    else
                        T = kron( MList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )', E );
                    end

                    % the rest of the degrees
                    for degi = 2 : nNonzLen
                        if deg( degi ) >= 0
                            T = kron( MList{ idx( degi ) }...
                                (  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
                        else
                            T = kron( MList{ idx( degi ) }...
                                ( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                        end
                    end

                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( T )';

                    Line = Line + 1;
                end
            end
        end
    else
        % real polynomial

        % the number of n-tuples
        NTuple = QOsi;
        nTuple = ntuple( nzoi, NTuple, 0 );

        if K == 1
            % scalar coefficients

            % find indexes in the "list"
            % of indexes
            idx = sparse( nNonzLen, 1 );
            for degi = 1 : nNonzLen
                idx( degi ) = find( MVecUnq == nzi( degi ) );
            end

            % for every n-tuple
            for nLine = 1 : NTuple

                deg = nTuple( nLine, : );

                % init with the
                % first degree
                T = MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : );

                % the rest of the degrees
                for degi = 2 : nNonzLen
                    T = kron( MList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                end

                % we add the vectorized
                % matrix from "trace"
                D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( T )';

                Line = Line + 1;
            end
        else
            % matrix coefficients

            % "half" of the
            % matrix
            hK = sumn( K );

            % find indexes in the "list"
            % of indexes
            idx = sparse( nNonzLen, 1 );
            for degi = 1 : nNonzLen
                idx( degi ) = find( MVecUnq == nzi( degi ) );
            end

            % for all the n-tuples
            for nLine = 1 : NTuple

                deg = nTuple( nLine, : );

                for elem = 1 : hK
                    [ row, col ] = sind2sub( K, elem );

                    % selector matrix     
                    E = sparse( col, row, 1, K, K, 1 ); 

                    % init with the
                    % first degree
                    T = kron( MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ), E );

                    % the rest of the degrees
                    for degi = 2 : nNonzLen
                        T = kron( MList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                    end

                    % we add the vectorized
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( T )';

                    Line = Line + 1;
                end
            end
        end
    end
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % BRL, must extend the vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if isT
    % trigonometric polynomial
    
    % the number of n-tuples
    NTuple = nch( nzi );
    nTuple = ntuple( nzi, NTuple, 1 );
    
    if K == 1
        % scalar coefficients
        
        % find indexes in the "list"
        % of indexes
        idx = sparse( nNonzLen, 1 );
        for degi = 1 : nNonzLen
            idx( degi ) = find( MVecUnq == nzi( degi ) );
        end
        
        % for every n-tuple
        for nLine = 1 : NTuple
            
            deg = nTuple( nLine, : );
            
            % init with the
            % first degree
            if deg( 1 ) >= 0
                T = MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : );
            else
                T = MList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )';
            end
            
            % the rest of the degrees
            for degi = 2 : nNonzLen
                if deg( degi ) >= 0
                    T = kron( MList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
                else
                    T = kron( MList{ idx( degi ) }( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                end
            end
            
            % we add the extended vectorized
            % matrix from "trace"
            D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', T, BRLK2i, [ 'r' 'd' ] ) )';
            
            Line = Line + 1;
        end
    else
        % matrix coefficients
        
        % number of elem.
        % in matrix
        K2 = K ^ 2;
        
        % "half" of the
        % matrix
        hK = sumn( K );
        
        % find indexes in the "list"
        % of indexes
        idx = sparse( nNonzLen, 1 );
        for degi = 1 : nNonzLen
            idx( degi ) = find( MVecUnq == nzi( degi ) );
        end
        
        % deg = (0,0,...,0);
        % the coefficient is a
        % symmetric matrix
        for elem = 1 : hK
            
            [ row, col ] = sind2sub( K, elem );
            
            % selector matrix     
            E = sparse( col, row, 1, K, K, 1 ); 
            
            % init with the
            % first degree
            T = kron( MList{ idx( 1 ) }(  1 : nzi1( 1 ), : ), E );
            
            % the rest of the degrees
            for degi = 2 : nNonzLen
                T = kron( MList{ idx( degi ) }( 1 : nzi1( degi ), : ), T );
            end
            
            % we add the vectorized
            % matrix from "trace"
            D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', T, BRLK2i, [ 'r' 'd' ] ) )';
                        
            Line = Line + 1;
        end
        
        % for the rest of the n-tuples
        for nLine = 2 : NTuple
            
            deg = nTuple( nLine, : );
            
            for elem = 1 : K2
                
                [ row, col ] = ind2sub( K, elem );
                
                % selector matrix     
                E = sparse( col, row, 1, K, K, 1 ); 
                
                % init with the
                % first degree
                if deg( 1 ) >= 0
                    T = kron( MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ),  E );
                else
                    T = kron( MList{ idx( 1 ) }( -deg( 1 ) * nzi1( 1 ) + 1 : ( -deg( 1 ) + 1 ) * nzi1( 1 ), : )', E );
                end
                
                % the rest of the degrees
                for degi = 2 : nNonzLen
                    if deg( degi ) >= 0
                        T = kron( MList{ idx( degi ) }...
                            (  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ),  T );
                    else
                        T = kron( MList{ idx( degi ) }...
                            ( -deg( degi ) * nzi1( degi ) + 1 : ( -deg( degi ) + 1 ) * nzi1( degi ), : )', T );
                    end
                end
                
                % we add the extended vectorized
                % matrix from "trace"
                D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', T, BRLK2i, [ 'r' 'd' ] ) )';
                
                Line = Line + 1;
            end
        end
    end
else
    % real polynomial
    
    % the number of n-tuples
    NTuple = QOsi;
    nTuple = ntuple( nzoi, NTuple, 0 );
    
    if K == 1
        % scalar coefficients
        
        % find indexes in the "list"
        % of indexes
        idx = sparse( nNonzLen, 1 );
        for degi = 1 : nNonzLen
            idx( degi ) = find( MVecUnq == nzi( degi ) );
        end
        
        % for every n-tuple
        for nLine = 1 : NTuple
            
            deg = nTuple( nLine, : );
            
            % init with the
            % first degree
            T = MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : );
            
            % the rest of the degrees
            for degi = 2 : nNonzLen
                T = kron( MList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
            end
            
            % we add the extended vectorized
            % matrix from "trace"
            D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', T, BRLK2i, [ 'r' 'd' ] ) )';
            
            Line = Line + 1;
        end
    else
        % matrix coefficients
        
        % "half" of the
        % matrix
        hK = sumn( K );
        
        % find indexes in the "list"
        % of indexes
        idx = sparse( nNonzLen, 1 );
        for degi = 1 : nNonzLen
            idx( degi ) = find( MVecUnq == nzi( degi ) );
        end
        
        % for all the n-tuples
        for nLine = 1 : NTuple
            
            deg = nTuple( nLine, : );
            
            for elem = 1 : hK
                [ row, col ] = sind2sub( K, elem );
                
                % selector matrix     
                E = sparse( col, row, 1, K, K, 1 ); 
                
                % init with the
                % first degree
                T = kron( MList{ idx( 1 ) }(  deg( 1 ) * nzi1( 1 ) + 1 : (  deg( 1 ) + 1 ) * nzi1( 1 ), : ), E );
                
                % the rest of the degrees
                for degi = 2 : nNonzLen
                    T = kron( MList{ idx( degi ) }(  deg( degi ) * nzi1( degi ) + 1 : (  deg( degi ) + 1 ) * nzi1( degi ), : ), T );
                end
                
                % we add the extended vectorized
                % matrix from "trace"
                D( Line, LimD( 1 ) : LimD( 2 ) ) = vec( mextend( '2D', 'zpd', T, BRLK2i, [ 'r' 'd' ] ) )';
                
                Line = Line + 1;
            end
        end
    end
    end
end