function [ R ] = sos_pol( p, varargin )

% SOS_POL Sum-of-squares polynomial
%
%   [ R ] = SOS_POL( P, PTYPE ) creates a polynomial variable for the
%   CVX library using the P and PTYPE structures.
%
%   [ R ] = SOS_POL( [ N K ] ) creates a trigonometric polynomial variable
%   according to the multivariate degree N and size of coefficients K.
%
%   [ R ] = SOS_POL( P, PTYPE, PARS ) uses also the structure PARS.
%
%   If one doesn't have a Bounded Real Lemma (BRL) then the variable R 
%   describes a sum-of-squares polynomial, i.e. is the coefficients of
%   the polynomial.
%
%   If one wants to describe a BRL then you describe the problem
%
%   ||H|| < |A|, globally or on a positivity domain,
%
%   and you obtain (RR = |A|^2) * I > |H|^2. In this case the output 
%   variable R will hold [ H; RR ].
%
%   When one describes a polynomial with matrix coefficients R contains 
%   stacked vectorized coefficients. (All the rules describing the 
%   polynomials apply here. See the User's Guide for more information.)
%   
%   Fields of the structure P: 
%   
%   [ N K ] - where N is a vector holding the degree of the multivariable 
%   polynomial and K is the size of the KxK coefficients.
%
%   Fields of the structure PTYPE: 
%
%   trigonometric - number of trigonometric variables for the polynomial
%   real          - number of real          variables for the polynomial
%   int           - positivity on interval for univariable polynomial
%   complex_coef  - set this to 1 when the polynomial is complex
%   dom           - used when the polynomial is positive only on a domain
%                   to describe the positivity polynomial          
%      - dom.deg  - matrix which hold on it's lines the degrees for the 
%                   polynomial describing the positivity domain    
%      - dom.coef - vector of coefficients describing the polynomial for
%                   the positivity domain (the order of the coefficients
%                   and of the degrees is the same)
%      - dom.nc   - number of coefficients for each polynomial describing
%                   the positivity domain          
%      - dom.nunion - vector with nr. of pols. for each domain from the union
%   brl           - the variable R will describe a BRL
%      - brl.hsize - describes the coefficients of A, of size K1xK2
%
%   Fields of the structure PARS:
%
%   fid           - file descriptor to redirect the output of POS3POLY;
%                   if set to 0 then no output will be provided

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 08.04.10-07.10.10

% check if the 'p' fields
% are nonempty
assert( ~isempty( p ), 'Empty variable ''p''.' );

% the version of POS3POLY
global VER
VER = '3.1';

% square root
% of -1;
% j
global SR FID;
SR = 1i;

% file descriptor to print at
lenIn = length( varargin );
if  lenIn == 2
    if isfield( varargin{ 2 }, 'fid' )
        FID = varargin{ 2 }.fid;
    else
        FID = 1;
    end
    % 'ptype'
    ptype = varargin{ 1 };
elseif lenIn == 1
    FID = 1;
    % 'ptype'
    ptype = varargin{ 1 };
elseif lenIn == 0
    % no 'ptype' structure
    FID = 1;
else
    error( 'Wrong number of input arguments.' );
end

% compute the D matrix
if ~lenIn
    % no 'ptype'
    [ D, Ks, nCoef, k, isT, isC, isBRL, BRLK1, BRLK2, IsBRLK1, nNonzProd1, nCoefPol, isnunion, nULen1, pNU, ptypeNU ] = dmat( p        );
else
    % with 'ptype'
    [ D, Ks, nCoef, k, isT, isC, isBRL, BRLK1, BRLK2, IsBRLK1, nNonzProd1, nCoefPol, isnunion, nULen1, pNU, ptypeNU ] = dmat( p, ptype );
end

% check if we have 'nunion'
if isnunion
    % we have reunion of domains
    
    DNU   = cell( nULen1, 1 );
    KsNU  = cell( nULen1, 1 );
    NQiNU = cell( nULen1, 1 );
    Q2SNU = cell( nULen1, 1 );
    L1NU  = cell( nULen1, 1 );
    L2NU  = cell( nULen1, 1 );
    
    for i = 1 : nULen1
        [ DNU{ i }, KsNU{ i } ] = dmat( pNU{ i }, ptypeNU{ i } );
        
        % number of Gram matrices
        NQiNU{ i } = length( KsNU{ i }.s );

        Q2SNU{ i } = KsNU{ i }.s .^ 2;

        L1NU{ i } = zeros( NQiNU{ i }, 1 );
        L2NU{ i } = zeros( NQiNU{ i }, 1 );
        L1NU{ i }( 1 ) = 1;
        L2NU{ i }( 1 ) = Q2SNU{ i }( 1 );
        for j = 2 : NQiNU{ i }
            L1NU{ i }( j ) = L1NU{ i }( j - 1 ) + Q2SNU{ i }( j - 1 );
            L2NU{ i }( j ) = L2NU{ i }( j - 1 ) + Q2SNU{ i }( j     );
        end
    end
end

% number of Gram matrices
NQi = length( Ks.s );

Q2S = Ks.s .^ 2;

L1 = zeros( NQi, 1 );
L2 = zeros( NQi, 1 );
L1( 1 ) = 1;
L2( 1 ) = Q2S( 1 );
for i = 2 : NQi
    L1( i ) = L1( i - 1 ) + Q2S( i - 1 );
    L2( i ) = L2( i - 1 ) + Q2S( i     );
end

% create de CVX set to describe 
% the positive polynomial
cvx_begin SET SDP
    if ~isBRL
        % no BRL
        
        if ~isC
            % real polynomial

            % the coefs of the pol.
            variable R( nCoef );

            % the Gram matrices 
            for i = 1 : NQi
                eval( [ 'variable Q', num2str( i - 1 ), '(', num2str( Ks.s( i ) ), ',', num2str( Ks.s( i ) ), ') symmetric;' ] );
            end
            
            % for vectorizing the Gram matrices
            variable V( sum( Q2S ) );
        else        
            % complex polynomial

            % the coefs of the pol.
            variable R( nCoef ) complex;

            % the Gram matrices 
            for i = 1 : NQi
                eval( [ 'variable Q', num2str( i - 1 ), '(', num2str( Ks.s( i ) ), ',', num2str( Ks.s( i ) ), ') hermitian;' ] );
            end
            
            % for vectorizing the Gram matrices
            variable V( sum( Q2S ) ) complex;
        end

        % PSD Gram matrices
        for i = 1 : NQi
            eval( [ 'Q', num2str( i - 1 ),  ' >= 0; ' ] );
        end
        
        % put the Gram matrices into the vector V
        for i = 1 : NQi
            V( L1( i ) : L2( i ) ) == vec( eval( [ 'Q', num2str( i - 1 ) ] ) );
        end

        subject to

        % "trace" constraints between the coefs.
        % of the pol. and the Gram matrix
        R == D * V;
    else
        % BRL
        
        kBRLK2           = k           * BRLK2;
        nNonzProd1k      = nNonzProd1  * k;
        nNonzProd1kBRLK2 = nNonzProd1k * BRLK2;
        
        if ~IsBRLK1
            % scalar BRL
            
            if ~isC
                % real polynomial
                
                % variable R contanins coefs. for filter H and coefs. of pol. R;
                % RVar = [ H; R ]
                % |H| < |A|
                % R = A * A'
                variable R( nNonzProd1kBRLK2 + nCoef );
                
                % the Gram matrices
                for i = 1 : NQi
                    eval( [ 'variable Q', num2str( i - 1 ), '(', num2str( Ks.s( i ) ), ',', num2str( Ks.s( i ) ), ') symmetric;' ] );
                end
                
                % for vectorizing the Gram matrices
                variable V( sum( Q2S ) );
            else
                % complex polynomial
                
                % variable R contanins coefs. for filter H and coefs. of pol. R;
                % RVar = [ H; R ]
                % |H| < |A|
                % R = A * A'
                variable R( nNonzProd1kBRLK2 + nCoef ) complex;
                
                % the Gram matrices
                for i = 1 : NQi
                    eval( [ 'variable Q', num2str( i - 1 ), '(', num2str( Ks.s( i ) ), ',', num2str( Ks.s( i ) ), ') hermitian;' ] );
                end
                
                % for vectorizing the Gram matrices
                variable V( sum( Q2S ) ) complex;
            end
            
            % PSD Gram matrices
            for i = 1 : NQi
                eval( [ 'Q', num2str( i - 1 ),  ' >= 0; ' ] );
            end
            
            % put the Gram matrices into the vector V
            for i = 1 : NQi
                V( L1( i ) : L2( i ) ) == vec( eval( [ 'Q', num2str( i - 1 ) ] ) );
            end
            
            subject to
            
            % "trace" constraints between the coefs.
            % of the pol. R and the Gram matrix
            R( nNonzProd1kBRLK2 + 1 : end ) == D * V;
            
            % introduce the filter H in Q0
            for i = 1 : nNonzProd1
                vec( Q0( ( i - 1 ) * k + 1 : i * k , nNonzProd1k + 1 : end ) ) == R( ( i - 1 ) * kBRLK2 + 1 : i * kBRLK2 );
            end
            
            % introduce "half" the identity matrix in Q0
            vecs2( Q0( nNonzProd1k + 1 : end, nNonzProd1k + 1 : end ) ) == vecs2( speye( BRLK2 ) );
            
        else
            % matrix BRL
            
            % number of variables added 
            % for the matrix coefficients
            nCoefPolDif = nCoef - nCoefPol;
            
            % number of variables in half a matrix
            BRLK1Half = sumn( BRLK1 );
            % number of variables in a whole matrix
            BRLK1All  = BRLK1 ^ 2;
            % we jump accord to weather the coefficient is symmetric (real pol.)
            % or not (trigonometric or hybrid)
            Jump = cf( ~isT, BRLK1Half, BRLK1All );
            
            if ~isC
                % real polynomial
                
                % variable R contanins coefs. for filter H and coefs. of pol. R;
                % RVar = [ H; R ]
                % ||H|| < |A|
                % R = A * A'
                variable R( nNonzProd1kBRLK2 + nCoefPol );
                % the variables added for the matrix coefs.
                variable RM( nCoefPolDif );
                % all the variables for the matrix coefficients
                variable RRM( nCoef );
                
                % the Gram matrices
                for i = 1 : NQi
                    eval( [ 'variable Q', num2str( i - 1 ), '(', num2str( Ks.s( i ) ), ',', num2str( Ks.s( i ) ), ') symmetric;' ] );
                end
                
                % for vectorizing the Gram matrices
                variable V( sum( Q2S ) );
            else
                % complex polynomial
                
                % variable R contanins coefs. for filter H and coefs. of pol. R;
                % RVar = [ H; R ]
                % ||H|| < |A|
                % R = A * A'
                variable R( nNonzProd1kBRLK2 + nCoefPol ) complex;
                % the variables added for the matrix coefs.
                variable RM( nCoefPolDif ) complex;
                % all the variables for the matrix coefficients
                variable RRM( nCoef ) complex;
                
                % the Gram matrices
                for i = 1 : NQi
                    eval( [ 'variable Q', num2str( i - 1 ), '(', num2str( Ks.s( i ) ), ',', num2str( Ks.s( i ) ), ') hermitian;' ] );
                end
                
                % for vectorizing the Gram matrices
                variable V( sum( Q2S ) ) complex;
            end
            
            % PSD Gram matrices
            for i = 1 : NQi
                eval( [ 'Q', num2str( i - 1 ),  ' >= 0; ' ] );
            end
            
            % put the Gram matrices into the vector V
            for i = 1 : NQi
                V( L1( i ) : L2( i ) ) == vec( eval( [ 'Q', num2str( i - 1 ) ] ) );
            end
            
            subject to
            
            % "trace" constraints between the coefs.
            % of the pol. R and the Gram matrix
            RRM == D * V;
            
            % constrains for the variables from the scalar constraints;
            % first variable, because the first coefficient is always symmetric
            R( nNonzProd1kBRLK2 + 1 ) == RRM( 1 );
            % the rest
            for i = 2 : nCoefPol
                R( nNonzProd1kBRLK2 + i ) == RRM( BRLK1Half + ( i - 2 ) * Jump + 1 );
            end
            
            % constrains for the variables added 
            % for the matrix coefficients
            Line = 1;
            Col  = 2;
            % the first column of the symmetric matrix
            for k1 = 2 : BRLK1
                RM( Line ) == RRM( Col );
                Line = Line + 1;
                Col = Col + 1;
            end
            % the rest of the columns
            for k2 = 2 : BRLK1
                for k1 = k2 : BRLK1
                    RM( Line ) == RRM( Col );
                    if k1 == k2
                        RM( Line ) == R( nNonzProd1kBRLK2 + 1 );
                    end
                    Line = Line + 1;
                    Col = Col + 1;
                end
            end
            Col = Col + 1;
            % the rest of the degrees; rest of the matrix coefficients
            %
            if ~isT
                % real polynomial: symmetric coefficients
                
                for i = 1 : nCoefPol - 1
                    % the first column of the symmetric matrix
                    for k1 = 2 : BRLK1
                        RM( Line ) == RRM( Col );
                        Line = Line + 1;
                        Col = Col + 1;
                    end
                    % the rest of the columns
                    for k2 = 2 : BRLK1
                        for k1 = k2 : BRLK1
                            RM( Line ) == RRM( Col );
                            if k1 == k2
                                RM( Line ) == R( nNonzProd1kBRLK2 + i + 1 );
                            end
                            Line = Line + 1;
                            Col = Col + 1;
                        end
                    end
                    
                    Col = Col + 1;
                end
            else
                % trig. or hybrid pol.: nonsymmetric coefficients
                
                for i = 1 : nCoefPol - 1
                    % the first column of the nonsymmetric matrix
                    for k1 = 2 : BRLK1
                        RM( Line ) == RRM( Col );
                        Line = Line + 1;
                        Col = Col + 1;
                    end
                    % the rest of the columns
                    for k2 = 2 : BRLK1
                        for k1 = 1 : BRLK1
                            RM( Line ) == RRM( Col );
                            if k1 == k2
                                RM( Line ) == R( nNonzProd1kBRLK2 + i + 1 );
                            end
                            Line = Line + 1;
                            Col = Col + 1;
                        end
                    end
                    
                    Col = Col + 1;
                end
            end
            
            % introduce the filter H in Q0
            for i = 1 : nNonzProd1
                vec( Q0( ( i - 1 ) * k + 1 : i * k , nNonzProd1k + 1 : end ) ) == R( ( i - 1 ) * kBRLK2 + 1 : i * kBRLK2 );
            end
            
            % introduce "half" the identity matrix in Q0
            vecs2( Q0( nNonzProd1k + 1 : end, nNonzProd1k + 1 : end ) ) == vecs2( speye( BRLK2 ) );
        end
    end
cvx_end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% for reunion of intervals, add polynomials
if isnunion
    % create de CVX set to describe 
    % the positive polynomial
    cvx_begin SET SDP
        if ~isBRL
            % no BRL

            if ~isC
                % real polynomial

                % the coefs of the pol.
                for j = 1 : nULen1
                    eval( [ 'variable RNU', num2str( j ), '(', num2str( nCoef ), ');' ] );
                end

                % the Gram matrices 
                for j = 1 : nULen1
                    for i = 1 : NQi
                        eval( [ 'variable QNU', num2str( j ), num2str( i - 1 ), '(', num2str( KsNU{ j }.s( i ) ), ',', ...
                                                                                     num2str( KsNU{ j }.s( i ) ), ') symmetric;' ] );
                    end
                end

                % for vectorizing the Gram matrices
                for j = 1 : nULen1
                    eval( [ 'variable VNU', num2str( j ), '(', num2str( sum( Q2SNU{ j } ) ), ');' ] );
                end
            else        
                % complex polynomial

                % the coefs of the pol.
                for j = 1 : nULen1
                    eval( [ 'variable RNU', num2str( j ), '(', num2str( nCoef ), ') complex;' ] );
                end

                % the Gram matrices 
                for j = 1 : nULen1
                    for i = 1 : NQi
                        eval( [ 'variable QNU', num2str( j ), num2str( i - 1 ), '(', num2str( KsNU{ j }.s( i ) ), ',', ...
                                                                                     num2str( KsNU{ j }.s( i ) ), ') hermitian;' ] );
                    end
                end

                % for vectorizing the Gram matrices
                for j = 1 : nULen1
                    eval( [ 'variable VNU', num2str( j ), '(', num2str( sum( Q2SNU{ j } ) ), ') complex;' ] );
                end
            end

            % PSD Gram matrices
            for j = 1 : nULen1
                for i = 1 : NQi
                    eval( [ 'QNU', num2str( j ), num2str( i - 1 ),  ' >= 0; ' ] );
                end
            end

            % put the Gram matrices into the vector V
            for j = 1 : nULen1
                for i = 1 : NQi
                    eval( [ 'VNU', num2str( j ), '(L1NU{', num2str( j ), '}(', num2str( i ), '): L2NU{', num2str( j ),'}(',num2str( i ), '))' ] ) == vec( eval( [ 'QNU', num2str( j ), num2str( i - 1 ) ] ) );
                end
            end

            subject to

            for j = 1 : nULen1
                % the polynomial is equal to the original one
                eval( [ 'RNU', num2str( j ) ] ) == R;
                
                % "trace" constraints between the coefs.
                % of the pol. and the Gram matrix
                eval( [ 'RNU', num2str( j ) ] ) == DNU{ j } *  eval( [ 'VNU', num2str( j ) ] );
            end
        else
            % BRL

            kBRLK2           = k           * BRLK2;
            nNonzProd1k      = nNonzProd1  * k;
            nNonzProd1kBRLK2 = nNonzProd1k * BRLK2;

            if ~IsBRLK1
                % scalar BRL

                if ~isC
                    % real polynomial

                    % variable R contanins coefs. for filter H and coefs. of pol. R;
                    % RVar = [ H; R ]
                    % ||H|| < |A|
                    % R = A * A'
                    for j = 1 : nULen1
                        eval( [ 'variable RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + nCoef ), ');' ] );
                    end

                    % the Gram matrices 
                    for j = 1 : nULen1
                        for i = 1 : NQi
                            eval( [ 'variable QNU', num2str( j ), num2str( i - 1 ), '(', num2str( KsNU{ j }.s( i ) ), ',', ...
                                                                                         num2str( KsNU{ j }.s( i ) ), ') symmetric;' ] );
                        end
                    end

                    % for vectorizing the Gram matrices
                    for j = 1 : nULen1
                        eval( [ 'variable VNU', num2str( j ), '(', num2str( sum( Q2SNU{ j } ) ), ');' ] );
                    end
                else
                    % complex polynomial

                    % variable R contanins coefs. for filter H and coefs. of pol. R;
                    % RVar = [ H; R ]
                    % ||H|| < |A|
                    % R = A * A'
                    for j = 1 : nULen1
                        eval( [ 'variable RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + nCoef ), ') complex;' ] );
                    end

                    % the Gram matrices 
                    for j = 1 : nULen1
                        for i = 1 : NQi
                            eval( [ 'variable QNU', num2str( j ), num2str( i - 1 ), '(', num2str( KsNU{ j }.s( i ) ), ',', ...
                                                                                         num2str( KsNU{ j }.s( i ) ), ') hermitian;' ] );
                        end
                    end

                    % for vectorizing the Gram matrices
                    for j = 1 : nULen1
                        eval( [ 'variable VNU', num2str( j ), '(', num2str( sum( Q2SNU{ j } ) ), ') complex;' ] );
                    end
                end

                % PSD Gram matrices
                for j = 1 : nULen1
                    for i = 1 : NQi
                        eval( [ 'QNU', num2str( j ), num2str( i - 1 ),  ' >= 0; ' ] );
                    end
                end

                % put the Gram matrices into the vector V
                for j = 1 : nULen1
                    for i = 1 : NQi
                        eval( [ 'VNU', num2str( j ), '(L1NU{', num2str( j ), '}(', num2str( i ), '): L2NU{', num2str( j ),'}(',num2str( i ), '))' ] ) == vec( eval( [ 'QNU', num2str( j ), num2str( i - 1 ) ] ) );
                    end
                end

                subject to

                % "trace" constraints between the coefs.
                % of the pol. R and the Gram matrix
                
                for j = 1 : nULen1
                    % the polynomial is equal to the original one
                    eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + 1 ), ':end )' ] ) == R( nNonzProd1kBRLK2 + 1 : end );
                    % H is equal to the original one
                    eval( [ 'RNU', num2str( j ), '(', num2str( 1 ), ':', num2str( nNonzProd1kBRLK2 ), ')' ] ) == R( 1 : nNonzProd1kBRLK2 );

                    % "trace" constraints between the coefs.
                    % of the pol. and the Gram matrix
                    eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + 1 ), ':end )' ] ) == DNU{ j } *  eval( [ 'VNU', num2str( j ) ] );
                end

                % introduce the filter H in Q0
                for j = 1 : nULen1
                    for i = 1 : nNonzProd1
                        vec( eval( [ 'QNU', num2str( j ), '0(', num2str( ( i - 1 ) * k + 1 ), ':', num2str( i * k ), ',', num2str( nNonzProd1k + 1 ), ': end )' ] ) ) == eval( [ 'RNU', num2str( j ), '(', num2str( ( i - 1 ) * kBRLK2 + 1 ), ':', num2str( i * kBRLK2 ), ')' ] );
                    end
                end

                % introduce "half" the identity matrix in Q0
                for j = 1 : nULen1
                    vecs2( eval( [ 'QNU', num2str( j ), '0(', num2str( nNonzProd1k + 1 ), ':end,', num2str( nNonzProd1k + 1 ), ':end )' ] ) ) == vecs2( speye( BRLK2 ) );
                end

            else
                % matrix BRL

                % number of variables added 
                % for the matrix coefficients
                nCoefPolDif = nCoef - nCoefPol;

                % number of variables in half a matrix
                BRLK1Half = sumn( BRLK1 );
                % number of variables in a whole matrix
                BRLK1All  = BRLK1 ^ 2;
                % we jump accord to weather the coefficient is symmetric (real pol.)
                % or not (trigonometric or hybrid)
                Jump = cf( ~isT, BRLK1Half, BRLK1All );

                if ~isC
                    % real polynomial

                    % variable R contanins coefs. for filter H and coefs. of pol. R;
                    % RVar = [ H; R ]
                    % ||H|| < |A|
                    % R = A * A'
                    for j = 1 : nULen1
                        eval( [ 'variable RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + nCoefPol ), ');' ] );
                    end
                    % the variables added for the matrix coefs.
                    for j = 1 : nULen1
                        eval( [ 'variable RMNU', num2str( j ), '(', num2str( nCoefPolDif ), ');' ] );
                    end
                    % all the variables for the matrix coefficients
                    for j = 1 : nULen1
                        eval( [ 'variable RRMNU', num2str( j ), '(', num2str( nCoef ), ');' ] );
                    end

                    % the Gram matrices
                    for j = 1 : nULen1
                        for i = 1 : NQi
                            eval( [ 'variable QNU', num2str( j ), num2str( i - 1 ), '(', num2str( KsNU{ j }.s( i ) ), ',', ...
                                                                                         num2str( KsNU{ j }.s( i ) ), ') symmetric;' ] );
                        end
                    end

                    % for vectorizing the Gram matrices
                    for j = 1 : nULen1
                        eval( [ 'variable VNU', num2str( j ), '(', num2str( sum( Q2SNU{ j } ) ), ');' ] );
                    end
                else
                    % complex polynomial

                    % variable R contanins coefs. for filter H and coefs. of pol. R;
                    % RVar = [ H; R ]
                    % ||H|| < |A|
                    % R = A * A'
                    for j = 1 : nULen1
                        eval( [ 'variable RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + nCoefPol ), ') complex;' ] );
                    end
                    % the variables added for the matrix coefs.
                    for j = 1 : nULen1
                        eval( [ 'variable RMNU', num2str( j ), '(', num2str( nCoefPolDif ), ') complex;' ] );
                    end
                    % all the variables for the matrix coefficients
                    for j = 1 : nULen1
                        eval( [ 'variable RRMNU', num2str( j ), '(', num2str( nCoef ), ') complex;' ] );
                    end

                    % the Gram matrices
                    for j = 1 : nULen1
                        for i = 1 : NQi
                            eval( [ 'variable QNU', num2str( j ), num2str( i - 1 ), '(', num2str( KsNU{ j }.s( i ) ), ',', ...
                                                                                         num2str( KsNU{ j }.s( i ) ), ') hermitian;' ] );
                        end
                    end

                    % for vectorizing the Gram matrices
                    for j = 1 : nULen1
                        eval( [ 'variable VNU', num2str( j ), '(', num2str( sum( Q2SNU{ j } ) ), ') complex;' ] );
                    end
                end

                % PSD Gram matrices
                for j = 1 : nULen1
                    for i = 1 : NQi
                        eval( [ 'QNU', num2str( j ), num2str( i - 1 ),  ' >= 0; ' ] );
                    end
                end

                % put the Gram matrices into the vector V
                for j = 1 : nULen1
                    for i = 1 : NQi
                        eval( [ 'VNU', num2str( j ), '(L1NU{', num2str( j ), '}(', num2str( i ), '): L2NU{', num2str( j ),'}(',num2str( i ), '))' ] ) == vec( eval( [ 'QNU', num2str( j ), num2str( i - 1 ) ] ) );
                    end
                end

                subject to

                % "trace" constraints between the coefs.
                % of the pol. R and the Gram matrix
                for j = 1 : nULen1
                    % the polynomial is equal to the original one
                    eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + 1 ), ':end )' ] ) == R( nNonzProd1kBRLK2 + 1 : end );
                    % H is equal to the original one
                    eval( [ 'RNU', num2str( j ), '(', num2str( 1 ), ':', num2str( nNonzProd1kBRLK2 ), ')' ] ) == R( 1 : nNonzProd1kBRLK2 );

                    % "trace" constraints between the coefs.
                    % of the pol. and the Gram matrix
                    eval( [ 'RRMNU', num2str( j ) ] ) == DNU{ j } *  eval( [ 'VNU', num2str( j ) ] );
                end

                % constrains for the variables from the scalar constraints;
                % first variable, because the first coefficient is always symmetric
                for j = 1 : nULen1
                    eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + 1 ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( 1 ), ')' ] );
                    % the rest
                    for i = 2 : nCoefPol
                        eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + i ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( BRLK1Half + ( i - 2 ) * Jump + 1 ), ')' ] );
                    end
                end

                % constrains for the variables added 
                % for the matrix coefficients
                if ~isT
                    % real polynomial: symmetric coefficients
                    
                    for j = 1 : nULen1
                        Line = 1;
                        Col  = 2;
                        % the first column of the symmetric matrix
                        for k1 = 2 : BRLK1
                            eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                            Line = Line + 1;
                            Col = Col + 1;
                        end
                        % the rest of the columns
                        for k2 = 2 : BRLK1
                            for k1 = k2 : BRLK1
                                eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                                if k1 == k2
                                    eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + 1 ), ')' ] );
                                end
                                Line = Line + 1;
                                Col = Col + 1;
                            end
                        end
                        Col = Col + 1;
                        % the rest of the degrees; rest of the matrix coefficients
                        %
                        % real polynomial: symmetric coefficients
                        
                        for i = 1 : nCoefPol - 1
                            % the first column of the symmetric matrix
                            for k1 = 2 : BRLK1
                                eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                                Line = Line + 1;
                                Col = Col + 1;
                            end
                            % the rest of the columns
                            for k2 = 2 : BRLK1
                                for k1 = k2 : BRLK1
                                    eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                                    if k1 == k2
                                        eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + i + 1 ), ')' ] );
                                    end
                                    Line = Line + 1;
                                    Col = Col + 1;
                                end
                            end
                            
                            Col = Col + 1;
                        end
                    end
                else
                    % trig. or hybrid pol.: nonsymmetric coefficients
                    
                    for j = 1 : nULen1
                        Line = 1;
                        Col  = 2;
                        % the first column of the symmetric matrix
                        for k1 = 2 : BRLK1
                            eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                            Line = Line + 1;
                            Col = Col + 1;
                        end
                        % the rest of the columns
                        for k2 = 2 : BRLK1
                            for k1 = k2 : BRLK1
                                eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                                if k1 == k2
                                    eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + 1 ), ')' ] );
                                end
                                Line = Line + 1;
                                Col = Col + 1;
                            end
                        end
                        Col = Col + 1;
                        % the rest of the degrees; rest of the matrix coefficients
                        %
                        % trig. or hybrid pol.: nonsymmetric coefficients
                        
                        for i = 1 : nCoefPol - 1
                            % the first column of the nonsymmetric matrix
                            for k1 = 2 : BRLK1
                                eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                                Line = Line + 1;
                                Col = Col + 1;
                            end
                            % the rest of the columns
                            for k2 = 2 : BRLK1
                                for k1 = 1 : BRLK1
                                    eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RRMNU', num2str( j ), '(', num2str( Col ), ')' ] );
                                    if k1 == k2
                                        eval( [ 'RMNU', num2str( j ), '(', num2str( Line ), ')' ] ) == eval( [ 'RNU', num2str( j ), '(', num2str( nNonzProd1kBRLK2 + i + 1 ), ')' ] );
                                    end
                                    Line = Line + 1;
                                    Col = Col + 1;
                                end
                            end
                            
                            Col = Col + 1;
                        end
                    end
                end

                % introduce the filter H in Q0
                for j = 1 : nULen1
                    for i = 1 : nNonzProd1
                        vec( eval( [ 'QNU', num2str( j ), '0(', num2str( ( i - 1 ) * k + 1 ), ':', num2str( i * k ), ',', num2str(nNonzProd1k + 1 ), ':end)' ] ) ) == eval( [ 'RNU', num2str( j ), '(', num2str( ( i - 1 ) * kBRLK2 + 1 ), ':', num2str( i * kBRLK2 ), ')' ] );
                    end
                end

                % introduce "half" the identity matrix in Q0
                for j = 1 : nULen1
                    vecs2( eval( [ 'QNU', num2str( j ), '0(', num2str( nNonzProd1k + 1 ), ': end,', num2str( nNonzProd1k + 1 ),  ':end)' ] ) ) == vecs2( speye( BRLK2 ) );
                end
            end
        end
    cvx_end
end