function [ SBDegsAddSub, SBNrDegs, SBPosDegs, SBNrDegsMono ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, IndDeg, isSB, nt )

% GENSPBASIS Generate degrees from sparse basis
%
%   This function computes the degrees for which we
%   write the trace equalities when we use sparse basis.
%
%   INPUT:
%
%     N            - Degree of the polynomial. Real degrees in half
%     NOLD         - Degree of the polynomial. No change to the real degrees
%     D            - D - the number of variable of the polynomial
%     IST          - IST - if we have trigonometric variables
%     ISR          - ISR - if we have real variables
%     NNONZPROD1   - The product of ( the degree plus one )
%     INFDEG       - IND/DEG according to ISSB
%     ISSB         - ISSB for the polynomial. If is 1 then INDDEG is a
%                    vector of indices which points the basis. If is 2 then
%                    INDDEG is a matrix, whose rows are the degrees from the
%                    sparse basis
%     NT           - Number of trigonometric variables. (Used only for hybrid
%                    polynomials.)
%
%   OUTPUT:
%
%     SBDEGSADDSUB - the degrees for which we write the equalities
%     SBNRDEGS     - the number of degrees
%     SBPOSDEGS    - the position of the degrees in the basis
%     SBNRDEGSMONO - the numnber of degrees in the basis

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 16.06.10-25.06.10

if isSB == 1
    % if isSB 1, then check the number of monomials from
    % the basis to be equal with nNonzProd1
    if ceil( nNonzProd1 ) ~= length( IndDeg )
        error( 'Wrong number of elements in the basis of the polynomial.' );
    end
    
    % position with nonzero degrees from the basis
    SBPosDegs = find( IndDeg );
    % number of nonzero degrees from the sparse basis
    SBNrDegs = length( SBPosDegs );
    % degrees from the sparse basis
    SBDegs = sparse( SBNrDegs, d );
    for j = 1 : SBNrDegs
        SBDegs( j, : ) = rind2sub( SBPosDegs( j ), n );
    end
    
    % create the degrees for which we write
    % the equalities
    if isT && isR
        % hybrid polynomial
        
        SBNrDegs2 = SBNrDegs ^ 2;
        % SBDegsRep makes a repmat on all the degrres once
        SBDegsRep = repmat( SBDegs, SBNrDegs, 1 );
        % SBDegsRepEach makes a repmat on every degree one by one
        SBDegsRepEach = zeros( SBNrDegs2, d );
        for j = 1 : SBNrDegs
            SBDegsRepEach( ( j - 1 ) * SBNrDegs + 1 : j * SBNrDegs, : ) = repmat( SBDegs( j, : ), SBNrDegs, 1 );
        end
        SBDegsAddSub = zeros( SBNrDegs ^ 2, d );
        
        % subtract and add degrees for hybrid
        SBDegsAddSub( :, 1 : nt       ) = SBDegsRep    ( :, 1 : nt       ) - ...
            SBDegsRepEach( :, 1 : nt       );
        SBDegsAddSub( :, nt + 1 : end ) = SBDegsRep    ( :, nt + 1 : end ) + ...
            SBDegsRepEach( :, nt + 1 : end );
        
        % all the n-tuples in the halfspace
        DegsAll = ntupleh( nOld, nchp( nOld, nt ), nt );
        DegsRem = zeros( SBNrDegs2, 1 );
        for j = 1 : SBNrDegs2
            if isempty( findvm( DegsAll', SBDegsAddSub( j, : ) ) )
                DegsRem( j ) = j;
            end
        end
        % delete the degrees that are not in the halfspace
        SBDegsAddSub( DegsRem( DegsRem ~= 0 ), : ) = [ ];
        % delete multiple instances of the same degree; leave just one
        SBDegsAddSub = unique( SBDegsAddSub, 'rows' );
        
        % number of monomials in the basis
        SBNrDegsMono = SBNrDegs;
        % number of degrees for which we write the equalities
        SBNrDegs = size( SBDegsAddSub, 1 );
        
        % position of the degrees in the halfspace
        SBPosInHalf = zeros( SBNrDegs, 1 );
        for j = 1 : SBNrDegs
            SBPosInHalf( j ) = findvm( DegsAll', SBDegsAddSub( j, : ) );
        end
        SBPosInHalf = sort( SBPosInHalf );
        
        % we order the degrees as in the halfspace in
        % the variable 'SBDegsAddSubCopy'
        SBDegsAddSubCopy = zeros( SBNrDegs, d );
        for j = 1 : SBNrDegs
            SBDegsAddSubCopy( j, : ) = DegsAll( SBPosInHalf( j ), : );
        end
        % the ordered degrees in the halfspace
        SBDegsAddSub = SBDegsAddSubCopy;
    elseif isT
        % trigonometric polynomial
        
        SBNrDegs2 = SBNrDegs ^ 2;
        % SBDegsRep makes a repmat on all the degrres once
        SBDegsRep = repmat( SBDegs, SBNrDegs, 1 );
        % SBDegsRepEach makes a repmat on every degree one by one
        SBDegsRepEach = zeros( SBNrDegs2 , d );
        for j = 1 : SBNrDegs
            SBDegsRepEach( ( j - 1 ) * SBNrDegs + 1 : j * SBNrDegs, : ) = repmat( SBDegs( j, : ), SBNrDegs, 1 );
        end
        
        % subtract degrees for trigonometric
        SBDegsAddSub = SBDegsRep - SBDegsRepEach;
        
        % all the n-tuples in the halfspace
        DegsHalf = ntuple( n, nch( n ), 1 );
        DegsRem = zeros( SBNrDegs2, 1 );
        for j = 1 : SBNrDegs2
            if isempty( findvm( DegsHalf', SBDegsAddSub( j, : ) ) )
                DegsRem( j ) = j;
            end
        end
        % delete the degrees that are not in the halfspace
        SBDegsAddSub( DegsRem( DegsRem ~= 0 ), : ) = [ ];
        % delete multiple instances of the same degree; leave just one
        SBDegsAddSub = unique( SBDegsAddSub, 'rows' );
        
        % number of monomials in the basis
        SBNrDegsMono = SBNrDegs;
        % number of degrees for which we write the equalities
        SBNrDegs = size( SBDegsAddSub, 1 );
        
        % position of the degrees in the halfspace
        SBPosInHalf = zeros( SBNrDegs, 1 );
        for j = 1 : SBNrDegs
            SBPosInHalf( j ) = findvm( DegsHalf', SBDegsAddSub( j, : ) );
        end
        SBPosInHalf = sort( SBPosInHalf );
        
        % we order the degrees as in the halfspace in
        % the variable 'SBDegsAddSubCopy'
        SBDegsAddSubCopy = zeros( SBNrDegs, d );
        for j = 1 : SBNrDegs
            SBDegsAddSubCopy( j, : ) = DegsHalf( SBPosInHalf( j ), : );
        end
        % the ordered degrees in the halfspace
        SBDegsAddSub = SBDegsAddSubCopy;
    else
        % real polynomial
        
        SBNrDegs2 = SBNrDegs ^ 2;
        % SBDegsRep makes a repmat on all the degrres once
        SBDegsRep = repmat( SBDegs, SBNrDegs, 1 );
        % SBDegsRepEach makes a repmat on every degree one by one
        SBDegsRepEach = zeros( SBNrDegs ^ 2, d );
        for j = 1 : SBNrDegs
            SBDegsRepEach( ( j - 1 ) * SBNrDegs + 1 : j * SBNrDegs, : ) = repmat( SBDegs( j, : ), SBNrDegs, 1 );
        end
        
        % add degrees for real
        SBDegsAddSub = SBDegsRep + SBDegsRepEach;
        
        % all the n-tuples
        AllDegs = ntuple( nOld, prod( nOld + 1 ), 0 );
        DegsRem = zeros( SBNrDegs2, 1 );
        for j = 1 : SBNrDegs2
            if isempty( findvm( AllDegs', SBDegsAddSub( j, : ) ) )
                DegsRem( j ) = j;
            end
        end
        % delete the degrees that are not among the degrees
        SBDegsAddSub( DegsRem( DegsRem ~= 0 ), : ) = [ ];
        % delete multiple instances of the same degree; leave just one
        SBDegsAddSub = unique( SBDegsAddSub, 'rows' );
        
        % number of monomials in the basis
        SBNrDegsMono = SBNrDegs;
        % number of degrees for which we write the equalities
        SBNrDegs = size( SBDegsAddSub, 1 );
        
        % position of the degrees
        SBPosInHalf = zeros( SBNrDegs, 1 );
        for j = 1 : SBNrDegs
            SBPosInHalf( j ) = findvm( AllDegs', SBDegsAddSub( j, : ) );
        end
        SBPosInHalf = sort( SBPosInHalf );
        
        % we order all the degrees
        % in the variable 'SBDegsAddSubCopy'
        SBDegsAddSubCopy = zeros( SBNrDegs, d );
        for j = 1 : SBNrDegs
            SBDegsAddSubCopy( j, : ) = AllDegs( SBPosInHalf( j ), : );
        end
        % the ordered degrees in the halfspace
        SBDegsAddSub = SBDegsAddSubCopy;
    end
end
if isSB == 2
    % if isSB 2, then check the number of monomials from
    % the basis to be less than or equal to nNonzProd1
    if nNonzProd1 < length( IndDeg )
        error( 'Too many elements in the basis of the polynomial.' );
    end
    
    % number of nonzero degrees from the sparse basis
    SBNrDegs = size( IndDeg, 1 );
    % position with nonzero degrees from the basis
    SBPosDegs = zeros( SBNrDegs, 1 );
    % degrees from the sparse basis
    SBDegs = IndDeg;
    
    % all the degrees from the basis
    MonoBasis = ntuple( n, prod( n + 1 ), 0 );
    for j = 1 : SBNrDegs
        SBPosDegs( j ) = findvm( MonoBasis', SBDegs( j, : ) );
    end
    
    % create the degrees for which we write
    % the equalities
    if isT && isR
        % hybrid polynomial
        
        SBNrDegs2 = SBNrDegs ^ 2;
        % SBDegsRep makes a repmat on all the degrres once
        SBDegsRep = repmat( SBDegs, SBNrDegs, 1 );
        % SBDegsRepEach makes a repmat on every degree one by one
        SBDegsRepEach = zeros( SBNrDegs ^ 2, d );
        for j = 1 : SBNrDegs
            SBDegsRepEach( ( j - 1 ) * SBNrDegs + 1 : j * SBNrDegs, : ) = repmat( SBDegs( j, : ), SBNrDegs, 1 );
        end
        SBDegsAddSub = zeros( SBNrDegs ^ 2, d );
        
        % subtract and add degrees for hybrid
        SBDegsAddSub( :, 1 : nt       ) = SBDegsRep    ( :, 1 : nt       ) - ...
            SBDegsRepEach( :, 1 : nt       );
        SBDegsAddSub( :, nt + 1 : end ) = SBDegsRep    ( :, nt + 1 : end ) + ...
            SBDegsRepEach( :, nt + 1 : end );
        
        % all the n-tuples in the halfspace
        DegsAll = ntupleh( nOld, nchp( nOld, nt ), nt );
        DegsRem = zeros( SBNrDegs2, 1 );
        for j = 1 : SBNrDegs2
            if isempty( findvm( DegsAll', SBDegsAddSub( j, : ) ) )
                DegsRem( j ) = j;
            end
        end
        % delete the degrees that are not in the halfspace
        SBDegsAddSub( DegsRem( DegsRem ~= 0 ), : ) = [ ];
        % delete multiple instances of the same degree; leave just one
        SBDegsAddSub = unique( SBDegsAddSub, 'rows' );
        
        % number of monomials in the basis
        SBNrDegsMono = SBNrDegs;
        % number of degrees for which we write the equalities
        SBNrDegs = size( SBDegsAddSub, 1 );
        
        % position of the degrees in the halfspace
        SBPosInHalf = zeros( SBNrDegs, 1 );
        for j = 1 : SBNrDegs
            SBPosInHalf( j ) = findvm( DegsAll', SBDegsAddSub( j, : ) );
        end
        SBPosInHalf = sort( SBPosInHalf );
        
        % we order the degrees as in the halfspace in
        % the variable 'SBDegsAddSubCopy'
        SBDegsAddSubCopy = zeros( SBNrDegs, d );
        for j = 1 : SBNrDegs
            SBDegsAddSubCopy( j, : ) = DegsAll( SBPosInHalf( j ), : );
        end
        % the ordered degrees in the halfspace
        SBDegsAddSub = SBDegsAddSubCopy;
    elseif isT
        % trigonometric polynomial
        
        SBNrDegs2 = SBNrDegs ^ 2;
        % SBDegsRep makes a repmat on all the degrres once
        SBDegsRep = repmat( SBDegs, SBNrDegs, 1 );
        % SBDegsRepEach makes a repmat on every degree one by one
        SBDegsRepEach = zeros( SBNrDegs2, d );
        for j = 1 : SBNrDegs
            SBDegsRepEach( ( j - 1 ) * SBNrDegs + 1 : j * SBNrDegs, : ) = repmat( SBDegs( j, : ), SBNrDegs, 1 );
        end
        
        % subtract degrees for trigonometric
        SBDegsAddSub = SBDegsRep - SBDegsRepEach;
        
        % all the n-tuples in the halfspace
        DegsHalf = ntuple( n, nch( n ), 1 );
        DegsRem = zeros( SBNrDegs2, 1 );
        for j = 1 : SBNrDegs2
            if isempty( findvm( DegsHalf', SBDegsAddSub( j, : ) ) )
                DegsRem( j ) = j;
            end
        end
        % delete the degrees that are not in the halfspace
        SBDegsAddSub( DegsRem( DegsRem ~= 0 ), : ) = [ ];
        % delete multiple instances of the same degree; leave just one
        SBDegsAddSub = unique( SBDegsAddSub, 'rows' );
        
        % number of monomials in the basis
        SBNrDegsMono = SBNrDegs;
        % number of degrees for which we write the equalities
        SBNrDegs = size( SBDegsAddSub, 1 );
        
        % position of the degrees in the halfspace
        SBPosInHalf = zeros( SBNrDegs, 1 );
        for j = 1 : SBNrDegs
            SBPosInHalf( j ) = findvm( DegsHalf', SBDegsAddSub( j, : ) );
        end
        SBPosInHalf = sort( SBPosInHalf );
        
        % we order the degrees as in the halfspace in
        % the variable 'SBDegsAddSubCopy'
        SBDegsAddSubCopy = zeros( SBNrDegs, d );
        for j = 1 : SBNrDegs
            SBDegsAddSubCopy( j, : ) = DegsHalf( SBPosInHalf( j ), : );
        end
        % the ordered degrees in the halfspace
        SBDegsAddSub = SBDegsAddSubCopy;
    else
        % real polynomial
        
        SBNrDegs2 = SBNrDegs ^ 2;
        % SBDegsRep makes a repmat on all the degrres once
        SBDegsRep = repmat( SBDegs, SBNrDegs, 1 );
        % SBDegsRepEach makes a repmat on every degree one by one
        SBDegsRepEach = zeros( SBNrDegs2, d );
        for j = 1 : SBNrDegs
            SBDegsRepEach( ( j - 1 ) * SBNrDegs + 1 : j * SBNrDegs, : ) = repmat( SBDegs( j, : ), SBNrDegs, 1 );
        end
        
        % add degrees for real
        SBDegsAddSub = SBDegsRep + SBDegsRepEach;
        
        % all the n-tuples
        AllDegs = ntuple( nOld, prod( nOld + 1 ), 0 );
        DegsRem = zeros( SBNrDegs2, 1 );
        for j = 1 : SBNrDegs2
            if isempty( findvm( AllDegs', SBDegsAddSub( j, : ) ) )
                DegsRem( j ) = j;
            end
        end
        % delete the degrees that are not among the degrees
        SBDegsAddSub( DegsRem( DegsRem ~= 0 ), : ) = [ ];
        % delete multiple instances of the same degree; leave just one
        SBDegsAddSub = unique( SBDegsAddSub, 'rows' );
        
        % number of monomials in the basis
        SBNrDegsMono = SBNrDegs;
        % number of degrees for which we write the equalities
        SBNrDegs = size( SBDegsAddSub, 1 );
        
        % position of the degrees
        SBPosInHalf = zeros( SBNrDegs, 1 );
        for j = 1 : SBNrDegs
            SBPosInHalf( j ) = findvm( AllDegs', SBDegsAddSub( j, : ) );
        end
        SBPosInHalf = sort( SBPosInHalf );
        
        % we order all the degrees
        % in the variable 'SBDegsAddSubCopy'
        SBDegsAddSubCopy = zeros( SBNrDegs, d );
        for j = 1 : SBNrDegs
            SBDegsAddSubCopy( j, : ) = AllDegs( SBPosInHalf( j ), : );
        end
        % the ordered degrees in the halfspace
        SBDegsAddSub = SBDegsAddSubCopy;
    end
end