function ncoefsb = getnrcoefsb( n, ptype, K, isT, isR, isSB )

% GETNRCOEFSB Get number of variables for a sparse polynomial
%
%   This function computes the number of variables for a sparse polynomial.
%
%   NCOEFSB = GETNRCOEFSB( N, PTYPE, K, IST, ISR, ISSB )
%
%   INPUTS:
%
%   N     - A vector which holds on its rows the degrees of the polynomial.
%   PTYPE - The 'ptype' term for the polynomial.
%   K    -  A vector containing the size of the coeffcients for every
%           polynomial.
%   IST  -  Describes the polynomials of having or not trigonometric variables.
%   ISR  -  Describes the polynomials of having or not real variables.
%   ISSB -  The ISSB term. Describes how the sparse basis is enforced. If ISSB
%           is 1 then the user uses the field 'ind' to enforce the sparse
%           basis. If ISSB is 2 then the user uses the field 'deg' to set
%           the sparse basis.
%
%   OUTPUT:
%
%   NCOEFSB - The number of coefficients

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.03.09-14.09.11

% check value of isSB
if isSB ~= 1 && isSB ~= 2
    error( '''isSB'' must be 1 or 2.' );
end

nOld = n;

if isT && isR
    % hybrid polynomial
    
    % check exitence of field 'trigonometric'
    if ~isfield( ptype, 'trigonometric' )
        error( 'Field ''trigonometric'' in structure ''ptype'' is missing.' );
    end
    % check exitence of field 'real'
    if ~isfield( ptype, 'real' )
        error( 'Field ''real'' in structure ''ptype'' is missing.' );
    end
    
    n( ptype.trigonometric + 1 : end ) = n( ptype.trigonometric + 1 : end ) / 2;
    
    d = ptype.trigonometric + ptype.real;
else
    if isR
        % real polynomial
        
        % check exitence of field 'real'
        if ~isfield( ptype, 'real' )
            error( 'Field ''real'' in structure ''ptype'' is missing.' );
        end
        
        n = n / 2;
        d = ptype.real;
    else
        % trigonometric polynomial
        
        % check exitence of field 'trigonometric'
        if ~isfield( ptype, 'trigonometric' )
            error( 'Field ''trigonometric'' in structure ''ptype'' is missing.' );
        end
        
        d = ptype.trigonometric;
    end
end

nNonzProd1 = prod( n + 1 );

% the degrees
if isSB == 1
    if isT && isR
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.ind, isSB, ptype.trigonometric );
    else
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.ind, isSB );
    end
else
    if isT && isR
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.deg, isSB, ptype.trigonometric );
    else
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.deg, isSB );
    end
end    

if isT && isR
    % hybrid polynomial
    
    if K == 1
        % scalar coefficients
        
        ncoefsb = SBNrDegsi;
    else
        % matrix coefficients
        
        % number of elem.
        % in matrix
        K2 = K ^ 2;
        
        % "half" of the
        % matrix
        hK = sumn( K );
        
        % number of  symmetric matrices 
        % according to the degrees (deg(1:nt)=0)
        NSymmMat = 0;      
        for i = 1 : SBNrDegsi
            if iszero( SBDegsAddSubi( i, 1 : ptype.trigonometric ) )
                NSymmMat = NSymmMat + 1;
            end
        end
        
        ncoefsb = hK * NSymmMat + ( SBNrDegsi - NSymmMat ) * K2;
    end
else
    if isT
        % trigonometric polynomial
        
        if K == 1
            % scalar coefficients
            
            ncoefsb = SBNrDegsi;
        else
            % matrix coefficients
            
            % number of elem.
            % in matrix
            K2 = K ^ 2;
            
            % "half" of the
            % matrix
            hK = sumn( K );
            
            ncoefsb = hK + ( SBNrDegsi - 1 ) * K2;
        end
    else
        % real polynomial
        
        if K == 1
            % scalar coefficients
            
            ncoefsb = SBNrDegsi;
        else
            % matrix coefficients
            
            % "half" of the
            % matrix
            hK = sumn( K );
            
            ncoefsb = SBNrDegsi * hK;
        end
    end
end