function [ LimD ] = lim_mat( nPol, n, k, isIVec, Q2size, KsPflqrs, dmax, IsI, IsD, ...
                             ISSB, isSB, SBNrDegsMonoInt, ...
                             isD, nDomPol, nDomPolCS, QD2size )

% LIM_MAT Limits in matrix
%
%   This function returns the limits of the multipliers for
%   the variables of each polynomial in the matrix D.
%
%   The problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 22.11.08-21.07.10

% LimD is a matrix that holds the
% bounds for the limits of the
% coefficients of the variables
% of the i-th polynomial in D;
% LimD(i,1) & LimD(i,2) S0
% LimD(i,3) & LimD(i,4) S1 (for pos. on int.)
% LimD(i,3) & LimD(i,4) & ... Si (for pos. on dom.)
if IsD
    % positivity on domain exists
    LimD = zeros( nPol, 2 + 2 * max( nDomPol ) );
elseif IsI
    % positivity on interval exists
    LimD = zeros( nPol, 4 );
else
    % only global positivity
    LimD = zeros( nPol, 2 );
end

LimD( 1, 1 ) = KsPflqrs + 1;

if ~ISSB || dmax > 1
    % no sparse basis or multivariate
    
    if ~IsD
        % there is no pos. dom.
        
        for i = 1 : nPol - 1
            
            % K = 1 - scalar
            % K > 1 - matrix dim.
            K = k( i );
            
            switch isIVec( i )
                case 0
                    % global positivity
                    
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i + 1, 1 ) = LimD( i, 2 ) + 1;
                case 1
                    % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 2
                    % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 3
                    % [ a, pi ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 4
                    % [ -pi, b ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 5
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 6
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                    
                    % size of Gram
                    % matrices
                    n2 = ( n{ i } * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 7
                    % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 8
                    % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 9
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 10
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                    
                    % size of Gram
                    % matrices
                    n2 = ( n{ i } * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 11
                    % [a, b] real pol even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 12
                    % [a, b] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    % size of Gram
                    % matrices
                    n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                    
                    LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 13
                    % [ a, Inf ) real pol. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 14
                    % [ a, Inf ) real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    % size of Gram
                    % matrices
                    n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                    
                    LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 15
                    % ( -Inf, b ] real pol. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 16
                    % ( -Inf, b ] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    % size of Gram
                    % matrices
                    n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                    
                    LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                case 17
                    % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i,     3 ) = LimD( i, 2 ) + 1;
                    LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                    LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
            end
        end
        
        % update for the last
        % polynomial
        
        % K = 1 - scalar
        % K > 1 - matrix dim.
        K = k( nPol );
        switch isIVec( nPol )
            case 0
                % global positivity
                
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
            case 1
                % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 2
                % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 3
                % [ a, pi ] trig. pol. complex coef.
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 4
                % [ -pi, b ] trig. pol. complex coef.
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 5
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                
                % S0: n+1, S1:n-1
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
            case 6
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                
                % size of Gram
                % matrices
                n2 = ( n{ nPol } * K ) ^ 2;
                
                % S0: n, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
            case 7
                % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 8
                % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 9
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                
                % S0: n+1, S1:n-1
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
            case 10
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                
                % size of Gram
                % matrices
                n2 = ( n{ nPol } * K ) ^ 2;
                
                % S0: n, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
            case 11
                % [ a, b ] real pol. even degree
                
                % S0: n+1, S1:n-1
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 12
                % [ a, b ] real pol. odd degree
                
                % S0: n, S1:n
                
                % size of Gram
                % matrices
                n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
            case 13
                % [ a, Inf ) real pol. even degree
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 14
                % [ a, Inf ) real pol. odd degree
                
                % S0: n, S1:n
                
                % size of Gram
                % matrices
                n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
            case 15
                % ( -Inf, b ] real pol. even degree
                
                % S0: n+1, S1:n
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            case 16
                % ( -Inf, b ] real pol. odd degree
                
                % S0: n, S1:n
                
                % size of Gram
                % matrices
                n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
            case 17
                % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                
                % S0: n+1, S1:n-1
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
        end
    else
        % with pos. dom.
        
        for i = 1 : nPol - 1
            
            % K = 1 - scalar
            % K > 1 - matrix dim.
            K = k( i );
            
            if ~isD( i )
                % glob. pos. or pos. int.
                
                switch isIVec( i )
                    case 0
                        % global positivity
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i + 1, 1 ) = LimD( i, 2 ) + 1;
                    case 1
                        % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                        
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 2
                        % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                        
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 3
                        % [ a, pi ] trig. pol. complex coef.
                        
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 4
                        % [ -pi, b ] trig. pol. complex coef.
                        
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 5
                        % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                        
                        % S0: n+1, S1:n-1
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 6
                        % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                        
                        % size of Gram
                        % matrices
                        n2 = ( n{ i } * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 7
                        % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                        
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 8
                        % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                        
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 9
                        % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                        
                        % S0: n+1, S1:n-1
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 10
                        % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                        
                        % size of Gram
                        % matrices
                        n2 = ( n{ i } * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 11
                        % [a, b] real pol even degree
                        
                        % S0: n+1, S1:n-1
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 12
                        % [a, b] real pol. odd degree
                        
                        % S0: n, S1:n
                        
                        % size of Gram
                        % matrices
                        n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 13
                        % [ a, Inf ) real pol. even degree
                        
                        % S0: n+1, S1:n-1
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 14
                        % [ a, Inf ) real pol. odd degree
                        
                        % S0: n, S1:n
                        
                        % size of Gram
                        % matrices
                        n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 15
                        % ( -Inf, b ] real pol. even degree
                        
                        % S0: n+1, S1:n-1
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 16
                        % ( -Inf, b ] real pol. odd degree
                        
                        % S0: n, S1:n
                        
                        % size of Gram
                        % matrices
                        n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    case 17
                        % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                        
                        % S0: n+1, S1:n-1
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                end
            else
                % pos. dom.
                
                % S0
                LimD( i, 2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                
                % Si
                for DomPol = 1 : nDomPol( i )
                    dmpIdx  = 2 * DomPol;
                    dmpIdx1 = dmpIdx  + 1;
                    dmpIdx2 = dmpIdx1 + 1;
                    LimD( i, dmpIdx1 ) = LimD( i, dmpIdx  ) + 1;
                    LimD( i, dmpIdx2 ) = LimD( i, dmpIdx1 ) + QD2size( nDomPolCS( i ) + DomPol ) - 1;
                end
                % for the next polynomial
                LimD( i + 1, 1 ) = LimD( i, dmpIdx2 ) + 1;
            end
        end
        
        % update for the last
        % polynomial
        
        % K = 1 - scalar
        % K > 1 - matrix dim.
        K = k( nPol );
        if ~isD( i )
            % glob. pos. or pos. int.
            
            switch isIVec( nPol )
                case 0
                    % global positivity
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                case 1
                    % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 2
                    % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 3
                    % [ a, pi ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 4
                    % [ -pi, b ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 5
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
                case 6
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                    
                    % size of Gram
                    % matrices
                    n2 = ( n{ nPol } * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                case 7
                    % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 8
                    % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 9
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
                case 10
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                    
                    % size of Gram
                    % matrices
                    n2 = ( n{ nPol } * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                case 11
                    % [ a, b ] real pol. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 12
                    % [ a, b ] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    % size of Gram
                    % matrices
                    n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                case 13
                    % [ a, Inf ) real pol. even degree
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 14
                    % [ a, Inf ) real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    % size of Gram
                    % matrices
                    n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                case 15
                    % ( -Inf, b ] real pol. even degree
                    
                    % S0: n+1, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                case 16
                    % ( -Inf, b ] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    % size of Gram
                    % matrices
                    n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                case 17
                    % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                    
                    % S0: n+1, S1:n-1
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
            end
        else
            % pos. dom.
            
            % S0
            LimD( nPol,     2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
            
            % Si
            for DomPol = 1 : nDomPol( nPol )
                dmpIdx  = 2 * DomPol;
                dmpIdx1 = dmpIdx  + 1;
                dmpIdx2 = dmpIdx1 + 1;
                LimD( nPol, dmpIdx1 ) = LimD( nPol, dmpIdx  ) + 1;
                LimD( nPol, dmpIdx2 ) = LimD( nPol, dmpIdx1 ) + QD2size( nDomPolCS( nPol ) + DomPol ) - 1;
            end
        end
    end
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % sparse basis, univariate
    
    if ~IsD
        % there is no pos. dom.
        
        for i = 1 : nPol - 1
            
            % K = 1 - scalar
            % K > 1 - matrix dim.
            K = k( i );
            
            switch isIVec( i )
                case 0
                    % global positivity
                    
                    LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                    LimD( i + 1, 1 ) = LimD( i, 2 ) + 1;
                case 1
                    % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 2
                    % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 3
                    % [ a, pi ] trig. pol. complex coef.
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 4
                    % [ -pi, b ] trig. pol. complex coef.
                    
                    if ~ isSB( i )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 5
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 6
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                    
                    if ~isSB( i )
                        % size of Gram
                        % matrices
                        n2 = ( n{ i } * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % size of Gram
                        % matrices
                        n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 7
                    % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 8
                    % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 9
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 10
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                    
                    if ~isSB( i )
                        % size of Gram
                        % matrices
                        n2 = ( n{ i } * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % size of Gram
                        % matrices
                        n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 11
                    % [a, b] real pol even degree
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 12
                    % [a, b] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    if ~isSB( i )
                        % size of Gram
                        % matrices
                        n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                        
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % size of Gram
                        % matrices
                        n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                        
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 13
                    % [ a, Inf ) real pol. even degree
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 14
                    % [ a, Inf ) real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    if ~isSB( i )
                        % size of Gram
                        % matrices
                        % no sparse basis
                        n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % size of Gram
                        % matrices
                        % sparse basis
                        n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 15
                    % ( -Inf, b ] real pol. even degree
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 16
                    % ( -Inf, b ] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    if ~isSB( i )
                        % size of Gram
                        % matrices
                        % no sparse basis
                        n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % size of Gram
                        % matrices
                        % sparse basis
                        n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
                case 17
                    % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                    
                    if ~isSB( i )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i,     3 ) = LimD( i, 2 ) + 1;
                        LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                        LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                    end
            end
        end
        
        % update for the last
        % polynomial
        
        % K = 1 - scalar
        % K > 1 - matrix dim.
        K = k( nPol );
        switch isIVec( nPol )
            case 0
                % global positivity
                
                LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
            case 1
                % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 2
                % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 3
                % [ a, pi ] trig. pol. complex coef.
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 4
                % [ -pi, b ] trig. pol. complex coef.
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 5
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n-1
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n-1
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 6
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                
                if ~isSB( nPol )
                    % size of Gram
                    % matrices
                    % no sparse basis
                    n2 = ( n{ nPol } * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                else
                    % size of Gram
                    % matrices
                    % sparse basis
                    n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                end
            case 7
                % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 8
                % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 9
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n-1
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n-1
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 10
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                
                if ~isSB( nPol )
                    % size of Gram
                    % matrices
                    % no sparse basis
                    n2 = ( n{ nPol } * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                else
                    % size of Gram
                    % matrices
                    % sparse basis
                    n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                    
                    % S0: n, S1:n
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                end
            case 11
                % [ a, b ] real pol. even degree
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n-1;
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n-1
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 12
                % [ a, b ] real pol. odd degree
                
                % S0: n, S1:n
                
                % size of Gram
                % matrices
                if ~isSB( nPol )
                    n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;

                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                else
                    n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                    
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                end
            case 13
                % [ a, Inf ) real pol. even degree
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 14
                % [ a, Inf ) real pol. odd degree
                
                % S0: n, S1:n
                
                if ~isSB( nPol )
                    % size of Gram
                    % matrices
                    % no sparse basis
                    n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                else
                    % size of Gram
                    % matrices
                    % sparse basis
                    n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                end
            case 15
                % ( -Inf, b ] real pol. even degree
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
            case 16
                % ( -Inf, b ] real pol. odd degree
                
                % S0: n, S1:n
                
                if ~isSB( nPol )
                    % size of Gram
                    % matrices
                    % no sparse basis
                    n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                else
                    % size of Gram
                    % matrices
                    % sparse basis
                    n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                end
            case 17
                % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                
                if ~isSB( nPol )
                    % S0: n+1, S1:n-1
                    % no sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                else
                    % S0: n+1, S1:n-1
                    % sparse basis
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                    LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                    LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                end
        end
    else
        % with pos. dom.
        
        for i = 1 : nPol - 1
            
            % K = 1 - scalar
            % K > 1 - matrix dim.
            K = k( i );
            
            if ~isD( i )
                % glob. pos. or pos. int.
                
                switch isIVec( i )
                    case 0
                        % global positivity
                        
                        LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                        LimD( i + 1, 1 ) = LimD( i, 2 ) + 1;
                    case 1
                        % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 2
                        % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 3
                        % [ a, pi ] trig. pol. complex coef.
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 4
                        % [ -pi, b ] trig. pol. complex coef.
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 5
                        % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n-1
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n-1
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 6
                        % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                        
                        if ~isSB( i )
                            % size of Gram
                            % matrices
                            % no sparse basis
                            n2 = ( n{ i } * K ) ^ 2;
                            
                            % S0: n, S1:n
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % size of Gram
                            % matrices
                            % sparse basis
                            n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                            
                            % S0: n, S1:n
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 7
                        % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 8
                        % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 9
                        % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n-1
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( ( n{ i } - 1 ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n-1
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 10
                        % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                        
                        if ~isSB( i )
                            % size of Gram
                            % matrices
                            % no sparse basis
                            n2 = ( n{ i } * K ) ^ 2;
                            
                            % S0: n, S1:n
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % size of Gram
                            % matrices
                            % sparse basis
                            n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                            
                            % S0: n, S1:n
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 11
                        % [a, b] real pol even degree
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n-1
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n-1
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 12
                        % [a, b] real pol. odd degree
                        
                        % S0: n, S1:n
                        
                        if ~isSB( i )
                            % size of Gram
                            % matrices
                            % no sparse basis
                            n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                            
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % size of Gram
                            % matrices
                            % sparse basis
                            n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                            
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 13
                        % [ a, Inf ) real pol. even degree
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n-1
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n-1
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 14
                        % [ a, Inf ) real pol. odd degree
                        
                        % S0: n, S1:n
                        
                        if ~isSB( i )
                            % size of Gram
                            % matrices
                            % no sparse basis
                            n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                            
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % size of Gram
                            % matrices
                            % sparse basis
                            n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                            
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 15
                        % ( -Inf, b ] real pol. even degree
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n-1
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n-1
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 16
                        % ( -Inf, b ] real pol. odd degree
                        
                        % S0: n, S1:n
                        
                        if ~isSB( i )
                            % size of Gram
                            % matrices
                            % no sparse basis
                            n2 = ( ( n{ i } + 0.5 ) * K ) ^ 2;
                            
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % size of Gram
                            % matrices
                            % sparse basis
                            n2 = ( SBNrDegsMonoInt( i ) * K ) ^ 2;
                            
                            LimD( i,     2 ) = LimD( i, 1 ) + n2 - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + n2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                    case 17
                        % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                        
                        if ~isSB( i )
                            % S0: n+1, S1:n-1
                            % no sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( n{ i } * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        else
                            % S0: n+1, S1:n-1
                            % sparse basis
                            LimD( i,     2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                            LimD( i,     3 ) = LimD( i, 2 ) + 1;
                            LimD( i,     4 ) = LimD( i, 3 ) + ( SBNrDegsMonoInt( i ) * K ) ^ 2 - 1;
                            LimD( i + 1, 1 ) = LimD( i, 4 ) + 1;
                        end
                end
            else
                % pos. dom.
                
                % S0
                LimD( i, 2 ) = LimD( i, 1 ) + Q2size( i ) - 1;
                
                % Si
                for DomPol = 1 : nDomPol( i )
                    dmpIdx  = 2 * DomPol;
                    dmpIdx1 = dmpIdx  + 1;
                    dmpIdx2 = dmpIdx1 + 1;
                    LimD( i, dmpIdx1 ) = LimD( i, dmpIdx  ) + 1;
                    LimD( i, dmpIdx2 ) = LimD( i, dmpIdx1 ) + QD2size( nDomPolCS( i ) + DomPol ) - 1;
                end
                % for the next polynomial
                LimD( i + 1, 1 ) = LimD( i, dmpIdx2 ) + 1;
            end
        end
        
        % update for the last
        % polynomial
        
        % K = 1 - scalar
        % K > 1 - matrix dim.
        K = k( nPol );
        if ~isD( i )
            % glob. pos. or pos. int.
            
            switch isIVec( nPol )
                case 0
                    % global positivity
                    
                    LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                case 1
                    % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 2
                    % [ -a, a ] in ( -pi, pi ) trig. pol. complex coef.
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 3
                    % [ a, pi ] trig. pol. complex coef.
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 4
                    % [ -pi, b ] trig. pol. complex coef.
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 5
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 6
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
                    
                    if ~isSB( nPol )
                        % size of Gram
                        % matrices
                        % no sparse basis
                        n2 = ( n{ nPol } * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    else
                        % size of Gram
                        % matrices
                        % sparse basis
                        n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    end
                case 7
                    % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 8
                    % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 9
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( ( n{ nPol } - 1 ) * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 10
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
                    
                    if ~isSB( nPol )
                        % size of Gram
                        % matrices
                        % no sparse basis
                        n2 = ( n{ nPol } * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    else
                        % size of Gram
                        % matrices
                        % sparse basis
                        n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                        
                        % S0: n, S1:n
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    end
                case 11
                    % [ a, b ] real pol. even degree
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n-1;
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 12
                    % [ a, b ] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    if ~isSB( nPol )
                        % size of Gram
                        % matrices
                        % no sparse basis
                        n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                        
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    else
                        % size of Gram
                        % matrices
                        % sparse basis
                        n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                        
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    end
                case 13
                    % [ a, Inf ) real pol. even degree
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 14
                    % [ a, Inf ) real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    if ~isSB( nPol )
                        % size of Gram
                        % matrices
                        % no sparse basis
                        n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                        
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    else
                        % size of Gram
                        % matrices
                        % sparse basis
                        n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                        
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    end
                case 15
                    % ( -Inf, b ] real pol. even degree
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
                case 16
                    % ( -Inf, b ] real pol. odd degree
                    
                    % S0: n, S1:n
                    
                    if ~isSB( nPol )
                        % size of Gram
                        % matrices
                        % no sparse basis
                        n2 = ( ( n{ nPol } + 0.5 ) * K ) ^ 2;
                        
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    else
                        % size of Gram
                        % matrices
                        % sparse basis
                        n2 = ( SBNrDegsMonoInt( nPol ) * K ) ^ 2;
                        
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + n2 - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + n2 - 1;
                    end
                case 17
                    % ( -Inf, a ] U [ b, Inf ) real pol. even degree
                    
                    if ~isSB( nPol )
                        % S0: n+1, S1:n-1
                        % no sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( n{ nPol } * K ) ^ 2 - 1;
                    else
                        % S0: n+1, S1:n-1
                        % sparse basis
                        LimD( nPol, 2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
                        LimD( nPol, 3 ) = LimD( nPol, 2 ) + 1;
                        LimD( nPol, 4 ) = LimD( nPol, 3 ) + ( SBNrDegsMonoInt( nPol ) * K ) ^ 2 - 1;
                    end
            end
        else
            % pos. dom.
            
            % S0
            LimD( nPol,     2 ) = LimD( nPol, 1 ) + Q2size( nPol ) - 1;
            
            % Si
            for DomPol = 1 : nDomPol( nPol )
                dmpIdx  = 2 * DomPol;
                dmpIdx1 = dmpIdx  + 1;
                dmpIdx2 = dmpIdx1 + 1;
                LimD( nPol, dmpIdx1 ) = LimD( nPol, dmpIdx  ) + 1;
                LimD( nPol, dmpIdx2 ) = LimD( nPol, dmpIdx1 ) + QD2size( nDomPolCS( nPol ) + DomPol ) - 1;
            end
        end
    end
end