function [ nVar, Ks, isIVec, TVecUnq, HVecUnq, TList, HList, IsT, IsR, SBNrDegsMonoInt, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt ] = ...
           nvar( KsPflqrs, Ks, ptype, nPol, n, k, nOld, QKsize, Q2size, isT, isR, isC, isI, ...
                 isD, ...
                 ISSB, isSB, SBNrDegs, SBDegsAddSub, ...
                 d, nDomPol, nDomPolCS, nSi, QDKsize, QD2size )

% NVAR Total number of variables for the Gram matrices
%
%   This function returns in NVAR the total number of 
%   variables, updates fields 's' and 'scomplex' in the 
%   structure KS and returns in ISIVEC a corresponding 
%   number for the type of positivity for each polynomial.

%   Types of positivity - values of ISIVEC( i ):
%
%   0  - global positivity
% 
%   1  - [ a, b ] in ( -pi, pi ) trig. pol. complex coef.
% 
%   2  - [ -a, a ] in ( - pi, pi ) trig. pol. complex coef.
% 
%   3  - [ a, pi ] trig. pol. complex coef.
% 
%   4  - [ -pi, b ] trig. pol. complex coef.
% 
%   5  - [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree
% 
%   6  - [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree
% 
%   7  - [ -pi,  a ] U [ b, pi ] trig. pol. complex coef. 
% 
%   8  - [ -pi, -a ] U [ a, pi ] trig. pol. complex coef. 
% 
%   9  - [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree
% 
%   10 - [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree
% 
%   11 - [ a,b ] real pol. even degree
% 
%   12 - [ a,b ] real pol. odd degree
% 
%   13 - [ a, Inf ) real pol. even degree
% 
%   14 - [ a, Inf ) real pol. odd degree
% 
%   15 - ( -Inf, b ] real pol. even degree
% 
%   16 - ( -Inf, b ] real pol. odd degree
% 
%   17 - ( -Inf, a ] U [ b, Inf ) real pol. even degree

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 22.11.08-22.07.10

% 'nVar' will denote the total number 
% of variables from the optimization
% problem;
%
% add first the ones from KsPflqrs
nVar = KsPflqrs;

% type of positivity
% for the polynomials
isIVec = zeros( nPol, 1 );

% number o blocks in Ks.s
nKss = length( Ks.s );

% cumulative sum for the degrees of 
% trigonometric & real variables
nTVecCS = zeros( nPol, 1 );
nHVecCS = zeros( nPol, 1 );
% degrees for trigonometric &
% real variables
nTVec = cell( nPol, 1 );
nHVec = cell( nPol, 1 );

% unique
TVecUnq = [ ];
HVecUnq = [ ];

% list of
% matrices
TList = [ ];
HList = [ ];

SBNrDegsMonoInt = zeros( nPol, 1 );
SBDegsAddSubDomPolInt = cell( nPol, 1 );
SBPosDegsDomPolInt = cell( nPol, 1 );

if ~ISSB
    % there is no sparse basis
    
    % add the number of variables
    % from the Gram matrices
    for i = 1 : nPol

        if ~isI( i ) && ~isD( i )
            % global positivity

            nVar = nVar + Q2size( i );

            % update Ks.s
            Ks.s = [ Ks.s QKsize( i ) ];        

            nKss = nKss + 1;
            % update Ks.scomplex
            if isC( i )
                Ks.scomplex = [ Ks.scomplex nKss ];
            end

            % count degrees
            if isT( i ) && ~isR( i )
                % trig. pol.

                % add degree
                nTVec{ i } = n{ i };

                % update cum. sum.      
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
            elseif ~isT( i ) && isR( i )        
                % real pol.

                % add degree
                nHVec{ i } = n{ i };

                % update cum. sum.
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            else
                % isT(i) && isR(i);
                % hybrid polynomial

                % add degree
                nTVec{ i } = n{ i }( 1 : ptype{ i }.trigonometric );
                nHVec{ i } = n{ i }( ptype{ i }.trigonometric + 1 : end );

                % update cum. sum.
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            end            
        elseif isI( i )
            % positivity on an interval        

            % K = 1 - scalar
            % K > 1 - matrix dim.
            K = k( i );

            if isT( i )
                % trigonometric polynomial

                % check whether field 'int'
                % has two or four elements;
                % else return error            
                switch length( ptype{ i }.int )  
                    case 2
                        % the lower & upper bounds
                        % of the positivity interval
                        inti1 = ptype{ i }.int( 1 );
                        inti2 = ptype{ i }.int( 2 );        


                        if isC( i )
                            % complex coefficients

                            % check whether the bounds
                            % are in [ -pi, pi ] &
                            % first element strictly smaller
                            % than the second
                            if inti1 >= -pi && inti1 <  pi && ...
                               inti2 <=  pi && inti2 > -pi && ...
                               inti1 < inti2                        

                                % the bounds are
                                % in [ -pi, pi ]
                                if inti1 ~= -pi && inti2 ~= pi && abs( inti1 ) ~= inti2
                                    % [ a, b ] in ( -pi, pi ) trig. pol. 
                                    % complex coef.

                                    % size of the 
                                    % second Gram matrix
                                    s2 = n{ i } * K;

                                    % S0: n+1, S1: n
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;      

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    nKss = nKss + 2;
                                    % update Ks.scomplex
                                    Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];                          

                                    % type of positivity
                                    isIVec( i ) = 1;

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 2;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                    end
                                elseif inti1 ~= -pi && inti2 ~= pi && abs( inti1 ) == inti2
                                    % [ -a, a ] in ( -pi, pi ) trig. pol. 
                                    % complex coef.

                                    % size of the 
                                    % second Gram matrix
                                    s2 = n{ i } * K;

                                    % S0: n+1, S1: n
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;   

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    nKss = nKss + 2;
                                    % update Ks.scomplex                               
                                    Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];                         

                                    % type of positivity
                                    isIVec( i ) = 2; 

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 2;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                    end
                                elseif inti1 ~= -pi && inti2 == pi
                                    % [ a, pi ] in [ -pi, pi ] trig. pol 
                                    % complex coef.

                                    % size of the 
                                    % second Gram matrix
                                    s2 = n{ i } * K;

                                    % S0: n+1, S1: n
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;  

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    nKss = nKss + 2;
                                    % update Ks.scomplex
                                    Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];

                                    % type of positivity
                                    isIVec( i ) = 3; 

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 2;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                    end
                                elseif inti1 == -pi && inti2 ~= pi
                                    % [ -pi, b ] in [ -pi, pi ] trig. pol 
                                    % complex coef.

                                    % size of the 
                                    % second Gram matrix
                                    s2 = n{ i } * K;

                                    % S0: n+1, S1: n
                                    nVar = nVar + Q2size( i ) + s2 ^ 2; 

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    nKss = nKss + 2;
                                    % update Ks.scomplex          
                                    Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];

                                    % type of positivity
                                    isIVec( i ) = 4;     

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 2;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                    end
                                elseif inti1 == -pi && inti2 == pi
                                    % [ -pi, pi ]                                
                                    % global positivity

                                    nVar = nVar + Q2size( i );

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) ];

                                    nKss = nKss + 1;
                                    % update Ks.scomplex                                
                                    Ks.scomplex = [ Ks.scomplex nKss ];

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = n{ i };

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 1;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                    end                                
                                else
                                    error( 'Positivity interval not well defined.' );
                                end
                            else
                                error( 'Positivity interval not well defined.' );
                            end
                        else        
                            % real coefficients

                            % check whether the bounds
                            % are in [ 0, pi ] &
                            % first element strictly smaller
                            % than the second
                            if inti1 >= 0    && inti1 <  pi && ...
                               inti2 <= pi   && inti2 >  0  && ...
                               inti1 < inti2

                                % the bounds are
                                % in [ 0, pi ]
                                if inti1 ~= 0 || inti2 ~= pi
                                    % [ a, b ] in [ 0, pi ] trig pol. 
                                    % real coefficients

                                    if iseven( nOld{ i } )
                                        % even degree

                                        % size of the 
                                        % second Gram matrix
                                        s2 = ( n{ i } - 1 ) * K;

                                        % S0: n+1, S1: n-1
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;

                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];

                                        % update nr. blocks
                                        % in Ks.s
                                        nKss = nKss + 2;

                                        % type of positivity
                                        isIVec( i ) = 5;

                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 2 ];

                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    else
                                        % odd degree

                                        % size of the 
                                        % second Gram matrix
                                        s2 = n{ i } * K;

                                        % S0: n, S1: n 
                                        nVar = nVar + 2 * s2 ^ 2;

                                        % update Ks.s
                                        Ks.s = [ Ks.s s2 s2 ];

                                        % update nr. blocks
                                        % in Ks.s
                                        nKss = nKss + 2;

                                        % type of positivity
                                        isIVec( i ) = 6;

                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = n{ i } - 1;

                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 1;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                        end
                                    end
                                elseif inti1 == 0 && inti2 == pi
                                    % [ 0, pi ]                                
                                    % global positivity

                                    nVar = nVar + Q2size( i ); 

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) ];

                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 1;

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = n{ i };

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 1;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                    end
                                else
                                    error( 'Positivity interval not well defined.' );
                                end
                            else
                                error( 'Positivity interval not well defined.' );
                            end
                        end
                    case 4
                        % the lower & upper bounds
                        % of the two positivity 
                        % intervals
                        inti1 = ptype{ i }.int( 1 );
                        inti2 = ptype{ i }.int( 2 );
                        inti3 = ptype{ i }.int( 3 );
                        inti4 = ptype{ i }.int( 4 );

                        if isC( i )
                            % complex coefficients

                            % check whether the bounds
                            % are in [ -pi, pi ] &
                            % left element strictly smaller
                            % than the one from its right
                            if inti1 >= -pi && inti1 <  pi && ...
                               inti2 <=  pi && inti2 > -pi && ...
                               inti3 <=  pi && inti3 > -pi && ...
                               inti4 <=  pi && inti4 > -pi && ...                                                                                 
                               inti1 < inti2 < inti3 < inti4                                                    

                                % the bounds are
                                % in [ -pi, pi ]
                                if abs( inti2 ) ~= inti3
                                    % [ -pi,  a ] U [ b, pi ] trig. pol. 
                                    % complex coef.

                                    % size of the 
                                    % second Gram matrix
                                    s2 = n{ i } * K;

                                    % S0: n+1, S1: n
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;  

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    nKss = nKss + 2;
                                    % update Ks.scomplex                                
                                    Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];

                                    % type of positivity
                                    isIVec( i ) = 7;   

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 2;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                    end
                                elseif abs( inti2 ) == inti3
                                    % [ -pi,  -a ] U [ a, pi ] trig. pol. 
                                    % complex coef.

                                    % size of the 
                                    % second Gram matrix
                                    s2 = n{ i } * K;

                                    % S0: n+1, S1: n
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;  

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    nKss = nKss + 2;
                                    % update Ks.scomplex                                
                                    Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];

                                    % type of positivity
                                    isIVec( i ) = 8;   

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 2;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                    end
                                else
                                    error( 'Positivity interval not well defined.' );
                                end                     
                            else
                                error( 'Positivity interval not well defined.' );
                            end
                        else
                            % real coefficients

                            % check whether the bounds
                            % are in [ 0, pi ] &
                            % left element strictly smaller
                            % than the one from its right
                            if inti1 >= 0 && inti1 <= pi && ...
                               inti2 >= 0 && inti2 <= pi && ...
                               inti3 >= 0 && inti3 <= pi && ...
                               inti4 >= 0 && inti4 <= pi && ...                                                                                 
                               inti1 < inti2 < inti3 < inti4                                                    

                                % the bounds are
                                % in [ 0, pi ]
                                % [ 0, a ] U [ b, pi ] trig pol. real coef.
                                if inti1 == 0 && inti4 == pi
                                    if iseven( nOld{ i } )
                                            % even degree

                                            % size of the 
                                            % second Gram matrix
                                            s2 = ( n{ i } - 1 ) * K;

                                            % S0: n+1, S1: n-1
                                            nVar = nVar + Q2size( i ) + s2 ^ 2;

                                            % update Ks.s
                                            Ks.s = [ Ks.s QKsize( i ) s2 ];

                                            % update nr. blocks
                                            % in Ks.s
                                            nKss = nKss + 2;                               

                                            % type of positivity
                                            isIVec( i ) = 9;

                                            % count degrees;
                                            % add degree
                                            nTVec{ i } = [ n{ i } n{ i } - 2 ];

                                            % update cum. sum.
                                            if i == 1
                                                nTVecCS( i ) = 2;
                                            else
                                                nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                            end
                                        else
                                            % odd degree

                                            % size of the 
                                            % second Gram matrix
                                            s2 = n{ i } * K;

                                            % S0: n, S1: n 
                                            nVar = nVar + 2 * s2 ^ 2;

                                            % update Ks.s
                                            Ks.s = [ Ks.s s2 s2 ];

                                            % update nr. blocks
                                            % in Ks.s
                                            nKss = nKss + 2;                                        

                                            % type of positivity
                                            isIVec( i ) = 10;   

                                            % count degrees;
                                            % add degree
                                            nTVec{ i } = n{ i } - 1;

                                            % update cum. sum.
                                            if i == 1
                                                nTVecCS( i ) = 1;
                                            else
                                                nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                            end
                                    end
                                else
                                    error( 'Positivity interval not well defined.' );
                                end
                            else
                                error( 'Positivity interval not well defined.' );
                            end                        
                        end               
                end
            else
                % real polynomial

                % check whether field 'int'
                % has two or four elements;
                % else return error
                switch length( ptype{ i }.int )                
                    case 2
                        % the lower & upper bounds
                        % of the positivity interval
                        inti1 = ptype{ i }.int( 1 );
                        inti2 = ptype{ i }.int( 2 );

                        % size n  : (n-1)/2+1 = n/2+0.5 = n{i}+0.5
                        % size n-1: (n-2)/2+1 = n/2 = n{i}                    

                        if ~isinf( inti1 ) && ~isinf( inti2 )                
                            % [ a, b ]

                            if inti1 >= inti2
                                error( 'Positivity interval not well defined.' );  
                            end                 

                            if iseven( nOld{ i } )
                                % even degree

                                % size of the 
                                % second Gram matrix
                                s2 = n{ i } * K;

                                % S0: n+1,  S1: n-1
                                nVar = nVar + Q2size( i ) + s2 ^ 2;

                                % update Ks.s
                                Ks.s = [ Ks.s QKsize( i ) s2 ];

                                % update nr. blocks
                                % in Ks.s
                                nKss = nKss + 2;                               

                                % type of positivity
                                isIVec( i ) = 11;

                                % count degrees;
                                % add degree
                                nHVec{ i } = [ n{ i } n{ i } - 1 ];

                                % update cum. sum.
                                if i == 1
                                    nHVecCS( i ) = 2;
                                else
                                    nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                end
                            else
                                % odd degree

                                % size of the 
                                % second Gram matrix
                                s2 = ( n{ i } + 0.5 ) * K;

                                % S0: n,  S1: n
                                nVar = nVar + 2 * s2 ^ 2;

                                % update Ks.s
                                Ks.s = [ Ks.s s2 s2 ];

                                % update nr. blocks
                                % in Ks.s
                                nKss = nKss + 2; 

                                % type of positivity
                                isIVec( i ) = 12;

                                % count degrees;
                                % add degree
                                nHVec{ i } = n{ i } - 0.5;

                                % update cum. sum.
                                if i == 1
                                    nHVecCS( i ) = 1;
                                else
                                    nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                end
                            end    
                        elseif ~isinf( inti1 ) && inti2 == Inf                                         
                            % [ a, Inf ) 

                            if iseven( nOld{ i } )
                                % even degree

                                % size of the 
                                % second Gram matrix
                                s2 = n{ i } * K;

                                % S0: n+1,  S1: n-1
                                nVar = nVar + Q2size( i ) + s2 ^ 2;

                                % update Ks.s
                                Ks.s = [ Ks.s QKsize( i ) s2 ];

                                % update nr. blocks
                                % in Ks.s
                                nKss = nKss + 2; 

                                % type of positivity
                                isIVec( i ) = 13; 

                                % count degrees;
                                % add degree
                                nHVec{ i } = [ n{ i } n{ i } - 1 ];

                                % update cum. sum.
                                if i == 1
                                    nHVecCS( i ) = 2;
                                else
                                    nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                end
                            else
                                % odd degree

                                % size of the 
                                % second Gram matrix
                                s2 = ( n{ i } + 0.5 ) * K;

                                % S0: n,  S1: n
                                nVar = nVar + 2 * s2 ^ 2;

                                % update Ks.s
                                Ks.s = [ Ks.s s2 s2 ];

                                % update nr. blocks
                                % in Ks.s
                                nKss = nKss + 2; 

                                % type of positivity
                                isIVec( i ) = 14; 

                                % count degrees;
                                % add degree
                                nHVec{ i } = n{ i } - 0.5;

                                % update cum. sum.
                                if i == 1
                                    nHVecCS( i ) = 1;
                                else
                                    nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                end
                            end     
                        elseif inti1 == -Inf   && ~isinf( inti2 )
                            % ( -Inf, b ]

                            if iseven( nOld{ i } )
                                % even degree

                                % size of the 
                                % second Gram matrix
                                s2 = n{ i } * K;

                                % S0: n+1,  S1: n-1
                                nVar = nVar + Q2size( i ) + s2 ^ 2;

                                % update Ks.s
                                Ks.s = [ Ks.s QKsize( i ) s2 ];

                                % update nr. blocks
                                % in Ks.s
                                nKss = nKss + 2; 

                                % type of positivity
                                isIVec( i ) = 15; 

                                % count degrees;
                                % add degree
                                nHVec{ i } = [ n{ i } n{ i } - 1 ];

                                % update cum. sum.
                                if i == 1
                                    nHVecCS( i ) = 2;
                                else
                                    nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                end
                            else
                                % odd degree

                                % size of the 
                                % second Gram matrix
                                s2 = ( n{ i } + 0.5 ) * K;

                                % S0: n,  S1: n
                                nVar = nVar + 2 * s2 ^ 2;

                                % update Ks.s
                                Ks.s = [ Ks.s s2 s2 ];

                                % update nr. blocks
                                % in Ks.s
                                nKss = nKss + 2; 

                                % type of positivity
                                isIVec( i ) = 16; 

                                % count degrees;
                                % add degree
                                nHVec{ i } = n{ i } - 0.5;

                                % update cum. sum.
                                if i == 1
                                    nHVecCS( i ) = 1;
                                else
                                    nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                end
                            end                
                        elseif inti1 == -Inf && inti2 == Inf                                                
                            % ( -Inf, Inf )
                            % global positivity

                            % check for even degree
                            if ~iseven( nOld{ i } )
                                error( 'Maximum degree for real variable not even.' );                                                
                            end

                            nVar = nVar + Q2size( i );                           

                            % update Ks.s
                            Ks.s = [ Ks.s QKsize( i ) ];

                            % update nr. blocks
                            % in Ks.s
                            nKss = nKss + 1; 

                            % count degrees;
                            % add degree
                            nHVec{ i } = n{ i };

                            % update cum. sum.
                            if i == 1
                                nHVecCS( i ) = 1;
                            else
                                nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                            end
                        else
                            error( 'Positivity interval not well defined.' );                
                        end 
                    case 4
                        if   ptype{ i }.int( 1 ) == -Inf && ptype{ i }.int( 4 ) == Inf && ...
                            ~isinf( ptype{ i }.int( 2 ) ) && ~isinf( ptype{ i }.int( 3 ) )
                            % ( -Inf, a } U [ b, Inf )

                            if iseven( nOld{ i } )
                                % even degree

                                % size of the 
                                % second Gram matrix
                                s2 = n{ i } * K;

                                % S0: n+1,  S1: n-1
                                nVar = nVar + Q2size( i ) + s2 ^ 2;

                                % update Ks.s
                                Ks.s = [ Ks.s QKsize( i ) s2 ];

                                % update nr. blocks
                                % in Ks.s
                                nKss = nKss + 2; 

                                % type of positivity
                                isIVec( i ) = 17;

                                % count degrees;
                                % add degree
                                nHVec{ i } = [ n{ i } n{ i } - 1 ];

                                % update cum. sum.
                                if i == 1
                                    nHVecCS( i ) = 2;
                                else
                                    nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                end
                            else
                                % odd degree

                                % this case is not possible

                                error( 'Odd degree positive polynomial on ( -Inf, a ] U [ b, Inf ) not possible.' );
                            end 
                        else
                            error( 'Positivity intervals not well defined.' );   
                        end
                end
            end  
        else
            % positivity on domain       

            % S0
            nVar = nVar + Q2size( i );
            Ks.s = [ Ks.s QKsize( i ) ];

            % for ever pol. from the pos. dom.
            for DomPol = 1 : nDomPol( i )
                idx = nDomPolCS( i ) + DomPol;
                % Si
                nVar = nVar + QD2size( idx );
                Ks.s = [ Ks.s QDKsize( idx ) ];
            end

            if isC( i )
                nKss = nKss + nDomPol( i ) + 1;
                % update Ks.scomplex
                Ks.scomplex = [ Ks.scomplex nKss - nDomPol( i ) : nKss ];
            end

            % count degrees
            if isT( i ) && ~isR( i )
                % trig. pol.

                % add degree
                nTVec{ i } = [ n{ i } reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ),...
                                               1, nDomPol( i ) * d( i ) ) ];

                % update cum. sum.      
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
            elseif ~isT( i ) && isR( i )        
                % real pol.

                % add degree
                nHVec{ i } = [ n{ i } reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ),...
                                               1, nDomPol( i ) * d( i ) ) ];

                % update cum. sum.
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            else
                % isT(i) && isR(i);
                % hybrid polynomial

                % add degree
                nTVec{ i } = [ n{ i }( 1 : ptype{ i }.trigonometric ), ...
                               reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), 1 : ptype{ i }.trigonometric ),...
                                        1, nDomPol( i ) * ptype{ i }.trigonometric ) ];
                nHVec{ i } = [ n{ i }( ptype{ i }.trigonometric + 1 : end ), ...
                               reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), ptype{ i }.trigonometric + 1 : end ),...
                                        1, nDomPol( i ) * ( d( i ) - ptype{ i }.trigonometric ) ) ];

                % update cum. sum.
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            end                
        end
    end
else
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % there is at least one polynomial 
    % with sparse basis
    
    % add the number of variables
    % from the Gram matrices
    for i = 1 : nPol

        if ~isI( i ) && ~isD( i )
            % global positivity

            nVar = nVar + Q2size( i );

            % update Ks.s
            Ks.s = [ Ks.s QKsize( i ) ];        

            nKss = nKss + 1;
            % update Ks.scomplex
            if isC( i )
                Ks.scomplex = [ Ks.scomplex nKss ];
            end

            % count degrees
            if isT( i ) && ~isR( i )
                % trig. pol.

                % add degree
                nTVec{ i } = n{ i };

                % update cum. sum.      
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
            elseif ~isT( i ) && isR( i )        
                % real pol.

                % add degree
                nHVec{ i } = n{ i };

                % update cum. sum.
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            else
                % isT(i) && isR(i);
                % hybrid polynomial

                % add degree
                nTVec{ i } = n{ i }( 1 : ptype{ i }.trigonometric );
                nHVec{ i } = n{ i }( ptype{ i }.trigonometric + 1 : end );

                % update cum. sum.
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            end            
        elseif isI( i )
            % positivity on an interval        

            % K = 1 - scalar
            % K > 1 - matrix dim.
            K = k( i );

            if isT( i )
                % trigonometric polynomial

                % check whether field 'int'
                % has two or four elements;
                % else return error            
                switch length( ptype{ i }.int )  
                    case 2
                        % the lower & upper bounds
                        % of the positivity interval
                        inti1 = ptype{ i }.int( 1 );
                        inti2 = ptype{ i }.int( 2 );        


                        if isC( i )
                            % complex coefficients

                            % check whether the bounds
                            % are in [ -pi, pi ] &
                            % first element strictly smaller
                            % than the second
                            if inti1 >= -pi && inti1 <  pi && ...
                               inti2 <=  pi && inti2 > -pi && ...
                               inti1 < inti2                        

                                % the bounds are
                                % in [ -pi, pi ]
                                if inti1 ~= -pi && inti2 ~= pi && abs( inti1 ) ~= inti2
                                    % [ a, b ] in ( -pi, pi ) trig. pol. 
                                    % complex coef.

                                    if ~isSB( i )
                                        % size of the
                                        % second Gram matrix
                                        s2 = n{ i } * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 1;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    else
                                        [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                        getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                  
                                        % size of the
                                        % second Gram matrix
                                        s2 = SBNrDegsMonoInt( i ) * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 1;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    end
                                elseif inti1 ~= -pi && inti2 ~= pi && abs( inti1 ) == inti2
                                    % [ -a, a ] in ( -pi, pi ) trig. pol. 
                                    % complex coef.

                                    if ~isSB( i )
                                        % size of the
                                        % second Gram matrix
                                        s2 = n{ i } * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 2;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    else
                                        [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                        getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                  
                                        % size of the
                                        % second Gram matrix
                                        s2 = SBNrDegsMonoInt( i ) * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 2;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    end
                                elseif inti1 ~= -pi && inti2 == pi
                                    % [ a, pi ] in [ -pi, pi ] trig. pol 
                                    % complex coef.
                                    
                                    if ~isSB( i )
                                        % size of the
                                        % second Gram matrix
                                        s2 = n{ i } * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 3;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    else
                                        [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                        getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                  
                                        % size of the
                                        % second Gram matrix
                                        s2 = SBNrDegsMonoInt( i ) * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 3;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    end
                                elseif inti1 == -pi && inti2 ~= pi
                                    % [ -pi, b ] in [ -pi, pi ] trig. pol 
                                    % complex coef.
                                    
                                    if ~isSB( i )
                                        % size of the
                                        % second Gram matrix
                                        s2 = n{ i } * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 4;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    else
                                        [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                        getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );

                                        % size of the
                                        % second Gram matrix
                                        s2 = SBNrDegsMonoInt( i ) * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 4;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    end
                                elseif inti1 == -pi && inti2 == pi
                                    % [ -pi, pi ]                                
                                    % global positivity

                                    nVar = nVar + Q2size( i );

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) ];

                                    nKss = nKss + 1;
                                    % update Ks.scomplex                                
                                    Ks.scomplex = [ Ks.scomplex nKss ];

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = n{ i };

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 1;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                    end                                
                                else
                                    error( 'Positivity interval not well defined.' );
                                end
                            else
                                error( 'Positivity interval not well defined.' );
                            end
                        else        
                            % real coefficients

                            % check whether the bounds
                            % are in [ 0, pi ] &
                            % first element strictly smaller
                            % than the second
                            if inti1 >= 0    && inti1 <  pi && ...
                               inti2 <= pi   && inti2 >  0  && ...
                               inti1 < inti2

                                % the bounds are
                                % in [ 0, pi ]
                                if inti1 ~= 0 || inti2 ~= pi
                                    % [ a, b ] in [ 0, pi ] trig pol. 
                                    % real coefficients

                                    if iseven( nOld{ i } )
                                        % even degree
                                        
                                        if ~isSB( i )
                                            % size of the
                                            % second Gram matrix
                                            s2 = ( n{ i } - 1 ) * K;
                                            
                                            % S0: n+1, S1: n-1
                                            nVar = nVar + Q2size( i ) + s2 ^ 2;
                                            
                                            % update Ks.s
                                            Ks.s = [ Ks.s QKsize( i ) s2 ];
                                            
                                            % update nr. blocks
                                            % in Ks.s
                                            nKss = nKss + 2;
                                            
                                            % type of positivity
                                            isIVec( i ) = 5;
                                            
                                            % count degrees;
                                            % add degree
                                            nTVec{ i } = [ n{ i } n{ i } - 2 ];
                                            
                                            % update cum. sum.
                                            if i == 1
                                                nTVecCS( i ) = 2;
                                            else
                                                nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                            end
                                        else
                                            [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                            getspbasisint( n{ i }, nOld{ i }, n{ i } - 2, nOld{ i } - 2, 1, 0, 3, [ 0; 1; 2 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                            
                                            % size of the
                                            % second Gram matrix
                                            s2 = SBNrDegsMonoInt( i ) * K;
                                            
                                            % S0: n+1, S1: n-1
                                            nVar = nVar + Q2size( i ) + s2 ^ 2;
                                            
                                            % update Ks.s
                                            Ks.s = [ Ks.s QKsize( i ) s2 ];
                                            
                                            % update nr. blocks
                                            % in Ks.s
                                            nKss = nKss + 2;
                                            
                                            % type of positivity
                                            isIVec( i ) = 5;
                                            
                                            % count degrees;
                                            % add degree
                                            nTVec{ i } = [ n{ i } n{ i } - 2 ];
                                            
                                            % update cum. sum.
                                            if i == 1
                                                nTVecCS( i ) = 2;
                                            else
                                                nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                            end
                                        end
                                    else
                                        % odd degree
                                        
                                        if ~isSB( i )
                                            % size of the
                                            % second Gram matrix
                                            s2 = n{ i } * K;
                                            
                                            % S0: n, S1: n
                                            nVar = nVar + 2 * s2 ^ 2;
                                            
                                            % update Ks.s
                                            Ks.s = [ Ks.s s2 s2 ];
                                            
                                            % update nr. blocks
                                            % in Ks.s
                                            nKss = nKss + 2;
                                            
                                            % type of positivity
                                            isIVec( i ) = 6;
                                            
                                            % count degrees;
                                            % add degree
                                            nTVec{ i } = n{ i } - 1;
                                            
                                            % update cum. sum.
                                            if i == 1
                                                nTVecCS( i ) = 1;
                                            else
                                                nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                            end
                                        else
                                            [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                            getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );

                                            % size of the
                                            % second Gram matrix
                                            s2 = SBNrDegsMonoInt( i ) * K;
                                            
                                            % S0: n, S1: n
                                            nVar = nVar + 2 * s2 ^ 2;
                                            
                                            % update Ks.s
                                            Ks.s = [ Ks.s s2 s2 ];
                                            
                                            % update nr. blocks
                                            % in Ks.s
                                            nKss = nKss + 2;
                                            
                                            % type of positivity
                                            isIVec( i ) = 6;
                                            
                                            % count degrees;
                                            % add degree
                                            nTVec{ i } = n{ i } - 1;
                                            
                                            % update cum. sum.
                                            if i == 1
                                                nTVecCS( i ) = 1;
                                            else
                                                nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                            end
                                        end
                                    end
                                elseif inti1 == 0 && inti2 == pi
                                    % [ 0, pi ]                                
                                    % global positivity

                                    nVar = nVar + Q2size( i ); 

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) ];

                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 1;

                                    % count degrees;
                                    % add degree
                                    nTVec{ i } = n{ i };

                                    % update cum. sum.
                                    if i == 1
                                        nTVecCS( i ) = 1;
                                    else
                                        nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                    end
                                else
                                    error( 'Positivity interval not well defined.' );
                                end
                            else
                                error( 'Positivity interval not well defined.' );
                            end
                        end
                    case 4
                        % the lower & upper bounds
                        % of the two positivity 
                        % intervals
                        inti1 = ptype{ i }.int( 1 );
                        inti2 = ptype{ i }.int( 2 );
                        inti3 = ptype{ i }.int( 3 );
                        inti4 = ptype{ i }.int( 4 );

                        if isC( i )
                            % complex coefficients

                            % check whether the bounds
                            % are in [ -pi, pi ] &
                            % left element strictly smaller
                            % than the one from its right
                            if inti1 >= -pi && inti1 <  pi && ...
                               inti2 <=  pi && inti2 > -pi && ...
                               inti3 <=  pi && inti3 > -pi && ...
                               inti4 <=  pi && inti4 > -pi && ...                                                                                 
                               inti1 < inti2 < inti3 < inti4                                                    

                                % the bounds are
                                % in [ -pi, pi ]
                                if abs( inti2 ) ~= inti3
                                    % [ -pi,  a ] U [ b, pi ] trig. pol. 
                                    % complex coef.
                                    
                                    if ~isSB( i )
                                        % size of the
                                        % second Gram matrix
                                        s2 = n{ i } * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 7;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    else
                                        [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                        getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                        
                                        % size of the
                                        % second Gram matrix
                                        s2 = SBNrDegsMonoInt( i ) * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 7;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    end
                                elseif abs( inti2 ) == inti3
                                    % [ -pi,  -a ] U [ a, pi ] trig. pol. 
                                    % complex coef.

                                    if ~isSB( i )
                                        % size of the
                                        % second Gram matrix
                                        s2 = n{ i } * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 8;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    else
                                        [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                        getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );

                                        % size of the
                                        % second Gram matrix
                                        s2 = SBNrDegsMonoInt( i ) * K;
                                        
                                        % S0: n+1, S1: n
                                        nVar = nVar + Q2size( i ) + s2 ^ 2;
                                        
                                        % update Ks.s
                                        Ks.s = [ Ks.s QKsize( i ) s2 ];
                                        
                                        nKss = nKss + 2;
                                        % update Ks.scomplex
                                        Ks.scomplex = [ Ks.scomplex nKss - 1 nKss ];
                                        
                                        % type of positivity
                                        isIVec( i ) = 8;
                                        
                                        % count degrees;
                                        % add degree
                                        nTVec{ i } = [ n{ i } n{ i } - 1 ];
                                        
                                        % update cum. sum.
                                        if i == 1
                                            nTVecCS( i ) = 2;
                                        else
                                            nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                        end
                                    end
                                else
                                    error( 'Positivity interval not well defined.' );
                                end                     
                            else
                                error( 'Positivity interval not well defined.' );
                            end
                        else
                            % real coefficients

                            % check whether the bounds
                            % are in [ 0, pi ] &
                            % left element strictly smaller
                            % than the one from its right
                            if inti1 >= 0 && inti1 <= pi && ...
                               inti2 >= 0 && inti2 <= pi && ...
                               inti3 >= 0 && inti3 <= pi && ...
                               inti4 >= 0 && inti4 <= pi && ...                                                                                 
                               inti1 < inti2 < inti3 < inti4                                                    

                                % the bounds are
                                % in [ 0, pi ]
                                % [ 0, a ] U [ b, pi ] trig pol. real coef.
                                if inti1 == 0 && inti4 == pi
                                    if iseven( nOld{ i } )
                                            % even degree
                                            
                                            if ~isSB( i )
                                                % size of the
                                                % second Gram matrix
                                                s2 = ( n{ i } - 1 ) * K;
                                                
                                                % S0: n+1, S1: n-1
                                                nVar = nVar + Q2size( i ) + s2 ^ 2;
                                                
                                                % update Ks.s
                                                Ks.s = [ Ks.s QKsize( i ) s2 ];
                                                
                                                % update nr. blocks
                                                % in Ks.s
                                                nKss = nKss + 2;
                                                
                                                % type of positivity
                                                isIVec( i ) = 9;
                                                
                                                % count degrees;
                                                % add degree
                                                nTVec{ i } = [ n{ i } n{ i } - 2 ];
                                                
                                                % update cum. sum.
                                                if i == 1
                                                    nTVecCS( i ) = 2;
                                                else
                                                    nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                                end
                                            else
                                                [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                                getspbasisint( n{ i }, nOld{ i }, n{ i } - 2, nOld{ i } - 2, 1, 0, 3, [ 0; 1; 2 ], SBNrDegs( i ), SBDegsAddSub{ i } );

                                                % size of the
                                                % second Gram matrix
                                                s2 = SBNrDegsMonoInt( i ) * K;
                                                
                                                % S0: n+1, S1: n-1
                                                nVar = nVar + Q2size( i ) + s2 ^ 2;
                                                
                                                % update Ks.s
                                                Ks.s = [ Ks.s QKsize( i ) s2 ];
                                                
                                                % update nr. blocks
                                                % in Ks.s
                                                nKss = nKss + 2;
                                                
                                                % type of positivity
                                                isIVec( i ) = 9;
                                                
                                                % count degrees;
                                                % add degree
                                                nTVec{ i } = [ n{ i } n{ i } - 2 ];
                                                
                                                % update cum. sum.
                                                if i == 1
                                                    nTVecCS( i ) = 2;
                                                else
                                                    nTVecCS( i ) = nTVecCS( i - 1 ) + 2;
                                                end
                                            end
                                        else
                                            % odd degree
                                            
                                            if ~isSB( i )
                                                % size of the
                                                % second Gram matrix
                                                s2 = n{ i } * K;
                                                
                                                % S0: n, S1: n
                                                nVar = nVar + 2 * s2 ^ 2;
                                                
                                                % update Ks.s
                                                Ks.s = [ Ks.s s2 s2 ];
                                                
                                                % update nr. blocks
                                                % in Ks.s
                                                nKss = nKss + 2;
                                                
                                                % type of positivity
                                                isIVec( i ) = 10;
                                                
                                                % count degrees;
                                                % add degree
                                                nTVec{ i } = n{ i } - 1;
                                                
                                                % update cum. sum.
                                                if i == 1
                                                    nTVecCS( i ) = 1;
                                                else
                                                    nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                                end
                                            else
                                                [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                                getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 1, 1, 0, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );

                                                % size of the
                                                % second Gram matrix
                                                s2 = SBNrDegsMonoInt( i ) * K;
                                                
                                                % S0: n, S1: n
                                                nVar = nVar + 2 * s2 ^ 2;
                                                
                                                % update Ks.s
                                                Ks.s = [ Ks.s s2 s2 ];
                                                
                                                % update nr. blocks
                                                % in Ks.s
                                                nKss = nKss + 2;
                                                
                                                % type of positivity
                                                isIVec( i ) = 10;
                                                
                                                % count degrees;
                                                % add degree
                                                nTVec{ i } = n{ i } - 1;
                                                
                                                % update cum. sum.
                                                if i == 1
                                                    nTVecCS( i ) = 1;
                                                else
                                                    nTVecCS( i ) = nTVecCS( i - 1 ) + 1;
                                                end
                                            end
                                    end
                                else
                                    error( 'Positivity interval not well defined.' );
                                end
                            else
                                error( 'Positivity interval not well defined.' );
                            end                        
                        end               
                end
            else
                % real polynomial

                % check whether field 'int'
                % has two or four elements;
                % else return error
                switch length( ptype{ i }.int )                
                    case 2
                        % the lower & upper bounds
                        % of the positivity interval
                        inti1 = ptype{ i }.int( 1 );
                        inti2 = ptype{ i }.int( 2 );

                        % size n  : (n-1)/2+1 = n/2+0.5 = n{i}+0.5
                        % size n-1: (n-2)/2+1 = n/2 = n{i}                    

                        if ~isinf( inti1 ) && ~isinf( inti2 )                
                            % [ a, b ]

                            if inti1 >= inti2
                                error( 'Positivity interval not well defined.' );  
                            end                 

                            if iseven( nOld{ i } )
                                % even degree

                                if ~isSB( i )

                                    % size of the 
                                    % second Gram matrix
                                    s2 = n{ i } * K;

                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;                               

                                    % type of positivity
                                    isIVec( i ) = 11;

                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                else
                                    [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                    getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 2, 0, 1, 3, [ 0; 1; 2 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                    % size of the 
                                    % second Gram matrix
                                    s2 = SBNrDegsMonoInt( i ) * K;

                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;

                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];

                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;                               

                                    % type of positivity
                                    isIVec( i ) = 11;

                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];

                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                end
                            else
                                % odd degree

                                if ~isSB( i )
                                    % size of the 
                                    % second Gram matrix
                                    s2 = ( n{ i } + 0.5 ) * K;

                                    % S0: n,  S1: n
                                    nVar = nVar + 2 * s2 ^ 2;

                                    % update Ks.s
                                    Ks.s = [ Ks.s s2 s2 ];

                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2; 

                                    % type of positivity
                                    isIVec( i ) = 12;

                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = n{ i } - 0.5;

                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 1;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                    end
                                else
                                    [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                    getspbasisint( n{ i } + 0.5, nOld{ i }, n{ i } - 0.5, nOld{ i } - 1, 0, 1, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );

                                    % size of the 
                                    % second Gram matrix
                                    s2 = SBNrDegsMonoInt( i ) * K;

                                    % S0: n,  S1: n
                                    nVar = nVar + 2 * s2 ^ 2;

                                    % update Ks.s
                                    Ks.s = [ Ks.s s2 s2 ];

                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2; 

                                    % type of positivity
                                    isIVec( i ) = 12;

                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = n{ i } - 0.5;

                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 1;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                    end

                                end
                            end    
                        elseif ~isinf( inti1 ) && inti2 == Inf                                         
                            % [ a, Inf ) 

                            if iseven( nOld{ i } )
                                % even degree
                                
                                if ~isSB( i )
                                    % size of the
                                    % second Gram matrix
                                    s2 = n{ i } * K;
                                    
                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 13;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                else
                                    [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                    getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 2, 0, 1, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                
                                    % size of the
                                    % second Gram matrix
                                    s2 = SBNrDegsMonoInt( i ) * K;
                                    
                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 13;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                end
                            else
                                % odd degree
                                
                                if ~isSB( i )
                                    % size of the
                                    % second Gram matrix
                                    s2 = ( n{ i } + 0.5 ) * K;
                                    
                                    % S0: n,  S1: n
                                    nVar = nVar + 2 * s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s s2 s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 14;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = n{ i } - 0.5;
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 1;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                    end
                                else
                                    [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                    getspbasisint( n{ i } + 0.5, nOld{ i }, n{ i } - 0.5, nOld{ i } - 1, 0, 1, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                
                                    % size of the
                                    % second Gram matrix
                                    s2 = SBNrDegsMonoInt( i ) * K;
                                    
                                    % S0: n,  S1: n
                                    nVar = nVar + 2 * s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s s2 s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 14;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = n{ i } - 0.5;
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 1;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                    end
                                end
                            end     
                        elseif inti1 == -Inf   && ~isinf( inti2 )
                            % ( -Inf, b ]

                            if iseven( nOld{ i } )
                                % even degree
                                
                                if ~isSB( i )
                                    % size of the
                                    % second Gram matrix
                                    s2 = n{ i } * K;
                                    
                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 15;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                else
                                    [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                    getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 2, 0, 1, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                
                                    % size of the
                                    % second Gram matrix
                                    s2 = SBNrDegsMonoInt( i ) * K;
                                    
                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 15;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                end
                            else
                                % odd degree
                                
                                if ~isSB( i )
                                    % size of the
                                    % second Gram matrix
                                    s2 = ( n{ i } + 0.5 ) * K;
                                    
                                    % S0: n,  S1: n
                                    nVar = nVar + 2 * s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s s2 s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 16;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = n{ i } - 0.5;
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 1;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                    end
                                else
                                    [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                    getspbasisint( n{ i } + 0.5, nOld{ i }, n{ i } - 0.5, nOld{ i } - 1, 0, 1, 2, [ 0; 1 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                
                                    % size of the
                                    % second Gram matrix
                                    s2 = SBNrDegsMonoInt( i ) * K;
                                    
                                    % S0: n,  S1: n
                                    nVar = nVar + 2 * s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s s2 s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 16;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = n{ i } - 0.5;
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 1;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                                    end
                                end
                            end                
                        elseif inti1 == -Inf && inti2 == Inf                                                
                            % ( -Inf, Inf )
                            % global positivity

                            % check for even degree
                            if ~iseven( nOld{ i } )
                                error( 'Maximum degree for real variable not even.' );                                                
                            end

                            nVar = nVar + Q2size( i );                           

                            % update Ks.s
                            Ks.s = [ Ks.s QKsize( i ) ];

                            % update nr. blocks
                            % in Ks.s
                            nKss = nKss + 1; 

                            % count degrees;
                            % add degree
                            nHVec{ i } = n{ i };

                            % update cum. sum.
                            if i == 1
                                nHVecCS( i ) = 1;
                            else
                                nHVecCS( i ) = nHVecCS( i - 1 ) + 1;
                            end
                        else
                            error( 'Positivity interval not well defined.' );                
                        end 
                    case 4
                        if   ptype{ i }.int( 1 ) == -Inf && ptype{ i }.int( 4 ) == Inf && ...
                            ~isinf( ptype{ i }.int( 2 ) ) && ~isinf( ptype{ i }.int( 3 ) )
                            % ( -Inf, a } U [ b, Inf )

                            if iseven( nOld{ i } )
                                % even degree
                                
                                if ~isSB( i )
                                    % size of the
                                    % second Gram matrix
                                    s2 = n{ i } * K;
                                    
                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 17;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                else
                                    [ SBNrDegsMonoInt( i ), SBDegsAddSubDomPolInt{ i }, SBPosDegsDomPolInt{ i } ] = ...
                                    getspbasisint( n{ i }, nOld{ i }, n{ i } - 1, nOld{ i } - 2, 0, 1, 3, [ 0; 1; 2 ], SBNrDegs( i ), SBDegsAddSub{ i } );
                                
                                    % size of the
                                    % second Gram matrix
                                    s2 = SBNrDegsMonoInt( i ) * K;
                                    
                                    % S0: n+1,  S1: n-1
                                    nVar = nVar + Q2size( i ) + s2 ^ 2;
                                    
                                    % update Ks.s
                                    Ks.s = [ Ks.s QKsize( i ) s2 ];
                                    
                                    % update nr. blocks
                                    % in Ks.s
                                    nKss = nKss + 2;
                                    
                                    % type of positivity
                                    isIVec( i ) = 17;
                                    
                                    % count degrees;
                                    % add degree
                                    nHVec{ i } = [ n{ i } n{ i } - 1 ];
                                    
                                    % update cum. sum.
                                    if i == 1
                                        nHVecCS( i ) = 2;
                                    else
                                        nHVecCS( i ) = nHVecCS( i - 1 ) + 2;
                                    end
                                end
                            else
                                % odd degree

                                % this case is not possible

                                error( 'Odd degree positive polynomial on ( -Inf, a ] U [ b, Inf ) not possible.' );
                            end 
                        else
                            error( 'Positivity intervals not well defined.' );   
                        end
                end
            end  
        else
            % positivity on domain       

            % S0
            nVar = nVar + Q2size( i );
            Ks.s = [ Ks.s QKsize( i ) ];

            % for ever pol. from the pos. dom.
            for DomPol = 1 : nDomPol( i )
                idx = nDomPolCS( i ) + DomPol;
                % Si
                nVar = nVar + QD2size( idx );
                Ks.s = [ Ks.s QDKsize( idx ) ];
            end

            if isC( i )
                nKss = nKss + nDomPol( i ) + 1;
                % update Ks.scomplex
                Ks.scomplex = [ Ks.scomplex nKss - nDomPol( i ) : nKss ];
            end

            % count degrees
            if isT( i ) && ~isR( i )
                % trig. pol.

                % add degree
                nTVec{ i } = [ n{ i } reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ),...
                                               1, nDomPol( i ) * d( i ) ) ];

                % update cum. sum.      
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
            elseif ~isT( i ) && isR( i )        
                % real pol.

                % add degree
                nHVec{ i } = [ n{ i } reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), : ),...
                                               1, nDomPol( i ) * d( i ) ) ];

                % update cum. sum.
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            else
                % isT(i) && isR(i);
                % hybrid polynomial

                % add degree
                nTVec{ i } = [ n{ i }( 1 : ptype{ i }.trigonometric ), ...
                               reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), 1 : ptype{ i }.trigonometric ),...
                                        1, nDomPol( i ) * ptype{ i }.trigonometric ) ];
                nHVec{ i } = [ n{ i }( ptype{ i }.trigonometric + 1 : end ), ...
                               reshape( nSi( nDomPolCS( i ) + 1 : nDomPolCS( i ) + nDomPol( i ), ptype{ i }.trigonometric + 1 : end ),...
                                        1, nDomPol( i ) * ( d( i ) - ptype{ i }.trigonometric ) ) ];

                % update cum. sum.
                if i == 1
                    nTVecCS( i ) = length( nTVec{ i } );
                else
                    nTVecCS( i ) = nTVecCS( i - 1 ) + length( nTVec{ i } );
                end
                if i == 1
                    nHVecCS( i ) = length( nHVec{ i } );
                else
                    nHVecCS( i ) = nHVecCS( i - 1 ) + length( nHVec{ i } );
                end
            end                
        end
    end
end

% TVec & HVec are the degrees for 
% trigonometric and real variables
TVec = nTVec{ 1 };
HVec = nHVec{ 1 };
for i = 2 : nPol
    if nTVecCS( i ) > nTVecCS( i - 1 )
        TVec( nTVecCS( i - 1 ) + 1 : nTVecCS( i ) ) = nTVec{ i };
    end
    if nHVecCS( i ) > nHVecCS( i - 1 )
        HVec( nHVecCS( i - 1 ) + 1 : nHVecCS( i ) ) = nHVec{ i };
    end
end

% check if we have trigonometric
% and/or real variables
IsT = nTVecCS( nPol );
IsR = nHVecCS( nPol );

% if we have trigonometric variables
% create list of Toeplitzs
if IsT        
    % TVec unique
    TVecUnq = unique( TVec );
    
    % 26.09.09 removed the elimination
    % of the 0 degree from TVecUnq
    
    % size for TVecUnq
    nTVecUnq = length( TVecUnq );
    
    % list of Toeplitz matrices
    TList = cell( nTVecUnq, 1 );
    
    % populate list
    for i = 1 : nTVecUnq
        TList{ i } = toeplitzs( TVecUnq( i ) );
    end
end

% if we have real variables
% create list of Hankels
if IsR    
    % HVec unique
    HVecUnq = unique( HVec );
    
    % 26.09.09 removed the elimination
    % of the 0 degree from HVecUnq

    % size for HVecUnq
    nHVecUnq = length( HVecUnq );

    % list of Hankel matrices
    HList = cell( nHVecUnq, 1 );

    % populate list
    for i = 1 : nHVecUnq
        HList{ i } = hankels( HVecUnq( i ) );
    end
end