function [ xy ] = pos3poly( AsP, bsP, csP, KsP, pars )

% POS3POLY SeDuMi Preprocessor for Positive Polynomials
%
%   This function can solve an optimization problems which
%   involves positive polynomials.
%
%   [ XY ] = POS3POLY( ASP, BSP, CSP, KSP ) 
%
%   or
%
%   [ XY ] = POS3POLY( ASP, BSP, CSP, KSP, PARS ) 
%
%   INPUTS:
%
%   ASP  - the matrix that describes the equality constraints
%   BSP  - the "free" term
%   CSP  - the objective function
%   KSP  - POS3POLY structure
%   PARS - structure for setting parameters of SeDuMi    
%
%   OUTPUT:
%
%   XY   - the solution of the POS3POLY problem

%
%   Fields of the structure KSP.p{i}: 
%   
%   [ N K ] - where N is a vector holding the degree of the multivariable 
%   polynomial and K is the size of the KxK coefficients.
%
%   Fields of the structure KSP.ptype{i}: 
%
%   trigonometric   - number of trigonometric variables for the polynomial
%   real            - number of real          variables for the polynomial
%   int             - positivity on interval for univariable polynomial
%   complex_coef    - set this to 1 when the polynomial is complex
%   dom             - used when the polynomial is positive only on a domain
%                     to describe the positivity polynomial          
%      - dom.deg    - matrix which hold on it's lines the degrees for the 
%                     polynomial describing the positivity domain    
%      - dom.coef   - vector of coefficients describing the polynomial for
%                     the positivity domain (the order of the coefficients
%                     and of the degrees is the same)
%      - dom.nc     - number of coefficients for each polynomial describing
%                     the positivity domain          
%      - dom.nunion - vector with nr. of pols. for each domain from the union
%   brl             - the variables describe a BRL ||H||<|A| (RR=|A|^2>|H|^2)
%      - brl.hsize  - describes the coefficients of A, of size K1xK2
%
%   Fields of the structure KSP.pars:
%
%   fid             - file descriptor to redirect the output of POS3POLY;
%                     if set to 0 then no output will be provided   
%
%   See also sedumi_data.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 5.08.08-27.09.11

% check number of
% input arguments
ni = nargin;
if ni ~= 4 && ni ~= 5
    error( 'Incorrect number of input arguments.' );
end

% get SeDuMi data
[ As, bsP, cs, Ks, D, IsBRL, isEQForm, bsPnConstrOld, isBRL, BRLlenpol, BRLlencpol, KsPflqrs, KsPflqrsOld, nPolNU ] = ...
  sedumi_data( AsP, bsP, csP, KsP );
         
% call SeDuMi to solve 
% optimization problem
if ni == 5
    % with 'pars'
    x = sedumi( As, bsP, cs, Ks, pars );
else
    % without 'pars'
    x = sedumi( As, bsP, cs, Ks );
end

% the solution of the SeDuMi problem
% is "converted" to the solution of 
% the POS3POLY problem, if the problem
% was solved
xy = sdm2p3p( x, D, IsBRL, isEQForm, bsPnConstrOld, isBRL, BRLlenpol, BRLlencpol, KsPflqrs, KsPflqrsOld, nPolNU );