function [ Qsize, QKsize, Q2size, QOldsize, n, nOld, nNonz, nNonzOld, ZeroDeg, nNonzProd1, ...
           SBNrDegs, SBDegsAddSub, SBPosDegs ] = ...
           qsize( nPol, ptype, d, n, k, isT, isR, isBRL, BRLK2, isSB )

% QSIZE Size of the Q matrices
%
%   This function returns the size of the Q matrices and whether
%   the polynomials have trigonometric and/or real variables. These
%   are the first Q matrices for each polynomial.
%
%   INPUTS:  NPOL, PTYPE, N, K, IST, ISR, ISBRL, BRLK2, ISSB
%   
%   OUTPUTS: QSIZE, QKSIZE, Q2SIZE, QOLDSIZE, IST, ISR, NOLD, NNONZ,
%            NNONZOLD, ZERODEG, NNONZPROD1

% Copyright (C) 2008, 2009, 2010, 2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 15.12.08-01.08.11

% Qsize is the degree + 1;
% these values are for the 
% dimensions of the Gram PSD
% matrices
Qsize   = zeros( nPol, 1 );
QKsize  = zeros( nPol, 1 );
Q2size  = zeros( nPol, 1 );

% nOld
nOld = n;
% QOldsize
QOldsize = zeros( nPol, 1 );

% nonzeros(n) && nonzeros(nOld)
nNonz    = cell( nPol, 1 );
nNonzOld = cell( nPol, 1 );

% zero degrees in the degree 
% of the polynomial
ZeroDeg  = cell( nPol, 1 ); 

% product of ( degree + 1 );
% "new" degree, not the "old" one
nNonzProd1 = zeros( nPol, 1 );

% sparse basis - number of monomials in basis
SBNrDegs = zeros( nPol, 1 );
% degrees for which we write the equalities
% when using sparse basis
SBDegsAddSub = cell( nPol, 1 );
% position with the degrees in 
% the set of degrees 
SBPosDegs = cell( nPol, 1 );
% number of degrees in the sparse basis
SBNrDegsMono = zeros( nPol, 1 );

for i = 1 : nPol

    if ~isT( i ) && ~isR( i )
        % no trigonometric && no real 
        % => trigonometric = d(i)

        % trigonometric polynomial
        % as default
        isT( i ) = 1;

        ptype{ i }.trigonometric = d( i );

    elseif isT( i ) && ~isR( i )
        % trigonometric polynomial;
        % check trigonometric == d(i)

        if ptype{ i }.trigonometric ~=  d( i )
            error( 'Number of variables: Inconsistency between ''ptype'' and ''p''.' );
        end                

    elseif ~isT( i ) && isR( i )
        % real polynomial;
        % check real == d(i)            

        if ptype{ i }.real ~= d( i )
            error( 'Number of variables: Inconsistency between ''ptype'' and ''p''.' );
        end  

        % check even maximum degree 
        % for real variables
        % for positivity on the
        % whole domain
        if  ~isfield( ptype{ i }, 'int' ) && ~isfield( ptype{ i }, 'dom' )
            for j = 1 : d( i )
                % check for nonzero degree
                if n{ i }( j )
                    % check if degree is even
                    if ~iseven( n{ i }( j ) )
                        % not even
                        error( 'Maximum degree for real variable not even.' );
                    else
                        % even
                        n{ i }( j ) = n{ i }( j ) / 2;
                    end
                end
            end            
        elseif isfield( ptype{ i }, 'int' )
            % positivity on an interval
            
            for j = 1 : d( i )
                % check for nonzero degree
                if n{ i }( j )
                    n{ i }( j ) = n{ i }( j ) / 2;
                end
            end    
        else
            % positivity on a domain
            
            for j = 1 : d( i )
                % check for nonzero degree
                if n{ i }( j )
                    % if degree is not even and no degree in pos. dom. then error; 
                    % global degrees must be even
                    if ~iseven( n{ i }( j ) )
                        % odd degree
                        assert( logical( nnz( ptype{ i }.dom.deg( :, j ) ) ), 'Global real degree not even; ppol: %d, deg: %d.', i, j );
                        % create a higher degree with 1 than the minimum
                        n{ i }( j ) = n{ i }( j ) / 2 + 0.5;
                    else
                        % even degree
                        n{ i }( j ) = n{ i }( j ) / 2;
                    end
                end
            end
        end
    else
        % isT(i) && isR(i);
        % hybrid polynomial
        
        ptypet = ptype{ i }.trigonometric;
        ptyper = ptype{ i }.real;

        % check trigonometric + real == d(i)
        if ptypet + ptyper ~= d( i )
            error( 'Number of variables: Inconsistency between ''ptype'' and ''p''.' );
        end  

        % check even maximum degree 
        % for real variables
        % for positivity on the 
        % whole domain
        if  ~isfield( ptype{ i }, 'int' ) && ~isfield( ptype{ i }, 'dom' )
            for j = ptypet + 1 : d( i )
                % check for nonzero degree
                if n{ i }( j )
                    % check if degree is even
                    if ~iseven( n{ i }( j ) )
                        % not even
                        error( 'Maximum degree for real variable not even.' );
                    else
                        % even
                        n{ i }( j ) = n{ i }( j ) / 2;
                    end
                end
            end
        elseif isfield( ptype{ i }, 'int' )
            % positivity on an interval
            
            for j = ptypet + 1 : d( i )
                n{ i }( j ) = n{ i }( j ) / 2;
            end
        else
            % positivity on a domain
            
            for j = ptypet + 1 : d( i )
                % check for nonzero degree
                if n{ i }( j )
                    % if degree is not even and no degree in pos. dom. then error; 
                    % global degrees must be even
                    if ~iseven( n{ i }( j ) )
                        % odd degree
                        assert( logical( nnz( ptype{ i }.dom.deg( :, j ) ) ), 'Global real degree not even; ppol: %d, deg: %d.', i, j );
                        % create a higher degree with 1 than the minimum
                        n{ i }( j ) = n{ i }( j ) / 2 + 0.5;
                    else
                        % even degree
                        n{ i }( j ) = n{ i }( j ) / 2;
                    end
                end
            end
        end
    end            
    
    % zero degrees for every polynomial
    ZeroDeg{ i } = find( n{ i } == 0 );    
    
    % nonzeros
    nNonz   { i } = nonzeros( n   { i } )';
    nNonzOld{ i } = nonzeros( nOld{ i } )';

    % Qsize && QKsize && Q2size;
    % Qsize depends on isBRL
    nNonzProd1( i ) = prod( nNonz{ i } + 1 );

    % SPARSE BASIS CHECK & COMPUTATION
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
    if isSB( i )
        % we have sparse basis
        if isSB( i ) == 1
            % isSB( i ) == 1
            
            if isT( i ) && isR( i )
                % hybrid
                [ SBDegsAddSub{ i }, SBNrDegs( i ), SBPosDegs{ i }, SBNrDegsMono( i ) ] = ...
                  genspbasis( n{ i }, nOld{ i }, d, isT( i ), isR( i ), nNonzProd1( i ), ...
                  ptype{ i }.spbasis.ind, 1, ptype{ i }.trigonometric );
            else
                % not hybrid
                [ SBDegsAddSub{ i }, SBNrDegs( i ), SBPosDegs{ i }, SBNrDegsMono( i ) ] = ...
                  genspbasis( n{ i }, nOld{ i }, d, isT( i ), isR( i ), nNonzProd1( i ), ...
                  ptype{ i }.spbasis.ind, 1 );
            end
        else
            % isSB( i ) == 2
            
            if isT( i ) && isR( i )
                % hybrid
                [ SBDegsAddSub{ i }, SBNrDegs( i ), SBPosDegs{ i }, SBNrDegsMono( i ) ] = ...
                  genspbasis( n{ i }, nOld{ i }, d, isT( i ), isR( i ), nNonzProd1( i ), ...
                  ptype{ i }.spbasis.deg, 2, ptype{ i }.trigonometric );
            else
                % not hybrid
                [ SBDegsAddSub{ i }, SBNrDegs( i ), SBPosDegs{ i }, SBNrDegsMono( i ) ] = ...
                  genspbasis( n{ i }, nOld{ i }, d, isT( i ), isR( i ), nNonzProd1( i ), ...
                  ptype{ i }.spbasis.deg, 2 );
            end
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if isSB( i )
        % with sparse basis
        Qsize ( i ) = cf( ~isBRL( i ), SBNrDegsMono( i )      ,  SBNrDegsMono( i )        + BRLK2( i ) );
    else
        % no sparse  basis
        Qsize ( i ) = cf( ~isBRL( i ), nNonzProd1( i )        ,  nNonzProd1( i )          + BRLK2( i ) );
    end
    
    if isSB( i )
        % with sparse basis
        QKsize( i ) = cf( ~isBRL( i ), SBNrDegsMono( i ) * k( i ), SBNrDegsMono( i ) * k( i ) + BRLK2( i ) ); 
    else
        % no sparse basis
        QKsize( i ) = cf( ~isBRL( i ), nNonzProd1( i )   * k( i ), nNonzProd1( i )   * k( i ) + BRLK2( i ) ); 
    end
    
    Q2size( i )     = QKsize( i ) ^ 2;      

    % QOldsize
    QOldsize( i )   = prod( nNonzOld{ i } + 1 );
end