function [ As, bsP, cs, Ks, D, IsBRL, isEQForm, bsPnConstrOld, isBRL, BRLlenpol, BRLlencpol, KsPflqrs, KsPflqrsOld, nPolNU ] = ...
           sedumi_data( AsP, bsP, csP, KsP )

% SEDUMI_DATA SeDuMi data for the POS3POLY problem
%
%   This function returns the SeDuMi data for the optimization
%   problem written in POS3POLY.
%
%   [ AS, BSP, CS, KS ] = SEDUMI_DATA( ASP, BSP, CSP, KSP )
%
%   INPUTS(POS3POLY data):
%
%   ASP  - the matrix that describes the equality constraints
%   BSP  - the "free" term
%   CSP  - the objective function
%   KSP  - POS3POLY structure
%
%   OUTPUTS(SeDuMi data and the D matrix):
%
%   AS   - the matrix that describes the equality constraints
%   BSP  - the "free" term
%   CS   - the objective function
%   KS   - SeDuMi structure
%   D    - the D matrix  
%
%   Fields of the structure KSP.p{i}: 
%   
%   [ N K ] - where N is a vector holding the degree of the multivariable 
%   polynomial and K is the size of the KxK coefficients.
%
%   Fields of the structure KSP.ptype{i}: 
%
%   trigonometric   - number of trigonometric variables for the polynomial
%   real            - number of real          variables for the polynomial
%   int             - positivity on interval for univariable polynomial
%   complex_coef    - set this to 1 when the polynomial is complex
%   dom             - used when the polynomial is positive only on a domain
%                     to describe the positivity polynomial          
%      - dom.deg    - matrix which hold on it's lines the degrees for the 
%                     polynomial describing the positivity domain    
%      - dom.coef   - vector of coefficients describing the polynomial for
%                     the positivity domain (the order of the coefficients
%                     and of the degrees is the same)
%      - dom.nc     - number of coefficients for each polynomial describing
%                     the positivity domain        
%      - dom.nunion - vector with nr. of pols. for each domain from the union
%   brl             - the variables describe a BRL ||H||<|A| (RR=|A|^2>|H|^2)
%      - brl.hsize  - describes the coefficients of A, of size K1xK2
%
%   Fields of the structure KSP.pars:
%
%   fid             - file descriptor to redirect the output of POS3POLY;
%                     if set to 0 then no output will be provided   
%
%   See also pos3poly.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 26.01.09-16.11.11

% square root
% of -1;
% j
global SR;
SR = 1i;
% fid for p3p_fprintf
global FID;
% the version of POS3POLY
VER = '3.1';

% start time
t0 = cputime;

% file descriptor to print at
if isfield( KsP, 'pars' )
    if isfield( KsP.pars, 'fid' )
        FID = KsP.pars.fid;
    else
        FID = 1;
    end
else
    FID = 1;
end
% copyright text when starting POS3POLY
p3p_fprintf( FID, [ 'POS3POLY ' VER ' (C) 2008-2011 by Bogdan C. Sicleru and Bogdan Dumitrescu\n' ] );

% check if 'bsP' 
% is a vector
assert( isvector( bsP ), 'Variable ''bsP'' must be a vector.' );

% check if 'csP'
% is a vector
assert( isvector( csP ), 'Variable ''csP'' must be a vector.' );

% number of elem. in 'bsP'
% (number of constraints)
bsPnConstr = length( bsP );
bsPnConstrOld = bsPnConstr;

% number of coefficients
% in the csP obj. func.
csPnCoef = length( csP );

[ nConstr, AsPnCoef ] = size( AsP ); 

if nConstr <= AsPnCoef
    % EQuality form
    %
    %   min  csP' * x
    %   s.t. AsP  * x = bsP
    %
    % check equality between number
    % of lines in 'AsP' and 'bsP'
    assert( nConstr == bsPnConstr, 'Inconsistency between the number of lines in ''AsP'' and ''bsP''.' );
    
    % check equality between number of columns
    % in 'AsP' and elements in 'csP'
    assert( AsPnCoef == csPnCoef , 'Inconsistency between the nr. of columns in ''AsP'' and the nr. of elements in ''csP''.' );
    
    % the problem is in EQuality form
    isEQForm = 1;
else
   % INEQuality form
   % nConstr > AsPnCoef
   % 
   %   min  bsP' * y
   %   s.t. csP - AsP * y >= 0
   %
   % check equality between number
   % of lines in 'AsP' and 'csP'
   assert( nConstr == csPnCoef, 'Inconsistency between the number of lines in ''AsP'' and ''csP''.' );
   
   % check equality between number of columns
   % in 'AsP' and elements in 'bsP'
   assert( AsPnCoef == bsPnConstr , 'Inconsistency between the nr. of columns in ''AsP'' and the nr. of elements in ''bsP''.' );
   
   % the problem is in INEQuality form
   isEQForm = 0;
end

% check the existence of 
% field 'p' in KsP
if isfield( KsP, 'p' )
    p = KsP.p; 
else
    error( 'Field ''p'' missing from structure ''KsP''.' );
end

% number of polynomials
nPol = length( p );

% check if the 'p' fields
% are nonempty
for i = 1 : nPol
    assert( ~isempty( p{ i } ), 'Empty field ''p''; ppol: %d.', i );
end        

% d - number of variables
% n - order of the polynomial
% k - the coefficients are k x k matrices
d = zeros( nPol, 1 ); 
n = cell ( nPol, 1 ); 
k = zeros( nPol, 1 );

% fill d, n, k variables
for i = 1 : nPol
    % number of elements in p{i}
    Npi = length( p{ i } );
    
    % check nr. of elem. in p{i}
    switch Npi
        case 1            
            % p{ i } == [ n ];
            d( i ) = 1;
            n{ i } = p{ i };
            k( i ) = 1;     
        otherwise
            % Npi > 1
            
            % p{ i } == [ n k ];
            d( i ) = Npi - 1;
            n{ i } = p{ i }( 1 : end - 1 );
            k( i ) = p{ i }( end ); 
    end    
end

% the greatest number of variables;
% used to determine whether we are in
% the univariate or multivariate case
dmax = max( d );

% check 'ptype' field
if isfield( KsP, 'ptype' )
    % check equality between length(KsP.p) and length(KsP.ptype)
    assert( length( KsP.ptype ) == nPol, 'The number of elements in ''KsP.p'' and ''KsP.ptype'' must be the same.' );
 
    ptype = KsP.ptype;
    
    % check if the 'ptype' fields
    % are nonempty
    for i = 1 : nPol
        assert( ~isempty( ptype{ i } ), 'Empty field ''ptype''; ppol: %d.', i );
    end
else
    % If field 'ptype' does not exist then we create one with
    % the default values for the environment.
    
    % 'ptype' 
    ptype = cell( nPol, 1 );
    
    % introduce default values
    for i = 1 : nPol        
        ptype{ i }.trigonometric = d( i );
    end   
end

% reshape 'bsP' and 'csP' just to be sure the sizes are OK
if isEQForm
    % EQForm
    bsP = reshape( bsP, bsPnConstr, 1 );
    csP = reshape( csP, 1, csPnCoef   );
else
    % INEQForm
    bsP = reshape( bsP, 1, bsPnConstr );
    csP = reshape( csP, csPnCoef, 1   );
end

% check type of polynomials
[ isT, isR ] = istisr( ptype, nPol );

% check if any BRL
[ isBRL, IsBRL, BRLniLines, BRLNLines, BRLK1, BRLK2, IsBRLK1 ] = brl_check( ptype, nPol, isEQForm, 0, n, n, k, isT, isR );
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% change size of matrix coefficient for matrix BRL
if IsBRLK1
    % at least one matrix BRL
    
    for i = 1 : nPol
        if isBRL( i ) && ( BRLK1( i ) > 1 )
            % change size of matrix
            
            k( i ) = BRLK1( i );
        end
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% check if the polynomials must use sparse basis
[ isSB, ISSB ] = spbasis_check( ptype, nPol );

% size of the Q matrices, isT, isR, nOld and nNonz
[ Qsize, QKsize, Q2size, QOldsize, n, nOld, nNonz, nNonzOld, ZeroDeg, nNonzProd1, ...
  SBNrDegs, SBDegsAddSub, SBPosDegs ] = ...
  qsize( nPol, ptype, d, n, k, isT, isR, isBRL, BRLK2, isSB );
% re-run brl_check because of nOld
[ isBRL, IsBRL, BRLniLines, BRLNLines, BRLK1, BRLK2, IsBRLK1 ] = brl_check( ptype, nPol, isEQForm, 0, n, nOld, k, isT, isR );

% init the fields of Ks ( f,l,q,r,s )
[ Ks, KsPflqrs ] = init_ks( KsP );
KsPflqrsOld = KsPflqrs; 

% total number of coefficients
% from the descriptions of the polynomials 
% and
% number of coefficients for every polynomial
[ nCoef, nCoefPol ] = ncoef( KsPflqrs, ptype, nOld, nNonzOld, k, nPol, isT, isR, dmax, isBRL, BRLK1, isSB );

% vector for complex coefficients
isC = zeros( nPol, 1 );
for i = 1 : nPol
    if isfield( ptype{ i }, 'complex_coef' )
        
        % if we have 'true' complex_coef
        if ptype{ i }.complex_coef
            
            isC( i ) = 1;
        end
    end
end

% if we are in EQForm and we have at least one BRL,
% we must move the filter from the BRL in the free 
% variables AND insert the constraints for the filter
BRLlenpol  = [ ];
BRLlencpol = [ ];
if IsBRL
    if isEQForm
        [ AsP, bsP, csP, ptype, nConstr, bsPnConstr, nCoef, Ks, KsPflqrs, BRLniLines, BRLNLines, BRLlenpol, BRLlencpol ] = ...
          brl_parse_eq( AsP, bsP, csP, nPol, nOld, ptype, k, KsPflqrs, isBRL, BRLK1, BRLK2, ...
                        isT, isR, nConstr, bsPnConstr, nCoef, Ks, ...
                        BRLniLines, BRLNLines );
    end
end

nConstrBeforeNU = nConstr;
nCoefBeforeNU   = nCoef;

% check for positivity of polynomials
[ nPol, nPolNU, p, ptype, n, nOld, k, d, isT, isR, isC, isI, isD, nDomPol, isNU, isnunion, nULen1, ...
             AsP, bsP, csP, bsPnConstr, csPnCoef, nConstr, AsPnCoef] = ...
  pos_check( nPol, p, ptype, n, nOld, k, d, dmax, isT, isR, isC, KsPflqrs, nCoefPol, ...
             AsP, bsP, csP, bsPnConstr, csPnCoef, nConstr, AsPnCoef, ...
             isBRL, BRLniLines, isEQForm );

if isNU
    % if we have 'nunion' recalculate
    
    % check if any BRL
    [ isBRL, IsBRL, BRLniLines, BRLNLines, BRLK1, BRLK2, IsBRLK1 ] = brl_check( ptype, nPol, isEQForm, isNU, n, nOld, k, isT, isR );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % change size of matrix coefficient for matrix BRL
    if IsBRLK1
        % at least one matrix BRL

        for i = 1 : nPol
            if isBRL( i ) && ( BRLK1( i ) > 1 )
                % change size of matrix

                k( i ) = BRLK1( i );
            end
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % check if the polynomials must use sparse basis
    [ isSB, ISSB ] = spbasis_check( ptype, nPol );

    % size of the Q matrices, isT, isR, nOld and nNonz
    [ Qsize, QKsize, Q2size, QOldsize, n, nOld, nNonz, nNonzOld, ZeroDeg, nNonzProd1, ...
      SBNrDegs, SBDegsAddSub, SBPosDegs ] = ...
      qsize( nPol, ptype, d, n, k, isT, isR, isBRL, BRLK2, isSB );
    
    % total number of coefficients
    % from the descriptions of the polynomials
    % and
    % number of coefficients for every polynomial
    [ nCoef, nCoefPol ] = ncoef( KsPflqrs, ptype, nOld, nNonzOld, k, nPol, isT, isR, dmax, isBRL, BRLK1, isSB );
end

% keep nConstrNU as nConstr
nConstrNU = nConstr;
%
% if we are in INEQForm then modify AsP, bsP and csP
% to match the EQForm
if ~isEQForm
[ AsP, bsP, csP, Ks, ptype, nCoef, nConstr, KsPflqrs, AsPnCoef, bsPnConstr, csPnCoef, nConstrNU, BRLniLines, BRLNLines ] = ...
  ineq_data( AsP, bsP, csP, Ks, ptype, n, nOld, nNonzOld, nPol, k, dmax, nCoef, nConstr, bsPnConstr, KsPflqrs, ...
             nCoefPol, IsBRL, isBRL, BRLniLines, isT, isR, isNU, BRLK1, BRLK2, isSB, isnunion, ...
             nPolNU, nULen1, nConstrNU, nConstrBeforeNU, nCoefBeforeNU );
end

% check if we have pos. int.
IsI = cf( nnz( isI ), 1, 0 );

% check if we have
% pos. dom.
if nnz( isD )
    % yes
    
    IsD = 1;
    % compute degrees for the pol. from the pos. dom. of each pol. and
    % the size for the Gram matrices
    [ nSi, QDKsize, QD2size, nNonzSi, nNonzOldSi, degDNonz, nDomPolCS, ncCS, SBDegsAddSubDomPol, SBPosDegsDomPol ] = ...
      posdom( nPol, ptype, n, nOld, k, dmax, isT, isR, isD, nDomPol, ZeroDeg, isSB, SBNrDegs, SBDegsAddSub );
else
    % no
    
    IsD = 0;
end

% 'nvar' function call
if ~IsD
    % no pos. dom.
    [ nVar, Ks, isIVec, TVecUnq, HVecUnq, TList, HList, IsT, IsR, SBNrDegsMonoInt, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt ] = ...
      nvar( KsPflqrs, Ks, ptype, nPol, n, k, nOld, QKsize, Q2size, isT, isR, isC, isI, ...
            isD, ...
            ISSB, isSB, SBNrDegs, SBDegsAddSub );
else
    % with pos. dom.
    [ nVar, Ks, isIVec, TVecUnq, HVecUnq, TList, HList, IsT, IsR, SBNrDegsMonoInt, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt ] = ...
      nvar( KsPflqrs, Ks, ptype, nPol, n, k, nOld, QKsize, Q2size, isT, isR, isC, isI, ...
            isD, ...
            ISSB, isSB, SBNrDegs, SBDegsAddSub, ...
            d, nDomPol, nDomPolCS, nSi, QDKsize, QD2size );
end

% removed field 'scomplex' 
% from 'Ks' if empty
if isempty( Ks.scomplex )
    Ks = rmfield( Ks, 'scomplex' );
end

% message to print number of variables
%
% when added dual BRL changed nConstr to nConstrNU
p3p_fprintf( FID, 'd: %d, nppol: %d, nConstr: %d, nVar: %d, KsPflqrs: %d, nSDMVar: %d\n', dmax, nPol, nConstrNU, nCoef, KsPflqrs, nVar );

% LimD is a matrix that holds the
% bounds for the limits of the 
% coefficients of the variables of the
% i-th polynomial in D;
% LimD(i,1) & LimD(i,2) S0
% LimD(i,3) & LimD(i,4) S1 (for pos. on int.)
% LimD(i,3) & LimD(i,4) & ... Si (for pos. on dom.)
if ~IsD
    % no pos. dom.
    
    [ LimD ] = lim_mat( nPol, n, k, isIVec, Q2size, KsPflqrs, dmax, IsI, IsD, ...
                        ISSB, isSB, SBNrDegsMonoInt );
    
    [ D ]    = coef_var_mat( AsPnCoef, LimD, ptype, nPol, n, k, nOld, nNonz, nNonzOld, KsPflqrs, nVar, dmax, IsT, IsR, isT, isR, isC, ...
                             isIVec, TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                             IsD, isBRL, BRLK2, ...
                             isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPolInt, SBPosDegsDomPolInt );
else
    % with pos. dom.
    
    [ LimD ] = lim_mat( nPol, n, k, isIVec, Q2size, KsPflqrs, dmax, IsI, IsD, ...
                        ISSB, isSB, SBNrDegsMonoInt, ...
                        isD, nDomPol, nDomPolCS, QD2size );
    
    [ D ]    = coef_var_mat( AsPnCoef, LimD, ptype, nPol, n, k, nOld, nNonz, nNonzOld, KsPflqrs, nVar, dmax, IsT, IsR, isT, isR, isC ,...
                             isIVec, TList, HList, TVecUnq, HVecUnq, Qsize, QOldsize, ...
                             IsD, isBRL, BRLK2, ...
                             isSB, SBNrDegs, SBDegsAddSub, SBPosDegs, SBDegsAddSubDomPol, SBPosDegsDomPol, ...
                             isD, degDNonz, nDomPol, ncCS, nDomPolCS, ...
                             nNonzSi, nNonzOldSi );
end

% if 'IsBRL' remove the lines in 'AsP' and 'bsP'
% describing the filter from the BRL
if IsBRL
    [ AsP, bsP, csP, bsPnConstr, csPnCoef, BRLAsP, BRLbsP, LimBRL, NBRL ] = ...
      brl_parse( AsP, bsP, csP, ptype, bsPnConstr, csPnCoef, nPol, nCoefPol, KsPflqrs, isEQForm, nPolNU, isT, isR, nOld, nNonzOld, dmax, ...
                 k, isBRL, IsBRLK1, BRLniLines, BRLNLines, BRLK1, BRLK2 );
end

% check if the nr. of coef. is equal to the nr. of elem. in the csP objective function;
%
% if no equality then the product csP * wtx is not compatible, so stop
assert( nCoef == csPnCoef, 'Inconsistency between ''csP'' and the data from ''KsP''.' );

% the matrix from
% the final problem
As = AsP * D;

% the objective function
% in the final problem
cs = csP * D;

% if 'IsBRL' introduce the constraints describing the filter
% in the SeDuMi problem
if IsBRL
    [ As, bsP, Ks ] = brl_set( As, bsP, k, bsPnConstr, LimD, KsPflqrs, nPol, QKsize, nNonzProd1, ...
                               isBRL, BRLAsP, BRLbsP, BRLniLines, BRLNLines, LimBRL, NBRL, BRLK2, ...
                               isC, Ks );
end

t0 = cputime - t0;
[ h, m, s ] = sec2hour( t0 );
% leave a blank line after POS3POLY
p3p_fprintf( FID, 'total time %2d:%2d:%2d\n\n', h, m, s );