function spdegs = sparsedegs( deg, n, nOld, nt, nr )

% SPARSEDEGS Generate sparse degrees
%
%   SPDEGS = SPARSEDEGS( DEG, N, NOLD, NT, NR )
%
%   This function generates the sparse degrees SPDEGS which generate
%   the set of  degrees DEG; DEG is matrix holding on its lines the 
%   degrees. NT and NR are the numbers of trigonometric and real 
%   degrees for the degrees in DEG, respectively. N and NOLD are the
%   maximum degree for the set of degrees. In N the real degrees are 
%   cut in half, while in NOLD are not.
%
%   Example:
%
%   deg( 1, : ) = [ 2 4 2 2 ];
%   deg( 2, : ) = [ 0 0 2 2 ];
%   deg( 3, : ) = [ 2 2 0 2 ];
%   deg( 4, : ) = [ 0 2 0 2 ];
%   deg( 5, : ) = [ 0 0 0 2 ];
%   deg( 6, : ) = [ 1 0 0 0 ];
%   deg( 7, : ) = [ 1 1 0 0 ];
%   deg( 8, : ) = [ 0 1 0 0 ];
%   deg( 9, : ) = [ 0 0 0 0 ];
%   deg( 10, : ) = [ 0 0 0 0 ];
%
%   spdegs = sparsedegs( deg, [2 4 1 1], [2 4 2 2], 2, 2 );

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 22.06.10-29.06.10

% total number of variables
d = nt + nr;

% number of columns
s2 = size( deg );

% check if numer of variables from degrees = nt + nr
if s2 ~= ( d )
    error( 'Number of variables from the degrees not equal with ''nt''+''nr''.' );
end

% sparse degs
spdegs = zeros( 0, d );

degInf = deg;
% if we have trigonometri degrees,
% we could have negative degrees
if nt 
    degInf( degInf < 0 ) = Inf;
end

% as long as we have degrees to generate
while ~isempty( deg )
    % get minimum 'sum' degree
    if nt && nr
        % hybrid polynomial
        
        % sumdegInf = sum( degInf( :, nt + 1 : end ), 2 );
        % replace the minimum from above with minimum
        % only on even degrees
        sumdegInf = sum( degInf( logical( iseven( degInf( :, nt + 1 : end ) ) ) ), 2 );
        I = min( sumdegInf );
        if isempty( I )
            sumdegInf = sum( degInf( :, nt + 1 : end ), 2 );
            I = min( sumdegInf );
        end
        [ I , J ] = max( sum( degInf( sumdegInf == I, 1 : nt ), 2 ) );
    elseif nt
        % trigonometric polynomial
        
        [ I , J ] = max( sum( degInf, 2 ) );
    else
        % real polynomial
        
        % [ I , J ] = min( sum( degInf, 2 ) );
        % replace the minimum from above with minimum
        % only on even degrees
        [ I, J ] = min( sum( degInf( logical( iseven( degInf ) ) ), 2 ) );
        if isempty( J )
            [ I , J ] = min( sum( degInf, 2 ) );
        end
    end    
    % extend matrix to add degree
    spdegs = mextend( 'ar', 'zpd', spdegs, 1, 'd' );
    % add degree
    spdegs( end, : ) = deg( J, : );
    % eliminate degree from 'deg'
    deg( J, : ) = [ ];
    degInf( J, : ) = [ ];
    % if the degrees are even and the
    % degrees are real then divide by 2
    if nt && nr
        % hybrid polynomial
        for i = nt + 1 : d
            if iseven( spdegs( end, i ) )
                spdegs( end, i ) = spdegs( end, i ) / 2;
            end
        end
    elseif nr
        % real polynomial
        for i = 1 : d
            if iseven( spdegs( end, i ) )
                spdegs( end, i ) = spdegs( end, i ) / 2;
            end
        end
    end
    % eliminate negative degrees because
    % we want to generate them with the basis
    spTmp = spdegs;
    spLen = size( spdegs, 1 );
    DegsRem = zeros( spLen, 1 );
    for i = 1 : spLen
        if nnz( spTmp( i, : ) < 0 )
            DegsRem( i ) = i;
        end
    end
    spTmp( DegsRem( DegsRem ~= 0 ), : ) = [ ];
    % generate degrees from the degrees in the sparse basis 
    [ SBDegsAddSub ] = genspbasis( n, nOld, d, nt, nr, prod( n + 1 ), spTmp, 2, nt );
    
    % eliminate the degrees from 'deg' which are generated 
    % by the current sparse basis
    degLen = size( deg, 1 );
    DegsRem = zeros( degLen, 1 );
    for i = 1 : degLen
        if ~isempty( findvm( SBDegsAddSub', deg( i, : ) ) )
            DegsRem( i ) = i;
        end
    end
    % remove degrees
    deg( DegsRem( DegsRem ~= 0 ), : ) = [ ];
    degInf( DegsRem( DegsRem ~= 0 ), : ) = [ ];
end