function G = adj_fir_approx_lp( n, gp, D, tl, tu, t0, tau )

% ADJ_FIR_APPROX_LP Design of approximately linear-phase adjustable FIR filters
%
%   G = ADJ_FIR_APPROX_LP( N, GP, D, TL, TU, T0, TAU ). N denotes the degree
%   of the hybrid polynomial and GP is the passband error bound.
%   
%   This function can be used to design adjustable filters, as
%   described in [1].
%
%   The function solves the problem
%
%   min  gs
%   s.t. gp ^ 2 = S0 + Dp1 * S1 + Dp2 * S2
%        gs ^ 2 = S3 + Dp3 * S4 + Dp2 * S5
%        [ Q0 f1; f1' 1 ] >= 0
%        [ Q3 f2; f2' 1 ] >= 0
%
%   where f1, f2 are the coefficients of G-z^{-tau} and G, respectively,
%   G = G(z,t) and Sx are sum-of-squares. 
%   Dp2 = (tau + t0 - tl) * (tu - tau - t0)
%   Dp1 = (z+z^{-1})/2 - tau - t0 - D
%   Dp3 = tau + t0 - D - (z+z^{-1})/2
%   TAU is the desired group delay.
%
%   (The algorithm for designing the filters uses a BRL characterization.)
%
%   Example:
%
%   G = adj_fir_approx_lp( [26 8], 0.01, 0.29, 0.019, 0.518, 0.2495, 10 );
%
%   [1] B. Dumitrescu, B.C. Sicleru, R. Stefan. Positive hybrid 
%       real-trigonometric polynomials and applications to adjustable 
%       filter design and absolute stability analysis. Circ. Syst. Sign. Proc.,
%       to appear, 2010.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 19.02.10-15.11.11

% number of coefs. for
% a hybrid pol.
N = lenpol( n, 1, 1, 1 );

N2 = N * 2;

% number of coefficients in the
% causal filter
M = lencpol( [ n( 1 ) n( 2 ) / 2 ] );

M2 = M * 2;

% number of constraints
nConstr = N2 + M2; 
% number of "variables"
nVar    = 1 + M;

AsP = sparse( nConstr, nVar );
bsP = -speye( 1,       nVar );
csP = sparse( nConstr, 1    );

Im = speye( M );

csP( tau + 1 ) = 1;
csP( M + 1 ) = gp ^ 2;

AsP( 1 : M, 2 : M + 1 ) = Im;
AsP( M + N + 1 : 2 * M + N, 2 : M + 1 ) = Im;

% gs
AsP( 2 * M + N + 1, 1 ) = -1;

% S0
KsP.ptype{ 1 }.trigonometric = 1;
KsP.ptype{ 1 }.real = 1;
KsP.p{ 1 } = [ n 1 ];
KsP.ptype{ 1 }.dom.nc   = [ 3 3 ];
KsP.ptype{ 1 }.dom.deg  = [ 0 0; ...
                            1 0; ...
                            0 1; ...
                            0 0; ...
                            0 1; ...
                            0 2  ];
KsP.ptype{ 1 }.dom.coef = [ -(D+t0) 0.5 -1 (tu-t0)*(t0-tl) (tu+tl-2*t0) -1 ];
KsP.ptype{ 1 }.brl.hsize = [ 1 1 ];

% S1
KsP.ptype{ 2 }.trigonometric = 1;
KsP.ptype{ 2 }.real = 1;
KsP.p{ 2 } = [ n 1 ];
KsP.ptype{ 2 }.dom.nc   = [ 3 3 ];
KsP.ptype{ 2 }.dom.deg  = [ 0 0; ...
                            1 0; ...
                            0 1; ...
                            0 0; ...
                            0 1; ...
                            0 2  ];
KsP.ptype{ 2 }.dom.coef = [ -D+t0 -0.5 1 (tu-t0)*(t0-tl) (tu+tl-2*t0) -1 ];
KsP.ptype{ 2 }.brl.hsize = [ 1 1 ];

[ y ] = pos3poly( AsP, bsP, csP, KsP );

% approximately linear-phase adjustable filters
G = y( 2 : M + 1 );