function G = adj_fir_v2( n, gp, D, tl, tu, t0 )

% ADJ_FIR_V2 Design of linear-phase adjustable FIR filters
%
%   G = ADJ_FIR_V2( N, GP, D, TL, TU, T0 ). N denotes the degree
%   of the hybrid polynomial and GP is the passband error bound.
%   
%   This function can be used to design adjustable filters, as
%   described in [1].
%
%   The function solves the problem
%
%   min  gs
%   s.t. G + gs     = S0 + Dz1 * S01 + Dt * S02
%        gs - G     = S1 + Dz1 * S11 + Dt * S12
%        G - 1 + gp = S2 + Dz2 * S21 + Dt * S22
%        1 + gp - G = S3 + Dz2 * S31 + Dt * S32
%
%   where G = G(z,t), Sx and Syy are sum-of-squares,
%   and Dt = (tu-t)*(t-tl), Dz1 = t0+t-D-(z+z^{-1})/2,
%   Dz2 = (z+z^{-1})/2-t-t0-D; Dt>=0, Dz1>=0, Dz2>=0.    
%
%   Example:
%
%   G = adj_fir( [ 13 6 ], 0.01, 0.25, 0, 0.56, 0.28 );
%
%   [1] B. Dumitrescu, R. Stefan. Positive hybrid real-trigonometric
%       polynomials and applications. In Proc. Math. Theory of
%       Networks and Systems, 2008.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 17.10.09-21.10.09

% nr. of coefs. in halfsapce for
% a hybrid pol. with 1 trig. var.
N = nchp( n, 1 );

% identity matrix
I = speye( N );

% nr. of constraints
nConstr = 4 * N;
% nr. of POS3POLY variables
nVar    = 1 + N + nConstr;

AsP = sparse( nConstr, nVar );
bsP = sparse( nConstr, 1    );
csP =  speye( 1,       nVar );

bsP( 2 * N + 1 ) =  1 - gp;
bsP( 3 * N + 1 ) = -1 - gp;

% S0
AsP( 1 : N, 2 : N + 1 ) = I;
AsP( 1 : N, N + 2 : 2 * N + 1 ) = -I;
AsP( 1, 1 ) = 1;

% S1
AsP( N + 1 : 2 * N, 2 : N + 1 ) = -I;
AsP( N + 1 : 2 * N, 2 * N + 2 : 3 * N + 1 ) = -I;
AsP( N + 1, 1 ) = 1;

% S2
AsP( 2 * N + 1 : 3 * N, 2 : N + 1 ) = I;
AsP( 2 * N + 1 : 3 * N, 3 * N + 2 : 4 * N + 1 ) = -I;

% S3
AsP( 3 * N + 1 : 4 * N, 2 : N + 1 ) = -I;
AsP( 3 * N + 1 : 4 * N, 4 * N + 2 : 5 * N + 1 ) = -I;

% gs + G
KsP.f = 1 + N;

% S0
KsP.ptype{ 1 }.trigonometric = 1;
KsP.ptype{ 1 }.real = 1;
KsP.p{ 1 } = [ n 1 ];
KsP.ptype{ 1 }.dom.deg  = [ 1 1; ...
                            0 2  ];                        
KsP.ptype{ 1 }.dom.coef = [ -D+t0 -0.5 1 0 (tu-t0)*(t0-tl) (tu+tl-2*t0) -1 ];

% S1
KsP.ptype{ 2 }.trigonometric = 1;
KsP.ptype{ 2 }.real = 1;
KsP.p{ 2 } = [ n 1 ];
KsP.ptype{ 2 }.dom.deg  = [ 1 1; ...
                            0 2  ];  
KsP.ptype{ 2 }.dom.coef = [ -D+t0 -0.5 1 0 (tu-t0)*(t0-tl) (tu+tl-2*t0) -1 ];

% S2
KsP.ptype{ 3 }.trigonometric = 1;
KsP.ptype{ 3 }.real = 1;
KsP.p{ 3 } = [ n 1 ];
KsP.ptype{ 3 }.dom.deg  = [ 1 1; ...
                            0 2  ];  
KsP.ptype{ 3 }.dom.coef = [ -D-t0 0.5 -1 0 (tu-t0)*(t0-tl) (tu+tl-2*t0) -1 ];

% S3
KsP.ptype{ 4 }.trigonometric = 1;
KsP.ptype{ 4 }.real = 1;
KsP.p{ 4 } = [ n 1 ];
KsP.ptype{ 4 }.dom.deg  = [ 1 1; ...
                            0 2  ];  
KsP.ptype{ 4 }.dom.coef = [ -D-t0 0.5 -1 0 (tu-t0)*(t0-tl) (tu+tl-2*t0) -1 ];

pars = [ ];
% pars.eps = 0;

[ x ] = pos3poly( AsP, bsP, csP, KsP, pars );

% adjustable filters
G = x( 2 : N + 1 );