function [ h ] = approx_lin_phase_fir( n, wp, ws, gp, gs, tau )

% APPROX_LIN_PHASE_FIR_5_1_3 Approximately linear phase FIR filter design
%
%   This function is an example of the design of approximately 
%   linear-phase FIR filters. The problem is discussed in 
%   [1, Section 5.1.3], page 142.
%
%   [ H ] = APPROX_LIN_PHASE_FIR_5_1_3( N, WP, WS, GP, GS, TAU )
%
%   INPUTS:
%
%   N   - degree of the filter
%   WP  - (frequency) upper limit of the passband
%   WS  - (frequency) lower limit of the stopband
%   GP  - magnitude error bound of the passband
%   GS  - magnitude error bound of the stopband
%   TAU - desired group delay
%
%   OUTPUT:
%
%   H = coefficients of the filter
%
%   H(z) = \sum_{k=0}^n h_k*z^{-k}.
%
%   The optimization problem solved is:
%
%   min  Es
%
%   s.t.  | H - z^{-tau} | <= gp, w in [ 0, wp ]
%         | H | <= gs, w in [ ws, pi ]
%
%   where Es is the stopband energy.
%
%   Example:
%
%   h = approx_lin_phase_fir_5_1_3( 50, 0.2*pi, 0.25*pi, 0.1, 0.0158, 22 );
%   freqz( h );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 09.03.10-10.11.11

% size (degree + 1)
n1 = n + 1;

nConstr = 5 * n1;

nVar    = 1 + n1 * 6;

% initialize POS3POLY 
% variables
AsP = sparse( nConstr, nVar );
bsP = sparse( nConstr, 1    );
csP = sparse( 1,       nVar );

% 1 for eps in the
% objective function
csP( n1 + 1 ) = 1;

% free terms from the
% two polynomial constraints
bsP( 3 * n1 + 1 ) = gp ^ 2;
bsP( 4 * n1 + 1 ) = gs ^ 2;

I = speye( n1 );

% dim. of vector c
c = zeros( n1, 1 );

% ck = 1-ws/pi, k = 0
% ck = -sin(k*ws)/(k*pi), k > 0
c( 1 ) = 1 - ws / pi;
for i = 2 : n1
    c( i ) = -sin( ( i - 1 ) * ws ) / ( ( i - 1 ) * pi );
end

% Es = h' * C * h
C = toeplitz( c );

% square matrix
C12 = real( sqrtm( C ) );

% y = C12 * h
AsP( 1 : n1, 1 : n1 ) = -C12;
AsP( 1 : n1, n1 + 2 : 2 * n1 + 1 ) = I;

% h, h0
AsP( n1 + 1 : 2 * n1, 1 : n1 ) = I;
AsP( n1 + 1 : 2 * n1, 2 * n1 + 2 : 3 * n1 + 1 ) = -I;
bsP( n1 + tau + 1 ) = 1;

% h, h1
AsP( 2 * n1 + 1 : 3 * n1, 1 : n1 ) = I;
AsP( 2 * n1 + 1 : 3 * n1, 4 * n1 + 2 : 5 * n1 + 1 ) = -I;

% s0
AsP( 3 * n1 + 1 : 4 * n1, 3 * n1 + 2 : 4 * n1 + 1 ) = I;

% s1
AsP( 4 * n1 + 1 : 5 * n1, 5 * n1 + 2 : 6 * n1 + 1 ) = I;

% h coefficients
% are free variables
KsP.f = n1;

% SOC: ||y|| <= eps
KsP.q = n1 + 1;

% the polynomial variables
KsP.ptype{ 1 }.trigonometric = 1;
KsP.ptype{ 2 }.trigonometric = 1;

% positivity domains
KsP.ptype{ 1 }.int = [ 0  wp ];
KsP.ptype{ 2 }.int = [ ws pi ];

% the degrees of the polynomials
% are n, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];
KsP.p{ 2 } = [ n 1 ];

% BRL

% 1
KsP.ptype{ 1 }.brl.hsize = 1;

% 2
KsP.ptype{ 2 }.brl.hsize = 1;

% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP );

% coefficients of the filter
h = x( 1 : n1 );