function [ h ] = approx_lin_phase_fir2d_5_2_3( n, gp, tau )

% APPROX_LIN_PHASE_FIR2D_5_2_3 Approximately linear phase 2-D FIR filter design
%
%   This function is an example of the design of approximately
%   linear-phase 2-D symmetric FIR filters. The problem is discussed 
%   in [1, Section 5.2.3]. According to the defined positivity 
%   domains, this function will design diamond shape filters.
%
%   [ H ] = APPROX_LIN_PHASE_FIR2D_5_2_3( N, GP, TAU )
%
%   INPUTS:
%
%   N   - degree of the filter (filter of size (N+1)x(N+1))
%   GP  - magnitude error bound of the passband
%   TAU - desired group delay
%
%   OUTPUT:
%
%   H = coefficients of the filter
%
%   H(z) = \sum_{k=0}^n h_k*z^{-k}, h_{-k}=h{k}, k=(k1, k2).
%
%   The optimization problem solved is:
%
%   min  gs
%
%   s.t.  | H(w) - exp(-j*tau*w) | <= gp, w in Dp
%         | H(w) | <= gs, w in Ds
%
%   Example:
%
%   h = approx_lin_phase_fir2d_5_2_3( [ 7 7 ], 0.1, [ 4 4 ] );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.02.10-02.11.11

% number of n-tuples
M = lenpol( n );

% number of filter coefs.
N = lencpol( n );

% number of eq. constraints
nConstr = 2 * M + 2 * N;

% number of variables:
nVar    = 2 * M + 3 * N + 1;

In = speye( N );
Im = speye( M );

% initialize POS3POLY 
% variables
AsP = sparse( nConstr, nVar );
bsP = sparse( nConstr, 1    );
csP =  speye( 1,       nVar );

% h0 and h
AsP( 1 : N, 2 : N + 1 ) = In; % h
AsP( 1 : N, N + 2 : 2 * N + 1 ) = In; % h0
bsP( tau( 2 ) * ( n( 1 ) + 1 ) + tau( 1 ) + 1 ) = 1;

% h1 and h
AsP( N + 1 : 2 * N, 2 : N + 1 ) = In; % h
AsP( N + 1 : 2 * N, 2 * N + M + 2 : 3 * N + M + 1 ) = In; % h1

% s0
AsP( 2 * N + 1 : 2 * N + M, 2 * N + 2 : 2 * N + M + 1 ) = Im;
bsP( 2 * N + 1 ) = gp ^ 2;

% s1
AsP( 2 * N + M + 1 : 2 * N + 2 * M, 3 * N + M + 2 : 3 * N + 2 * M + 1 ) = Im;
AsP( 2 * N + M + 1, 1 ) = -1; 

% h and gs are
% free variables
KsP.f = 1 + N;

% the polynomial variables
KsP.ptype{ 1 }.trigonometric = 2;
KsP.ptype{ 2 }.trigonometric = 2;

% positivity domains
% Dp
KsP.ptype{ 1 }.dom.nc   = [ 2 2 2 ];
KsP.ptype{ 1 }.dom.deg = [  0 0; ...
                            1 1; ...
                            0 0; ...
                           -1 1; ...
                            1 0; ...
                            0 1; ];
                        
KsP.ptype{ 1 }.dom.coef = [ -0.1 0.5 -0.1 0.5 0.5 0.5 ];

% Ds1 U Ds2 U Ds3
KsP.ptype{ 2 }.dom.nunion = [ 1 1 1 ];
KsP.ptype{ 2 }.dom.nc   = [ 2 2 2 ];
KsP.ptype{ 2 }.dom.deg  = [ 0  0; ...
                            1  1; ...
                            0  0; ...
                            1 -1; ...
                            1  0; ...
                            0  1  ];
KsP.ptype{ 2 }.dom.coef = [ -0.7 -0.5 -0.7 -0.5 -0.5 -0.5 ];

% BRL settings

% 1
KsP.ptype{ 1 }.brl.hsize = 1;

% 2
KsP.ptype{ 2 }.brl.hsize = 1;

% the degrees of the polynomials
% are 'n', and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];
KsP.p{ 2 } = [ n 1 ];

% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP );

% the coefficients of the filter
h = x( 2 : N + 1 );