function [ h ] = approx_lin_phase_fir2d_5_2_3_cvx( n, gp, tau )

% APPROX_LIN_PHASE_FIR2D_5_2_3_CVX Approximately linear phase 2-D FIR filter design
%
%   This function is an example of the design of approximately
%   linear-phase 2-D symmetric FIR filters. The problem is discussed 
%   in [1, Section 5.2.3]. According to the defined positivity 
%   domains, this function will design diamond shape filters.
%
%   [ H ] = APPROX_LIN_PHASE_FIR2D_5_2_3_CVX( N, GP, TAU )
%
%   INPUTS:
%
%   N   - degree of the filter (filter of size (N+1)x(N+1))
%   GP  - magnitude error bound of the passband
%   TAU - desired group delay
%
%   OUTPUT:
%
%   H = coefficients of the filter
%
%   H(z) = \sum_{k=0}^n h_k*z^{-k}, h_{-k}=h{k}, k=(k1, k2).
%
%   The optimization problem solved is:
%
%   min  gs
%
%   s.t.  | H(w) - exp(-j*tau*w) | <= gp, w in Dp
%         | H(w) | <= gs, w in Ds
%
%   Example:
%
%   h = approx_lin_phase_fir2d_5_2_3_cvx( [ 7 7 ], 0.1, [ 4 4 ] );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.02.10

% number of n-tuples
N = nch( n );

% number of filter coefs.
M = prod( n + 1 );

% the polynomial variables
ptype{ 1 }.trigonometric = 2;
ptype{ 2 }.trigonometric = 2;

% positivity domains
% Dp
ptype{ 1 }.dom.nc   = [ 2 2 2 ];
ptype{ 1 }.dom.deg = [  0 0; ...
                        1 1; ...
                        0 0; ...
                       -1 1; ...
                        1 0; ...
                        0 1; ];
                        
ptype{ 1 }.dom.coef = [ -0.1 0.5 -0.1 0.5 0.5 0.5 ];

% Ds1 U Ds2 U Ds3
ptype{ 2 }.dom.nunion = [ 1 1 1 ];
ptype{ 2 }.dom.nc   = [ 2 2 2];
ptype{ 2 }.dom.deg  = [ 0 0; ...
                        1 1; ...
                        0 0; ...
                        1 -1;...
                        1 0; ...
                        0 1  ];
ptype{ 2 }.dom.coef = [ -0.7 -0.5 -0.7 -0.5 -0.5 -0.5 ];

% BRL settings

% 1
ptype{ 1 }.brl.hsize = 1;

% 2
ptype{ 2 }.brl.hsize = 1;

% the degrees of the polynomials
% are 'n', and its coefficients are
% scalars (1x1)
p{ 1 } = [ n 1 ];
p{ 2 } = [ n 1 ];

Ivec = sparse( M, 1 );
Ivec( tau( 2 ) * ( n( 1 ) + 1 ) + tau( 1 ) + 1 ) = 1;

Ieye = speye( N, 1 );

cvx_begin
    variable gs;
    variable HR1( M + N );
    variable HR2( M + N );
    variable h( M );
    
    minimize gs;
    
    HR1 == sos_pol( p{ 1 }, ptype{ 1 } );
    HR2 == sos_pol( p{ 2 }, ptype{ 2 } );
    
    Ieye * gp ^ 2 == HR1( M + 1 : end );
    Ieye * gs     == HR2( M + 1 : end );
    
    h - Ivec == HR1( 1 : M );
    h        == HR2( 1 : M );
cvx_end