function [ h ] = approx_lin_phase_fir_5_1_3_cvx( n, wp, ws, gp, gs, tau )

% APPROX_LIN_PHASE_FIR_5_1_3_CVX Approximately linear phase FIR filter design 
%
%   This function is an example of the design of approximately 
%   linear-phase FIR filters. The problem is discussed in 
%   [1, Section 5.1.3], page 142.
%
%   [ H ] = APPROX_LIN_PHASE_FIR_5_1_3_CVX( N, WP, WS, GP, GS, TAU )
%
%   INPUTS:
%
%   N   - degree of the filter
%   WP  - (frequency) upper limit of the passband
%   WS  - (frequency) lower limit of the stopband
%   GP  - magnitude error bound of the passband
%   GS  - magnitude error bound of the stopband
%   TAU - desired group delay
%
%   OUTPUT:
%
%   H = coefficients of the filter
%
%   H(z) = \sum_{k=0}^n h_k*z^{-k}.
%
%   The optimization problem solved is:
%
%   min  Es
%
%   s.t.  | H - z^{-tau} | <= gp, w in [ 0, wp ]
%         | H | <= gs, w in [ ws, pi ]
%
%   where Es is the stopband energy.
%
%   Example:
%
%   h = approx_lin_phase_fir_5_1_3_cvx( 50, 0.2*pi, 0.25*pi, 0.1, 0.0158, 22 );
%   freqz( h );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 09.03.10-15.04.10

% size (degree + 1)
n1 = n + 1;

% dim. of vector c
c = zeros( n1, 1 );

% ck = 1-ws/pi, k = 0
% ck = -sin(k*ws)/(k*pi), k > 0
c( 1 ) = 1 - ws / pi;
for i = 2 : n1
    c( i ) = -sin( ( i - 1 ) * ws ) / ( ( i - 1 ) * pi );
end

% Es = h' * C * h
C = toeplitz( c );

% square matrix
C12 = real( sqrtm( C ) );

% the polynomial variables
ptype{ 1 }.trigonometric = 1;
ptype{ 2 }.trigonometric = 1;

% positivity domains
ptype{ 1 }.int = [ 0  wp ];
ptype{ 2 }.int = [ ws pi ];

% the degrees of the polynomials
% are n, and its coefficients are
% scalars (1x1)
p{ 1 } = [ n 1 ];
p{ 2 } = [ n 1 ];

% BRL

% 1
ptype{ 1 }.brl.hsize = 1;

% 2
ptype{ 2 }.brl.hsize = 1;

O = sparse( n, 1 );

Ivec = sparse( n1, 1);
% desired delay
Ivec( tau + 1 ) = 1;

cvx_begin
    variable e;
    variables h( n1 ) y( n1 );
    
    variable HR1( 2 * n1 );
    variable HR2( 2 * n1 );
    
    HR1 == sos_pol( p{ 1 }, ptype{ 1 } );
    HR2 == sos_pol( p{ 2 }, ptype{ 2 } );
    
    subject to
    
    norm( y ) <= e;
    y == C12 * h;
    
    gp ^ 2 == HR1( n1 + 1 );
    gs ^ 2 == HR2( n1 + 1 );
    
    O == HR1( n1 + 2 : end );
    O == HR2( n1 + 2 : end );
    
    h - Ivec == HR1( 1 : n1 );
    h == HR2( 1 : n1 );
cvx_end