function hin = hinf_fir_cvx( n, h )

% HINF_FIR_CVX H_inf norm for FIR filter in cvx style
%
%   HIN = HINF_FIR_CVX( N, H ) returns the H infinity norm HIN for
%   the FIR filter described by the coefficients from the vector
%   H of degree N. The coefficients are chosen from a halfspace.
%
%   Example:
%
%   n = [ 3 3 ];
%   deg = [ 3 0; 0 3; 2 1; 1 2; 0 2; 0 1 ];
%   coef = { 1, 1, 3, 3, 1, 1 };              
%   h = sppol2fvec( 1, 6, [ 3 3 ], deg, coef, 1, 0, 1 );             
%   hin = hinf_fir_cvx( n, h );

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 10.02.10-15.04.10

% number of n-tuples
N = nch( n );

% number of n-tuples in the
% positive orthant
Nt = prod( n + 1 );

if Nt ~= length( h )
    error( 'Inconsistency between Nt and length(H).' );
end

% S0
ptype{ 1 }.trigonometric = length( n );
ptype{ 1 }.brl.hsize = 1;
p{ 1 } = [ n 1 ];

cvx_begin
    variable gs;
    variable HR( Nt + N );
    
    HR == sos_pol( p{ 1 }, ptype{ 1 } );
    
    minimize gs;
    
    subject to
    
    gs == HR( Nt + 1 );
    
    HR( Nt + 2 : end ) == zeros( N - 1, 1 );
    
    HR( 1 : Nt ) == h;
cvx_end

% H_inf norm
hin = sqrt( gs );