function [ h ] = lin_phase_fir2d_5_2_2_cvx( n, gp )

% LIN_PHASE_FIR2D_5_2_2_CVX Linear phase 2-D FIR filter design
%
%   This function is an example of the design of linear-phase
%   2-D symmetric FIR filters. The problem is discussed in 
%   [1, Section 5.2.2]. According to the defined positivity 
%   domains, this function will design diamond shape filters.
%
%   [ H ] = LIN_PHASE_FIR2D_5_2_2_CVX( N, GP )
%
%   INPUTS:
%
%   N  - ("half the") degree of the filter
%   GP - magnitude error bound of the passband
%
%   OUTPUT:
%
%   H = coefficients of the filter belonging to a halfspace 
%
%   H(z) = \sum_{k=-n}^n h_k*z^{-k}, h_{-k}=h{k}, k=(k1, k2).
%
%   The optimization problem solved is:
%
%   min  gs
%
%   s.t.  1 + gp - H(w) >= 0, for all w
%        -1 + gp + H(w) >= 0, w in Dp = Dp1 * Dp2 * Dp3
%             gs - H(w) >= 0, w in Ds1 + Ds2 + Ds3
%             gs + H(w) >= 0, w in Ds1 + Ds2 + Ds3
%
%   Denoting
%
%   S0 =  1 + gp - H(w)
%   S1 = -1 + gp + H(w)
%   S2 =      gs - H(w)
%   S3 =      gs - H(w)
%   S4 =      gs - H(w)
%   S5 =      gs + H(w)
%   S6 =      gs + H(w)
%   S7 =      gs + H(w)
%
%   the problem for POS3POLY has the following form:
%
%   min  gs
%
%   s.t.  1 + gp =  H(w) + S0, S0 >= 0, for all w
%        -1 + gp = -H(w) + S1, S1 >= 0, w in Dp = Dp1 * Dp2 * Dp3
%             gs =  H(w) + S2, S2 >= 0, w in Ds1
%             gs =  H(w) + S3, S3 >= 0, w in Ds2
%             gs =  H(w) + S4, S4 >= 0, w in Ds3
%             gs = -H(w) + S5, S5 >= 0, w in Ds1
%             gs = -H(w) + S6, S6 >= 0, w in Ds2
%             gs = -H(w) + S7, S7 >= 0, w in Ds3
%
%   Example:
%
%   h = lin_phase_fir2d_5_2_2_cvx( [ 7 7 ], 0.1 );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 15.06.09-17.06.09

% number of n-tuples
N = nch( n );

% the polynomial variables
ptype{ 1 }.trigonometric = 2;
ptype{ 2 }.trigonometric = 2;
ptype{ 3 }.trigonometric = 2;
ptype{ 4 }.trigonometric = 2;
ptype{ 5 }.trigonometric = 2;
ptype{ 6 }.trigonometric = 2;
ptype{ 7 }.trigonometric = 2;
ptype{ 8 }.trigonometric = 2;

% positivity domains
% Dp
ptype{ 2 }.dom.nc   = [ 2 2 2 ];
ptype{ 2 }.dom.deg = [  0 0; ...
                        1 1; ...
                        0 0; ...
                       -1 1; ...
                        1 0; ...
                        0 1; ];
                        
ptype{ 2 }.dom.coef = [ -0.1 0.5 -0.1 0.5 0.5 0.5 ];

% Ds1
ptype{ 3 }.dom.nc   = 2;
ptype{ 3 }.dom.deg  = [ 0 0; ...
                            1 1 ];
ptype{ 3 }.dom.coef = [ -0.7 -0.5 ];

% Ds2
ptype{ 4 }.dom.nc   = 2;   
ptype{ 4 }.dom.deg  = [  0 0; ...
                            -1 1 ];
ptype{ 4 }.dom.coef = [ -0.7 -0.5 ];

% Ds3
ptype{ 5 }.dom.nc   = 2;  
ptype{ 5 }.dom.deg  = [ 1 0; ...
                            0 1 ];
ptype{ 5 }.dom.coef = [ -0.5 -0.5 ];

% Ds1
ptype{ 6 }.dom.nc   = 2;
ptype{ 6 }.dom.deg  = [ 0 0; ...
                            1 1 ];
ptype{ 6 }.dom.coef = [ -0.7 -0.5 ];

% Ds2
ptype{ 7 }.dom.nc   = 2;   
ptype{ 7 }.dom.deg  = [  0 0; ...
                            -1 1 ];
ptype{ 7 }.dom.coef = [ -0.7 -0.5 ];

% Ds3
ptype{ 8 }.dom.nc   = 2;  
ptype{ 8 }.dom.deg  = [ 1 0; ...
                            0 1 ];
ptype{ 8 }.dom.coef = [ -0.5 -0.5 ];

% the degrees of the polynomials
% are wtn, and its coefficients are
% scalars (1x1)
p{ 1 } = [ n 1 ];
p{ 2 } = [ n 1 ];
p{ 3 } = [ n 1 ];
p{ 4 } = [ n 1 ];
p{ 5 } = [ n 1 ];
p{ 6 } = [ n 1 ];
p{ 7 } = [ n 1 ];
p{ 8 } = [ n 1 ]; 

Ipol = eye( N, 1 );

cvx_begin
    cvx_solver sedumi;
    
    variable  gs;
    variables h( N );
    variables R1( N ) R2( N ) R3( N ) R4( N ) R5( N ) R6( N ) R7( N ) R8( N );
    
    R1 == sos_pol( p{ 1 }, ptype{ 1 } );
    R2 == sos_pol( p{ 2 }, ptype{ 2 } );
    R3 == sos_pol( p{ 3 }, ptype{ 3 } );
    R4 == sos_pol( p{ 4 }, ptype{ 4 } );
    R5 == sos_pol( p{ 5 }, ptype{ 5 } );
    R6 == sos_pol( p{ 6 }, ptype{ 6 } );
    R7 == sos_pol( p{ 7 }, ptype{ 7 } );
    R8 == sos_pol( p{ 8 }, ptype{ 8 } );
    
    minimize gs;
    
    % 1st
    (  1 + gp ) * Ipol - h == R1;
    % 2nd
    ( -1 + gp ) * Ipol + h == R2;
    % 3rd
    gs * Ipol - h == R3;
    % 4th
    gs * Ipol - h == R4;
    % 5th
    gs * Ipol - h == R5;
    % 6th
    gs * Ipol + h == R6;
    % 7th
    gs * Ipol + h == R7;
    % 8th
    gs * Ipol + h == R8;
cvx_end    