function [ h ] = lin_phase_fir2d_5_2_2_cvx_v3( n, gp )

% LIN_PHASE_FIR2D_5_2_2_CVX_V3 Linear phase 2-D FIR filter design
%
%   This function is an example of the design of linear-phase
%   2-D symmetric FIR filters. The problem is discussed in 
%   [1, Section 5.2.2]. According to the defined positivity 
%   domains, this function will design diamond shape filters.
%
%   [ H ] = LIN_PHASE_FIR2D_5_2_2_CVX_V3( N, GP )
%
%   INPUTS:
%
%   N  - ("half the") degree of the filter
%   GP - magnitude error bound of the passband
%
%   OUTPUT:
%
%   H = coefficients of the filter belonging to a halfspace 
%
%   H(z) = \sum_{k=-n}^n h_k*z^{-k}, h_{-k}=h{k}, k=(k1, k2).
%
%   The optimization problem solved is:
%
%   min  gs
%
%   s.t.  1 + gp - H(w) >= 0, for all w
%        -1 + gp + H(w) >= 0, w in Dp = Dp1 * Dp2 * Dp3
%             gs - H(w) >= 0, w in Ds1 + Ds2 + Ds3
%             gs + H(w) >= 0, w in Ds1 + Ds2 + Ds3
%
%   Denoting
%
%   S0 =  1 + gp - H(w)
%   S1 = -1 + gp + H(w)
%   S2 =      gs - H(w)
%   S3 =      gs + H(w)
%
%   the problem for POS3POLY has the following form:
%
%   min  gs
%
%   s.t.  1 + gp =  H(w) + S0, S0 >= 0, for all w
%        -1 + gp = -H(w) + S1, S1 >= 0, w in Dp = Dp1 * Dp2 * Dp3
%             gs =  H(w) + S2, S2 >= 0, w in Ds1 + Ds2 + Ds3
%             gs = -H(w) + S5, S5 >= 0, w in Ds1 + Ds2 + Ds3
%
%   Example:
%
%   h = lin_phase_fir2d_5_2_2_cvx_v3( [ 7 7 ], 0.1 );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 15.06.09-17.06.09

% number of n-tuples
N = nch( n );

% the polynomial variables
ptype{ 1 }.trigonometric = 2;
ptype{ 2 }.trigonometric = 2;
ptype{ 3 }.trigonometric = 2;
ptype{ 4 }.trigonometric = 2;

% positivity domains
% Dp
ptype{ 2 }.dom.deg = [ 1 1; ...
                       1 1; ...
                       1 1  ];
ptype{ 2 }.dom.coef = [ -0.1 0 0 0 0.5 -0.1 0 0.5 0 0 0 0.5 0 0.5 0 ];

% Ds1, Ds2, Ds3
ptype{ 3 }.dom.nunion = [ 1 1 1 ];
ptype{ 3 }.dom.deg  = [ 1 1; ...
                        1 1; ...
                        1 1  ];
ptype{ 3 }.dom.coef = [ -0.7 0 0 0 -0.5 -0.7 0 -0.5 0 0 0 -0.5 0 -0.5 0 ];

% Ds1, Ds2, Ds3
ptype{ 4 }.dom.nunion = [ 1 1 1 ];
ptype{ 4 }.dom.deg  = [ 1 1; ...
                        1 1; ...
                        1 1  ];
ptype{ 4 }.dom.coef = [ -0.7 0 0 0 -0.5 -0.7 0 -0.5 0 0 0 -0.5 0 -0.5 0 ];

% the degrees of the polynomials
% are wtn, and its coefficients are
% scalars (1x1)
p{ 1 } = [ n 1 ];
p{ 2 } = [ n 1 ];
p{ 3 } = [ n 1 ];
p{ 4 } = [ n 1 ];

Ipol = eye( N, 1 );

cvx_begin
    cvx_solver sedumi;
    
    variable  gs;
    variables h( N );
    variables R1( N ) R2( N ) R3( N ) R4( N );
    
    R1 == sos_pol( p{ 1 }, ptype{ 1 } );
    R2 == sos_pol( p{ 2 }, ptype{ 2 } );
    R3 == sos_pol( p{ 3 }, ptype{ 3 } );
    R4 == sos_pol( p{ 4 }, ptype{ 4 } );
    
    minimize gs;
    
    % 1st
    (  1 + gp ) * Ipol - h == R1;
    % 2nd
    ( -1 + gp ) * Ipol + h == R2;
    % 3rd
    gs * Ipol - h == R3;
    % 4th
    gs * Ipol + h == R4;
cvx_end    