function [ wth ] = lin_phase_fir_5_1_1_cvx( wtn, wp, ws, gp, gs )

% LIN_PHASE_FIR_5_1_1_cvx Linear phase FIR filter design
%
%   This function is an example of the design of linear-phase
%   symmetric FIR filters, by optimizing the magnitude. The 
%   problem is discussed in [1] Section 5.1.1, page 139.
%
%   [ WTH ] = LIN_PHASE_FIR_5_1_1_CVX( WTN, WP, WS, GP, GS )
%
%   INPUTS:
%
%   WTN - ("half the") degree of the filter
%   WP  - (frequency) upper limit of the passband
%   WS  - (frequency) lower limit of the stopband
%   GP  - magnitude error bound of the passband
%   GS  - magnitude error bound of the stopband
%
%   OUTPUT:
%
%   WTH = unique coefficients of the (symmetric) filter
%
%   wtH(z) = \sum_{k=-wtn}^wtn wth_k*z^{-k}, wth_{-k}=wth{k}.
%
%   The optimization problem solved is:
%
%   min  eps
%
%   s.t.  1 + gp - wtH(w) >= 0, w in [ 0, wp ]
%        -1 + gp + wtH(w) >= 0, w in [ 0, wp ]
%             gs - wtH(w) >= 0, w in [ ws, pi ]
%             gs + wtH(w) >= 0, w in [ ws, pi ]
%             y = wtC12 * wth
%             ||y|| <= eps
%
%   Denoting
%
%   S0 =  1 + gp - wtH(w)
%   S1 = -1 + gp + wtH(w)
%   S2 =      gs - wtH(w)
%   S3 =      gs + wtH(w)
%
%   the problem for POS3POLY has the following form:
%
%   min  eps
%
%   s.t.  1 + gp =  wtH(w) + S0, S0 >= 0, w in [ 0, wp ]
%        -1 + gp = -wtH(w) + S1, S1 >= 0, w in [ 0, wp ]
%             gs =  wtH(w) + S2, S2 >= 0, w in [ ws, pi ]
%             gs = -wtH(w) + S3, S3 >= 0, w in [ ws, pi ]
%             y = wtC12 * wth
%             ||y|| <= eps   
%
%   Example:
%
%   wth = lin_phase_fir_5_1_1_cvx( 25, 0.2 * pi, 0.25 * pi, 0.1, 0.0158 );
%   fvtool( [ flipud( wth( 2 : end ) ); wth ] );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 27.11.08-12.04.10

% wtn = widetilde{n}
% n = 2 * wtn

% size (degree + 1)
wtn1 = wtn + 1;

% constraints;
%  1 + gp - wtH = S0
% -1 + gp + wtH = S1
%      gs - wtH = S2
%      gs + wtH = S3
%  y = wtC^{1/2}*wth

% dim. of vector c
wtn21 = 2 * wtn + 1;
c = zeros( wtn21, 1 );

% ck = 1-ws/pi, k = 0
% ck = -sin(k*ws)/(k*pi), k > 0
c( 1 ) = 1 - ws / pi;
for i = 2 : wtn21
    c( i ) = -sin( ( i - 1 ) * ws ) / ( ( i - 1 ) * pi );
end

% Es = h' * C * h
C = toeplitz( c );

% h = P * wth
P = [ zeros( wtn, 1 ) fliplr( eye( wtn ) ); ...
            1          zeros( 1, wtn )    ; ...
      zeros( wtn, 1 )         eye( wtn )   ];

% Es = wth' * wtC * wth
wtC = P' * C * P;

% square matrix
wtC12 = real( sqrtm( wtC ) );

% the polynomial variables
ptype{ 1 }.trigonometric = 1;
ptype{ 2 }.trigonometric = 1;
ptype{ 3 }.trigonometric = 1;
ptype{ 4 }.trigonometric = 1;

% positivity domains
ptype{ 1 }.int = [ 0  wp ];
ptype{ 2 }.int = [ 0  wp ];
ptype{ 3 }.int = [ ws pi ];
ptype{ 4 }.int = [ ws pi ];

% the degrees of the polynomials
% are wtn, and its coefficients are
% scalars (1x1)
p{ 1 } = [ wtn 1 ];
p{ 2 } = [ wtn 1 ];
p{ 3 } = [ wtn 1 ];
p{ 4 } = [ wtn 1 ];

Ipol = eye( wtn1, 1 );

cvx_begin
    cvx_solver sedumi;
    
    variable  e;
    variables wth( wtn1 ) y( wtn1 );
    variables R1( wtn1 ) R2( wtn1 ) R3( wtn1 ) R4( wtn1 );
    
    R1 == sos_pol( p{ 1 }, ptype{ 1 } );
    R2 == sos_pol( p{ 2 }, ptype{ 2 } );
    R3 == sos_pol( p{ 3 }, ptype{ 3 } );
    R4 == sos_pol( p{ 4 }, ptype{ 4 } );
    
    minimize e;
    
    norm( y ) <= e;
    
    % 1st positive polynomial
    ( 1 + gp ) * Ipol - wth == R1;
    
    % 2nd positive polynomial
    wth - ( 1 - gp ) * Ipol == R2;
    
    % 3rd positive polynomial
    gs * Ipol - wth == R3;
    
    % 4th positive polynomial
    wth + gs * Ipol == R4;
    
    y == wtC12 * wth;
cvx_end