function H = mat_filt_cvx( N, wp, ws1, ws2 )

% MAT_FILT_CVX Matrix FIR filter design for cvx style
%
%   H = MAT_FILT_CVX( N, WP, WS1, WS2 )
%
%   This function can be used to design a matrix filter as 
%   described in [1].
%
%   The matrix filter H, of size NxN will have passband [0 WP] and
%   stopbands [WS1 pi] and [-pi WS2].
%   
%   Example:
% 
%   H = mat_filt_cvx( 15, 0.2*pi, 0.3*pi, -0.1*pi );        
%   A = mat( H );
%   w = -pi : 0.01 : pi;
%   len = length( w );
%   Hw = zeros( N, len );
%   for i = 0 : 15 - 1
%       Hw( i + 1, : ) = freqz( A( i + 1, : ) / sqrt( N ), 1, w );
%   end
%   Hwn = zeros( len,1);
%   for i = 1 : len
%       Hwn( i ) = norm( Hw( :, i ) );
%   end
%   plot( w / pi, mag2db( Hwn ) );
%
%   [1] B. Dumitrescu. Bouded real lemma for multivariate trigonometric
%   matrix polynomials and FIR filter design applications. EUSIPCO, 2009.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 23.03.10-16.04.10

N2 = N ^ 2;

I  =  speye( N  );
O  = sparse( N - 1, 1 );

% the polynomial variables
ptype{ 1 }.trigonometric = 1;
ptype{ 2 }.trigonometric = 1;

% positivity domains
ptype{ 1 }.int = [ 0   wp ];
ptype{ 2 }.int = [ ws1 pi -pi ws2 ];

% the degrees of the polynomials
% are n, and its coefficients are
% scalars (1x1)
p{ 1 } = [ N - 1 1 ];
p{ 2 } = [ N - 1 1 ];

% BRL 1
ptype{ 1 }.brl.hsize = [ 1 N ];

% BRL 2
ptype{ 2 }.brl.hsize = [ 1 N ];

% complex
ptype{ 1 }.complex_coef = 1;
ptype{ 2 }.complex_coef = 1;

cvx_begin    
    variable H( N2 ) complex;
    
    variable HR1( N2 + N ) complex;
    variable HR2( N2 + N ) complex;
    
    HR1 == sos_pol( p{ 1 }, ptype{ 1 } );
    HR2 == sos_pol( p{ 2 }, ptype{ 2 } );
    
    minimize real( HR1( N2 + 1 ) );

    subject to
    
    real( HR1( N2 + 1 ) ) == real( HR2( N2 + 1 ) );
    
    HR1( 1 : N2 ) == H - vec( I );
    HR2( 1 : N2 ) == H;
    
    O == HR1( N2 + 2 : end );
    O == HR2( N2 + 2 : end );
cvx_end

H = H';