function H = mimo_fir2d_cvx_v3( n, gp, K, degD, coefD, wp, ws )

% MIMO_FIR2D 2-D FIR MIMO filter design
%
%   H = MIMO_FIR2D_CVX_V3( N, GP, K, DEGD, COEFD, WP, WS ) can be used to
%   design a 2-D FIR MIMO filter of size (N(1)+1)x(N(2)+1). The 
%   coefficients of the filter are of size K(1)xK(2). DEGD and COEFD
%   denote the degrees and coeficients of the delay filter, respectively.
%   WP and WS are the passband and stopband edges, respectively. COEFD is
%   a vector of cells.
%
%   Example:
%   
%   H = mimo_fir2d_cvx_v3( [4 4], 0.1, [2 2], [0 0], {eye(2)}, 0.4*pi, 0.9*pi );
%   freqz2d( mat( H( 1 : 4 : 100 ) ) );   

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.02.10-16.04.10

% number of n-tuples
N = nch( n );

% number of filter coefs.
M = prod( n + 1 );

K1   = K( 1 );
K2   = K( 2 );
K1K2 = K1 * K2;

MK1K2 = M * K1 * K2;

% number of eq. constraints for one polynomial
% in the Gram matrix characterization
OneSOSVar = N;  

% the polynomial variables
ptype{ 1 }.trigonometric = 2;
ptype{ 2 }.trigonometric = 2;
ptype{ 3 }.trigonometric = 2;

% positivity domains
% Dp
ptype{ 1 }.dom.nc   = [ 2 2 ];
ptype{ 1 }.dom.deg = [ 0 0; ...
                       1 0; ...
                       0 0; ...
                       0 1  ];
                        
ptype{ 1 }.dom.coef = [ -2 * cos( wp ) 1 -2 * cos( wp ) 1 ];

% Ds1 U Ds2
ptype{ 2 }.dom.nunion = [ 1 1 ];
ptype{ 2 }.dom.nc   = [ 2 2 ];
ptype{ 2 }.dom.deg  = [ 0 0; ...
                        1 0; ...
                        0 0; ...
                        0 1  ];
ptype{ 2 }.dom.coef = [ 2 * cos( ws ) -1 2 * cos( ws ) -1 ];

% BRL settings

% 1
ptype{ 1 }.brl.hsize = [ K1 K2 ];

% 2
ptype{ 2 }.brl.hsize = [ K1 K2 ];

% the degrees of the polynomials
% are 'n', and its coefficients are
% scalars (1x1)
p{ 1 } = [ n 1 ];
p{ 2 } = [ n 1 ];

O = sparse( OneSOSVar - 1, 1 );

D = sparse( MK1K2, 1 );

% introduce D(z)
for i = 1 : size( degD, 1 )
    nrdeg = degD( i, 2 ) * ( n( 1 ) + 1 ) + degD( i, 1 ) + 1;
    
    D( ( nrdeg - 1 ) * K1K2 + 1 : nrdeg * K1K2 ) = vec( coefD{ i } );
end

cvx_begin
    cvx_solver sedumi;
    
    variable gs;
    variable H( MK1K2 );
    
    variable HR1( MK1K2 + OneSOSVar );
    variable HR2( MK1K2 + OneSOSVar );
    
    HR1 == sos_pol( p{ 1 }, ptype{ 1 } );
    HR2 == sos_pol( p{ 2 }, ptype{ 2 } );
    
    minimize gs;
    
    subject to
    
    gp ^ 2 == HR1( MK1K2 + 1 );
    gs     == HR2( MK1K2 + 1 );
    
    O == HR1( MK1K2 + 2 : end );
    O == HR2( MK1K2 + 2 : end );
    
    HR1( 1 : MK1K2 ) == H - D;
    HR2( 1 : MK1K2 ) == H;
cvx_end