function [ m ] = min_eig_trig_cvx( r, n, K )

% MIN_EIG_TRIG_CVX Minimum eigenvalue of a multivariate matrix trig. pol.
%
%   This function solves the problem of finding the minimum of a 
%   multivariate trigonometric polynomial, given in Section 3.5, 
%   page 79, in [1], using POS3POLY.
%
%   [ M ] = MIN_EIG_TRIG_CVX( R, N, K )
%
%   INPUT:
%     
%   N - degree of the multivariate polynomial
%   R - coefficients of the trigonometric polynomial
%
%   OUTPUTS:
%
%   MU - minimum value
%   
%   The original problem is
%
%   m* = max  m
%           
%         s.t. R(z) - m >= 0, z in T^d.
%      
%   Denoting S0 = R(z) - m we have
%
%   m* = max  m
%           
%         s.t. S0 >= 0, z in T^d.
%
%   The function solves using POS3POLY the 
%   relaxed problem
%
%   m* = max  m
%           
%         s.t. S0 in RS_n^m, z in T^d, where
%   
%   RS_n^m is the set of SOS with degree n and factors
%   of degree m.
%
%   Example:
%
%   m = min_eig_trig_cvx( [ 38 18 4 1 2 1 -8 -5 ]', [ 2 1 ], 1 );
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 12.03.09-26.10.11

cvx_begin
    variable m;
    
    maximize m;

    subject to

    r - m * unitpol( n, K ) == sos_pol( [ n K ] );
cvx_end