function sigma = dist2unstab_r_dual( A, B, n )

% DIST2UNSTAB_R_DUAL Distance to unstabilizability with real polynomials
%
%   SIGMA = DIST2UNSTAB_R_DUAL( A, B, N ) returns the distance to 
%   unstabilizability SIGMA for the pair (A, B).
%
%   The function solves the following problem:
%
%   max tau
%
%       P(t1,t2) - tau * I = S0 + t1*S1 + (r^2 - t1^2 - t2^2)*S2,
%
%   where S0, S1 and S2 are sum-of-squares and
%
%   P_{2,0}  = I
%   P_{0,2}  = I
%   P_{1,0}  = -( A + A' )
%   P_{0,1} = -j * ( A' - A )
%   P_{0,0}  = A * A' + B * B'
%
%   SIGMA = sqrt(TAU). N is the degree for the polynomial P(t1,t2).
%
%   The algorithm is discussed in [1].
%
%   [1] B. Dumitrescu, B.C. Sicleru, R. Stefan, Computing the 
%   controllability radius: a semi-definite programming approach,
%   IET Constrol Theory and Applications, 2009, vol. 3, no. 6,
%   pp. 654-660.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 09.10.09-08.11.11

% size of the matrices A and B
K = size( A, 1 );

% number of elements in 
% "half" a matrix
hK = sumn( K );

% identity matrix
I = speye( K );

% j
SR = 1i;

% upper bound
r = sqrt( max( abs( eig( ( A + A' ) / 2 ) ) ) ^ 2 + ...
          max( abs( eig( ( A - A' ) / ( 2 * SR ) ) ) ) ^ 2 ); 

% degrees of the nonzero coefficients
% of the polynomial P(t1,t2)      
deg = [ 2 0; ...
        0 2; ...
        1 0; ...
        0 1; ...
        0 0  ];

% nonzero coefficiets of the polynomial P(t1,t2)        
coef{ 1 } =  I;   
coef{ 2 } =  I;
coef{ 3 } =  -( A + A' );
coef{ 4 } = -SR * ( A' - A );
coef{ 5 } = A * A' + B * B';

% objective function
bsP = 1;    

% number of coefficients
N = prod( n + 1 );

% number of constraints in the POS3POLY problem
nConstr = N * hK;

AsP = unitpol( n, K, 0, 2 );
csP = sppol2fvec( n, deg, coef, 0, 2 ); 

% 2-D polynomial P(t1,t2)
KsP.ptype{ 1 }.real = 2;
% degree and size of the coefficients
KsP.p{ 1 } = [ n K ];

% positivity domain (pos. dom.)

% number of coefficients
KsP.ptype{ 1 }.dom.nc   = [ 1 3 ];
% degrees of the coefficients
KsP.ptype{ 1 }.dom.deg = [  1 0; ...
                            0 0; ...
                            2 0; ...
                            0 2  ];
% coefficients from the pos. dom.                            
KsP.ptype{ 1 }.dom.coef = [ 1, r ^ 2, -1, -1 ];

% complex coefficients
KsP.ptype{ 1 }.complex_coef = 1;
% complex constraints
KsP.ycomplex = 1 : nConstr;

% call POS3POLY
[ y ] = pos3poly( AsP, bsP, csP, KsP );

% distance to unstabilizability
sigma = sqrt( y );