function sigma = dist2unstab_t( A, B, n )

% DIST2UNSTAB_T Distance to unstabilizability with trigonometric polynomials
%
%   SIGMA = DIST2UNSTAB_T( A, B, N ) returns the distance to 
%   unstabilizability SIGMA for the pair (A, B).
%
%   The function solves the following problem:
%
%   max tau
%
%       P(z1,z2) - tau * I = S0 + 1/2(z1 + z1^{-1})*S1,
%
%   where S0 and S1 are sum-of-squares and
%
%   P_{0,0}  = 3 / 8 * r ^ 2 * I + A * A' + B * B' 
%   P_{1,0}  = -1 / 2 * r * A 
%   P_{-1,1} = -1 / 4 * r * A'
%   P_{0,1}  = 1 / 4 * r ^ 2 *I
%   P_{1,1}  = -1 / 4 * r * A
%   P_{-1,2} = O
%   P_{0,2}  = 1 / 16 * r ^ 2 * I
%   P_{1,2}  = O
%
%   SIGMA = sqrt(TAU). N is the degree for the polynomial P(z1,z2).
%
%   The algorithm is discussed in [1].
%
%   [1] B. Dumitrescu, B.C. Sicleru, R. Stefan, Computing the 
%   controllability radius: a semi-definite programming approach,
%   IET Constrol Theory and Applications, 2009, vol. 3, no. 6,
%   pp. 654-660.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 11.10.09

% size of the matrices A and B
K = size( A, 1 );

% number of elements in 
% "half" a matrix
hK = sumn( K );

% nr. of elements
% in a matrix
K2 = K ^ 2;

% identity matrix
I = speye( K );
% zero matrix
O = sparse( K, K );

% j
SR = 1i;

% upper bound
r = sqrt( max( abs( eig( ( A + A' ) / 2 ) ) ) ^ 2 + ...
          max( abs( eig( ( A - A' ) / ( 2 * SR ) ) ) ) ^ 2 ); 

% degrees of the nonzero coefficients
% of the polynomial P(z1,z2)
deg = [ 0 0; ...
        1 0; ...
       -1 1; ...
        0 1; ...
        1 1; ...
       -1 2; ...
        0 2; ...
        1 2  ];
    
% nonzero coefficiets of the polynomial P(z1,z2)    
coef{ 1 } = 3 / 8 * r ^ 2 * I + A * A' + B * B';   
coef{ 2 } = -1 / 2 * r * A ;
coef{ 3 } = -1 / 4 * r * A';
coef{ 4 } = 1 / 4 * r ^ 2 *I;
coef{ 5 } = -1 / 4 * r * A;
coef{ 6 } = O; 
coef{ 7 } = 1 / 16 * r ^ 2 * I;
coef{ 8 } = O;

% build 'bsP' term
bsP = sppol2fvec( n, deg, coef, 2 );    

% number of coefficiets 
% in a halfspace
N = nch( n );

% number of constraints in the POS3POLY problem
nConstr = hK + ( N - 1 ) * K2;
% number of variables in the POS3POLY problem
nVar = 1 + nConstr;

AsP = sparse( nConstr, nVar );
csP = -speye( 1,       nVar );

AsP( 1 : hK, 1 ) = vecs( I );

AsP( 1 : nConstr, 2 : nVar ) = speye( nConstr );

% the 'tau' variable
KsP.f = 1;

% 2-D polynomial P(z1,z2)
KsP.ptype{ 1 }.trigonometric = 2;
% degree and size of the coefficients
KsP.p{ 1 } = [ n K ];

% positivity domain (pos. dom.)

% number of coefficients
KsP.ptype{ 1 }.dom.nc   = 1;
% degree of the coefficient
KsP.ptype{ 1 }.dom.deg  = [ 1 0 ];
% coefficients from the pos. dom.                        
KsP.ptype{ 1 }.dom.coef = 0.5;

% call POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP );

% distance to unstabilizability
sigma = sqrt( x( 1 ) );