function [ h ] = fir3d_lp( n, gp, c )

% FIR3D_LP 3-D linear-phase FIR filter
%
%   This function is an example of the design of linear-phase
%   3-D symmetric FIR fan filters.
%
%   [ H ] = FIR3D_LP( N, GP, C )
%
%   INPUTS:
%
%   N  - ("half the") degree of the filter
%   GP - magnitude error bound of the passband
%   C  - parameter to adjust the shape of the passband
%
%   OUTPUT:
%
%   H = coefficients of the filter belonging to a halfspace 
%
%   H(z) = \sum_{k=-n}^n h_k*z^{-k}, h_{-k}=h{k}, k=(k1, k2, k3).
%
%   The optimization problem solved is:
%
%   min  gs
%
%   s.t.  1 + gp - H(w) >= 0, for all w
%        -1 + gp + H(w) >= 0, w in Dp1 * Dp2
%             gs - H(w) >= 0, w in Ds1 + Ds2
%             gs + H(w) >= 0, w in Ds1 + Ds2
%
%   Denoting
%
%   S0 =  1 + gp - H(w)
%   S1 = -1 + gp + H(w)
%   S2 =      gs - H(w)
%   S3 =      gs - H(w)
%   S4 =      gs + H(w)
%   S5 =      gs + H(w)
%
%   the problem for POS3POLY has the following form:
%
%   min  gs
%
%   s.t.  1 + gp =  H(w) + S0, S0 >= 0, for all w
%        -1 + gp = -H(w) + S1, S1 >= 0, w in Dp1 * Dp2
%             gs =  H(w) + S2, S2 >= 0, w in Ds1
%             gs =  H(w) + S3, S3 >= 0, w in Ds2
%             gs = -H(w) + S4, S4 >= 0, w in Ds1
%             gs = -H(w) + S5, S5 >= 0, w in Ds2
%
%   Example:
%
%   n = [ 3 3 3 ];
%   h = fir3d_lp( n, 0.1, 0.7 );
%   H = half2all3d( h, n );
%   Hw = freqz3d( H);
%   f1 = -1 : 1e-2 : 1;
%   f2 = -1 : 1e-2 : 1;
%   % plot on w3
%   plotf3d2d( Hw, 0 : 0.1 : 1, f1, f2, 1e-2, 1, 0, '', 0 );
%
%   Bibliography:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 20.06.09-24.09.09

% number of n-tuples
N = nch( n );

% number of constraints
nConstr = 6 * N;

% number of variables:
nVar    = nConstr + N + 1;

% identity matrix of 
% size wtn1 x wtn1
I = speye( N );

% initialize POS3POLY 
% variables
AsP = sparse( nConstr, nVar );
bsP = sparse( nConstr, 1    );
csP = sparse( 1,       nVar );

% 1 for eps in the
% objective function
csP( 1 ) = 1;

% free terms from the the
% four polynomial contraints
bsP( 1     ) =  1 + gp;
bsP( 1 + N ) = -1 + gp;

% 1 + gp = H + S0
AsP( 1: N,     2 : N + 1 ) = I;
AsP( 1: N, N + 2 : 2 * N + 1 ) = I;

% -1 + gp = -H + S1, Dp
AsP( N + 1 : 2 * N,         2 : N + 1 ) = -I;
AsP( N + 1 : 2 * N, 2 * N + 2 : 3 * N + 1 ) = I;

% gs = H + S2, Ds1
AsP( 2 * N + 1 , 1 ) = -1;
AsP( 2 * N + 1 : 3 * N, 2 : N + 1 ) = I;
AsP( 2 * N + 1 : 3 * N, 3 * N + 2 : 4 * N + 1 ) = I;

% gs = H + S3, Ds2
AsP( 3 * N + 1 , 1 ) = -1;
AsP( 3 * N + 1 : 4 * N, 2 : N + 1 ) = I;
AsP( 3 * N + 1 : 4 * N, 4 * N + 2 : 5 * N + 1 ) = I;

% gs = -H + S4, Ds1
AsP( 4 * N + 1 , 1 ) = -1;
AsP( 4 * N + 1 : 5 * N, 2 : N + 1 ) = -I;
AsP( 4 * N + 1 : 5 * N, 5 * N + 2 : 6 * N + 1 ) = I;

% gs = -H + S5, Ds2
AsP( 5 * N + 1 , 1 ) = -1;
AsP( 5 * N + 1 : 6 * N, 2 : N + 1 ) = -I;
AsP( 5 * N + 1 : 6 * N, 6 * N + 2 : 7 * N + 1 ) = I;

% h and gs are
% free variables
KsP.f = 1 + N;

% the polynomial variables
KsP.ptype{ 1 }.trigonometric = 3;
KsP.ptype{ 2 }.trigonometric = 3;
KsP.ptype{ 3 }.trigonometric = 3;
KsP.ptype{ 4 }.trigonometric = 3;
KsP.ptype{ 5 }.trigonometric = 3;
KsP.ptype{ 6 }.trigonometric = 3;

% positivity domains
% Dp
KsP.ptype{ 2 }.dom.nc   = [ 2 2 ];
KsP.ptype{ 2 }.dom.deg = [  1 0 0; ...
                            0 0 1; ...
                            0 1 0; ...
                            0 0 1  ];
                        
KsP.ptype{ 2 }.dom.coef = [ 0.5 -0.5 0.5 -0.5 ];

% Ds1
KsP.ptype{ 3 }.dom.nc   = 3;
KsP.ptype{ 3 }.dom.deg  = [ 1 0 0; ...
                            0 0 1; ...
                            0 0 0  ];
KsP.ptype{ 3 }.dom.coef = [ -0.5 0.5 -c ];

% Ds2
KsP.ptype{ 4 }.dom.nc   = 3;  
KsP.ptype{ 4 }.dom.deg  = [ 0 1 0; ...
                            0 0 1; ...
                            0 0 0  ];
KsP.ptype{ 4 }.dom.coef = [ -0.5 0.5 -c ];

% Ds1
KsP.ptype{ 5 }.dom.nc   = 3;
KsP.ptype{ 5 }.dom.deg  = [ 1 0 0; ...
                            0 0 1; ...
                            0 0 0  ];
KsP.ptype{ 5 }.dom.coef = [ -0.5 0.5 -c ];

% Ds2
KsP.ptype{ 6 }.dom.nc   = 3;   
KsP.ptype{ 6 }.dom.deg  = [ 0 1 0; ...
                            0 0 1; ...
                            0 0 0  ];
KsP.ptype{ 6 }.dom.coef = [ -0.5 0.5 -c ];

% the degrees of the polynomials
% are wtn, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];
KsP.p{ 2 } = [ n 1 ];
KsP.p{ 3 } = [ n 1 ];
KsP.p{ 4 } = [ n 1 ];
KsP.p{ 5 } = [ n 1 ];
KsP.p{ 6 } = [ n 1 ];

% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP );

% "half" of the filter's
% coefficients
h = x( 2 : N + 1 );