function hin = hinf_fir( n, h )

% HINF_FIR H_inf norm for FIR filter
%
%   HIN = HINF_FIR( N, H ) returns the H infinity norm HIN for
%   the FIR filter described by the coefficients from the vector
%   H of degree N. The coefficients are chosen from a halfspace.
%
%   Example:
%
%   n = [ 3 3 ];
%   deg = [ 3 0; 0 3; 2 1; 1 2; 0 2; 0 1 ];
%   coef = { 1, 1, 3, 3, 1, 1 };              
%   h = sppol2fvec( 1, 6, [ 3 3 ], deg, coef, 1, 0, 1 );             
%   hin = hinf_fir( n, h );

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 10.02.10-15.11.11

% number of n-tuples
N = lenpol( n );

% number of n-tuples in the
% positive orthant
Nt = lencpol( n );

if Nt ~= length( h )
    error( 'Inconsistency between Nt and length(H).' );
end

AsP = sparse( N + Nt, 1 );
bsP = -1;
csP = sparse( size( AsP, 1 ), 1 );

% c = [ 0; vec( H ) ]
csP( 1 : Nt ) = h;

% A^T = [ 0; -1; 0 ]
AsP( Nt + 1 ) = -1;

KsP.ptype{ 1 }.trigonometric = length( n );
KsP.p{ 1 } = [ n 1 ];
KsP.ptype{ 1 }.brl.hsize = 1;

% call POS3POLY 
[ y ] = pos3poly( AsP, bsP, csP, KsP );

% H_inf norm
hin = sqrt( y );