function [ h ] = lin_phase_fir2d_dual( n, gp )

% LIN_PHASE_FIR2D_DUAL Linear phase 2-D FIR filter design
%
%   This function is an example of the design of linear-phase
%   2-D symmetric FIR filters. The problem is discussed in 
%   [1, Section 5.2.2]. According to the defined positivity 
%   domains, this function will design diamond shape filters.
%
%   [ H ] = LIN_PHASE_FIR2D_DUAL( N, GP )
%
%   INPUTS:
%
%   N  - ("half the") degree of the filter
%   GP - magnitude error bound of the passband
%
%   OUTPUT:
%
%   H = coefficients of the filter belonging to a halfspace 
%
%   H(z) = \sum_{k=-n}^n h_k*z^{-k}, h_{-k}=h{k}, k=(k1, k2).
%
%   The optimization problem solved is:
%
%   min  gs
%
%   s.t.  1 + gp - H(w) >= 0, for all w
%        -1 + gp + H(w) >= 0, w in Dp = Dp1 * Dp2 * Dp3
%             gs - H(w) >= 0, w in Ds1
%             gs - H(w) >= 0, w in Ds2
%             gs - H(w) >= 0, w in Ds3
%             gs + H(w) >= 0, w in Ds1
%             gs + H(w) >= 0, w in Ds2
%             gs + H(w) >= 0, w in Ds3
%
%   Example:
%
%   h = lin_phase_fir2d_dual( [ 7 7 ], 0.1 );
%
%   The function implements the dual form.
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 15.06.09-07.11.11

% number of n-tuples
M = lenpol( n );

% number of constraints
nConstr = 8 * M;

% number of variables:
nVar    = M + 1;

% identity matrix of 
% size M x M
I = speye( M );

% initialize POS3POLY 
% variables
AsP = sparse( nConstr, nVar );
bsP = sparse( 1,       nVar );
csP = sparse( nConstr, 1    );

% 1 for eps in the
% objective function
bsP( 1 ) = -1;

% free terms for the 
% polynomial contraints
csP( 1     ) =  1 + gp;
csP( 1 + M ) = -1 + gp;

% 1 + gp - H(w) >= 0, for all w
AsP( 1: M,     2 : M + 1 ) = I;

% -1 + gp + H(w) >= 0, w in Dp = Dp1 * Dp2 * Dp3
AsP( M + 1 : 2 * M,         2 : M + 1 ) = -I;

% gs - H(w) >= 0, w in Ds1
AsP( 2 * M + 1 , 1 ) = -1;
AsP( 2 * M + 1 : 3 * M, 2 : M + 1 ) = I;

% gs - H(w) >= 0, w in Ds2
AsP( 3 * M + 1 , 1 ) = -1;
AsP( 3 * M + 1 : 4 * M, 2 : M + 1 ) = I;

% gs - H(w) >= 0, w in Ds3
AsP( 4 * M + 1 , 1 ) = -1;
AsP( 4 * M + 1 : 5 * M, 2 : M + 1 ) = I;

% gs + H(w) >= 0, w in Ds1
AsP( 5 * M + 1 , 1 ) = -1;
AsP( 5 * M + 1 : 6 * M, 2 : M + 1 ) = -I;

% gs + H(w) >= 0, w in Ds2
AsP( 6 * M + 1 , 1 ) = -1;
AsP( 6 * M + 1 : 7 * M, 2 : M + 1 ) = -I;

% gs + H(w) >= 0, w in Ds3
AsP( 7 * M + 1 , 1 ) = -1;
AsP( 7 * M + 1 : 8 * M, 2 : M + 1 ) = -I;

% the polynomial variables
KsP.ptype{ 1 }.trigonometric = 2;
KsP.ptype{ 2 }.trigonometric = 2;
KsP.ptype{ 3 }.trigonometric = 2;
KsP.ptype{ 4 }.trigonometric = 2;
KsP.ptype{ 5 }.trigonometric = 2;
KsP.ptype{ 6 }.trigonometric = 2;
KsP.ptype{ 7 }.trigonometric = 2;
KsP.ptype{ 8 }.trigonometric = 2;

% positivity domains
% Dp
KsP.ptype{ 2 }.dom.nc   = [ 2 2 2 ];
KsP.ptype{ 2 }.dom.deg = [  0 0; ...
                            1 1; ...
                            0 0; ...
                           -1 1; ...
                            1 0; ...
                            0 1; ];
                        
KsP.ptype{ 2 }.dom.coef = [ -0.1 0.5 -0.1 0.5 0.5 0.5 ];

% Ds1
KsP.ptype{ 3 }.dom.nc   = 2;
KsP.ptype{ 3 }.dom.deg  = [ 0 0; ...
                            1 1 ];
KsP.ptype{ 3 }.dom.coef = [ -0.7 -0.5 ];

% Ds2
KsP.ptype{ 4 }.dom.nc   = 2;   
KsP.ptype{ 4 }.dom.deg  = [  0 0; ...
                            -1 1 ];
KsP.ptype{ 4 }.dom.coef = [ -0.7 -0.5 ];

% Ds3
KsP.ptype{ 5 }.dom.nc   = 2;  
KsP.ptype{ 5 }.dom.deg  = [ 1 0; ...
                            0 1 ];
KsP.ptype{ 5 }.dom.coef = [ -0.5 -0.5 ];

% Ds1
KsP.ptype{ 6 }.dom.nc   = 2;
KsP.ptype{ 6 }.dom.deg  = [ 0 0; ...
                            1 1 ];
KsP.ptype{ 6 }.dom.coef = [ -0.7 -0.5 ];

% Ds2
KsP.ptype{ 7 }.dom.nc   = 2;   
KsP.ptype{ 7 }.dom.deg  = [  0 0; ...
                            -1 1 ];
KsP.ptype{ 7 }.dom.coef = [ -0.7 -0.5 ];

% Ds3
KsP.ptype{ 8 }.dom.nc   = 2;  
KsP.ptype{ 8 }.dom.deg  = [ 1 0; ...
                            0 1 ];
KsP.ptype{ 8 }.dom.coef = [ -0.5 -0.5 ];

% the degrees of the polynomials
% are n, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];
KsP.p{ 2 } = [ n 1 ];
KsP.p{ 3 } = [ n 1 ];
KsP.p{ 4 } = [ n 1 ];
KsP.p{ 5 } = [ n 1 ];
KsP.p{ 6 } = [ n 1 ];
KsP.p{ 7 } = [ n 1 ];
KsP.p{ 8 } = [ n 1 ];

% use POS3POLY
[ y ] = pos3poly( AsP, bsP, csP, KsP );

% "half" of the filter's
% coefficients
h = y( 2 : M + 1 );