function [ h ] = lin_phase_fir2d_v3( n, gp )

% LIN_PHASE_FIR2D_V3 Linear phase 2-D FIR filter design
%
%   This function is an example of the design of linear-phase
%   2-D symmetric FIR filters. The problem is discussed in 
%   [1, Section 5.2.2]. According to the defined positivity 
%   domains, this function will design diamond shape filters.
%
%   [ H ] = LIN_PHASE_FIR2D_V3( N, GP )
%
%   INPUTS:
%
%   N  - ("half the") degree of the filter
%   GP - magnitude error bound of the passband
%
%   OUTPUT:
%
%   H = coefficients of the filter belonging to a halfspace 
%
%   H(z) = \sum_{k=-n}^n h_k*z^{-k}, h_{-k}=h{k}, k=(k1, k2).
%
%   The optimization problem solved is:
%
%   min  gs
%
%   s.t.  1 + gp - H(w) >= 0, for all w
%        -1 + gp + H(w) >= 0, w in Dp = Dp1 * Dp2 * Dp3
%             gs - H(w) >= 0, w in Ds1 + Ds2 + Ds3
%             gs + H(w) >= 0, w in Ds1 + Ds2 + Ds3
%
%   Denoting
%
%   S0 =  1 + gp - H(w)
%   S1 = -1 + gp + H(w)
%   S2 =      gs - H(w)
%   S3 =      gs + H(w)
%
%   the problem for POS3POLY has the following form:
%
%   min  gs
%
%   s.t.  1 + gp =  H(w) + S0, S0 >= 0, for all w
%        -1 + gp = -H(w) + S1, S1 >= 0, w in Dp = Dp1 * Dp2 * Dp3
%             gs =  H(w) + S2, S2 >= 0, w in Ds1 U Ds2 U Ds3
%             gs = -H(w) + S5, S5 >= 0, w in Ds1 U Ds2 U Ds3
%
%   Example:
%
%   h = lin_phase_fir2d_v3( [ 7 7 ], 0.1 );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 15.06.09-17.06.09

% number of n-tuples
N = nch( n );

% number of constraints
nConstr = 4 * N;

% number of variables:
nVar    = nConstr + N + 1;

% identity matrix of 
% size wtn1 x wtn1
I = speye( N );

% initialize POS3POLY 
% variables
AsP = sparse( nConstr, nVar );
bsP = sparse( nConstr, 1    );
csP = sparse( 1,       nVar );

% 1 for eps in the
% objective function
csP( 1 ) = 1;

% free terms for the 
% polynomial constraints
bsP( 1     ) =  1 + gp;
bsP( 1 + N ) = -1 + gp;

% 1 + gp = H + S0
AsP( 1: N,     2 : N + 1 ) = I;
AsP( 1: N, N + 2 : 2 * N + 1 ) = I;

% -1 + gp = -H + S1, Dp
AsP( N + 1 : 2 * N,         2 : N + 1 ) = -I;
AsP( N + 1 : 2 * N, 2 * N + 2 : 3 * N + 1 ) = I;

% gs = H + S2, Ds1, Ds2, Ds3
AsP( 2 * N + 1 , 1 ) = -1;
AsP( 2 * N + 1 : 3 * N, 2 : N + 1 ) = I;
AsP( 2 * N + 1 : 3 * N, 3 * N + 2 : 4 * N + 1 ) = I;

% gs = -H + S3, Ds1, Ds2, Ds3
AsP( 3 * N + 1 , 1 ) = -1;
AsP( 3 * N + 1 : 4 * N, 2 : N + 1 ) = -I;
AsP( 3 * N + 1 : 4 * N, 4 * N + 2 : 5 * N + 1 ) = I;

% h and gs are
% free variables
KsP.f = 1 + N;

% the polynomial variables
KsP.ptype{ 1 }.trigonometric = 2;
KsP.ptype{ 2 }.trigonometric = 2;
KsP.ptype{ 3 }.trigonometric = 2;
KsP.ptype{ 4 }.trigonometric = 2;

% positivity domains
% Dp
KsP.ptype{ 2 }.dom.nc   = [ 2 2 2 ];
KsP.ptype{ 2 }.dom.deg = [  0 0; ...
                            1 1; ...
                            0 0; ...
                           -1 1; ...
                            1 0; ...
                            0 1; ];
KsP.ptype{ 2 }.dom.coef = [ -0.1 0.5 -0.1 0.5 0.5 0.5 ];

% Ds1, Ds2, Ds3
KsP.ptype{ 3 }.dom.nunion = [ 1 1 1 ];
KsP.ptype{ 3 }.dom.nc   = [ 2 2 2 ];
KsP.ptype{ 3 }.dom.deg  = [ 0 0; ...
                            1 1; ...
                            0 0; ...
                           -1 1; ...
                            1 0; ...
                            0 1  ];
KsP.ptype{ 3 }.dom.coef = [ -0.7 -0.5 -0.7 -0.5 -0.5 -0.5 ];

% Ds1, Ds2, Ds3
KsP.ptype{ 4 }.dom.nunion = [ 1 1 1 ];
KsP.ptype{ 4 }.dom.nc   = [ 2 2 2 ];
KsP.ptype{ 4 }.dom.deg  = [ 0 0; ...
                            1 1; ...
                            0 0; ...
                           -1 1; ...
                            1 0; ...
                            0 1  ];
KsP.ptype{ 4 }.dom.coef = [ -0.7 -0.5 -0.7 -0.5 -0.5 -0.5 ];

% the degrees of the polynomials
% are wtn, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];
KsP.p{ 2 } = [ n 1 ];
KsP.p{ 3 } = [ n 1 ];
KsP.p{ 4 } = [ n 1 ];

pars = [ ];
% set accuracy; SeDuMi will run
% as long as it can progress
% pars.eps = 0;    

% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP, pars );

% "half" of the filter's
% coefficients
h = x( 2 : N + 1 );