function H = mimo_fir2d( n, gp, K, degD, coefD, wp, ws )

% MIMO_FIR2D 2-D FIR MIMO filter design
%
%   H = MIMO_FIR2D( N, GP, K, DEGD, COEFD, WP, WS ) can be used to
%   design a 2-D FIR MIMO filter of size (N(1)+1)x(N(2)+1). The 
%   coefficients of the filter are of size K(1)xK(2). DEGD and COEFD
%   denote the degrees and coeficients of the delay filter, respectively.
%   WP and WS are the passband and stopband edges, respectively. COEFD is
%   a vector of cells.
%
%   Example:
%   
%   H = mimo_fir2d( [4 4], 0.1, [2 2], [0 0], {eye(2)}, 0.4*pi, 0.9*pi );
%   freqz2d( mat( H( 1 : 4 : 100 ) ) );   

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.02.10-15.11.11

% number of n-tuples
M = lenpol( n );

% number of filter coefs.
N = lencpol( n, K );

In = speye( N );

% number of eq. constraints
nConstr = 3 * ( M + N );

% number of variables:
nVar    = 1 + N;

% initialize POS3POLY 
% variables
AsP = sparse( nConstr, nVar );
bsP = -speye( 1,       nVar );
csP = sparse( nConstr, 1    );

% 1st BRL 
AsP( 1 : N, 2 : 1 + N ) = In;
% introduce D(z)
for i = 1 : size( degD, 1 )
    nrdeg = degD( i, 2 ) * ( n( 1 ) + 1 ) + degD( i, 1 ) + 1;
    
    csP( ( nrdeg - 1 ) * prod( K ) + 1 : nrdeg * prod( K ) ) = vec( coefD{ i } );
end
csP( N + 1 ) = gp ^ 2;

% 2nd BRL
AsP( M + N + 1 : 2 * N + M, 2 : 1 + N ) = -In;
AsP( 2 * N + M + 1, 1 ) = -1;

% 3rd BRL
AsP( 2 * ( M + N ) + 1 : 2 * ( N + M ) + N, 2 : 1 + N ) = -In;
AsP( 2 * ( N + M ) + N + 1, 1 ) = -1;

% the polynomial variables
KsP.ptype{ 1 }.trigonometric = 2;
KsP.ptype{ 2 }.trigonometric = 2;
KsP.ptype{ 3 }.trigonometric = 2;

% positivity domains
% Dp
KsP.ptype{ 1 }.dom.nc   = [ 2 2 ];
KsP.ptype{ 1 }.dom.deg = [ 0 0; ...
                           1 0; ...
                           0 0; ...
                           0 1  ];
                        
KsP.ptype{ 1 }.dom.coef = [ -2 * cos( wp ) 1 -2 * cos( wp ) 1 ];

% Ds1
KsP.ptype{ 2 }.dom.nc   = 2;
KsP.ptype{ 2 }.dom.deg  = [ 0 0; ...
                            1 0 ];
KsP.ptype{ 2 }.dom.coef = [ 2 * cos( ws ) -1 ];

% Ds2
KsP.ptype{ 3 }.dom.nc   = 2;   
KsP.ptype{ 3 }.dom.deg  = [ 0  0; ...
                            0  1 ];
KsP.ptype{ 3 }.dom.coef = [ 2 * cos( ws ) -1 ];


% BRL settings

% 1
KsP.ptype{ 1 }.brl.hsize = K;

% 2
KsP.ptype{ 2 }.brl.hsize = K;

% 3
KsP.ptype{ 3 }.brl.hsize = K;

% the degrees of the polynomials
% are 'n', and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];
KsP.p{ 2 } = [ n 1 ];
KsP.p{ 3 } = [ n 1 ];

% use POS3POLY
[ y ] = pos3poly( AsP, bsP, csP, KsP );

% coefs. of the filter
H = y( 2 : N + 1 );