function [ mu ] = min_matrix_poly_eigen_general_real( P )

%  MIN_MATRIX_POLY_EIGEN_GENERAL_REAL 
%   
%   Minimization of the smallest eigenvalue of a real matrix univariate polynomial
%
%   This function solves the problem of the minimizing the smallest 
%   the eigenvalue a real matrix univariate polynomial. 
%
%   [ MU ] = MIN_MATRIX_POLY_EIGEN_GENERAL_REAL( P )
%
%   INPUT:
%   
%   P - an array of matrices for the matrix coefficients of the polynomial
%
%   OUTPUT:
%
%   MU - minimal value of the smallest eigenvalue
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. P(t) - mu * I >= 0, t real.
%   
%   Denoting S0(t) = P(t) - mu * I we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, t real.
%
%   The variables are mu and the coefficients of S0.
%
%   Because {nonnegative} = {SOS} in the univariate
%   case, the constraint above is equivalent with
%
%   S0 = SOS, ( S0 is sum-of-squares (SOS) ).
%
%   Example:
%
%   P( :, :, 1 ) = [ 5 0; 0 5 ];
%   P( :, :, 2 ) = [ 0 1; 1 0 ];
%   P( :, :, 3 ) = [ 2 0; 0 2 ];
%   mu = min_matrix_poly_eigen_general_real( P );
%   
%   Bibliography:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.11.08

% n1 = n + 1 (degree + 1)
n1 = size( P, 3 );

% the degree
% of the polynomial
n = n1 - 1;

% matrix dimensions
K  = size( P, 1 );

% number of elements
% in "half" a matrix
% N = sumn( K );
N = K * ( K + 1 ) / 2;

% total number of
% constraints
nConstr = n1 * N;

% total number of
% POS3POLY variables
% (coefficients + var.)
nCoef = 1 + nConstr;

% coeficients of the variables 
% in the objective function
csP = -speye( nCoef, 1 );

% AsP is an nConstr x (1+nConstr) matrix;
% nConstr constriaints;
% 1+nConstr variables;
AsP = sparse( nConstr, nCoef );

% coefficients of mu
AsP( 1 : N, 1 ) = vecs( speye( K ) );

% multipliers for the 
% scalar elements of the
% matrix coefficients of S0
AsP( :, 2 : end ) = speye( nConstr );

% mu is a free
% variable
KsP.f = 1;

% the polynomial is dependent
% on one real variable
KsP.ptype{ 1 }.real = 1;

% the degree of the polynomial
% is n1-1, and its coefficients are
% matrices (KxK)
KsP.p{ 1 } = [ n K ];

% use POS3POLY
[ x ] = pos3poly( AsP, vecg( P, 0 ), csP, KsP );

% minimum
mu = x( 1 );