function [ mu ] = min_matrix_poly_eigen_general_real_dual( P )

%  MIN_MATRIX_POLY_EIGEN_GENERAL_REAL_DUAL 
%   
%   Minimization of the smallest eigenvalue of a real matrix univariate polynomial
%
%   This function solves the problem of the minimizing the smallest 
%   the eigenvalue a real matrix univariate polynomial. 
%
%   [ MU ] = MIN_MATRIX_POLY_EIGEN_GENERAL_REAL_DUAL( P )
%
%   INPUT:
%   
%   P - an array of matrices for the matrix coefficients of the polynomial
%
%   OUTPUT:
%
%   MU - minimal value of the smallest eigenvalue
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. P(t) - mu * I >= 0, t real.
%
%   Because {nonnegative} = {SOS} in the univariate
%   case, the constraint above is equivalent with
%
%   S0 = SOS, ( S0 is sum-of-squares (SOS) ).
%
%   Example:
%
%   P( :, :, 1 ) = [ 5 0; 0 5 ];
%   P( :, :, 2 ) = [ 0 1; 1 0 ];
%   P( :, :, 3 ) = [ 2 0; 0 2 ];
%   mu = min_matrix_poly_eigen_general_real_dual( P );
%   
%   Bibliography:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.11.08-17.11.11

% n1 = n + 1 (degree + 1)
n1 = size( P, 3 );

% the degree
% of the polynomial
n = n1 - 1;

% matrix dimensions
K = size( P, 1 );

% AsP is a vector
AsP = unitpol( n, K, 0, 1 );

% the polynomial is dependent
% on one real variable
KsP.ptype{ 1 }.real = 1;

% the degree of the polynomial
% is n1-1, and its coefficients are
% matrices (KxK)
KsP.p{ 1 } = [ n K ];

% use POS3POLY
mu = pos3poly( AsP, 1, vecg( P, 0 ), KsP );