function [ mu ] = min_matrix_poly_eigen_multi_general_real( n, bsP, K )

%  MIN_MATRIX_POLY_EIGEN_MULTI_GENERAL_REAL 
%   
%   Minimization of the smallest eigenvalue of a real matrix univariate polynomial
%
%   This function solves the problem of minimizing the smallest 
%   eigenvalue of a real matrix multivariate polynomial. 
%
%   [ MU ] = MIN_MATRIX_POLY_EIGEN_MULTI_GENERAL_REAL( N, BSP, K )
%
%   INPUTS:
%   
%   N   - degree of the multivariate polynomial
%   BSP - the 'bsP' term
%   K   - size of the matrices
%
%   OUTPUT:
%
%   MU - minimal value of the smallest eigenvalue
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. P(t) - mu * I >= 0, t real.
%   
%   Denoting S0(t) = P(t) - mu * I we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, t real.
%
%   The function solves using POS3POLY the 
%   relaxed problem
%
%   mu* = max  mu
%           
%         s.t. S0 in RS_n^m, t in R^d, where
%   
%   RS_n^m is the set of SOS with degree n and factors
%   of degree m.
%   
%   Bibliography:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.11.08

% total number of
% constraints
nConstr = length( bsP );

% number of elements
% in "half" a matrix
hK = sumn( K );

% total number of
% POS3POLY variables
% (coefficients + var.)
nCoef = 1 + nConstr;

% coeficients of the variables 
% in the objective function
csP = -speye( nCoef, 1 );

% AsP is an nConstr x (1+nConstr) matrix;
% nConstr constriaints;
% 1+nConstr variables;
AsP = sparse( nConstr, nCoef );

% coefficients of mu
AsP( 1 : hK, 1 ) = vecs( speye( K ) );

% multipliers for the 
% scalar elements of the
% matrix coefficients of S0
AsP( :, 2 : end ) = speye( nConstr );

% mu is a free
% variable
KsP.f = 1;

% the polynomial is dependent
% on length(n) variables
KsP.ptype{ 1 }.real = length( n );

% the degree of the polynomial
% is n, and its coefficients are
% matrices (KxK)
KsP.p{ 1 } = [ n K ];
    
% set accuracy; SeDuMi will run
% as long as it can progress
% pars.eps = 0;    

% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP );

% minimum
mu = x( 1 );