function [ mu ] = min_matrix_poly_eigen_multi_general_trig( n, bsP, K )

%  MIN_MATRIX_POLY_EIGEN_MULTI_GENERAL_TRIG
%   
%   Minimization of the smallest eigenvalue of a trig. matrix multivariate polynomial
%
%   This function solves the problem of minimizing the smallest 
%   eigenvalue of a trig. matrix multivariate polynomial, problem
%   which can be found in [ 1, Section 3.10 ].
%
%   [ MU ] = MIN_MATRIX_POLY_EIGEN_MULTI_GENERAL_TRIG( N, BSP, K )
%
%   INPUTS:
%   
%   N   - degree of the multivariate polynomial
%   BSP - the 'bsP' term
%   K   - size of the matrices
%
%   OUTPUT:
%
%   MU - minimal value of the smallest eigenvalue
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu * I >= 0, z in T.
%   
%   Denoting S0(z) = R(z) - mu * I we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, z in T.
%   
%   The function solves using POS3POLY the 
%   relaxed problem
%
%   mu* = max  mu
%           
%         s.t. S0 in RS_n^m, z in T^d, where
%   
%   RS_n^m is the set of SOS with degree n and factors
%   of degree m.
%
%   Bibliography:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.11.08-26.11.08

% total number of
% constraints
nConstr = length( bsP );

% total number of
% POS3POLY variables
% (coefficients + var.)
nCoef = 1 + nConstr;

% coeficient of the variable 
% mu in the objective function
csP = -speye( nCoef, 1 );

% AsP is an nConstr x (1+nConstr) matrix;
% nConstr constriaints;
% 1+nConstr variables;
AsP = zeros( nConstr, nCoef );

% coefficients of mu; we put 1
% for each element on the diagonal
% corresponding to mu * I
%
% in di we hold the indexes for the
% diagonal elements
di      = zeros( K, 1 );
di( 1 ) = 1;
for i = 2 : K
    di( i ) = di( i - 1 ) + K - i + 2;
end
% update AsP matrix with the 
% coefficients of mu
for i = 1 : K
    AsP( di( i ), 1 ) = 1;
end

% multipliers for the 
% scalar elements of the
% matrix coefficients of S0
for c = 1 : nConstr
    AsP( c, 1 + c ) = 1;
end

% mu is a free
% variable
KsP.f = 1;

% the polynomial is dependent
% on length(n) variables
KsP.ptype{ 1 }.trigonometric = length( n );

% the degree of the polynomial
% is n, and its coefficients are
% matrices (KxK)
KsP.p{ 1 } = [ n K ];
    
pars = [ ];
% set accuracy; SeDuMi will run
% as long as it can progress
% pars.eps = 0;    

% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP, pars );

% minimum 
% eigenvalue
mu = x( 1 );