function [ mu ] = min_matrix_poly_eigen_multi_general_trig_dual( n, csP, K )

%  MIN_MATRIX_POLY_EIGEN_MULTI_GENERAL_TRIG_DUAL
%   
%   Minimization of the smallest eigenvalue of a trig. matrix multivariate polynomial
%
%   This function solves the problem of minimizing the smallest 
%   eigenvalue of a trig. matrix multivariate polynomial, problem
%   which can be found in [ 1, Section 3.10 ].
%
%   [ MU ] = MIN_MATRIX_POLY_EIGEN_MULTI_GENERAL_TRIG_DUAL( N, BSP, K )
%
%   INPUTS:
%   
%   N   - degree of the multivariate polynomial
%   BSP - the 'bsP' term
%   K   - size of the matrices
%
%   OUTPUT:
%
%   MU - minimal value of the smallest eigenvalue
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu * I >= 0, z in T.
%   
%   Denoting S0(z) = R(z) - mu * I we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, z in T.
%   
%   The function solves using POS3POLY the 
%   relaxed problem
%
%   mu* = max  mu
%           
%         s.t. S0 in RS_n^m, z in T^d, where
%   
%   RS_n^m is the set of SOS with degree n and factors
%   of degree m.
%
%   The function implements the dual form.
%
%   Bibliography:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.11.08-17.11.11

% coeficient of the variable 
% mu in the objective function
bsP = 1;

% AsP is an nC size vector;
AsP = unitpol( n, K, length( n ) );

% the polynomial is dependent
% on length(n) variables
KsP.ptype{ 1 }.trigonometric = length( n );

% the degree of the polynomial
% is n, and its coefficients are
% matrices (KxK)
KsP.p{ 1 } = [ n K ];

% use POS3POLY
mu = pos3poly( AsP, bsP, csP, KsP );