function [ mu ] = min_poly_value

% MIN_POLY_VALUE Minimum value of a polynomial
%
%   This function solves the problem of finding the minimum of an 
%   univariate trigonometric polynomial, given in Section 2.2, 
%   Example 2.12, page 29, in [1], using POS3POLY.
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu >= 0, z in T,
%
%   R(z) =  2z^(-2) - 3z^(-1) + 6 -3z + 2z^2.
%   
%   Denoting S0(z) = R(z) - mu we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, z in T.
%
%   The variables are mu and the coefficients of S0.
%
%   Because {nonnegative} = {SOS} in the univariate
%   case, the constraint above is equivalent with
%
%   S0 = SOS, ( S0 is sum-of-squares (SOS) ).
%   
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 28.10.08-1.11.08

% bsP = [ r0 r1 r2 ]'
bsP = [ 6 -3 2 ]';

% coeficients of the variables
% in the objective function
csP = [ -1 0 0 0 ]';

% AsP is a 3x4 matrix
% 3 constraints
% 4 variables: mu, s0, s1, s2,
% S0 = s2z^(-2) + s1z^(-1) + s0 + s1z + s2z^2
AsP = [ 1 1 0 0 ;
        0 0 1 0 ;
        0 0 0 1 ; ];

% mu is a free
% variable
KsP.f = 1;

% the first polynomial is dependent
% on one trigonometric variable
KsP.ptype{ 1 }.trigonometric = 1;

% the degree of the polynomial
% is 2, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ 2 1 ];
    
% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP );

% minimum
mu = x( 1 );