function [ mu ] = min_poly_value_dual

% MIN_POLY_VALUE_DUAL Minimum value of a polynomial
%
%   This function solves the problem of finding the minimum of an 
%   univariate trigonometric polynomial, given in Section 2.2, 
%   Example 2.12, page 29, in [1], using POS3POLY.
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu >= 0, z in T,
%
%   R(z) =  2z^(-2) - 3z^(-1) + 6 -3z + 2z^2.
%   
%   The problem is in dual form (with inequalities).
%   
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 28.10.08-09.10.11

% the coefficients of R(z)
csP = [ 6 -3 2 ]';

% 'AsP' is a 3x1 matrix multiplying 'mu'
AsP = [ 1 0 0 ]';

% the objective function is 'mu'
bsP = 1;

% the first polynomial is dependent
% on one trigonometric variable
KsP.ptype{ 1 }.trigonometric = 1;

% the degree of the polynomial
% is 2, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ 2 1 ];
    
% use POS3POLY
[ y ] = pos3poly( AsP, bsP, csP, KsP );

% minimum
mu = y( 1 );