function [ mu ] = min_poly_value_general_trig( r, parsfid )

% MIN_POLY_VALUE_GENERAL_TRIG Minimum value of a polynomial
%
%   This function solves the problem of finding the minimum of an 
%   univariate trigonometric polynomial, given in Section 2.2, 
%   page 28, in [1], using POS3POLY.
%
%   [ MU ] = MIN_POLY_VALUE_GENERAL_TRIG( R )
%
%   INPUT:
%   
%   R - coefficients of the trigonometric polynomial
%
%   OUTPUTS:
%
%   MU - minimum value
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu >= 0, z in T.
%      
%   Denoting S0 = R(z) - mu we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, z in T.
%
%   The variables are mu and the coefficients of S0.
%
%   Because {nonnegative} = {SOS} in the univariate
%   case, the constraint above is equivalent with
%
%   S0 = SOS, ( S0 is sum-of-squares (SOS) ).
%
%   Example:
%
%   mu = min_poly_value_general_trig( [ 6 -3 2 ] );
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 1.11.08

% n1 = n + 1 (degree + 1)
n1 = length( r );

% n2 = n + 2
% mu + coeffcients
n2 = n1 + 1;

% coeficients of the variables 
% in the objective function
csP = -speye( n2, 1 );

% AsP is an n1x(1+n1) matrix
% n1 constriaints
% 1+n1 variables: mu, s0, ..., sn
AsP = zeros( n1, n2 );

% coefficient of mu
AsP( 1, 1 ) = 1;

% multipliers for the 
% coefficients of S0
for c = 1 : n1
    AsP( c, 1 + c ) = 1;
end

% mu is a free
% variable
KsP.f = 1;

% the first polynomial is dependent
% on one trigonometric variable
KsP.ptype{ 1 }.trigonometric = 1;

% the degree of the polynomial
% is n1-1, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n1 - 1 1 ];
    
pars = [ ];
% set accuracy; SeDuMi will run
% as long as it can progress
% pars.eps = 0;    

if nargin == 2
    pars.fid = parsfid;
end

% use POS3POLY
[ x ] = pos3poly( AsP, r, csP, KsP, pars );

% minimum
mu = x( 1 );