function [ mu ] = min_poly_value_general_trig_dual( r )

% MIN_POLY_VALUE_GENERAL_TRIG Minimum value of a polynomial
%
%   This function solves the problem of finding the minimum of an 
%   univariate trigonometric polynomial, given in Section 2.2, 
%   page 28, in [1], using POS3POLY.
%
%   [ MU ] = MIN_POLY_VALUE_GENERAL_TRIG( R )
%
%   INPUT:
%   
%   R - coefficients of the trigonometric polynomial
%
%   OUTPUTS:
%
%   MU - minimum value
%   
%   The problem solved by POS3POLY is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu >= 0, z in T.
%      
%   The function implements the dual form.
%
%   Example:
%
%   mu = min_poly_value_general_trig_dual( [ 6 -3 2 ]' );
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 1.11.08

% n1 = n + 1 (degree + 1)
n1 = length( r );

% AsP is an n1x1 matrix
AsP = eye( n1, 1 );

% the first polynomial is dependent
% on one trigonometric variable
KsP.ptype{ 1 }.trigonometric = 1;

% the degree of the polynomial
% is n1-1, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n1 - 1 1 ];

% use POS3POLY
mu = pos3poly( AsP, 1, r, KsP );