function [ mu ] = min_poly_value_general_trig_v2( r )

% MIN_POLY_VALUE_GENERAL_TRIG_V2 Minimum value of a polynomial
%
%   This function solves the problem of finding the minimum of an 
%   univariate trigonometric polynomial, given in Section 2.2, 
%   page 28, in [1], using POS3POLY.
%
%   [ MU ] = MIN_POLY_VALUE_GENERAL_TRIG_V2( R )
%
%   INPUT:
%   
%   R - coefficients of the trigonometric polynomial
%
%   OUTPUTS:
%
%   MU - minimum value
%   
%   The problem solved by POS3POLY is
%
%   s0* = min  s0
%           
%         s.t. sk = rk, k ~= 0
%              S(z) >= 0, z in T.      
%
%   The variables are the coefficients of S.
%
%   Because {nonnegative} = {SOS} in the univariate
%   case, the constraint above is equivalent with
%
%   S = SOS, ( S is sum-of-squares (SOS) ).
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 1.11.08

% the degree
n = length( r ) - 1;

% n1 = n + 1 (degree + 1)
n1 = n + 1;

% coeficients of the variables 
% in the objective function
csP = speye( n1, 1 );

% AsP is an nx(n1) matrix
% n constriaints
% n1 variables: s0, ..., sn
AsP = zeros( n, n1 );

% multipliers for the 
% coefficients of S0
for c = 1 : n
    AsP( c, c + 1 ) = 1;
end

% the first polynomial is dependent
% on one trigonometric variable
KsP.ptype{ 1 }.trigonometric = 1;

% the degree of the polynomial
% is n1-1, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n1 - 1 1 ];
    
% use POS3POLY
[ x ] = pos3poly( AsP, r( 2 : end ), csP, KsP );

% s0*
s0 = x( 1 );

% the minimum
mu = r( 1 ) - s0;