function [ mu ] = min_poly_value_multi_general_hybrid( n, bsP, nt )

% MIN_POLY_VALUE_MULTI_GENERAL_HYBRID Minimum value of a multivariate hybrid polynomial
%
%   This function solves the problem of finding the minimum of a
%   multivariate real polynomial using POS3POLY.
%
%   [ MU ] = MIN_POLY_VALUE_MULTI_GENERAL_HYBRID( N, BSP, NT )
%
%   INPUTS:
%   
%   N   - degree of the multivariate polynomial
%   BSP - the 'bsP' term
%   NT  - number of trigonometric variables of the pol.
%
%   OUTPUT:
%
%   MU - minimum value
%   
%   The original problem is
%
%   mu* = max  mu
%           
%         s.t. H(z,t) - mu >= 0.
%      
%   Denoting S0 = H(z,t) - mu we have
%
%   mu* = max  mu
%           
%         s.t. S0(z,t) >= 0.
%
%   The function solves using POS3POLY the 
%   relaxed problem.
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 5.04.09

% number of coefficients
nC = length( bsP );

nC1 = nC + 1;

% coeficients of the variables 
% in the objective function
csP = -speye( nC1, 1 );

% AsP matrix
AsP = sparse( nC, nC1 );

% coefficient of mu
AsP( 1, 1 ) = 1;

% multipliers for the 
% coefficients of S0
AsP( :, 2 : end ) = speye( nC, nC );

% mu is a free
% variable
KsP.f = 1;

% the first polynomial is dependent
% on nt trig. variables and 
% length(n)-nt real variables
KsP.ptype{ 1 }.trigonometric = nt;
KsP.ptype{ 1 }.real = length( n ) - nt;

% the degree of the polynomial
% is n1-1, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];     

% use POS3POLY
[ x ] = pos3poly( AsP, bsP, csP, KsP );

% minimum
mu = x( 1 );