function [ mu ] = min_poly_value_multi_general_real( n, p )

% MIN_POLY_VALUE_MULTI_GENERAL_REAL Minimum value of a multivariate real polynomial
%
%   This function solves the problem of finding the minimum of a
%   multivariate real polynomial using POS3POLY.
%
%   [ MU ] = MIN_POLY_VALUE_MULTI_GENERAL_REAL( N, P )
%
%   INPUTS:
%   
%   N - degree of the multivariate polynomial
%   P - coefficients of the real polynomial
%
%   OUTPUT:
%
%   MU - minimum value
%   
%   The original problem is
%
%   mu* = max  mu
%           
%         s.t. P(t) - mu >= 0, t in R^d.
%      
%   Denoting S0 = P(t) - mu we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, t in R^d.
%
%   The function solves using POS3POLY the 
%   relaxed problem
%
%   mu* = max  mu
%           
%         s.t. S0 in RS_n^m, t in R^d, where
%   
%   RS_n^m is the set of SOS with degree n and factors
%   of degree m.
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 12.03.09-28.03.09

nC = prod( n + 1 );
np = length( p );
if np ~= nC
    error( 'Inconsistency between ''p'' and ''n''.' );
end

% n2 = n + 2
% mu + coeffcients
nC1 = nC + 1;

% coeficients of the variables 
% in the objective function
csP = -speye( nC1, 1 );

% AsP is an n1x(1+n1) matrix
% n1 constriaints
% 1+n1 variables: mu, s0, ..., sn
AsP = sparse( nC, nC1 );

% coefficient of mu
AsP( 1, 1 ) = 1;

% multipliers for the 
% coefficients of S0
% for c = 1 : nC
%     AsP( c, 1 + c ) = 1;
% end
AsP( :, 2 : end ) = speye( nC, nC );

% mu is a free
% variable
KsP.f = 1;

% the first polynomial is dependent
% on length(n) real variables
KsP.ptype{ 1 }.real = length( n );

% the degree of the polynomial
% is n1-1, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];     

% use POS3POLY
[ x ] = pos3poly( AsP, p, csP, KsP );

% minimum
mu = x( 1 );