function [ mu ] = min_poly_value_multi_general_trig_3_5( n, r )

% MIN_POLY_VALUE_MULTI_GENERAL_TRIG_3_5 Minimum value of a multivariate trig. polynomial
%
%   This function solves the problem of finding the minimum of a 
%   multivariate trigonometric polynomial, given in Section 3.5, 
%   page 79, in [1], using POS3POLY.
%
%   [ MU ] = MIN_POLY_VALUE_MULTI_GENERAL_TRIG_3_5( N, R )
%
%   INPUT:
%     
%   N - degree of the multivariate polynomial
%   R - coefficients of the trigonometric polynomial
%
%   OUTPUTS:
%
%   MU - minimum value
%   
%   The original problem is
%
%   mu* = max  mu
%           
%         s.t. R(z) - mu >= 0, z in T^d.
%      
%   Denoting S0 = R(z) - mu we have
%
%   mu* = max  mu
%           
%         s.t. S0 >= 0, z in T^d.
%
%   The function solves using POS3POLY the 
%   relaxed problem
%
%   mu* = max  mu
%           
%         s.t. S0 in RS_n^m, z in T^d, where
%   
%   RS_n^m is the set of SOS with degree n and factors
%   of degree m.
%
%   Example:
%
%   mu = min_poly_value_multi_general_trig_3_5( [ 2 1 ], [ 38 18 4 1 2 1 -8 -5 ] );
%      
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 12.03.09-28.03.09

nC = nch( n );
nr = length( r );
if nr ~= nC
    error( 'Inconsistency between ''r'' and ''n''.' );
end

% n2 = n + 2
% mu + coeffcients
nr1 = nr + 1;

% coeficients of the variables 
% in the objective function
csP = -speye( nr1, 1 );

% AsP is an nr x (1+nr) matrix
% nr constriaints
% 1+nr variables: mu, s0, ..., sn
AsP = zeros( nr, nr1 );

% coefficient of mu
AsP( 1, 1 ) = 1;

% multipliers for the 
% coefficients of S0
for c = 1 : nr
    AsP( c, 1 + c ) = 1;
end

% mu is a free
% variable
KsP.f = 1;

% the first polynomial is dependent
% on length(n) trigonometric variables
KsP.ptype{ 1 }.trigonometric = length( n );

% the degree of the polynomial
% is n, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];     

% use POS3POLY
[ x ] = pos3poly( AsP, r, csP, KsP );

% minimum
mu = x( 1 );