function r = nearest_autocorrelation( whr )

% NEAREST_AUTOCORRELATION Nearest autocorrelation problem
% 
%   This function treats the problem of finding the nearest auto-
%   correlation. 
%
%   R = NEAREST_AUTOCORRELATION( WHR )
%
%   INPUT:
%   
%   WHR - given Hermitian sequence (k=0:n)
% 
%   OUTPUT:
%
%   R   - nearest autocorrelatation sequence (k=0:n)
% 
%   As stated in Section 1.2, page 7, [1], the problem 
%   can be written as follows:
%
%   min  ( r - r^ )^H * G * ( r - r^ )
%    r
%
%   s.t. R( z ) >= 0, z in T,
%
%   R( z ) = \sum_{k=-n}^n r_k * z^{-n}, r_{-k}=r_k*.
%
%   r = [ r0 r1 ... rn ], G = diag( 1, 2, ..., 2 ).
%
%   As it is explained in Section 2.2 [1], the problem from above
%   can be reformulated as
% 
%     min       alpha
%   alpha, y
%
%     s.t.      ||y||  <= alpha   
%               R( z ) >= 0
%
%   where y = G^(1/2) * ( r - r^ ).
%
%   The variables are alpha, y, r.
%
%   Example:
%
%   r = nearest_autocorrelation( [ 3 2 - j 1 + j ] );
%
%   References:
%
%   [1] B. Dumitrescu. "Positive Trigonometric Polynomials and Signal 
%   Processing Applications". Springer, 2007.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 30.10.08-15.12.08

% whr = widehat{r} = r^

% n1 = n + 1 (degree + 1)
n1 = length( whr);

% n - the degree
n = n1 - 1;

% number of POS3POLY
% variables
nVarCoef = 2 * n1 + 1;

% we have n1 scalar constraints and 
% 2 * n1 + 1 variables
AsP = zeros( n1, nVarCoef );
% the objective function has 1 + n1 elements;
% we have one for alpha and zeros for all elements of 'y'
csP = speye( nVarCoef, 1 );

% k = 0
AsP( 1, 2 )      = -1;
AsP( 1, 2 + n1 ) =  1;

% for every coefficient (k = 1 : n)
for k = 2 : n1
    AsP( k, 1 + k )      = -1 / sqrt( 2 );
    AsP( k, 1 + n1 + k ) =  1;
end

% number of variables
% in SOC
KsP.q = 1 + n1;

% the polynomial is dependent
% on one trigonometric variable
KsP.ptype{ 1 }.trigonometric = 1;

% complex numbers in the sequence
KsP.ptype{ 1 }.complex_coef = 1;

% the equality constraints
% are between complex numbers
KsP.ycomplex = 1 : n1;

% the degree of the polynomial
% is n, and its coefficients are
% scalars (1x1)
KsP.p{ 1 } = [ n 1 ];

% use POS3POLY
[ x ] = pos3poly( AsP, whr, csP, KsP ); 

% nearest autocorrelation
r = spdiags( [ 1; 1 / sqrt( 2 ) * ones( n, 1 ) ], 0, n1, n1 ) * x( 2 : n1 + 1 ) + ...
    reshape( whr, n1, 1 );