function [ lambda ] = neutral_stab( A0, A1, B, ovTau, m )

% NEUTRAL_STAB Neutral systems stability
%
%   [ LAMBDA ] = NEUTRAL_STAB( A0, A1, B, OVTAU, M )
%
%   This function tests the stability of the neutral system [1]
%
%   dx/dt x(t) - B*dx/dt x(t-OVTAU) = A0*x(t) + A1*x(t-OVTAU).
%
%   M is the order of the Pade approximation used in the algorithm.
%
%   If LAMBDA > 0, the system is stable. (Due to the numerical 
%   errors a threshold must be considered.)
%
%   References:
%   
%   [1] B. Dumitrescu, B.C. Sicleru and R. Stefan. Delay-dependent 
%   stability analysis of neutral systems using positive polynomials 
%   optimization, Proc. Control Applications of Optimisation (CAO),
%   Jyvaskyla, Finland, May 2009.   

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 07.01.10-08.02.10

SR = sqrt( -1 ); % j

K = size( A0, 1 );

% sizes for Num & Den
nNum = m + 1;
nDen = m + 1;

[ w_cm, num, den ] = wcm( m );
am = w_cm / ( 2 * pi );     % alfa_m

% division for the coefficients 
% from the Pade approximation
num = num / num( end );     
den = den / den( end );
% num = num / 500;
% den = den / 500;

% flip num & den vectors
num = fliplr( num );
den = fliplr( den );

% R_m( j * am * t )
for i = 2 : m + 1 % m + 1 coefficients
    num( i ) = num( i ) * ( SR * am ) ^ ( i - 1 );
    den( i ) = den( i ) * ( SR * am ) ^ ( i - 1 );
end

Num = cell( nNum, 1 );
Den = cell( nDen, 1 );

% R_m = Num / Den
for i = 1 : nNum
    Num{ i, 1 } = num( i );
    Den{ i, 1 } = den( i );
end

[ DenDen_, nN ] = poly_prod_mono( Den, poly_conj( Den, [ nDen 1 ] ), nDen, nDen );
  NumDen_       = poly_prod_mono( Num, poly_conj( Den, [ nDen 1 ] ), nNum, nDen );
  Num_Den       = poly_prod_mono( poly_conj( Num, [ nNum 1 ] ), Den, nNum, nDen );

% size for polT
nT1 = nN + 2;
nT2 = 3;

polT = cell( nT1, nT2 );

% fill polynomial with zero matrices
[ polT ] = poly_fill_zeros( polT, [ nT1 nT2 ], K );

%-----------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------
for i = 0 : nN - 1
    polT{ i + 3, 1 } = DenDen_{ i + 1, 1 } * ( eye( K ) + B * B' );
end

for i = 0 : nN - 1
    polT{ i + 3, 1 } = polT{ i + 3, 1 } - NumDen_{ i + 1, 1 } * B;
end

for i = 0 : nN - 1
    polT{ i + 3, 1 } = polT{ i + 3, 1 } - Num_Den{ i + 1, 1 } * B';
end

%-----------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------
for i = 0 : nN - 1
    polT{ i + 2, 1 } =    polT{ i + 2, 1 } - ...
                       DenDen_{ i + 1, 1 } * ( A0' - B * A1' - A0 + A1 * B' ) * ovTau / 2 * SR;
end

for i = 0 : nN - 1
    polT{ i + 2, 2 } =    polT{ i + 2, 2 } - ...
                       DenDen_{ i + 1, 1 } * ( A0' - B * A1' - A0 + A1 * B' ) * ovTau / 4 * SR;
end

%-----------------------------------------------------------------------------------

for i = 0 : nN - 1
    polT{ i + 2, 1 } = polT{ i + 2, 1 } + NumDen_{ i + 1, 1 } * B * A0' * ovTau / 2 * SR;
end

for i = 0 : nN - 1
    polT{ i + 2, 2 } = polT{ i + 2, 2 } + NumDen_{ i + 1, 1 } * B * A0' * ovTau / 4 * SR;
end

%-----------------------------------------------------------------------------------

for i = 0 : nN - 1
    polT{ i + 2, 1 } = polT{ i + 2, 1 } + NumDen_{ i + 1, 1 } * A1 * ovTau / 2 * SR;
end

for i = 0 : nN - 1
    polT{ i + 2, 2 } = polT{ i + 2, 2 } + NumDen_{ i + 1, 1 } * A1 * ovTau / 4 * SR;
end

%-----------------------------------------------------------------------------------

for i = 0 : nN - 1
    polT{ i + 2, 1 } = polT{ i + 2, 1 } - Num_Den{ i + 1, 1 } * A1' * ovTau / 2 * SR;
end

for i = 0 : nN - 1
    polT{ i + 2, 2 } = polT{ i + 2, 2 } - Num_Den{ i + 1, 1 } * A1' * ovTau / 4 * SR;
end

%-----------------------------------------------------------------------------------

for i = 0 : nN - 1
    polT{ i + 2, 1 } = polT{ i + 2, 1 } - Num_Den{ i + 1, 1 } * A0 * B' * ovTau / 2 * SR;
end

for i = 0 : nN - 1
    polT{ i + 2, 2 } = polT{ i + 2, 2 } - Num_Den{ i + 1, 1 } * A0 * B' * ovTau / 4 * SR;
end

%-----------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------
for i = 0 : nN - 1
    polT{ i + 1, 1 } =    polT{ i + 1, 1 } + ...
                       DenDen_{ i + 1, 1 } * ( A0 * A0' + A1 * A1' ) * ovTau ^ 2  / 8 * 3;
end

for i = 0 : nN - 1
    polT{ i + 1, 2 } =    polT{ i + 1, 2 } + ...
                       DenDen_{ i + 1, 1 } * ( A0 * A0' + A1 * A1' ) * ovTau ^ 2 / 4;
end

for i = 0 : nN - 1
    polT{ i + 1, 3 } =    polT{ i + 1, 3 } + ...
                       DenDen_{ i + 1, 1 } * ( A0 * A0' + A1 * A1' ) * ovTau ^ 2 / 16;
end

%-----------------------------------------------------------------------------------

for i = 0 : nN - 1
    polT{ i + 1, 1 } =    polT{ i + 1, 1 } + ...
                       Num_Den{ i + 1, 1 } * ( A0 * A1' ) * ovTau ^ 2  / 8 * 3;
end

for i = 0 : nN - 1
    polT{ i + 1, 2 } =    polT{ i + 1, 2 } + ...
                       Num_Den{ i + 1, 1 } * ( A0 * A1' ) * ovTau ^ 2 / 4;
end

for i = 0 : nN - 1
    polT{ i + 1, 3 } =    polT{ i + 1, 3 } + ...
                       Num_Den{ i + 1, 1 } * ( A0 * A1' ) * ovTau ^ 2 / 16;
end

%-----------------------------------------------------------------------------------

for i = 0 : nN - 1
    polT{ i + 1, 1 } =    polT{ i + 1, 1 } + ...
                       NumDen_{ i + 1, 1 } * ( A1 * A0' ) * ovTau ^ 2  / 8 * 3;
end

for i = 0 : nN - 1
    polT{ i + 1, 2 } =    polT{ i + 1, 2 } + ...
                       NumDen_{ i + 1, 1 } * ( A1 * A0' ) * ovTau ^ 2 / 4;
end

for i = 0 : nN - 1
    polT{ i + 1, 3 } =    polT{ i + 1, 3 } + ...
                       NumDen_{ i + 1, 1 } * ( A1 * A0' ) * ovTau ^ 2 / 16;
end
%-----------------------------------------------------------------------------------
%-----------------------------------------------------------------------------------

hK = sumn( K );
K2 = K ^ 2;
P = nT1 * nT2;

nConstr = hK * nT1 + K2 * ( P - nT1 );
bsP = zeros( nConstr, 1 );
Lim1 = 1;

for i = 1 : nT1
    for j = 1 : nT2
        if j ~= 1
            Lim2 = Lim1 + K2 - 1;
            bsP( Lim1 : Lim2 ) = vec( polT{ i, j } );
            Lim1 = Lim2 + 1;
        else
            Lim2 = Lim1 + hK - 1;
            bsP( Lim1 : Lim2 ) = vecs( polT{ i, j } );
            Lim1 = Lim2 + 1;
        end
    end
end

nVar = 1 + nConstr;

AsP =  zeros( nConstr, nVar );
csP = -speye( 1      , nVar );

line = 1;


%line = hK + 1;

for k1 = 0 : nT1 - 1
    for k2 = 0 : nT2 - 1        
        if k2 == 0
            for i = 1 : K
                for j = i : K
                    if k1 <= 2 * m && i == j
                        AsP( line, 1 ) = DenDen_{ k1 + 1, 1} * ovTau ^ 2 / 8 * 3;
                    end
                    
                    line = line + 1;
                end
            end
        else
            for i = 1 : K
                for j = 1 : K
                    
                    if k1 <= 2 * m && i == j
                        if k2 == 0
                            AsP( line, 1 ) = DenDen_{ k1 + 1, 1} * ovTau ^ 2 / 8 * 3;
                        elseif k2 == 1
                            AsP( line, 1 ) = DenDen_{ k1 + 1, 1} * ovTau ^ 2 / 4;
                        elseif k2 == 2
                            AsP( line, 1 ) = DenDen_{ k1 + 1, 1} * ovTau ^ 2 / 16;
                        end
                    end
                    
                    line = line + 1;
                end
            end
        end
    end
end

AsP( :, 2 : end ) = speye( nConstr );

KsP.l = 1;

KsP.ptype{ 1 }.trigonometric = 1;
KsP.ptype{ 1 }.real = 1;
KsP.p{ 1 } = [ nT2-1 nT1-1 K ];
KsP.ptype{ 1 }.dom.nc   = 1;
KsP.ptype{ 1 }.dom.deg  = [ 0 1 ];
KsP.ptype{ 1 }.dom.coef = 1;

KsP.scomplex = [ 1 2 ];
KsP.ycomplex = 1 : nConstr;

pars = [ ];
% pars.eps = 0;

% solve optimization problem
[ x ] = pos3poly( AsP, bsP, csP, KsP, pars );

lambda = x( 1 );

function [ w_cm, num, den ] = wcm( m )
%
%   This function determines w_cm and the Pade approximation
%   for the requested degree.

% Author: Bogdan C. Sicleru

[ num, den ] = pade( 1, m );
Roots = roots( num - den );

w_cm = imag( Roots );
w_cm = positive( w_cm );
w_cm = min( w_cm );

function [ pv ] = positive ( v )
%
% POSITIVE 
%
%   This function returns the positive values in the given vector V.

% Author: Bogdan C. Sicleru

pv = v( v > 0 );

function [ polC, nC ] = poly_prod_mono( polA, polB, nA, nB )

% POLY_PRODUCT_MONO Polynomial product for monovariabile polynomials

% Author: Bogdan C. Sicleru
% Date  : 21.07.08

% size for polC
nC = nA + nB - 1;

polC = cell( nC, 1 );

% fill polC with zeros
for i = 1 : nC
    polC{ i, 1 } = 0;
end

% polynomial product
for i = 0 : nA - 1
    for j = 0 : nB - 1
        polC{ i + j + 1, 1 } = polC{ i + j + 1, 1 } + ...
                               polA{ i + 1, 1 } * polB{ j + 1, 1 };
    end
end

function [ polA ] = poly_fill_zeros( polA, Asize, K )

% POLY_FILL_ZEROS Fill polynomial with zeros

%   For each empty element of the polynomial we
%   assign the zero matrix of K dimension. 
%   (If K is 1, then we insert the zero element.)

% Author: Bogdan C. Sicleru
% Date  : 18.07.08-7.10.08

for i = 1 : Asize( 1 )    
    for j = 1 : Asize( 2 ) 
        if ( isempty( polA{ i, j } ) )
            polA{ i, j } = zeros( K );
        end
    end
end

function [ polAConj ] = poly_conj( polA, Asize )

% POL_CONJ Polynomial with conjugate coefficients

% Author: Bogdan C. Sicleru
% Date  : 16.07.08

nA1 = Asize( 1 );
nA2 = Asize( 2 );

polAConj = cell( nA1, nA2 );

for i = 1 : nA1
    for j = 1 : nA2
        polAConj{ i, j } = conj( polA{ i, j } );
    end
end