function [ pos, posM, poss ] = coefpos( n, deg, nt, nr, K, i, j )

% COEFPOS Get position of coefficient
%
%   This function returns the position of a coefficient in the vector 
%   that describes the polynomial.
%
%   For scalar polynomials: 
%   POS = COEFPOS( N, DEG, NT, NR )
%   
%   The function returns in the vector POS the position of the scalar
%   coefficients. 
%   
%   N is the degree of the polynomials. 
%   DEG is a matrix whose rows are the degrees of the coefficients. 
%   NT is the number of trigonometric variables of the polynomial. 
%   NR is the number of real variables. 
%   
%   If NR is omitted, a trigonometric polynomial is considered. If NT
%   is omitted too, a univariate trigonometric polynomial is considered.
%
%   For matrix polynomials: 
%   [ POS, POSM, POSS ] = COEFPOS( N, DEG, NT, NR, K, I, J )
%
%   The significance of the first four inputs is kept. We the following
%   inputs:
%   
%   K is the size of the matrix KxK coefficients.
%   I,J describe the position of some scalar coefficients inside the matrix
%   coefficients. The length of I,J is the number of matrix coefficients.
%
%   POS is a vector with the scalar positions for all the matrix coefficients.
%   POSM is a vector with the positions of the matrix coefficients.
%   POSS is a vector with the liniar indexes for the scalar elements of the
%   matrix coefficients described by I,J.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 24.10.11-26.10.11

% number of degrees
ndeg  = size( deg, 1 );

% vector with positions
pos = zeros( ndeg, 1 );

if nargin > 2
    % check if 'nt' is empty
    ntE = isempty( nt );
    if nargin > 3
        % check if 'nr' is empty
        nrE = isempty( nr );
        if ntE && nrE
            % if 'nt' and 'nr' are empty
            % we are in the univariate trig. case
            nt = 1;
            nr = 0;
        elseif ntE && ~nrE
            % if only 'nt' is empty we have real pol.
            nt = 0;
        elseif ~ntE && nrE
            % if only 'nr' is empty we have trig. pol.
            nr = 0;
        end
    else
        if ntE
            % if 'nr' is missing and 'nt' empty
            % we are in the univariate trig. case
            nt = 1;
        end
    end
end

if nargin < 5
    % default, scalar case
    K = 1;
    if nargin < 4
        % default, trigonometric case
        nr = 0;
        if nargin < 3
            % default, univariate trig. case
            nt = 1;
        end
    end
end

% total number of coefficients
N = lenpol( n, 1, nt, nr );

if nt && nr
    % hybrid polynomial
    
    nTuple = ntupleh( n, N, nt );
elseif nt
    % trigonometric polynomial
    
    nTuple = ntuple( n, N, 1 );
else
    % real polynomial
    
    nTuple = ntuple( n, N, 0 );
end

% get the position of the coefficients
for k = 1 : ndeg
    pos( k ) = findvm( nTuple', deg( k, : ) );
end

if K > 1
    % position of matrix coefficients
    posM = pos;
    pos = [ ];
    
    if nargin > 5
        ii = i;
        jj = j;
        poss = zeros( ndeg, 1 );
    
        if nt && nr
            % hybrid polynomial
            
            hK = sumn( K );
            K2 = K ^ 2;
            
            for k = 1 : ndeg
                
                % start index
                Start = 0;
                
                for i = 1 : nC
                    
                    % if no more coef. than we have nothing to insert
                    if ~ncoef
                        break;
                    end
                    
                    % linear index in halfspace
                    idx = findvm( deg( k, : )', nTuple( i, : ) );
                    
                    % while we have coef. we insert them in 'fvec'
                    if iszero( nTuple( i, 1 : nt ) )
                        % (0,0,...,0) for the trig. degrees;
                        % symmetric coef.
                        
                        if ~isempty( idx )
                            % degree exists
                            
                            pos = [ pos; ( Start + 1 : Start + hK )' ];
                            poss( k ) = Start + ssub2ind( K, ii( k ), jj( k ) );
                            break;
                        end
                        Start = Start + hK;
                    else
                        % non-symmetric coef.
                        
                        if ~isempty( idx )
                            % degree exists
                            
                            pos = [ pos; ( Start + 1 : Start + K2 )' ];
                            poss( k ) = Start + sub2ind( [ K K ], ii( k ), jj( k ) );
                            break;
                        end
                        Start = Start + K2;
                    end
                end
            end
        elseif nt
            % trigonometric polynomial
            
            hK = sumn( K );
            K2 = K ^ 2;
            
            % for every matrix coefficient
            for i = 1 : ndeg
                % linear index in halfspace
                idx = gsub2ind( n, deg( i, : ), 1 );
                
                if idx > 1
                    Start = hK + ( idx - 2 ) * K2;
                    pos = [ pos; ( Start + 1 : Start + K2 )' ];
                    poss( i ) = Start + sub2ind( [ K K ], ii( i ), jj( i ) );
                else
                    % (0,0,...,0)
                    pos = [ pos; ( 1 : hK )' ];
                    poss( i ) = ssub2ind( K, ii( i ), jj( i ) );
                end
            end
        else
            % real polynomial
            
            hK = sumn( K );
            
            % for every nonzero matrix coefficient
            for i = 1 : ndeg
                % linear index in halfspace
                idx = gsub2ind( n, deg( i, : ), 0 );
                
                pos  = [ pos;  ( ( idx - 1 ) * hK + 1 : idx * hK )' ];
                poss( i ) = ( idx - 1 ) * hK + ssub2ind( K, ii( i ), jj( i ) );
            end
        end
    else
        % nargin <= 5
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        if nt && nr
            % hybrid polynomial
            
            hK = sumn( K );
            K2 = K ^ 2;
            
            for k = 1 : ndeg
                
                % start index
                Start = 0;
                
                for i = 1 : nC
                    
                    % if no more coef. than we have nothing to insert
                    if ~ncoef
                        break;
                    end
                    
                    % linear index in halfspace
                    idx = findvm( deg( k, : )', nTuple( i, : ) );
                    
                    % while we have coef. we insert them in 'fvec'
                    if iszero( nTuple( i, 1 : nt ) )
                        % (0,0,...,0) for the trig. degrees;
                        % symmetric coef.
                        
                        if ~isempty( idx )
                            % degree exists
                            
                            pos = [ pos; ( Start + 1 : Start + hK )' ];
                            break;
                        end
                        Start = Start + hK;
                    else
                        % non-symmetric coef.
                        
                        if ~isempty( idx )
                            % degree exists
                            
                            pos = [ pos; ( Start + 1 : Start + K2 )' ];
                            break;
                        end
                        Start = Start + K2;
                    end
                end
            end
        elseif nt
            % trigonometric polynomial
            
            hK = sumn( K );
            K2 = K ^ 2;
            
            % for every matrix coefficient
            for i = 1 : ndeg
                % linear index in halfspace
                idx = gsub2ind( n, deg( i, : ), 1 );
                
                if idx > 1
                    Start = hK + ( idx - 2 ) * K2;
                    pos = [ pos; ( Start + 1 : Start + K2 )' ];
                else
                    % (0,0,...,0)
                    pos = [ pos; ( 1 : hK )' ];
                end
            end
        else
            % real polynomial
            
            hK = sumn( K );
            
            % for every nonzero matrix coefficient
            for i = 1 : ndeg
                % linear index in halfspace
                idx = gsub2ind( n, deg( i, : ), 0 );
                
                pos = [ pos; ( ( idx - 1 ) * hK + 1 : idx * hK )' ];
            end
        end
    end
end