function [ varargout ] = freqz2d( B, A, Eps1, Eps2 )

% FREQZ2D 2-D Frequency response
%
%   [ HW, F1, F2, L1, L2 ] = FREQZ2D( B, A, EPS1, EPS2 ) returns
%   the frequency response HW of the function B/A with EPS1, EPS2
%   being the accuracy on the two axes. F1, F2 are the frequencies 
%   in 2-D and L1, L2 are the number of points considered for the 
%   frequencies. EPS1 and EPS2 are optional.
%
%   [ ... ] = FREQZ2D( B ) uses A as the identity polynomial.
%
%   FREQZ2D( ... ) plots the 2-D frequency response.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received A copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 20.02.02-08.03.10

% checks
if nargin < 3
  Eps1 = 1e-2;
  Eps2 = 1e-2;
  if nargin == 1
      A = 1;
  end
end

% frequencies
f1 = -1 : Eps1 : 1;
f2 = -1 : Eps2 : 1;

% j
SR = 1i;

% number of frequencies
L1 = length( f1 );
L2 = length( f2 );

Bw = polyval2d( B, exp( -SR * pi * f1 ), exp( -SR * pi * f2 ) );
Aw = polyval2d( A, exp( -SR * pi * f1 ), exp( -SR * pi * f2 ) );

% frequency response
Hw = Bw./Aw;

if nargout == 0
    % draw
    
    mesh( f1, f2, 20 * log10( abs( Hw ) ) );
    set( get( gca, 'XLabel' ), 'String', 'Frequency (\omega_1/\pi)' );
    set( get( gca, 'YLabel' ), 'String', 'Frequency (\omega_2/\pi)' );
    set( get( gca, 'ZLabel' ), 'String', 'Magnitude (dB)'           );
    return;
else
    % output parameters
    
    varargout = cell( nargout, 1 );
    
    switch nargout
        case 1
            varargout{ 1 } = Hw;
        case 2
            varargout{ 1 } = Hw;
            varargout{ 2 } = f1; 
        case 3
            varargout{ 1 } = Hw;
            varargout{ 2 } = f1;
            varargout{ 3 } = f2;
        case 4
            varargout{ 1 } = Hw;
            varargout{ 2 } = f1;
            varargout{ 3 } = f2;
            varargout{ 4 } = L1;
        case 5
            varargout{ 1 } = Hw;
            varargout{ 2 } = f1;
            varargout{ 3 } = f2;
            varargout{ 4 } = L1;
            varargout{ 5 } = L2;
        otherwise
            error( 'Wrong number of output arguments.' );
    end
end