function x = gsub2ind( n, s, isT )

% GSUB2IND General linear index from subscripts
%
%   This function returns the equivalent single index
%   corresponding to a given set of subscript values.
%
%   X = GSUB2IND( N, S, IST ) returns the index X which 
%   corresponds to the subscripts found in the vector S, for 
%   a multidimensional array of size N, where S belongs to a
%   set depending on IST.
%
%   If IST = 1, then the n-tuples correspond to the 
%   coefficients of a trig. pol.
%
%   If IST = 0, then the n-tuples correspond to the 
%   coefficients of a real pol. (The subscripts start from 0.)
%
%   Example:
%
%   x = gsub2ind( [ 1 2 1 ], [ -1 0 1 ], 1 );
%   disp( x );
%
%   See also gsub2indh, sub2ind, ind2sub, ntuple, nch.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.03.09-23.04.09

% number of dimensions
% for the array
N = length( n );

% check if the same number
% of elements
if length( s ) ~= N
    error( 'The number of dimensions must be equal for ''n'' and ''s''.' );
end

% check for valid subscripts
if nnz( ( n >= s ) - 1 ) ~= 0
    error( 'Out of range subscript.' );
end

if isT
    % trig. variables
    
    % find the last nonzero degree
    % for the vector s
    d = find( s, 1, 'last' );
    
    % check (k,0,...,0)
    if isempty( d ) || ( d == 1 )
        % x + 1
        x = s( 1 ) + 1;
    else
        % "add" the first d-1 dimensions
        x = nch( n( 1 : d - 1 ) );
        
        % "add" if s(d) > 1
        x = x + prod( 2 * n( 1 : d - 1 ) + 1 ) * ( s( d ) - 1 );
        
        % for all but the 1st dim.
        % add n-tuples
        while d > 2
            d = d - 1;
            x = x + prod( 2 * n( 1 : d - 1 ) + 1 ) * ( s( d ) + n( d ) );
        end
        
        % the n-tuples for the 1st dim.
        x = x + s( 1 ) + n( 1 ) + 1;
    end
else
    % real variables
    
    % init linear index with
    % first subscript
    x = s( 1 ) + 1;
    
    % product variable
    P = 1;
    % for the rest of the
    % N-1 dimensions
    for i = 2 : N
        P = P * ( n( i - 1 ) + 1 );
        x = x + s( i ) * P;
    end
end