function x = gsub2indh( n, s, nt )

% GSUB2INDH General linear index from subscripts
%
%   This function returns the equivalent single index
%   corresponding to a given set of subscript values of an
%   n-tuple describing a coefficient of a hybrid polynomial.
%
%   X = GSUB2INDH( N, S, NT ) returns the index X which 
%   corresponds to the subscripts found in the vector S, for 
%   a multidimensional array of size N. NT is the number of
%   trigonometric variables for the hybrid polynomial.
%
%   Example:
%
%   x = gsub2indh( [ 2 3 2 2 ], [ 0 3 2 2 ], 2 );
%   disp( x );
%
%   See also gsub2ind, sub2ind, ind2sub, ntuple, ntupleh, nch.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 23.04.09

% number of dimensions
% for the array
N = length( n );

% check if the same number
% of elements
if length( s ) ~= N
    error( 'The number of dimensions must be equal for ''n'' and ''s''.' );
end

% check for valid subscripts
if nnz( ( n >= s ) - 1 ) ~= 0
    error( 'Out of range subscript.' );
end

% trig. part in degree
nnt = n( 1 : nt );
% real part in degree
nnr = n( nt + 1 : end );

% trig. part in subscripts
st = s( 1 : nt );
% real part in subscripts
sr = s( nt + 1 : end );

% linear index for the real part
xr = gsub2ind( nnr, sr, 0 );

% linear index
x = nch( nnt ) * ( xr - 1 ) + gsub2ind( nnt, st, 1 );