function [ H ] = hankels( N )

% HANKELS Hankel matrices
%
%   [ H ] = HANKELS( N ) returns 2*N+1 Hankel matrices of
%   size (N+1)x(N+1).
%
%   Each k-th matrix, k=0:2*N, has ones on the k-th anti-
%   diagonal and zeros elsewhere and will lie in matrix H
%   at H(k*(N+1)+1:(k+1)*(N+1),:).
%
%   See also toeplitzs.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 18.10.08-6.11.08

% CHECKS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check if N is numeric
if ~isnumeric( N )
    error( 'N must be numeric.' );
end

% check for NaN and Inf
if isnan( N ) || isinf( N )
    error( 'NaN and Inf not allowed.' );
end

% check if imaginary part is ~= 0
if imag( N )
    error( 'N''s imaginary part must be 0. ' );
end

% check if N is nonnegative
if N < 0
    error( 'N must be nonnegative.' );
end

% check if N is an integer number
fN = fix( N );
if fN ~= N
    warning( 'MATLAB:SeDuMiPol:NotIntegerNum', ...
             'N should be an integer number; we continue with fix(N).' );
    N = fN;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% number of matrices
nH = 2 * N + 1;
% size of matrices
sH = N + 1;

% matrix for Hankel matrices
H = sparse( nH * sH, sH );

% diagonal for Hankel
% matrices
d = ones( sH, 1 );

% fill 2*N+1 Hankel
% matrices
for k = N : -1 : -N
    H( ( k + N ) * sH + 1 : ( k + sH ) * sH, : ) = fliplr( spdiags( d, -k, sH, sH ) );
end