function Y = mextend( type, mode, X, l, loc )

% MEXTEND Matrix extend
%
%   Y = MEXTEND( TYPE, MODE, X, L, LOC ) extends the matrix X
%   to the matrix Y with zero padding. The function is similar
%   to the 'wextend' function from Matlab, Wavelet Toolbox, for
%   the case MODE='zpd'.
%
%   TYPE denotes the type of extension:
%   1, '1', '1d' or '1D'  :  1-D extension  
%   2, '2', '2d' or '2D'  :  2-D extension
%   'ar' or 'addrow'      :  add rows
%   'ac' or 'addcol'      :  add columns
%
%   MODE can *only* be 'zpd' (zero padding).
%
%   For TYPE={1,'1','1d' or '1D'}
%   LOC *must* be one of the following:
%   'l'/'u' - left/up extension
%   'r'/'d' - right/down extension
%   'b'     - extension on both sides
%   'n'     - no extension
%   L is the length of the extension.
%
%   For TYPE={2, '2', '2d' or '2D'}
%   LOC *must* be of the form [ c1 c2 ], where 
%   {c1, c2} in { 'l', 'r', 'u', 'd', 'b', 'n' }.
%   L(1) and L(2) are the lengths of the extension for
%   each dimension. If L is scalar, then L <- [ L L ].
%
%   For TYPE={'ar' or 'addrow'} / {'ac' or 'addcol'}
%   LOC *must* be one of the following:
%   'l'/'u' - left/up extension
%   'r'/'d' - right/down extension
%   'b'     - extension on both sides
%   'n'     - no extension
%   L is the number of rows / columns to add.
%
%   There are no defaults; all input parameters must be specified.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 31.03.10-03.05.10

% 'mode' must be 'zpd'
assert( strcmp( mode, 'zpd' ), 'MODE must be ''zpd''.' );

% matrix dimensions
[ m, n ] = size( X );

switch type
    case { 1, '1', '1d', '1D' }
        switch loc
            case { 'l', 'u' }
                Y = [ sparse( l, 1 ); X( : ) ];
            case { 'r', 'd' }
                Y = [ X( : ); sparse( l, 1 ) ];
            case 'b'
                O = sparse( l, 1 );
                Y = [ O; X( : ); O ];
            case 'n'
                Y = X;
            otherwise
                error( 'Wrong LOC argument.' );
        end
    case { 2, '2', '2d', '2D' }
        len = length( l );
        if len == 1
            l = [ l l ];
        elseif len ~= 2
            error( 'L must be of length one or two.' );
        end
        
        len = length( loc );
        if len == 1
            loc = [ loc loc ];
        elseif len ~= 2
            error( 'LOC must be of length one or two.' );
        end
        
        switch loc
            case { [ 'l' 'l' ], [ 'u' 'l' ], [ 'l' 'u' ], [ 'u' 'u' ] }
                O1 = sparse( l( 1 ), l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [    O1  ; ...
                      O2 X  ];
            case { [ 'l' 'r' ], [ 'l' 'd' ], [ 'u' 'r' ], [ 'u' 'd' ] }
                O1 = sparse( l( 1 ), l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [ O1     ; ...
                      X  O2 ];
            case { [ 'r' 'l' ], [ 'r' 'u' ], [ 'd' 'l' ], [ 'd' 'u' ] }
                O1 = sparse( l( 1 ), l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [ O2 X   ; ...
                        O1  ];
            case { [ 'r' 'r' ], [ 'r' 'd' ], [ 'd' 'r' ], [ 'd' 'd' ] }
                O1 = sparse( l( 1 ),l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [ X  O2  ; ...
                      O1    ];
            case { [ 'b' 'l' ], [ 'b' 'u' ] }
                O1 = sparse( l( 1 ), 2 * l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [    O1 ; ...
                      O2  X ; ...
                         O1   ];
            case { [ 'b' 'd' ], [ 'b' 'r' ] }
                O1 = sparse( l( 1 ), 2 * l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [  O1    ; ...
                       X  O2 ; ...
                       O1      ];
            case { [ 'l' 'b' ], [ 'u' 'b' ] }
                O1 = sparse( l( 1 ), 2 * l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [    O1    ; ...
                      O2 X  O2 ; ];
            case { [ 'r' 'b' ], [ 'd' 'b' ] }
                O1 = sparse( l( 1 ), 2 * l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [ O2 X  O2 ; ...
                         O1      ];
            case [ 'b' 'b' ]
                O1 = sparse( l( 1 ), 2 * l( 2 ) + n );
                O2 = sparse( m, l( 2 ) );
                Y = [    O1    ; ...
                      O2 X  O2 ; ...
                         O1      ];
            case { [ 'l' 'n' ], [ 'u' 'n' ] }
                O1 = sparse( l( 1 ), n );
                Y = [ O1   ;...
                      X   ];
            case { [ 'r' 'n' ], [ 'd' 'n' ] }
                O1 = sparse( l( 1 ), n );
                Y = [ X   ;...
                      O1 ];
            case { [ 'n' 'l' ], [ 'n' 'u' ] }
                O2 = sparse( m, l( 2 ) );
                Y = [ O2 X ];
            case { [ 'n' 'r' ], [ 'n' 'd' ] }
                O2 = sparse( m, l( 2 ) );
                Y = [ X O2 ];
            case [ 'n' 'n' ]
                Y = X;
            otherwise
                error( 'Wrong LOC argument.' );
        end
    case { 'ar', 'addrow' }
        switch loc
            case { 'l', 'u' }
                Y = [ sparse( l, n ); X ];
            case { 'r', 'd' }
                Y = [ X; sparse( l, n ) ];
            case 'b'
                O = sparse( l, n );
                Y = [ O; X; O ];
            case 'n'
                Y = X;
            otherwise
                error( 'Wrong LOC argument.' );
        end
    case { 'ac', 'addcol' }
        switch loc
            case { 'l', 'u' }
                Y = [ sparse( m, l ) X ];
            case { 'r', 'd' }
                Y = [ X sparse( m, l ) ];
            case 'b'
                O = sparse( m, l );
                Y = [ O X O ];
            case 'n'
                Y = X;
            otherwise
                error( 'Wrong LOC argument.' );
        end
    otherwise
        error( 'Wrong TYPE argument' );
end