function P = polconv2d( X, dX, Y, dY, KX, KY, Xvar )

% POLCONV2D 2-D Polynomial Convolution
%   
%   This function does the multiplication of causal multidimensional 
%   polynomials with matrix coefficients, represented in vector form.
%
%   Input:
%       X, Y    - vectors representing the polynomials
%       dX, dY  - degrees of the polynomials (vectors)
%       KX      - size of matrix coefficients for X (default = 1)
%       KY      - size of matrix coefficients for Y (default = KX)
%       Xvar    - if is 0, the variable in a possible CVX call is Y, 
%                 otherwise is X (this variable is not updated in the 
%                 function)
%   Output:
%       P      - the polynomial X*Y (of degree dX+dY)

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 14.01.09-08.06.10

% check number of variables of the polynomials 
% to be equal
nvar = length( dX );
if nvar ~= length( dY )
    error( 'Polynomials have different number of variables.' );
end

% check arguments
if nargin < 5
    KX = 1;
end
if nargin < 6
    KY = KX;
end
if nargin < 7
    Xvar = 0;
end

% make the size variable a vector
if length( KX ) == 1
    KX = [ KX KX ];
end
if length( KY ) == 1
    KY = [ KY KY ];
end

% number of elements in a matrix coef.
K2X = KX( 1 ) * KX( 2 );   
K2Y = KY( 1 ) * KY( 2 );
K2P = KX( 1 ) * KY( 2 );

dX = dX( : )';
dY = dY( : )';
dP = dX + dY; % degree of the result P

% size
sX = dX + 1;  
sY = dY + 1;
sP = dP + 1;

nX = prod( sX );  % number of coefficients
if length( X ) ~= nX * K2X
    error( 'Length of X does not correspond to given dX' );
end
nY = prod( sY );
if length(Y) ~= nY * K2Y
    error( 'Length of Y does not correspond to given dY' );
end
nP = prod( sP );

% Build convolution matrix, then multiply
if Xvar       % if X is the variable, build convolution matrix from Y
    C = sparse( nP * K2P, nX * K2X );
    I = eye( KX( 1 ) );
    for ix = 1 : nX
        for iy = 1 : nY
            eX = rind2sub( ix, dX );
            eY = rind2sub( iy, dY );
            ip = exp2ind_nd(eX + eY + 1, dP);
            C( ( ip - 1 ) * K2P + 1 : ip * K2P, ( ix - 1 ) * K2X + 1 : ix * K2X ) = ...
                 kron( reshape( Y( ( iy - 1 ) * K2Y + 1 : iy * K2Y ), KY( 1 ), KY( 2 ) )', I );
        end
    end
    P = C * X;
else          % Y is the variable, build convolution matrix from X
    C = sparse( nP * K2P, nY * K2Y );
    I = eye( KY( 2 ) );
    for ix = 1 : nX
        for iy = 1 : nY
            eX = rind2sub( ix, dX );
            eY = rind2sub( iy, dY );
            ip = exp2ind_nd(eX + eY + 1, dP);
            C( ( ip - 1 ) * K2P + 1 : ip * K2P, ( iy - 1 ) * K2Y + 1 : iy * K2Y ) = ...
                 kron( I , reshape( X( ( ix - 1 ) * K2X + 1 : ix * K2X ), KX( 1 ), KX( 2 ) ) );
        end
    end
    P = C * Y;
end

function k = exp2ind_nd( vexp, ndeg )

% Transforms the exponent 'vexp' into an index,
% for a polynomial of degree 'ndeg'.

% Date: 11.12.2003

Cp = cumprod( ndeg + 1 );
Cp = [ 1 Cp( 1 : end - 1 ) ];
k = Cp * ( vexp( : ) - 1 ) + 1;  % index starts from 1