function H = polyval3d( A, x1, x2, x3 )

% POLYVAL3D Value of 3-D polynomial
%
%   H = POLYVAL3D( A, X1, X2, X3 ) returns in the matrix H
%   the value of a 3-D polinomial retained in the 3-D array
%   A, in the triplets defined by the three vectors X1, X2, X3.
%
%   Each matrix A(:,:,i) hold the coefficients for the all the 
%   degrees of the 1st and 2nd variable with i-1 the degree for 
%   the 3rd variable.
%
%   H will be of size length( X1 ) x length( X2 ) * length( X3 ).
%
%   Example:
%
%   >> A( :, :, 1 ) = [ 3 2; 2  2 ];
%   >> A( :, :, 2 ) = [ 1 2; 3 -1 ];
%   >> A( :, :, 3 ) = [ 5 0; 7  0 ];
%   >> x1 = [  3 1 ];
%   >> x2 = [ -1 2 ];
%   >> x3 = [  1 5 2 ];
%
%   >> H = polyval3d( A, x1, x2, x3 );
%   >> H
%
%      38    59   706   715   127   145
%      16    31   316   343    55    73

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date:   29.05.09-02.06.09

% sizes
[ m, n, p ] = size( A );

% column vectors
x1 = x1( : );
x2 = x2( : );
x3 = x3( : );

% length of the vectors
Len1 = length( x1 );
Len2 = length( x2 );
Len3 = length( x3 );

% create matrices with degrees
X1 = deg_var( x1, Len1, m );
X2 = deg_var( x2, Len2, n );
X3 = deg_var( x3, Len3, p );

% init matrix with all the degrees and
% values for the 1st and 2nd variable
Y = zeros( Len1, Len2 * p );

% values for 1st and 2nd variable for all
% the degrees of the 3rd variable
for i = 1 : p
    Y( :, 1 + ( i - 1 ) * Len2 : i * Len2 ) = X1.' * A( :, :, i ) * X2;
end

% Z is the matrix for the values
% of the third variable;
% the structure of Z:
%
% 1 0 0 1 0 0
% 0 1 0 0 1 0
% 0 0 1 0 0 1
% a 0 0 b 0 0 ... Len3 blocks
% 0 a 0 0 b 0
% 0 0 a 0 0 b
%  2     2
% a  2  b  2
%   a  2  b  2
%     a     b
% .
% .
% . p blocks

Z = kron( X3, sparse( eye( Len2 ) ) );

% final matrix
% with values
H = Y * Z;

function X = deg_var( x, Len, Siz )

% This function creates a matrix with all the
% degrees for a vector of values.

% The function creates a matrix of the form:
%
% 1 1 1 1 1
% a b c d e 
%  2 2 2 2 2
% a b c d e  ... LEN times
%  3 3 3 3 3
% a b c d e
% .
% .
% . SIZ times
%
% a,b,c,d,e, etc. belong to the vector 'x'.

X = repmat( x.', Siz, 1 );
X( 1, : ) = ones( 1, Len );
X = cumprod( X );