function [ deg, coef ] = prod_poly( deg1, deg2, coef1, coef2, type, ke, nt )

% PROD_POLY Polynomial product
%
%   This function computes the multivariate product of two 
%   real/trigonometric/hybrid polynomials.
%
%   [ DEG, COEF ] = PROD_POLY( DEG1, DEG2, COEF1, COEF2, TYPE, KE, NT )
%
%   INPUTS:
%
%   DEG1  - A matrix containing on its rows the degrees of the
%           nonzero coefficients of the 1st polynomial.
%
%   DEG2  - A matrix containing on its rows the degrees of the
%           nonzero coefficients of the 2nd polynomial.
%
%   COEF1 - A vector of cells containing the nonzero scalar/matrix
%           coefficients of the 1st polynomial.
%
%   COEF2 - A vector of cells containing the nonzero scalar/matrix
%           coefficients of the 2nd polynomial.
%
%   TYPE  - A variable denoting the type of the polynomials.
%
%           If TYPE=0, then the polynomials are real.
%           If TYPE=1, then the polynomials are trigonometric.
%           If TYPE=2, then the polynomials are hybrid.
%
%   KE    - Keep or remove zero coefficients from polynomial.
%
%           If KE=0, then the zero coefficients are removed.
%           If KE=1, then the zero coefficients are kept.
%
%           The use of the variable is optional. By default, the
%           zero coefficients are removed.
%
%   NT    - A variable denoting the number of trigonometric 
%           variables when the polynomials are hybrid. The variable
%           is used only for hybrid polynomials, otherwise is not
%           required.
%
%   OUTPUTS:
%
%   DEG   - A matrix containing on its rows the degrees of the
%           nonzero coefficients of the resultant polynomial.
%
%   COEF  - A vector of cells containing the nonzero scalar/matrix
%           coefficients of the resultant polynomial.
%
%   Example:
%
%   H1 = t + z1^{-1}*t + z2^{-1}*t + 2 + z2^{-1} + sym^{-1}
%   H2 = z1^{-1}z2^{-2} + z1^{-2}z2^{-1}t + sym^{-1}
%
%   H1 and H2 are hybrid polynomials with two trig. variables and
%   one real variable. sym^{-1} is the "symmetric" part, the
%   coefficients from the other halfspace of the hybrid polynomial.
%
%   deg1  = [ 0 0 1; ...
%             1 0 1; ...
%             0 1 1; ...
%             0 0 0; ...
%             0 1 0; ];
%   deg2  = [ 1 2 0; ...
%             2 1 1; ];
%   coef1 = { 1, 1, 1, 2, 1 }; 
%   coef2 = { 1, 1 }; 
%   [ deg, coef ] = prod_poly( deg1, deg2, coef1, coef2, 2, 0, 2 );
%
%   See also sum_poly.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 14.05.09-27.09.09

% size for scalar/matrix
K = size( coef1{ 1 }, 1 );
% number of variables
d = size( deg1, 2 );

% number of coef. 
nC1 = length( coef1 );
nC2 = length( coef2 );

% max degree for each of
% the polynomials
n1 = max( abs( deg1 ) );
n2 = max( abs( deg2 ) );

% max. deg. for the
% final polynomial
n = n1 + n2;

% zero matrix
O = sparse( K, K );

% compute product according to 
% the type of polynomials
switch type
    case 0
        % real polynomials
        
        % number of coef.
        nC = prod( n + 1 );
        
        nTuple = ntuple( n, nC, 0 );
        
        % coefficients of the
        % temporar polynomial;
        % (final pol. with zeros)
        coefT = cell( nC, 1 );
        % init with zero (scalar/matrix)
        for i = 1 : nC
            coefT{ i } = O;
        end
        
        % compute product
        for i = 1 : nC1
            for j = 1 : nC2
                idx = gsub2ind( n, deg1( i, : ) + deg2( j, : ), type );
                coefT{ idx } = coefT{ idx } + coef1{ i } * coef2{ j };
            end
        end
        
        if nargin > 5
            if ~ke
                % retain only nonzero coefficients
                [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
            else
                deg  = nTuple;
                coef = coefT;
            end
        else
            % by default, retain only nonzero coefficients
            [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
        end        
    case 1
        % trigonometric polynomials
        
        % number of coef.
        nC = nch( n );
        
        nTuple = ntuple( n, nC, 1 );
        
        % coefficients of the
        % temporar polynomial;
        % (final pol. with zeros)
        coefT = cell( nC, 1 );
        % init with zero (scalar/matrix)
        for i = 1 : nC
            coefT{ i } = O;
        end
        
        % add "symmetric degrees"
        [ deg1S, coefS1, nCS1 ] = add_symm_part( deg1, coef1, nC1, d );
        [ deg2S, coefS2, nCS2 ] = add_symm_part( deg2, coef2, nC2, d );
        
        % compute product
        for i = 1 : nCS1
            for j = 1 : nCS2
                % temporary degree
                degT = deg1S( i, : ) + deg2S( j, : );
                
                if isinhalf( degT )
                    % resulting degree is in the halfspace,
                    % so we compute de product of the coefficients
                    
                    idx = gsub2ind( n, degT, type );
                    coefT{ idx } = coefT{ idx } + coefS1{ i } * coefS2{ j };
                end
            end
        end
        
        if nargin > 5
            if ~ke
                % retain only nonzero coefficients
                [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
            else
                deg  = nTuple;
                coef = coefT;
            end
        else
            % by default, retain only nonzero coefficients
            [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
        end
    case 2
        % hybrid polynomials
        
        % number of coef.
        nC = nchp( n, nt );
        
        nTuple = ntupleh( n, nC, nt );
        
        % coefficients of the
        % temporar polynomial;
        % (final pol. with zeros)
        coefT = cell( nC, 1 );
        % init with zero (scalar/matrix)
        for i = 1 : nC
            coefT{ i } = O;
        end
        
        % add "symmetric hybrid degrees"
        [ deg1S, coefS1, nCS1 ] = add_symm_part_h( deg1, coef1, nC1, d, nt );
        [ deg2S, coefS2, nCS2 ] = add_symm_part_h( deg2, coef2, nC2, d, nt );
        
        % compute product
        for i = 1 : nCS1
            for j = 1 : nCS2
                % temporary degree
                degT = deg1S( i, : ) + deg2S( j, : );
                
                if isinhalf( degT( 1 : nt ) )
                    % resulting degree for the trig. part is in the halfspace,
                    % so we compute de product of the coefficients
                    
                    idx = gsub2indh( n, degT, nt );
                    coefT{ idx } = coefT{ idx } + coefS1{ i } * coefS2{ j };
                end
            end
        end
        
        if nargin > 5
            if ~ke
                % retain only nonzero coefficients
                [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
            else
                deg  = nTuple;
                coef = coefT;
            end
        else
            % by default, retain only nonzero coefficients
            [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
        end
    otherwise
        error( 'Invalid value for the variable ''type''.' );
end

function [ degS, coefS, nCS ] = add_symm_part( deg, coef, nC, d )

% This function adds the symmetric degrees and
% coefficients for the trigonometric polynomial.

% zero degree (0,...,0)
o = sparse( d, 1 );

% find index for zero degree
i = findvm( deg', o );
if isempty( i )
    % no zero degree
    
    % add "symmetric" degrees
    degS = [ -deg ; ...
              deg   ];
    
    % total number of coef.
    % for the symm. pol.
    nCS = 2 * nC;
    
    % extend coef. to have
    % the "symmetric" ones too;
    coefS = cell( nCS, 1 );
    % new coef.
    [ coefS ] = conj_cells( coefS, coef, 1 : nC, 1 : nC );
    % old coef.
    coefS( nC + 1 : end ) = coef;
else
    % with zero degree
    
    % copy deg
    degS = deg;
    % check if the zero degree
    % is the 1st one
    if i ~= 1
        % the zero degree is not the 1st one,
        % so we put it 1st
        
        % change deg. 1 with deg. i
        [ degS( 1, : ), degS( i, : ) ] = deal( degS( i, : ), degS( 1, : ) );

        % put the first coef. the one 
        % for the zero degree
        [ coef{ 1 }, coef{ i } ] = deal( coef{ i }, coef{ 1 } );
    end
    
    % add "symmetric" degrees
    degS = [ -degS( 2 : end, : ) ; ...
                   degS            ];
                   
    % total number of coef.
    % for the symm. pol.
    nCS = 2 * nC - 1;
    
    % extend coef. to have
    % the "symmetric" ones too;
    coefS = cell( nCS, 1 );
    % new coef.
    [ coefS ] = conj_cells( coefS, coef, 1 : nC - 1, 2 : nC );
    % old coef.
    coefS( nC : end ) = coef;
end

function [ degS, coefS, nCS ] = add_symm_part_h( deg, coef, nC, d, nt )

% This function adds the symmetric degrees and
% coefficients for the hybrid polynomial.

% zero degree (0,...,0)
o = sparse( d, 1 );

% find index for zero degree
i = findvm( deg', o );
if isempty( i )
    % no zero degree
    
    % add "symmetric" degrees and coefficients
    [ degS, coefS, nCS ] = add_symm_deg_coef_h( deg, coef, nC, d, nt, 0 );
else
    % with zero degree
    
    % check if the zero degree
    % is the 1st one
    if i ~= 1
        % no
        
        % put the zero degree first
        [ deg( 1, : ), deg( i, : ) ] = deal( deg( i, : ), deg( 1, : ) );
              
        % put the first coef. the one
        % for the zero degree
        [ coef{ 1 }, coef{ i } ] = deal( coef{ i }, coef{ 1 } );
        
        % add "symmetric" degrees and coefficients
        [ degS, coefS, nCS ] = add_symm_deg_coef_h( deg, coef, nC, d, nt, 1 );
    else
        % yes
        
        % add "symmetric" degrees and coefficients
        [ degS, coefS, nCS ] = add_symm_deg_coef_h( deg, coef, nC, d, nt, 1 );
    end
end

function [ degS, coefS, nCS ] = add_symm_deg_coef_h( deg, coef, nC, d, nt, Offset )

% This function adds the "symmetric" degrees and coefficients
% for the hybrid polynomial. The OFFSET variable is used for 
% the zero degree. 

% OFFSET = 1, if the zero degree exists.
% OFFSET = 0, if the zero degree does not exist.

% zero degree for trig. part
ont = sparse( nt, 1 );

% indexes for the n-tuples with zero
% degree for the trig. part
idxZ = findvm( deg( 1 + Offset : end, 1 : nt )', ont ) + Offset;
idxZLen = length( idxZ );
% number of degrees and coef. to be added
symmLen = nC - idxZLen - Offset;

% total number of coef.
% for the symm. pol.
nCS = nC + symmLen;
degS = zeros( nCS, d );
% old+new coefficients
coefS = cell( nCS, 1 );

% old degrees and coefficients
degS ( symmLen + 1 : end, : ) = deg;
coefS( symmLen + 1 : end )    = coef;

% add "symmetric" degrees and coef.
symmCnt = 1;

for i = 1 + Offset : nC
    if ~isinvec( idxZ, i )
        % the degree does not have zero degree
        % for the trig. part
        
        degS ( symmCnt, : ) = [ -deg( i, 1 : nt ) deg( i, nt + 1 : end ) ];
        coefS{ symmCnt    } =  coef{ i }';
        symmCnt = symmCnt + 1;
    end
end

function [ coef1 ] = conj_cells( coef1, coef2, idx1, idx2 )

% This function sets the cells of COEF1 with indexes in IDX1
% to be the conjugate cells of COEF2 with indexes in IDX2.

Len1 = length( idx1 );

if Len1 ~= length( idx2 )
    error( 'Number of indexes must be the same.' );
end

% conjugate
for i = 1 : Len1
    coef1{ idx1( i ) } = coef2{ idx2( i ) }';
end

function [ i ] = isinvec( v, a )

% This  function tests if the value of A is among
% the elements of the vector V.

vLen = length( v );

i = cf( nnz( reshape( v, vLen, 1 ) - repmat( a, vLen, 1 ) ) == vLen, 0, 1 );

function [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O )

% This function retains only the nonzero coefficients
% and their degrees.

% nC final; we retain the
% the nonzero coef.
nCFin = 0;
% indexes where we have
% nonzero coefficients
idxFin  = zeros( nC, 1 );
for i = 1 : nC
    if ~eqm( coefT{ i }, O )
        % nonzero coefficient
        
        nCFin = nCFin + 1;
        idxFin( nCFin ) = i;
    end
end

% retain nonzero coefficients
% and their degrees
coef = cell( nCFin, 1 );
deg  = zeros( nCFin, d );
for i = 1 : nCFin
    coef{ i }   = coefT{ idxFin( i ) };
    deg( i, : ) = nTuple( idxFin( i ), : );
end