function fvec = spcausal2fvec( n, deg, coef, K )

% SPCAUSAL2FVEC Sparse causal polynomial to full vector
%
%   This function builds an associated vector for causal polynomials.
%
%   FVEC = SPPOL2FVEC( N, DEG, COEF, K )
%
%   INPUTS:
%
%   N    - A vector with the degrees of the polynomial.
%   DEG  - DEG is a matrix whose rows are the n-tuples which represent
%          the nonzero coefficients of the polynomial.
%          Ex: DEG( 5, : ) = [ 4 2 4 ] - the 5th coefficient has the
%          triplet [ 4 2 4 ].
%   COEF - COEF is a vector of cells, holding the coefficients of the 
%          polynomial, scalars or matrices.
%          Ex: COEF{ 3 } = [ 2 1; 0 1 ] - the 3rd coefficient of the 
%          polynomial is the matrix [ 2 1; 0 1 ].
%   K    - K = [ K1 K2 ] is the size of the coefficients. If K is scalar
%          then K1 = K2 = K.
%
%   OUTPUT:
%
%   FVEC  - The 'fvec' term (the full vector)
%
%   See also sppol2fvec, sppols2fvec.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.03.09-27.09.10

% number of n-tuples
ndeg  = size( deg, 1 );
% number of coefficients
ncoef = length( coef );

if ndeg ~= ncoef
    error( 'The number of n-tuples and coefficients must be equal.' );
end

% number of coefficients
nC = prod( n + 1 );

% scalar to vector
if length( K ) == 1
    K = [ K K ];
end

if K == 1
    % scalar coefficients
    
    % init fvec
    fvec = sparse( nC, 1 );
    
    % for every nonzero coefficient
    for i = 1 : ndeg
        fvec( gsub2ind( n, deg( i, : ), 0 ) ) = coef{ i };
    end
else
    % matrix coefficients
    
    % "half" of the
    % matrix
    KK = prod( K );
    
    % init fvec
    fvec = sparse( nC * KK, 1 );
    
    % for every nonzero matrix coefficient
    for i = 1 : ndeg
        % linear index in halfspace
        idx = gsub2ind( n, deg( i, : ), 0 );
        
        fvec( ( idx - 1 ) * KK + 1 : idx * KK ) = vec( coef{ i } );
    end
end