function fvec = sppol2fvec( n, deg, coef, nt, nr )

% SPPOL2FVEC Sparse polynomial to full vector
%
%   This function builds an associated vector for a single polynomial.
%   Practically it can be used to generate at least a part of the free
%   term (bsP).
%
%   FVEC = SPPOL2FVEC( N, DEG, COEF, K, NT, NR )
%
%   INPUTS:
%
%   N    - A vector with the degree of the polynomial.
%   DEG  - A matrix whose rows are the n-tuples which represent the 
%          nonzero coefficients of the polynomial.
%          Ex: DEG( 5, : ) = [ 4 2 4 ] - the 5th coefficient has the
%          triplet [ 4 2 4 ]. It is associated with COEF{ 5 }.
%   COEF - A vector of cells, holding the coefficients of the polynomial, 
%          scalars or matrices.
%          Ex: COEF{ 3 } = [ 2 1; 0 1 ] - the 3rd coefficient of the 
%          polynomial is the matrix [ 2 1; 0 1 ]. It associated with
%          DEG( 3, : ).
%   NT   - The number of trigonometric variables. If the parameter is 
%          missing NT=1.
%   NR   - The number of real variables. If the parameter is missing NR=0.
%
%   OUTPUT:
%
%   FVEC  - The 'fvec' term (the full vector)
%
%   See also sppols2fvec, spcausal2fvec.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.03.09-28.10.11

% number of n-tuples
ndeg  = size( deg, 1 );
% number of coefficients
ncoef = length( coef );

if ndeg ~= ncoef
    error( 'The number of n-tuples and coefficients must be equal.' );
end

% set defaults
if nargin < 5
    % default trig. pol.
    nr = 0;
    if nargin < 4
        % default univariate trig. pol.
        nt = 1;
    else
        if isempty( nt )
            nt = 1;
        end
    end
else
    if isempty( nr )
        nr = 0;
    end
end

K = length( coef{ 1 } );
% check coefficients size
for i = 2 : length( coef )
    if K ~= length( coef{ i } )
        error( 'The coefficients have different sizes.' );
    end
end

if nt && nr
    % hybrid polynomial
    
    % number of coefficients
    nC = nchp( n, nt );
    
    if K == 1
        % scalar coefficients
        
        % init fvec
        fvec = sparse( nC, 1 );
        
        % for every nonzero coefficient
        for i = 1 : ndeg
            fvec( gsub2indh( n, deg( i, : ), nt ) ) = coef{ i };
        end
    else
        % matrix coefficients
        
        % number of elem.
        % in matrix
        K2 = K ^ 2;
        
        % "half" of the
        % matrix
        hK = sumn( K );
        
        % number of coef. for the real part
        Nr = prod( nonzeros( n( nt + 1 : end ) ) + 1 );
        
        % init fvec
        fvec = sparse( ( nC - Nr ) * K2 + Nr * hK, 1 );
        
        % all the hybrid n-tuples
        nTuple = ntupleh( n, nC, nt );
        
        % start index
        Start = 0;
        
        % for every coefficient
        for i = 1 : nC
            
            % if no more coef. than we have nothing to insert
            if ~ncoef
                break;
            end
            
            % linear index in halfspace
            idx = findvm( deg', nTuple( i, : ) );
            
            % while we have coef. we insert them in 'fvec'
            if iszero( nTuple( i, 1 : nt ) )
                % (0,0,...,0) for the trig. degrees;
                % symmetric coef.
                
                if ~isempty( idx )
                    % degree exists
                    fvec( Start + 1 : Start + hK ) = vecs( coef{ idx } );
                    ncoef = ncoef - 1;
                end
                Start = Start + hK;
            else
                % non-symmetric coef.
                
                if ~isempty( idx )
                    % degree exists
                    fvec( Start + 1 : Start + K2 ) = vec( coef{ idx } );
                    ncoef = ncoef - 1;
                end
                Start = Start + K2;
            end
            
        end
    end
else
    if nt
        % trigonometric polynomial
        
        % number of coefficients
        nC = nch( n );
        
        if K == 1
            % scalar coefficients
            
            % init fvec
            fvec = sparse( nC, 1 );
            
            % for every nonzero coefficient
            for i = 1 : ndeg
                fvec( gsub2ind( n, deg( i, : ), 1 ) ) = coef{ i };
            end
        else
            % matrix coefficients
            
            % number of elem.
            % in matrix
            K2 = K ^ 2;
            
            % "half" of the
            % matrix
            hK = sumn( K );
            
            % init fvec
            fvec = sparse( hK + ( nC - 1 ) * K2, 1 );
            
            % for every nonzero matrix coefficient
            for i = 1 : ndeg
                % linear index in halfspace
                idx = gsub2ind( n, deg( i, : ), 1 );
                
                if idx > 1
                    Start = hK + ( idx - 2 ) * K2;
                    fvec( Start + 1 : Start + K2 ) = vec( coef{ i } );
                else
                    % (0,0,...,0)
                    fvec( 1 : hK ) = vecs( coef{ i } );
                end
            end
        end
    else
        % real polynomial
        
        % number of coefficients
        nC = prod( n + 1 );
        
        if K == 1
            % scalar coefficients
            
            % init fvec
            fvec = sparse( nC, 1 );
            
            % for every nonzero coefficient
            for i = 1 : ndeg
                fvec( gsub2ind( n, deg( i, : ), 0 ) ) = coef{ i };
            end
        else
            % matrix coefficients
            
            % "half" of the
            % matrix
            hK = sumn( K );
            
            % init fvec
            fvec = sparse( nC * hK, 1 );
            
            % for every nonzero matrix coefficient
            for i = 1 : ndeg
                % linear index in halfspace
                idx = gsub2ind( n, deg( i, : ), 0 );
                
                fvec( ( idx - 1 ) * hK + 1 : idx * hK ) = vecs( coef{ i } );
            end
        end
    end
end    