function spvec = sppol2spvec( n, ptype, deg, coef, K, isT, isR, isSB )

% SPPOL2SPVEC Sparse polynomial to sparse vector
%
%   This function has the role of 'sppol2fvec' for sparse polynomials.
%
%   SPVEC = SPPOL2SPVEC( N, PTYPE, DEG, COEF, K, IST, ISR, ISSB )
%
%   INPUTS:
%
%   N     - A vector which holds on its rows the degrees of the polynomial.
%   PTYPE - The 'ptype' term for the polynomial.
%   DEG   - DEG is a matrix whose rows are the n-tuples which represent the 
%           nonzero coefficients of the polynomial.
%           Ex: DEG( 5, : ) = [ 4 2 4 ] - the 5th coefficient has the
%           triplet [ 4 2 4 ].
%   COEF -  COEF is a vector of cells, holding the coefficients of the polynomial, 
%           scalars or matrices.
%           Ex: COEF{ 3 } = [ 2 1; 0 1 ] - the 3rd coefficient of the
%           polynomial is the matrix [ 2 1; 0 1 ].
%   K    -  A vector containing the size of the coeffcients for every
%           polynomial.
%   IST  -  Describes the polynomials of having or not trigonometric variables.
%   ISR  -  Describes the polynomials of having or not real variables.
%   ISSB -  The ISSB term. Describes how the sparse basis is enforced. If ISSB
%           is 1 then the user uses the field 'ind' to enforce the sparse
%           basis. If ISSB is 2 then the user uses the field 'deg' to set
%           the sparse basis.
%
%   OUTPUT:
%
%   SPVEC - The 'spvec' term

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.03.09-14.09.11

% check value of isSB
if isSB ~= 1 && isSB ~= 2
    error( '''isSB'' must be 1 or 2.' );
end

nOld = n;

if isT && isR
    % hybrid polynomial
    
    % check exitence of field 'trigonometric'
    if ~isfield( ptype, 'trigonometric' )
        error( 'Field ''trigonometric'' in structure ''ptype'' is missing.' );
    end
    % check exitence of field 'real'
    if ~isfield( ptype, 'real' )
        error( 'Field ''real'' in structure ''ptype'' is missing.' );
    end
    
    n( ptype.trigonometric + 1 : end ) = n( ptype.trigonometric + 1 : end ) / 2;
    
    d = ptype.trigonometric + ptype.real;
else
    if isR
        % real polynomial
        
        % check exitence of field 'real'
        if ~isfield( ptype, 'real' )
            error( 'Field ''real'' in structure ''ptype'' is missing.' );
        end
        
        n = n / 2;
        d = ptype.real;
    else
        % trigonometric polynomial
        
        % check exitence of field 'trigonometric'
        if ~isfield( ptype, 'trigonometric' )
            error( 'Field ''trigonometric'' in structure ''ptype'' is missing.' );
        end
        
        d = ptype.trigonometric;
    end
end

nNonzProd1 = prod( n + 1 );

% the degrees
if isSB == 1
    if isT && isR
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.ind, isSB, ptype.trigonometric );
    else
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.ind, isSB );
    end
else
    if isT && isR
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.deg, isSB, ptype.trigonometric );
    else
        [ SBDegsAddSubi, SBNrDegsi ] = genspbasis( n, nOld, d, isT, isR, nNonzProd1, ptype.spbasis.deg, isSB );
    end
end    

% number of n-tuples
ndeg  = size( deg, 1 );
% number of coefficients
ncoef = length( coef );

if ndeg ~= ncoef
    error( 'The number of n-tuples and coefficients must be equal.' );
end

if isT && isR
    % hybrid polynomial
    
    if K == 1
        % scalar coefficients
        
        % indices in the vector
        I    = zeros( ndeg, 1 );
        % coefficients
        Coef = zeros( ndeg, 1 );
        
        % for every nonzero coefficient
        for i = 1 : ndeg
            I( i )    = findvm( SBDegsAddSubi', deg( i, : ) );
            % check that degree is among those
            % from the sparse basis
            if isempty( I( i ) )
                error( 'Degree for coefficient not among the degrees obtained using the chosen sparse basis.' );
            end
            Coef( i ) = coef{ i };
        end
        spvec = sparse( I, ones( ndeg, 1 ), Coef, SBNrDegsi, 1 );
    else
        % matrix coefficients
        
        % number of elem.
        % in matrix
        K2 = K ^ 2;
        
        % "half" of the
        % matrix
        hK = sumn( K );
        
        % number of  symmetric matrices 
        % according to the degrees (deg(1:nt)=0)
        NSymmMat = 0;      
        for i = 1 : SBNrDegsi
            if iszero( SBDegsAddSubi( i, 1 : ptype.trigonometric ) )
                NSymmMat = NSymmMat + 1;
            end
        end
        
        % init spvec
        spvec = sparse( hK * NSymmMat + ( SBNrDegsi - NSymmMat ) * K2, 1 );
        
        for i = 1 : ndeg
            % linear index in halfspace
            idx = findvm( SBDegsAddSubi', deg( i, : ) );
            
            % check that degree is among those
            % from the sparse basis
            if isempty( idx )
                error( 'Degree for coefficient not among the degrees obtained using the chosen sparse basis.' );
            end
        end
        
        % start index
        Start = 0;
        
        % for every nonzero matrix coefficient
        for i = 1 : SBNrDegsi
            
            % if no more coef. than we have nothing to insert
            if ~ncoef
                break;
            end
            
            % linear index in halfspace
            idx = findvm( deg', SBDegsAddSubi( i, : ) );
            
            if iszero( SBDegsAddSubi( i, 1 : ptype.trigonometric ) )
                % (0,0,...,0) for the trig. degrees;
                % symmetric coef.
                
                if ~isempty( idx )
                    % degree exists
                    spvec( Start + 1 : Start + hK ) = vecs( coef{ idx } );
                    ncoef = ncoef - 1;
                end
                
                Start = Start + hK;                
            else
                % non-symmetric coef.
                
                if ~isempty( idx )
                    % degree exists
                    spvec( Start + 1 : Start + K2 ) = vec( coef{ idx } );
                    ncoef = ncoef - 1;
                end
                
                Start = Start + K2;
            end
        end
    end
else
    if isT
        % trigonometric polynomial
        
        if K == 1
            % scalar coefficients
            
            % indices in the vector
            I    = zeros( ndeg, 1 );
            % coefficients
            Coef = zeros( ndeg, 1 );
            
            % for every nonzero coefficient
            for i = 1 : ndeg
                I( i )    = findvm( SBDegsAddSubi', deg( i, : ) );
                % check that degree is among those
                % from the sparse basis
                if isempty( I( i ) )
                    error( 'Degree for coefficient not among the degrees obtained using the chosen sparse basis.' );
                end
                Coef( i ) = coef{ i };
            end
            spvec = sparse( I, ones( ndeg, 1 ), Coef, SBNrDegsi, 1 );
        else
            % matrix coefficients
            
            % number of elem.
            % in matrix
            K2 = K ^ 2;
            
            % "half" of the
            % matrix
            hK = sumn( K );
            
            % init spvec
            spvec = sparse( hK + ( SBNrDegsi - 1 ) * K2, 1 );
            
            % for every nonzero matrix coefficient
            for i = 1 : ndeg
                % linear index in the set of degrees
                idx = findvm( SBDegsAddSubi', deg( i, : ) );
                
                % check that degree is among those
                % from the sparse basis
                if isempty( idx )
                    error( 'Degree for coefficient not among the degrees obtained using the chosen sparse basis.' );
                end
                
                if idx > 1
                    Start = hK + ( idx - 2 ) * K2;
                    spvec( Start + 1 : Start + K2 ) = vec( coef{ i } );
                else
                    % (0,0,...,0)
                    spvec( 1 : hK ) = vecs( coef{ i } );
                end
            end
        end
    else
        % real polynomial
        
        if K == 1
            % scalar coefficients
            
            % indices in the vector
            I    = zeros( ndeg, 1 );
            % coefficients
            Coef = zeros( ndeg, 1 );
            
            % for every nonzero coefficient
            for i = 1 : ndeg
                I( i )    = findvm( SBDegsAddSubi', deg( i, : ) );
                % check that degree is among those
                % from the sparse basis
                if isempty( I( i ) )
                    error( 'Degree for coefficient not among the degrees obtained using the chosen sparse basis.' );
                end
                Coef( i ) = coef{ i };
            end
            spvec = sparse( I, ones( ndeg, 1 ), Coef, SBNrDegsi, 1 );
        else
            % matrix coefficients
            
            % "half" of the
            % matrix
            hK = sumn( K );
            
            % init spvec
            spvec = sparse( SBNrDegsi * hK, 1 );
            
            % for every nonzero matrix coefficient
            for i = 1 : ndeg
                % linear index in the set of degrees
                idx = findvm( SBDegsAddSubi', deg( i, : ) );
                
                % check that degree is among those
                % from the sparse basis
                if isempty( idx )
                    error( 'Degree for coefficient not among the degrees obtained using the chosen sparse basis.' );
                end
                
                spvec( ( idx - 1 ) * hK + 1 : idx * hK ) = vecs( coef{ i } );
            end
        end
    end
end