function fvec = sppols2fvec( nPol, nPC, n, deg, coef, K, nt, nr )

% SPPOL2FVEC Sparse polynomials to full vector
%
%   This function builds an associated vector for some polynomials.
%   Practically it can be used to generate the the whole free term (bsP) 
%   or only a part of it, as one needs.
%
%   FVEC = SPPOLS2FVEC( NPOL, NPC, N, DEG, COEF, K, NT, NR )
%
%   INPUTS:
%
%   NPOL - The number of polynomials in the optimization problem.
%          According to this parameter, some of the other parameters 
%          have one of the two forms.
%   NPC  - A vector containing the number of nonzero (scalar/matrix)
%          coefficients for each of the polynomials.
%   N    - A vector (NPOL=1) or a matrix (NPOL>1) which holds on 
%          its rows the degrees of the polynomials.
%   DEG  - If NPOL=1, then DEG is a matrix whose rows are the n-tuples
%          which represent the nonzero coefficients of the polynomial.
%          Ex: DEG( 5, : ) = [ 4 2 4 ] - the 5th coefficient has the
%          triplet [ 4 2 4 ].
%          If NPOL>1, then DEG is a 3-D array which holds the n-tuples of
%          the coefficients of all the polynomials.
%          Ex: DEG( 3, 6, : ) = [ -2 2 0 1 ] - the 6th coefficient of the
%          3rd polynomial has the quadruple [ -2 2 0 1 ].
%   COEF - If NPOL=1, then COEF is a vector of cells, holding the
%          coefficients of the polynomial, scalars or matrices.
%          Ex: COEF{ 3 } = [ 2 1; 0 1 ] - the 3rd coefficient of the 
%          polynomial is the matrix [ 2 1; 0 1 ].
%          IF NPOL>1, then DEG is matrix of cells, holding the coeffcients
%          for all the polynomials.
%          Ex: COEF{ 5, 4 } = 1 - the 4th coefficient of the 5th polynomial
%          is the scalar 1.
%   K    - A vector containing the size of the coeffcients for every
%          polynomial.
%   NT   - A vector which contains the number of trigonometric variables for
%          each polynomial. 
%          Ex: NT = [ 3 2 0 2 ] - the 1st polynomial has three trigonometric 
%          variables, while the 2nd and the 4th have two trigonometric variables
%          each; these polynomials might be trigonometric or hybrid, while the 
%          3rd is a real polynomial.
%   NR   - A vector which contains the number of real variables for each polynomial. 
%
%   OUTPUT:
%
%   FVEC  - The 'fvec' term (the full vector)
%
%   See also sppol2fvec, spcausal2fvec.

% Copyright (C) 2008-2011 Bogdan C. Sicleru, Bogdan Dumitrescu
%
% This file is part of POS3POLY.
%
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.03.09-28.10.11

if nPol == 1
    % one polynomial
    
    % number of n-tuples
    ndeg  = size( deg, 1 );
    % number of coefficients
    ncoef = length( coef );
    
    if ndeg ~= ncoef
        error( 'The number of n-tuples and coefficients must be equal.' );
    end
    
    if nt && nr
        % hybrid polynomial
        
        % number of coefficients
        nC = nchp( n, nt );
        
        if K == 1
            % scalar coefficients
            
            % init fvec
            fvec = sparse( nC, 1 );
            
            % for every nonzero coefficient
            for i = 1 : ndeg
                fvec( gsub2indh( n, deg( i, : ), nt ) ) = coef{ i };
            end
        else
            % matrix coefficients
            
            % number of elem.
            % in matrix
            K2 = K ^ 2;
            
            % "half" of the
            % matrix
            hK = sumn( K );
            
            % number of coef. for the real part
            Nr = prod( nonzeros( n( nt + 1 : end ) ) + 1 );
            
            % init fvec
            fvec = sparse( ( nC - Nr ) * K2 + Nr * hK, 1 );            
            
            % all the hybrid n-tuples
            nTuple = ntupleh( n, nC, nt );
            
            % start index
            Start = 0;
            
            % for every coefficient
            for i = 1 : nC
                
                % if no more coef. than we have nothing to insert
                if ~ncoef
                    break;
                end
                
                % linear index in halfspace
                idx = findvm( deg', nTuple( i, : ) );
                
                % while we have coef. we insert them in 'fvec'
                if iszero( nTuple( i, 1 : nt ) )
                    % (0,0,...,0) for the trig. degrees;
                    % symmetric coef.
                    
                    if ~isempty( idx )
                        % degree exists
                        fvec( Start + 1 : Start + hK ) = vecs( coef{ idx } );
                        ncoef = ncoef - 1;
                    end
                    Start = Start + hK;
                else
                    % non-symmetric coef.
                    
                    if ~isempty( idx )
                        % degree exists
                        fvec( Start + 1 : Start + K2 ) = vec( coef{ idx } );
                        ncoef = ncoef - 1;
                    end
                    Start = Start + K2;
                end
                
            end
        end
    else
        if nt
            % trigonometric polynomial
            
            % number of coefficients
            nC = nch( n );
            
            if K == 1
                % scalar coefficients
                
                % init fvec
                fvec = sparse( nC, 1 );
                
                % for every nonzero coefficient
                for i = 1 : ndeg
                    fvec( gsub2ind( n, deg( i, : ), 1 ) ) = coef{ i };
                end
            else
                % matrix coefficients
                
                % number of elem.
                % in matrix
                K2 = K ^ 2;
                
                % "half" of the
                % matrix
                hK = sumn( K );
                
                % init fvec
                fvec = sparse( hK + ( nC - 1 ) * K2, 1 );
                
                % for every nonzero matrix coefficient
                for i = 1 : ndeg
                    % linear index in halfspace
                    idx = gsub2ind( n, deg( i, : ), 1 );
                    
                    if idx > 1
                        Start = hK + ( idx - 2 ) * K2;
                        fvec( Start + 1 : Start + K2 ) = vec( coef{ i } );
                    else
                        % (0,0,...,0)
                        fvec( 1 : hK ) = vecs( coef{ i } );
                    end
                end
            end
        else
            % real polynomial
            
            % number of coefficients
            nC = prod( n + 1 );
            
            if K == 1
                % scalar coefficients
                
                % init fvec
                fvec = sparse( nC, 1 );
                
                % for every nonzero coefficient
                for i = 1 : ndeg
                    fvec( gsub2ind( n, deg( i, : ), 0 ) ) = coef{ i };
                end
            else
                % matrix coefficients
                
                % "half" of the
                % matrix
                hK = sumn( K );
                
                % init fvec
                fvec = sparse( nC * hK, 1 );
                
                % for every nonzero matrix coefficient
                for i = 1 : ndeg
                    % linear index in halfspace
                    idx = gsub2ind( n, deg( i, : ), 0 );
                    
                    fvec( ( idx - 1 ) * hK + 1 : idx * hK ) = vecs( coef{ i } );
                end
            end
        end
    end
else
    % nPol > 1; we have more
    % than one polynomial
    
    % nr. of coef. for
    % every pol.
    nC = zeros( nPol, 1 );
    
    % "whole" and "half" a matrix
    K2 = sparse( nPol, 1 );
    hK = sparse( nPol, 1 );
    
    % compute number of coefficients
    % for every polynomial
    for p = 1 : nPol
        % for every polynomial
        
        if nt( p ) && nr( p )
            % hybrid polynomial
            
            if K( p ) == 1
                % scalar coefficients
                
                nC( p ) = nchp( n( p, : ), nt( p ) );
            else
                % matrix coefficients
                
                K2( p ) = K( p ) ^ 2;
                hK( p ) = sumn( K( p ) );
                
                nC( p ) = hK( p ) + ( nchp( n( p, : ), nt( p ) ) - 1 ) * K2( p );
            end
        else
            if nt( p )
                % trigonometric polynomial
                
                if K( p ) == 1
                    % scalar coefficients
                    
                    nC( p ) = nch( n( p, : ) );
                else
                    % matrix coefficients
                    
                    K2( p ) = K( p ) ^ 2;
                    hK( p ) = sumn( K( p ) );
                    
                    nC( p ) = hK( p ) + ( nch( n( p, : ) ) - 1 ) * K2( p );
                end
            else
                % real polynomial
                
                if K( p ) == 1
                    % scalar coefficients
                    
                    nC( p ) = prod( n( p, : ) + 1 );
                else
                    % matrix coefficients
                    
                    hK( p ) = sumn( K( p ) );
                    
                    nC( p ) = prod( n( p, : ) + 1 ) * hK( p );
                end
            end
        end
    end
    
    % init 'fvec'
    fvec = sparse( sum( nC ), 1 );
    
    % cumulative sum for the
    % number of coefficients
    nCCum = cumsum( nC( 1 : end - 1 ) );
    nCCum = [ 0 nCCum ];
    
    for p = 1 : nPol
        % for every polynomial
        
        if nt( p ) && nr( p )
            % hybrid polynomial
            
            if K( p ) == 1
                % scalar coefficients
                
                % for every nonzero coefficient
                for i = 1 : nPC( p )
                    fvec( nCCum( p ) + gsub2indh( n( p, : ), deg( p, i, : ), nt( p ) ) ) = coef{ p, i };
                end
            else
                % matrix coefficients
                
                % for every nonzero matrix coefficient
                for i = 1 : nPC( p )
                    % linear index in halfspace
                    idx = gsub2indh( n( p, : ), deg( p, i, : ), nt( p ) );
                    
                    if idx > 1
                        Start = hK( p ) + ( idx - 2 ) * K2( p );
                        fvec( nCCum( p ) + Start + 1 : nCCum( p ) + Start + K2( p ) ) = vec( coef{ p, i } );
                    else
                        % (0,0,...,0)
                        fvec( nCCum( p ) + 1 : nCCum( p ) + hK( p ) ) = vecs( coef{ p, i } );
                    end
                end
            end
        else
            if nt( p )
                % trigonometric polynomial
                
                if K( p ) == 1
                    % scalar coefficients
                    
                    % for every nonzero coefficient
                    for i = 1 : nPC( p )
                        fvec( nCCum( p ) + gsub2ind( n( p, : ), deg( p, i, : ), 1 ) ) = coef{ p, i };
                    end
                else
                    % matrix coefficients
                    
                    % for every nonzero matrix coefficient
                    for i = 1 : nPC( p )
                        % linear index in halfspace
                        idx = gsub2ind( n( p, : ), deg( p, i, : ), 1 );
                        
                        if idx > 1
                            Start = hK( p ) + ( idx - 2 ) * K2( p );
                            fvec( nCCum( p ) + Start + 1 : nCCum( p ) + Start + K2( p ) ) = vec( coef{ p, i } );
                        else
                            % (0,0,...,0)
                            fvec( nCCum( p ) + 1 : nCCum( p ) + hK( p ) ) = vecs( coef{ p, i } );
                        end
                    end
                end
            else
                % real polynomial
                
                if K( p ) == 1
                    % scalar coefficients
                    
                    % for every nonzero coefficient
                    for i = 1 : nPC( p )
                        fvec( nCCum( p ) + gsub2ind( n( p, : ), deg( p, i, : ), 0 ) ) = coef{ p, i };
                    end
                else
                    % matrix coefficients
                    
                    % for every nonzero matrix coefficient
                    for i = 1 : nPC( p )
                        % linear index in halfspace
                        idx = gsub2ind( n( p, : ), deg( p, i, : ), 0 );
                        
                        fvec( nCCum( p ) + ( idx - 1 ) * hK( p ) + 1 : nCCum( p ) + idx * hK( p ) ) = vecs( coef{ p, i } );
                    end
                end
            end
        end
    end
end