function [ deg, coef ] = sum_poly( deg1, deg2, coef1, coef2, type, ke, nt )

% SUM_POLY Polynomial sum
%
%   This function computes the multivariate sum of two 
%   real/trigonometric/hybrid polynomials.
%
%   [ DEG, COEF ] = SUM_POLY( DEG1, DEG2, COEF1, COEF2, TYPE, KE, NT )
%
%   INPUTS:
%
%   DEG1  - A matrix containing on its rows the degrees of the
%           nonzero coefficients of the 1st polynomial.
%
%   DEG2  - A matrix containing on its rows the degrees of the
%           nonzero coefficients of the 2nd polynomial.
%
%   COEF1 - A vector of cells containing the nonzero scalar/matrix
%           coefficients of the 1st polynomial.
%
%   COEF2 - A vector of cells containing the nonzero scalar/matrix
%           coefficients of the 2nd polynomial.
%
%   TYPE  - A variable denoting the type of the polynomials.
%
%           If TYPE=0, then the polynomials are real.
%           If TYPE=1, then the polynomials are trigonometric.
%           If TYPE=2, then the polynomials are hybrid.
%
%   KE    - Keep or remove zero coefficients from polynomial.
%
%           If KE=0, then the zero coefficients are removed.
%           If KE=1, then the zero coefficients are kept.
%
%           The use of the variable is optional. By default, the
%           zero coefficients are removed.
%
%   NT    - A variable denoting the number of trigonometric 
%           variables when the polynomials are hybrid. The variable
%           is used only for hybrid polynomials, otherwise is not
%           required.
%
%   OUTPUTS:
%
%   DEG   - A matrix containing on its rows the degrees of the
%           nonzero coefficients of the resultant polynomial.
%
%   COEF  - A vector of cells containing the nonzero scalar/matrix
%           coefficients of the resultant polynomial.
%
%   Example:
%
%   H1 = t + z1^{-1}*t + z2^{-1}*t + 2 + z2^{-1} + sym^{-1}
%   H2 = z1^{-1}z2^{-2} + z1^{-2}z2^{-1}t + sym^{-1}
%
%   H1 and H2 are hybrid polynomials with two trig. variables and
%   one real variable. sym^{-1} is the "symmetric" part, the
%   coefficients from the other halfspace of the hybrid polynomial.
%
%   deg1  = [ 0 0 1; ...
%             1 0 1; ...
%             0 1 1; ...
%             0 0 0; ...
%             0 1 0; ];
%   deg2  = [ 1 2 0; ...
%             2 1 1; ];
%   coef1 = { 1, 1, 1, 2, 1 }; 
%   coef2 = { 1, 1 }; 
%   [ deg, coef ] = sum_poly( deg1, deg2, coef1, coef2, 2, 0, 2 );
%
%   See also prod_poly.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 08.07.09-27.09.09

% size for scalar/matrix
K = size( coef1{ 1 }, 1 );
% number of variables
d = size( deg1, 2 );

% number of coef. 
nC1 = length( coef1 );
nC2 = length( coef2 );

% max degree for each of
% the polynomials
n1 = max( abs( deg1 ) );
n2 = max( abs( deg2 ) );

% max. deg. for the
% final polynomial
n = max( [ n1; n2 ] );

% zero matrix
O = sparse( K, K );

% compute n-tuples according to 
% the type of polynomials
switch type
    case 0
        % real polynomials
        
        % number of coef.
        nC = prod( n + 1 );
        
        nTuple = ntuple( n, nC, 0 );       
    case 1
        % trigonometric polynomials
        
        % number of coef.
        nC = nch( n );
        
        nTuple = ntuple( n, nC, 1 );               
    case 2
        % hybrid polynomials
        
        % number of coef.
        nC = nchp( n, nt );
        
        nTuple = ntupleh( n, nC, nt );
    otherwise
        error( 'Invalid value for the variable ''type''.' );
end

% coefficients of the
% temporar polynomial;
% (final pol. with zeros)
coefT = cell( nC, 1 );
% init with zero (scalar/matrix)
for i = 1 : nC
    coefT{ i } = O;
end

% compute sum;
% 1st pol.
for i = 1 : nC1
    idx = findvm( nTuple', deg1( i, : ) );
    if ~isempty( idx )
        coefT{ idx } = coef1{ i };
    end
end
% add 2nd pol.
for i = 1 : nC2
    idx = findvm( nTuple', deg2( i, : ) );
    if ~isempty( idx )
        coefT{ idx } = coefT{ idx } + coef2{ i };
    end
end

if nargin > 5
    if ~ke
        % retain only nonzero coefficients
        [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
    else
        deg  = nTuple;
        coef = coefT;
    end
else
    % by default, retain only nonzero coefficients
    [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O );
end

function [ deg, coef ] = get_nonzero_coef( coefT, nC, nTuple, d, O )

% This function retains only the nonzero coefficients
% and their degrees.

% nC final; we retain the
% the nonzero coef.
nCFin = 0;
% indexes where we have
% nonzero coefficients
idxFin  = zeros( nC, 1 );
for i = 1 : nC
    if ~eqm( coefT{ i }, O )
        % nonzero coefficient
        
        nCFin = nCFin + 1;
        idxFin( nCFin ) = i;
    end
end

% retain nonzero coefficients
% and their degrees
coef = cell( nCFin, 1 );
deg  = zeros( nCFin, d );
for i = 1 : nCFin
    coef{ i }   = coefT{ idxFin( i ) };
    deg( i, : ) = nTuple( idxFin( i ), : );
end