function [ T ] = toeplitzs( N )

% TOEPLITZS Toeplitz matrices
%
%   [ T ] = TOEPLITZS( N ) returns N+1 Toeplitz 
%   matrices of size (N+1)x(N+1).
%
%   Each k-th matrix, k=0:N, has ones on the k-th 
%   diagonal and zeros elsewhere and will lie in  
%   matrix T at T(k*(N+1)+1:(k+1)*(N+1),:).
%
%   See also hankels.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 19.10.08-18.02.10

% CHECKS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% check if N is numeric
if ~isnumeric( N )
    error( 'N must be numeric.' );
end

% check for NaN and Inf
if isnan( N ) || isinf( N )
    error( 'NaN and Inf not allowed.' );
end

% check if imaginary part is ~= 0
if imag( N )
    error( 'N''s imaginary part must be 0. ' );
end

% check if N is nonnegative
if N < 0
    error( 'N must be nonnegative.' );
end

% check if N is an integer number
fN = fix( N );
if fN ~= N
    warning( 'MATLAB:SeDuMiPol:NotIntegerNum', ...
             'N should be an integer number; we continue with fix(N).' );
    N = fN;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% number of matrices
% &
% size of matrices
N1 = N + 1;

% number of ones in all the 
% Toeplitz matrices
nOnes = sumn( N1 );

i = zeros( nOnes, 1 );
j = zeros( nOnes, 1 );
s = ones ( nOnes, 1 );

% build the lines and columns
% for the values of one
Line = 1;
for Shift = 0 : N
    for k = 1 : N1 - Shift
        i( Line ) = Shift * N1 + k;
        j( Line ) = Shift + k;
        
        Line = Line + 1;
    end
end

% the N+1 Toeplitz matrices
T = sparse( i, j, s, N1 ^ 2, N1, nOnes );