function [ v ] = vecg( M, type )

% VECG General vectorization
%
%   [ V ] = VECG( M, TYPE ) vectorizes a matrix M
%   or an (N-2)-D array M of matrices, N>2,
%   according to the value of TYPE and returns
%   the resulting vector in V.
%
%   If TYPE is 0, then all the matrices from
%   M will be vectorized with 'vecsm' (matrix
%   vectorization of lower triangular part).
%
%   If TYPE IS 1, the first matrix is 
%   vectorized with 'vecsm' and the rest with
%   'vec' (vectorization of the whole matrix).
%
%   [ V ] = VECG( M ) vectorizes only one matrix
%   (with 'vecsm').
%
%   See also vec, vecs.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 26.11.08

% size vector for N-D array M
d = size( M );

% the first two elements of 'd'
% representing the matrix dimensions
d1 = d( 1 );
d2 = d( 2 );

% check if 'd' has the first two elements > 1 (2-D array)
if nnz( [ d1 d2 ] - 1 ) ~= 2
    error( 'The first two elements of ''d'' must indicate a matrix size.' );
end

% check if the first two elements of 'd'
% are equal (square matrix)
if d1 ~= d2
    error( 'The first two elements of ''d'' must be equal (square matrix).' );
end

if length( d ) == 2
    % we have one matrix
    v = vecsm( M );   
else
    if nargin ~= 2
        error( 'Incorrect number of input arguments.' );
    end
    % length(d) > 2; we have
    % an array of matrices    
    
    % number of matrices in 
    % the (N-2)-D array
    nM =  prod( d( 3 : end ) );
    
    % check type
    % value
    switch type
        case 0
            % vecsm

            % "half" a square matrix
            nElem = d1 * ( d1 + 1 ) / 2;

            % the resulting vector, vectorizing
            % all nM matrices
            v = zeros( nElem * nM, 1 );

            % step for putting
            % a new vector
            Step = 0;

            % create vector v
            for m = 1 : nM
                % "concatenate" a new vector obtained
                % by vectorizing a matrix
                v( 1 + Step : nElem + Step ) = vecsm( M( :, :, m ) );

                % increase Step
                Step = Step + nElem;
            end            
        case 1
            % vecsm + vec
            
            % "half" a square matrix
            hd1 = d1 * ( d1 + 1 ) / 2;
            % the "whole" matrix
            d12 = d1 ^ 2;
            
            % the resulting vector, vectorizing
            % all nM matrices
            v = zeros( hd1 + ( nM - 1 ) * d12, 1 );
            
            v( 1 : hd1 ) = vecsm( M( :, :, 1 ) );
            
            % step for putting
            % a new vector;
            % initialize
            Step = hd1;
            
            % coutinue to fill
            % vector v
            for m = 2 : nM
                % "concatenate" a new vector obtained
                % by vectorizing a matrix
                v( 1 + Step : d12 + Step ) = vec( M( :, :, m ) );

                % increase Step
                Step = Step + d12;
            end              
        otherwise
            error( '''type'' must be 0 or 1.' );
    end
end