function [ v ] = vecras( M )

% VECRAS Vectorize real antisymmetric
%
%   This function returns a column vector V from the 
%   the strict lower part of a real antisymmetric matrix 
%   in an (N-2)-D array M of matrices.
%
%   V = VECRAS( M ) returns a column vector of a matrix
%   if M is a matrix, or a of an (N-2)-D array of
%   matrices is M is an N-D array, N>2.
%
%   For complex matrices we consider the imaginary part.
%
%   Example 1:
%
%   >> M
%   M =
%       0     2     0     9
%      -2     0     1    -7
%       0    -1     0    -2
%      -9     7     2     0
%
%   >> v = vecras( M );
%
%   >> v'
%
%     -2 0 -9 -1 7 2
%
%   Example 2:
%
%   >> M
%   M(:,:,1) =
%              0     3     0          
%             -3     0     2i
%              0     2i    0          
%   M(:,:,2) =
%              0    -9    -1
%              9     0    -5
%              1     5     0
%   M(:,:,3) =
%              0     0    -1
%              0     0    -4
%              1     4     0
%
%   >> v = vecras( M )
%
%   >> v'
%
%      0 0 2 9 1 5 0 1 4

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 5.10.08-27.10.08

% size vector for N-D array M
d = size( M );

% the first two elements of 'd'
% representing the matrix dimensions
d1 = d( 1 );
d2 = d( 2 );

% check if 'd' has the first two elements > 1 (2-D array)
if nnz( [ d1 d2 ] - 1 ) ~= 2
    error( 'The first two elements of ''d'' must indicate a matrix size.' );
end

% check if the first two elements of 'd'
% are equal (square matrix)
if d1 ~= d2
    error( 'The first two elements of ''d'' must be equal (square matrix).' );
end

if length( d ) == 2
    % we have one matrix
    v = vecrasm( M );   
else
    % length(d) > 2; we have
    % an array of matrices    
    
    % number of matrices in 
    % the (N-2)-D array
    nM =  prod( d( 3 : end ) );
    
    % number of elements in
    % "half-1" a square matrix
    nElem = ( d1 - 1 ) * d1 / 2;
    
    % the resulting vector, vectorizing
    % all nM matrices
    v  = zeros( nElem * nM, 1 );
    
    % step for putting
    % a new vector
    Step = 0;
    
    % create vector v
    for m = 1 : nM
        % "concatenate" a new vector obtained
        % by vectorizing a matrix
        v( 1 + Step : nElem + Step ) = vecrasm( M( :, :, m ) );
        
        % increase Step
        Step = Step + nElem;
    end            
end