function [ v ] = vltri( M, k )

% VLTRI Vectorize lower triangular part
%
%   This functions returns a column vector V vectorizing 
%   the lower triangular part of a matrix M.
%
%   V = VLTRI( M ) returns a column vector from the lower
%   triangular part of matrix M
%
%   V = VLTRI( M, K ) returns a column vector from
%   the lower part below the K-th diagonal (including it).
%
%   K = 0 is the main diagonal, K < 0 is below the main
%   diagonal and K > 0 is above the main diagonal.
%
%   Example 1:
%
%   >> M
%   M =
%       1     2     3
%       5    -1     1
%       9     7     2
%       4     0     2
%       1     2     9
%
%   >> v = vltri( M );
%
%   >> v'
%
%      1 5 9 4 1 -1 7 0 2 2 2 9
%
%   Example 2:
%
%   >> M
%   M =
%       3     2     0     9     5
%       1     0     1     7     5
%       9     1     2     2     5
%
%   >> v = vltri( M, 2 );
%
%   >> v'
%  
%      3 1 9 2 0 1 0 1 2 7 2 5
%
%   See also vec, vecs, vecg.

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 25.10.08-27.10.08

% check for second parameter k;
% if it is missing, consider it 0
if nargin < 2
    k = 0;
else
    % check if 'k' is an integer number
    fk = fix( k );
    if fk ~= k
        warning( 'MATLAB:SeDuMiPol:NotIntegerNum', ...
                   '''k'' should be an integer number; we continue with fix(k).' );
        k = fk;
    end
end

% dimensions for matrix M
[ m, n ] = size( M );

% initialize vector v
v = [ ];

% check if 'k' is nonnegative 
% or positive
if k <= 0
    % k <= 0
  
    % number of lines to
    % consider
    nLin = m + k;
    
    % check if diagonal k exists
    if nLin <= 0
        % diagonal non-existent (too low), no columns
        % to take into account
        warning( 'MATLAB:SeDuMiPol:NonExistentDiagNoElem', ...
                 'Non-existent diagonal; no elements to consider.' );
        return;
    end
    
    % number of columns to consider
    nCol = min( n, nLin ); 

    % create vector v
    for c = 1 : nCol
        % copy of vector v
        vOld = v;
        % new vector v
        v = [ vOld; M( c - k : m, c ) ];
    end
else
    % k > 0
    
    % check if diagonal k exists
    if n - k <= 0
        % diagonal non-existent (too high), we consider
        % the whole matrix
        warning( 'MATLAB:SeDuMiPol:NonExistentDiagAllElem', ...
                 'Non-existent diagonal; we consider the whole matrix.' );
    end
    
    % number of columns to consider
    nCol = min( n, m + k ); 
    
    % create vector v
    for c = 1 : nCol
        % copy of vector v
        vOld = v;
        % new vector v
        v = [ vOld; M( max( c - k, 1 ) : m, c ) ];
    end
end