function [ D, Line ] = coef_var_mat_uni_pol( i, D, Line, LimD, ptype, n, nOld, K, IsR, ...
                                             isIVeci, MList, MVecUnq, Qsi, QOsi, ...
                                             isBRLi, BRLK2i )

% COEF_VAR_MAT_UNI_POL Create SeDuMi matrix for the univariate case
%
%   This function returns the D matrix, that we use
%   to transform the user's problem into a SeDuMi
%   problem.
%
%   The problem
%
%   min  csP' * wtx
%   s.t. AsP  * wtx = bsP
%
%   becomes
%
%   min  csP' * D * x
%   s.t. AsP  * D * x = bsP

% Copyright (C) 2008, 2009, 2010 Bogdan C. Sicleru, Bogdan Dumitrescu
% 
% This file is part of POS3POLY.
% 
% POS3POLY is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% POS3POLY is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with POS3POLY.  If not, see <http://www.gnu.org/licenses/>.

% Date: 22.11.08-05.05.10                   

global SR;


    
if ~isBRLi

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % no BRL, no need to extend the vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if K == 1
        % scalar coefficients

        switch isIVeci                    
            case 0
                % global positivity

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    % we add the vectorized 
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    Line = Line + 1;
                end
            case 1
                % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % two tangents
                ta = tan( a / 2 );
                tb = tan( b / 2 );  

                % coeficients of Dab
                d0 = -( ta * tb + 1 ) / 2;
                d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi                                   

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' + ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 2
                % [ -a, a ] in ( - pi, pi ) trig. pol. complex coef.

                % the upper bound of the
                % positivity interval 
                a = ptype.int( 2 );

                % cosine
                ca = cos( a ); 

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( -cosa + (z+z^{-1})/2 )
                    %      I          II

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )' + ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )';
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end 

                    Line = Line + 1;
                end
            case 3
                % [ a, pi ] trig. pol. complex coef.

                % the lower bound of the
                % positivity interval 
                a = ptype.int( 1 );

                % one tangent
                ta = tan( a / 2 );

                % coeficients of Dab
                d0 = -ta / 2;
                d1 = -ta / 4 + SR / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' + ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 4
                % [ -pi, b ] trig. pol. complex coef.

                % the upper bound of the
                % positivity interval 
                b = ptype.int( 2 );

                % one tangent
                tb = tan( b / 2 );

                % coeficients of Dab
                d0 = -tb / 2;
                d1 = -tb / 4 + SR / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % -( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' - ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end  

                    Line = Line + 1;
                end                      
            case 5
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % two cosines
                ca = cos( a );
                cb = cos( b );  

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                n1 = n - 1;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 2; 

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % -( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^2+z^{-2})/4
                    %            I                        II                       III

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            if nOld >= 4
                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                ( ca + cb ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % III
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                0.5 * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';
                            end
                        case 2                                       
                            if nOld >= 4 
                                % I
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -( ca * cb + 0.5 ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                0.5 * ( ca + cb ) * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';                                         
                            end

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                         
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            % III
                            if nOld >= 4
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...                                         
                                0.25 * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';
                            end                                    
                            if nOld >= 6                                            
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                0.25 * vec( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ) )';                                   
                            end
                        case QOsi - 3
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' + ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 2
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 1
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi                                        
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            -0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' + ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ) )' - ...
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                    end

                    Line = Line + 1;
                end
            case 6
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % two cosines
                ca = cos( a );
                cb = cos( b ); 

                % S0: n, S1:n

                % size for Gram 
                % matrices
                nn = n;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    switch deg        
                            %---------------------------------------------------------------------------------------
                            % coefficient from S0
                            % -cosb + (z + z^{-1})/2
                            %   I           II
                        case 1                                        
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            -cb * vec( MList{ idx }( 1 : nn, : ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                                D( Line, LimD( 1 ) : LimD( 2 ) ) + ...  
                                vec( MList{ idx }( nn + 1 : 2 * nn, : ) )';                                            
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            -cb * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            -cb * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                            0.5 * vec( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end
                    switch deg
                            %---------------------------------------------------------------------------------------
                            % coefficient from S1
                            % cosa - (z + z^{-1})/2
                            %   I           II
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx }( 1 : nn, : ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                vec( MList{ idx }( nn + 1 : 2 * nn, : ) )';   
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                            -0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * vec( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 7
                % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % two tangents
                ta = tan( a / 2 );
                tb = tan( b / 2 );  

                % coeficients of Dab
                d0 = -( ta * tb + 1 ) / 2;
                d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % -( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' - ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 8
                % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.

                % the upper bound of the
                % positivity interval 
                a = ptype.int( 3 );

                % cosine
                ca = cos( a ); 

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( cosa - (z+z^{-1})/2 )
                    %     I          II 

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )' - ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )';
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            -0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end  

                    Line = Line + 1;
                end
            case 9
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % two cosines
                ca = cos( a );
                cb = cos( b );  

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                n1 = n - 1;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 2; 

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                    %            I                        II                       III

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            if nOld >= 4
                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                ( ca + cb ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % III
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                0.5 * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';
                            end
                        case 2
                            if nOld >= 4
                                % I
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                ( ca * cb + 0.5 ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                0.5 * ( ca + cb ) * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';
                            end

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            % III
                            if nOld >= 4
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                              
                                0.25 * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';
                            end
                            if nOld >= 6
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                0.25 * vec( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ) )';                                        
                            end
                        case QOsi - 3
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' - ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 2
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 1
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            -0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi                                        
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' - ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ) )' + ...
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                    end

                    Line = Line + 1;
                end
            case 10
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % two cosines
                ca = cos( a );
                cb = cos( b ); 

                % S0: n, S1:n

                % size for Gram 
                % matrices
                nn = n;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    switch deg        
                            %---------------------------------------------------------------------------------------
                            % coefficient from S0
                            % cosb - (z + z^{-1})/2
                            %  I           II
                        case 1                                        
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            cb * vec( MList{ idx }( 1 : nn, : ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                                D( Line, LimD( 1 ) : LimD( 2 ) ) - ...  
                                vec( MList{ idx }( nn + 1 : 2 * nn, : ) )';                                            
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            cb * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                            -0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            cb * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                            0.5 * vec( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end
                    switch deg
                            %---------------------------------------------------------------------------------------
                            % coefficient from S1
                            % -cosa + (z + z^{-1})/2
                            %    I           II
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx }( 1 : nn, : ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                vec( MList{ idx }( nn + 1 : 2 * nn, : ) )';   
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * vec( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end    

                    Line = Line + 1;
                end
            case 11
                % [ a, b ] real pol. even degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (-ab + (a+b)*t - t^2 )

                    % term multiplied with -ab   
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -a * b * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with (a+b)*t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        ( a + b ) * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    % term multiplied with -t^2
                    if deg >= 3
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end
            case 12
                % [ a, b ] real pol. odd degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );                       

                % S0: n, S1:n

                % size for Gram 
                % matrices
                n05 = n + 0.5;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 0.5;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0
                    % (-a + t)                               

                    % term multiplied with -a
                    if deg ~= QOsi
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        -a * vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    % term multiplied with t
                    if deg ~= 1
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...   
                        D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                        vec( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ) )';
                    end

                    %---------------------------------------------------------------------------------------
                    % coefficient from S1
                    % (b - t)                              

                    % term multiplied with b
                    if deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        b * vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    % term multiplied with -t
                    if deg ~= 1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...   
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ) )';
                    end

                    Line = Line + 1;
                end
            case 13
                % [ a, Inf ) real pol. even degree

                % the lower bound
                % of the positivity interval 
                a = ptype.int( 1 );

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram  matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (-a + t)

                    % term multiplied with -a  
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -a * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end
            case 14
                % [ a, Inf ) real pol. odd degree

                % the lower bound
                % of the positivity interval 
                a = ptype.int( 1 );

                % S0: n, S1:n 

                % size for the second
                % Gram  matrix
                n05 = n + 0.5;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 0.5;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    if deg ~= QOsi
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (-a + t)

                    % term multiplied with -a  
                    if deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -a * vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    % term multiplied with t
                    if deg ~= 1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        vec( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ) )';
                    end

                    Line = Line + 1;
                end
            case 15
                % ( -Inf, b ] real pol. even degree

                % the upper bound
                % of the positivity interval 
                b = ptype.int( 2 );

                % S0: n+1, S1:n 

                % size for the second
                % Gram  matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (b - t)

                    % term multiplied with b  
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        b * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with -t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end 
            case 16
                % ( -Inf, b ] real pol. odd degree

                % the upper bound
                % of the positivity interval 
                b = ptype.int( 2 );

                % S0: n+1, S1:n 

                % size for the second
                % Gram  matrix
                n05 = n + 0.5;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 0.5;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    if deg ~= QOsi
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (b - t)

                    % term multiplied with b  
                    if deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        b * vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    % term multiplied with -t
                    if deg ~= 1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ) )';
                    end

                    Line = Line + 1;
                end
            case 17
                % ( -Inf, a ] U [ b, Inf ) real pol. even degree

                % bounds
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (ab - (a+b)*t + t^2 )

                    % term multiplied with ab   
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        a * b * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with -(a+b)*t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        ( a + b ) * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    % term multiplied with t^2
                    if deg >= 3
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        vec( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end
        end
    else
        % matrix coefficients

        % "half" of the
        % matrix
        hK = sumn( K );

        % sind2sub is used go get
        % the subscripts from a "half"
        % a matrix index

        if  IsR
            % real polynomial

            switch isIVeci 
                case 0
                    % global positivity

                    for deg = 1 : QOsi
                        for elem = 1 : hK

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                            vec( kron( MList{ i }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            Line = Line + 1;
                        end
                    end
                case 11
                    % [ a, b ] real pol. even degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK     

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (-ab + (a+b)*t - t^2 )

                            % term multiplied with -ab   
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -a * b * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with (a+b)*t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                ( a + b ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            % term multiplied with -t^2
                            if deg >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end                     
                case 12
                    % [ a, b ] real pol. odd degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );                       

                    % S0: n, S1:n

                    % size for Gram 
                    % matrices
                    n05 = n + 0.5;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 0.5;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 ); 

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0
                            % (-a + t)                               

                            % term multiplied with -a
                            if deg ~= QOsi
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                -a * vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            % term multiplied with t
                            if deg ~= 1
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...   
                                D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                                vec( kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ) )';
                            end

                            %---------------------------------------------------------------------------------------
                            % coefficient from S1
                            % (b - t)                              

                            % term multiplied with b
                            if deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                b * vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            % term multiplied with -t
                            if deg ~= 1
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...   
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 13
                    % [ a, Inf ) real pol. even degree

                    % the lower bound
                    % of the positivity interval 
                    a = ptype.int( 1 );

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram  matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK               

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (-a + t)

                            % term multiplied with -a  
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -a * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 14
                    % [ a, Inf ) real pol. odd degree

                    % the lower bound
                    % of the positivity interval 
                    a = ptype.int( 1 );

                    % S0: n, S1:n 

                    % size for the second
                    % Gram  matrix
                    n05 = n + 0.5;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 0.5;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK        

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 ); 

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            if deg ~= QOsi
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (-a + t)

                            % term multiplied with -a  
                            if deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -a * vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            % term multiplied with t
                            if deg ~= 1
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                vec( kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 15
                    % ( -Inf, b ] real pol. even degree

                    % the upper bound
                    % of the positivity interval 
                    b = ptype.int( 2 );

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram  matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK    

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (b - t)

                            % term multiplied with b  
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                b * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with -t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end   
                case 16
                    % ( -Inf, b ] real pol. odd degree

                    % the upper bound
                    % of the positivity interval 
                    b = ptype.int( 2 );

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram  matrix
                    n05 = n + 0.5;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 0.5;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK        

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            if deg ~= QOsi
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (b - t)

                            % term multiplied with b  
                            if deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                b * vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            % term multiplied with -t
                            if deg ~= 1
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 17
                    % ( -Inf, a ] U [ b, Inf ) real pol. even degree

                    % bounds
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (ab - (a+b)*t + t^2 )

                            % term multiplied with ab   
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                a * b * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with -(a+b)*t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                ( a + b ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            % term multiplied with t^2
                            if deg >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                vec( kron( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
            end
        else
            % trigonometric polynomial          

            K2 = K ^ 2;

            switch isIVeci 
                case 0
                    % global positivity                     

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                    
                        D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                        vec( kron( MList{ i }( 1 : Qsi, : ), E ) )';

                        Line = Line + 1;
                    end

                    for deg = 2 : QOsi
                        for elem = 1 : K2

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                            vec( kron( MList{ i }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            Line = Line + 1;
                        end
                    end   
                case 1
                    % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % two tangents
                    ta = tan( a / 2 );
                    tb = tan( b / 2 );  

                    % coeficients of Dab
                    d0 = -( ta * tb + 1 ) / 2;
                    d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( d1'*z + d0 + d1*z^{-1} )

                        % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' + ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2            

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( d1'*z + d0 + d1*z^{-1} )

                            % coefficients according
                            % to the degrees
                            switch deg                                                
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end       

                            Line = Line + 1;
                        end
                    end
                case 2
                    % [ -a, a ] in ( - pi, pi ) trig. pol. complex coef.

                    % the upper bound of the
                    % positivity interval 
                    a = ptype.int( 2 );

                    % cosine
                    ca = cos( a ); 

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( -cosa + (z+z^{-1})/2 )
                        %      I          II

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -ca * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % II
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )' + ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )';                                    

                        Line = Line + 1;
                    end                           

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2    

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( -cosa + (z+z^{-1})/2 )
                            %      I          II 

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    0.5 * vec( kron( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end  

                            Line = Line + 1;
                        end
                    end
                case 3
                    % [ a, pi ] trig. pol. complex coef.

                    % the lower bound of the
                    % positivity interval 
                    a = ptype.int( 1 );

                    % one tangent
                    ta = tan( a / 2 );

                    % coeficients of Dab
                    d0 = -ta / 2;
                    d1 = -ta / 4 + SR / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( d1'*z + d0 + d1*z^{-1} )

                        % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' + ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end                            

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2      

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( d1'*z + d0 + d1*z^{-1} )
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            ( d1' + d1 ) * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ), E ) )';

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end  

                            Line = Line + 1;
                        end
                    end
                case 4
                    % [ -pi, b ] trig. pol. complex coef.

                    % the upper bound of the
                    % positivity interval 
                    b = ptype.int( 2 );

                    % one tangent
                    tb = tan( b / 2 );

                    % coeficients of Dab
                    d0 = -tb / 2;
                    d1 = -tb / 4 + SR / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % -( d1'*z + d0 + d1*z^{-1} )

                         % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' - ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2              

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % -( d1'*z + d0 + d1*z^{-1} )

                            % coefficients according
                            % to the degrees
                            switch deg                                               
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end                        

                            Line = Line + 1;
                        end
                    end                        
                case 5
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b );  

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    n1 = n - 1;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 2; 

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % -( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                        %            I                        II                       III

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                        if nOld >= 4
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            ( ca + cb ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2  

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % -( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                            %            I                        II                       III

                            % coefficients according
                            % to the degrees
                            switch deg
                                case 2
                                    if nOld >= 4 
                                    % I
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                        -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                                        % II
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                        0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';                                                   
                                    end

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                                 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                                    % III
                                    if nOld >= 4
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...                                                      
                                        0.25 * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';
                                    end
                                    if nOld >= 6
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                        0.25 * vec( kron( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ), E ) )';                                                    
                                    end                                                
                                case QOsi - 3
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' + ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 2
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 1
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi                                        
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    -0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' + ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ), E ) )' - ...
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 6
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b ); 

                    % S0: n, S1:n

                    % size for Gram 
                    % matrices
                    nn = n;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0
                        % -cosb + (z + z^{-1})/2
                        %   I           II

                        % I
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        -cb * vec( kron( MList{ idx }( 1 : nn, : ), E ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ...  
                            vec( kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ) )';  
                        end

                        %---------------------------------------------------------------------------------------
                        % coefficient from S1
                        % cosa - (z + z^{-1})/2
                        %   I           II

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        ca * vec( kron( MList{ idx }( 1 : nn, : ), E ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            vec( kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ) )';   
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2   

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            switch deg        
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S0
                                    % -cosb + (z + z^{-1})/2
                                    %   I           II                                                     
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    -cb * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    -cb * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                                    0.5 * vec( kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end
                            switch deg
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S1
                                    % cosa - (z + z^{-1})/2
                                    %   I           II
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                                    -0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * vec( kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end  

                            Line = Line + 1;
                        end
                    end
                case 7
                    % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % two tangents
                    ta = tan( a / 2 );
                    tb = tan( b / 2 );  

                    % coeficients of Dab
                    d0 = -( ta * tb + 1 ) / 2;
                    d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % -( d1'*z + d0 + d1*z^{-1} )

                         % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' - ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end                            

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % -( d1'*z + d0 + d1*z^{-1} )

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end   

                            Line = Line + 1;
                        end
                    end
                case 8
                    % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.

                    % the upper bound of the
                    % positivity interval 
                    a = ptype.int( 3 );

                    % cosine
                    ca = cos( a ); 

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( cosa - (z+z^{-1})/2 )
                        %     I          II 

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        ca * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % II
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )' - ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )';

                        Line = Line + 1;
                    end                            

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2 

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( cosa - (z+z^{-1})/2 )
                            %     I          II 

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    -0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    0.5 * vec( kron( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end   

                            Line = Line + 1;
                        end
                    end
                case 9
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b );  

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    n1 = n - 1;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 2; 

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( kron( MList{ idx0 }( 1 : Qsi, : ), E ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                        %            I                        II                       III

                         % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                        if nOld >= 4
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            ( ca + cb ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2        

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( kron( MList{ idx0 }( ( deg - 1 ) * Qsi + 1 : deg * Qsi, : ), E ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                            %            I                        II                       III

                            % coefficients according
                            % to the degrees
                            switch deg
                                case 2
                                    if nOld >= 4 
                                        % I
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                        ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                                        % II
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                        0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';                                                   
                                    end

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...                                                 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                                    % III
                                    if nOld >= 4
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                                      
                                        0.25 * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';
                                    end
                                    if nOld >= 6
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                        0.25 * vec( kron( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ), E ) )';                                                    
                                    end                                               
                                case QOsi - 3
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' - ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 2
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 1
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    -0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi                                        
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' - ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ), E ) )' + ...
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 10
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b ); 

                    % S0: n, S1:n

                    % size for Gram 
                    % matrices
                    nn = n;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0
                        % cosb - (z + z^{-1})/2
                        %  I           II

                        % I
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        cb * vec( kron( MList{ idx }( 1 : nn, : ), E ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                            D( Line, LimD( 1 ) : LimD( 2 ) ) - ...  
                            vec( kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ) )';  
                        end

                        %---------------------------------------------------------------------------------------
                        % coefficient from S1
                        % -cosa + (z + z^{-1})/2
                        %    I           II

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -ca * vec( kron( MList{ idx }( 1 : nn, : ), E ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            vec( kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ) )';   
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2 

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            switch deg        
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S0
                                    % cosb - (z + z^{-1})/2
                                    %  I           II                                       
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    cb * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                                    -0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    cb * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                                    0.5 * vec( kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end
                            switch deg
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S1
                                    % -cosa + (z + z^{-1})/2
                                    %    I           II
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * vec( kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end   

                            Line = Line + 1;
                        end
                    end                        
            end
        end               
    end
else

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % BRL, must extend the vectorized matrix
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    ni1 = n + 1;

    if K == 1
        % scalar coefficients

        switch isIVeci                    
            case 0
                % global positivity

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    % we add the vectorized 
                    % matrix from "trace"
                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    Line = Line + 1;
                end
            case 1
                % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % two tangents
                ta = tan( a / 2 );
                tb = tan( b / 2 );  

                % coeficients of Dab
                d0 = -( ta * tb + 1 ) / 2;
                d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi                                   

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' + ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 2
                % [ -a, a ] in ( - pi, pi ) trig. pol. complex coef.

                % the upper bound of the
                % positivity interval 
                a = ptype.int( 2 );

                % cosine
                ca = cos( a ); 

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( -cosa + (z+z^{-1})/2 )
                    %      I          II

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )' + ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )';
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end 

                    Line = Line + 1;
                end
            case 3
                % [ a, pi ] trig. pol. complex coef.

                % the lower bound of the
                % positivity interval 
                a = ptype.int( 1 );

                % one tangent
                ta = tan( a / 2 );

                % coeficients of Dab
                d0 = -ta / 2;
                d1 = -ta / 4 + SR / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' + ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            d1' * vec( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 4
                % [ -pi, b ] trig. pol. complex coef.

                % the upper bound of the
                % positivity interval 
                b = ptype.int( 2 );

                % one tangent
                tb = tan( b / 2 );

                % coeficients of Dab
                d0 = -tb / 2;
                d1 = -tb / 4 + SR / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % -( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' - ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end  

                    Line = Line + 1;
                end                      
            case 5
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % two cosines
                ca = cos( a );
                cb = cos( b );  

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                n1 = n - 1;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 2; 

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % -( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^2+z^{-2})/4
                    %            I                        II                       III

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            if nOld >= 4
                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                ( ca + cb ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % III
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                0.5 * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';
                            end
                        case 2                                       
                            if nOld >= 4 
                                % I
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -( ca * cb + 0.5 ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                0.5 * ( ca + cb ) * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';                                         
                            end

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                         
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            % III
                            if nOld >= 4
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...                                         
                                0.25 * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';
                            end                                    
                            if nOld >= 6                                            
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                0.25 * vec( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ) )';                                   
                            end
                        case QOsi - 3
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' + ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 2
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 1
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi                                        
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            -0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' + ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            0.25 * vec( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ) )' - ...
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                    end

                    Line = Line + 1;
                end
            case 6
                % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % two cosines
                ca = cos( a );
                cb = cos( b ); 

                % S0: n, S1:n

                % size for Gram 
                % matrices
                nn = n;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    switch deg        
                            %---------------------------------------------------------------------------------------
                            % coefficient from S0
                            % -cosb + (z + z^{-1})/2
                            %   I           II
                        case 1                                        
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            -cb * vec( mextend( '2D', 'zpd', MList{ idx }( 1 : nn, : ), BRLK2i, [ 'r' 'd' ] ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                                D( Line, LimD( 1 ) : LimD( 2 ) ) + ...  
                                vec( mextend( '2D', 'zpd', MList{ idx }( nn + 1 : 2 * nn, : ), BRLK2i, [ 'r' 'd' ] ) )';                                            
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            -cb * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                            0.5 * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                            0.5 * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )';
                        otherwise
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            -cb * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                            0.5 * vec( mextend( '2D', 'zpd', MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )' + ...
                            0.5 * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )';
                    end
                    switch deg
                            %---------------------------------------------------------------------------------------
                            % coefficient from S1
                            % cosa - (z + z^{-1})/2
                            %   I           II
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx }( 1 : nn, : ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                vec( MList{ idx }( nn + 1 : 2 * nn, : ) )';   
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                            -0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * vec( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 7
                % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % two tangents
                ta = tan( a / 2 );
                tb = tan( b / 2 );  

                % coeficients of Dab
                d0 = -( ta * tb + 1 ) / 2;
                d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % -( d1'*z + d0 + d1*z^{-1} )

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )' - ...
                             d1 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )';
                        case QOsi - 1
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d1 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -d0 * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            d1' * vec( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            d1  * vec( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end   

                    Line = Line + 1;
                end
            case 8
                % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.

                % the upper bound of the
                % positivity interval 
                a = ptype.int( 3 );

                % cosine
                ca = cos( a ); 

                % S0: n+1, S1:n 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( cosa - (z+z^{-1})/2 )
                    %     I          II 

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx1 }( 1 : nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )' )' - ...
                            0.5 * vec( MList{ idx1 }( nn + 1 : 2 * nn, : )  )';
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            -0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ca * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' - ...
                            0.5 * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end  

                    Line = Line + 1;
                end
            case 9
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % two cosines
                ca = cos( a );
                cb = cos( b );  

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                n1 = n - 1;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 2; 

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % ( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                    %            I                        II                       III

                    % coefficients according
                    % to the degrees
                    switch deg
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            if nOld >= 4
                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                ( ca + cb ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % III
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                0.5 * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';
                            end
                        case 2
                            if nOld >= 4
                                % I
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                ( ca * cb + 0.5 ) * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';

                                % II
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                0.5 * ( ca + cb ) * vec( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ) )';
                            end

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( 1 : n1, : ) )';

                            % III
                            if nOld >= 4
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                              
                                0.25 * vec( MList{ idx1 }( n1 + 1 : 2 * n1, : ) )';
                            end
                            if nOld >= 6
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                0.25 * vec( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ) )';                                        
                            end
                        case QOsi - 3
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' - ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 2
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi - 1
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            -0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        case QOsi                                        
                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            ( ca * cb + 0.5 ) * vec( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ) )' - ...
                            0.5 * ( ca + cb ) * vec( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            0.25 * vec( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ) )' + ...
                            0.25 * vec( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ) )';
                    end

                    Line = Line + 1;
                end
            case 10
                % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % two cosines
                ca = cos( a );
                cb = cos( b ); 

                % S0: n, S1:n

                % size for Gram 
                % matrices
                nn = n;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    switch deg        
                            %---------------------------------------------------------------------------------------
                            % coefficient from S0
                            % cosb - (z + z^{-1})/2
                            %  I           II
                        case 1                                        
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            cb * vec( mextend( '2D', 'zpd', MList{ idx }( 1 : nn, : ), BRLK2i, [ 'r' 'd' ] ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                                D( Line, LimD( 1 ) : LimD( 2 ) ) - ...  
                                vec( mextend( '2D', 'zpd', MList{ idx }( nn + 1 : 2 * nn, : ), BRLK2i, [ 'r' 'd' ] ) )';                                            
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            cb * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), BRLK2i, ...
                                               [ 'r' 'd' ] ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                            0.5 * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                            -0.5 * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), BRLK2i, ...
                                                 [ 'r' 'd' ] ) )';
                        otherwise
                            % I
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            cb * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), BRLK2i, ...
                                               [ 'r' 'd' ] ) )';

                            % II
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                            0.5 * vec( mextend( '2D', 'zpd', MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )' - ...
                            0.5 * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), BRLK2i, ...
                                                [ 'r' 'd' ] ) )';
                    end
                    switch deg
                            %---------------------------------------------------------------------------------------
                            % coefficient from S1
                            % -cosa + (z + z^{-1})/2
                            %    I           II
                        case 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx }( 1 : nn, : ) )';

                            % II
                            if nOld >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                vec( MList{ idx }( nn + 1 : 2 * nn, : ) )';   
                            end
                        case QOsi - 1
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        case QOsi
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                        otherwise
                            % I
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            -ca * vec( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';

                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                            0.5 * vec( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ) )' + ...
                            0.5 * vec( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end    

                    Line = Line + 1;
                end
            case 11
                % [ a, b ] real pol. even degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (-ab + (a+b)*t - t^2 )

                    % term multiplied with -ab   
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -a * b * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with (a+b)*t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        ( a + b ) * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    % term multiplied with -t^2
                    if deg >= 3
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end
            case 12
                % [ a, b ] real pol. odd degree

                % the lower & upper bounds
                % of the positivity interval 
                a = ptype.int( 1 );
                b = ptype.int( 2 );                       

                % S0: n, S1:n

                % size for Gram 
                % matrices
                n05 = n + 0.5;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 0.5;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0
                    % (-a + t)                               

                    % term multiplied with -a
                    if deg ~= QOsi
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        -a * vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), BRLK2i, ...
                                           [ 'r' 'd' ] ) )';
                    end

                    % term multiplied with t
                    if deg ~= 1
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...   
                        D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                        vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), BRLK2i, ...
                                      [ 'r' 'd' ] ) )';
                    end

                    %---------------------------------------------------------------------------------------
                    % coefficient from S1
                    % (b - t)                              

                    % term multiplied with b
                    if deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        b * vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    % term multiplied with -t
                    if deg ~= 1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...   
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ) )';
                    end

                    Line = Line + 1;
                end
            case 13
                % [ a, Inf ) real pol. even degree

                % the lower bound
                % of the positivity interval 
                a = ptype.int( 1 );

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram  matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (-a + t)

                    % term multiplied with -a  
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -a * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end
            case 14
                % [ a, Inf ) real pol. odd degree

                % the lower bound
                % of the positivity interval 
                a = ptype.int( 1 );

                % S0: n, S1:n 

                % size for the second
                % Gram  matrix
                n05 = n + 0.5;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 0.5;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    if deg ~= QOsi
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), BRLK2i, ...
                                      [ 'r' 'd' ] ) )';
                    end

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (-a + t)

                    % term multiplied with -a  
                    if deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -a * vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    % term multiplied with t
                    if deg ~= 1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        vec( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ) )';
                    end

                    Line = Line + 1;
                end
            case 15
                % ( -Inf, b ] real pol. even degree

                % the upper bound
                % of the positivity interval 
                b = ptype.int( 2 );

                % S0: n+1, S1:n 

                % size for the second
                % Gram  matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (b - t)

                    % term multiplied with b  
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        b * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with -t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end 
            case 16
                % ( -Inf, b ] real pol. odd degree

                % the upper bound
                % of the positivity interval 
                b = ptype.int( 2 );

                % S0: n+1, S1:n 

                % size for the second
                % Gram  matrix
                n05 = n + 0.5;

                % index for the degree in
                % the list of unique degrees
                idx = MVecUnq == n - 0.5;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    if deg ~= QOsi
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), BRLK2i, ...
                                      [ 'r' 'd' ] ) )';
                    end

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (b - t)

                    % term multiplied with b  
                    if deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        b * vec( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ) )';
                    end

                    % term multiplied with -t
                    if deg ~= 1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        vec( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ) )';
                    end

                    Line = Line + 1;
                end
            case 17
                % ( -Inf, a ] U [ b, Inf ) real pol. even degree

                % bounds
                a = ptype.int( 2 );
                b = ptype.int( 3 );

                % S0: n+1, S1:n-1 

                % size for the second
                % Gram matrix
                nn = n;

                % indexes for the degrees in
                % the list of unique degrees
                idx0 = MVecUnq == n    ;
                idx1 = MVecUnq == n - 1;

                % for every degree of
                % the polynomial
                for deg = 1 : QOsi

                    %---------------------------------------------------------------------------------------
                    % coefficient from S0

                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                    vec( mextend( '2D', 'zpd', MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), BRLK2i, ...
                                  [ 'r' 'd' ] ) )';

                    %---------------------------------------------------------------------------------------
                    % coefficients from S1
                    % (ab - (a+b)*t + t^2 )

                    % term multiplied with ab   
                    if deg <= QOsi - 2
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        a * b * vec( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ) )';
                    end

                    % term multiplied with -(a+b)*t
                    if deg ~= 1 && deg ~= QOsi
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        ( a + b ) * vec( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ) )';
                    end

                    % term multiplied with t^2
                    if deg >= 3
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        vec( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ) )';
                    end

                    Line = Line + 1;
                end
        end
    else
        % matrix coefficients

        % "half" of the
        % matrix
        hK = sumn( K );

        % sind2sub is used go get
        % the subscripts from a "half"
        % a matrix index

        if  IsR
            % real polynomial

            switch isIVeci 
                case 0
                    % global positivity

                    for deg = 1 : QOsi
                        for elem = 1 : hK

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                            vec( mextend( '2D', 'zpd', kron( MList{ i }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            Line = Line + 1;
                        end
                    end
                case 11
                    % [ a, b ] real pol. even degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK     

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (-ab + (a+b)*t - t^2 )

                            % term multiplied with -ab   
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -a * b * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with (a+b)*t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                ( a + b ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            % term multiplied with -t^2
                            if deg >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end                     
                case 12
                    % [ a, b ] real pol. odd degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );                       

                    % S0: n, S1:n

                    % size for Gram 
                    % matrices
                    n05 = n + 0.5;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 0.5;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 ); 

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0
                            % (-a + t)                               

                            % term multiplied with -a
                            if deg ~= QOsi
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                -a * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ), ...
                                                   BRLK2i, [ 'r' 'd' ] ) )';
                            end

                            % term multiplied with t
                            if deg ~= 1
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...   
                                D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                                vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ), ...
                                              BRLK2i, [ 'r' 'd' ] ) )';
                            end

                            %---------------------------------------------------------------------------------------
                            % coefficient from S1
                            % (b - t)                              

                            % term multiplied with b
                            if deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                b * vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            % term multiplied with -t
                            if deg ~= 1
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...   
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 13
                    % [ a, Inf ) real pol. even degree

                    % the lower bound
                    % of the positivity interval 
                    a = ptype.int( 1 );

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram  matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK               

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (-a + t)

                            % term multiplied with -a  
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -a * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 14
                    % [ a, Inf ) real pol. odd degree

                    % the lower bound
                    % of the positivity interval 
                    a = ptype.int( 1 );

                    % S0: n, S1:n 

                    % size for the second
                    % Gram  matrix
                    n05 = n + 0.5;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 0.5;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK        

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 ); 

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            if deg ~= QOsi
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ), ...
                                              BRLK2i, [ 'r' 'd' ] ) )';
                            end

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (-a + t)

                            % term multiplied with -a  
                            if deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                -a * vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            % term multiplied with t
                            if deg ~= 1
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                vec( kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 15
                    % ( -Inf, b ] real pol. even degree

                    % the upper bound
                    % of the positivity interval 
                    b = ptype.int( 2 );

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram  matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK    

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (b - t)

                            % term multiplied with b  
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                b * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with -t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end   
                case 16
                    % ( -Inf, b ] real pol. odd degree

                    % the upper bound
                    % of the positivity interval 
                    b = ptype.int( 2 );

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram  matrix
                    n05 = n + 0.5;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 0.5;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK        

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            if deg ~= QOsi
                                D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ), ...
                                              BRLK2i, [ 'r' 'd' ] ) )';
                            end

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (b - t)

                            % term multiplied with b  
                            if deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                b * vec( kron( MList{ idx }( ( deg - 1 ) * n05 + 1 : deg * n05, : ), E ) )';
                            end

                            % term multiplied with -t
                            if deg ~= 1
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                vec( kron( MList{ idx }( ( deg - 2 ) * n05 + 1 : ( deg - 1 ) * n05, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 17
                    % ( -Inf, a ] U [ b, Inf ) real pol. even degree

                    % bounds
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % for every degree of
                    % the polynomial
                    for deg = 1 : QOsi
                        for elem = 1 : hK

                            [ row, col ] = sind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % (ab - (a+b)*t + t^2 )

                            % term multiplied with ab   
                            if deg <= QOsi - 2
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                a * b * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';
                            end

                            % term multiplied with -(a+b)*t
                            if deg ~= 1 && deg ~= QOsi
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                ( a + b ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end

                            % term multiplied with t^2
                            if deg >= 3
                                D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                                D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                vec( kron( MList{ idx1 }( ( deg - 3 ) * nn + 1 : ( deg - 2 ) * nn, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
            end
        else
            % trigonometric polynomial          

            K2 = K ^ 2;

            switch isIVeci 
                case 0
                    % global positivity                     

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                    
                        D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                        vec( mextend( '2D', 'zpd', kron( MList{ i }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        Line = Line + 1;
                    end

                    for deg = 2 : QOsi
                        for elem = 1 : K2

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                    
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ...
                            vec( mextend( '2D', 'zpd', kron( MList{ i }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            Line = Line + 1;
                        end
                    end   
                case 1
                    % [ a, b ] in ( -pi, pi ) trig. pol. complex coef.

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % two tangents
                    ta = tan( a / 2 );
                    tb = tan( b / 2 );  

                    % coeficients of Dab
                    d0 = -( ta * tb + 1 ) / 2;
                    d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( d1'*z + d0 + d1*z^{-1} )

                        % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' + ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2            

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( d1'*z + d0 + d1*z^{-1} )

                            % coefficients according
                            % to the degrees
                            switch deg                                                
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end       

                            Line = Line + 1;
                        end
                    end
                case 2
                    % [ -a, a ] in ( - pi, pi ) trig. pol. complex coef.

                    % the upper bound of the
                    % positivity interval 
                    a = ptype.int( 2 );

                    % cosine
                    ca = cos( a ); 

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( -cosa + (z+z^{-1})/2 )
                        %      I          II

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -ca * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % II
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )' + ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )';                                    

                        Line = Line + 1;
                    end                           

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2    

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( -cosa + (z+z^{-1})/2 )
                            %      I          II 

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    0.5 * vec( kron( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end  

                            Line = Line + 1;
                        end
                    end
                case 3
                    % [ a, pi ] trig. pol. complex coef.

                    % the lower bound of the
                    % positivity interval 
                    a = ptype.int( 1 );

                    % one tangent
                    ta = tan( a / 2 );

                    % coeficients of Dab
                    d0 = -ta / 2;
                    d1 = -ta / 4 + SR / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( d1'*z + d0 + d1*z^{-1} )

                        % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' + ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end                            

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2      

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( d1'*z + d0 + d1*z^{-1} )
                            % d0
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                            d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                            % d1'+d1
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            ( d1' + d1 ) * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ), E ) )';

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end  

                            Line = Line + 1;
                        end
                    end
                case 4
                    % [ -pi, b ] trig. pol. complex coef.

                    % the upper bound of the
                    % positivity interval 
                    b = ptype.int( 2 );

                    % one tangent
                    tb = tan( b / 2 );

                    % coeficients of Dab
                    d0 = -tb / 2;
                    d1 = -tb / 4 + SR / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % -( d1'*z + d0 + d1*z^{-1} )

                         % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' - ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2              

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % -( d1'*z + d0 + d1*z^{-1} )

                            % coefficients according
                            % to the degrees
                            switch deg                                               
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end                        

                            Line = Line + 1;
                        end
                    end                        
                case 5
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. even degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b );  

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    n1 = n - 1;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 2; 

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % -( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                        %            I                        II                       III

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                        if nOld >= 4
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            ( ca + cb ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            0.5 * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2  

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % -( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                            %            I                        II                       III

                            % coefficients according
                            % to the degrees
                            switch deg
                                case 2
                                    if nOld >= 4 
                                    % I
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                        -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                                        % II
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                        0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';                                                   
                                    end

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                                 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                                    % III
                                    if nOld >= 4
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...                                                      
                                        0.25 * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';
                                    end
                                    if nOld >= 6
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                        0.25 * vec( kron( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ), E ) )';                                                    
                                    end                                                
                                case QOsi - 3
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' + ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 2
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 1
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi                                        
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    -0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' + ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ), E ) )' - ...
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 6
                    % [ a, b ] in [ 0, pi ] trig. pol. real coef. odd degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 1 );
                    b = ptype.int( 2 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b ); 

                    % S0: n, S1:n

                    % size for Gram 
                    % matrices
                    nn = n;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0
                        % -cosb + (z + z^{-1})/2
                        %   I           II

                        % I
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        -cb * vec( mextend( '2D', 'zpd', kron( MList{ idx }( 1 : nn, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                            D( Line, LimD( 1 ) : LimD( 2 ) ) + ...  
                            vec( mextend( '2D', 'zpd', kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ), BRLK2i, ...
                                          [ 'r' 'd' ] ) )';  
                        end

                        %---------------------------------------------------------------------------------------
                        % coefficient from S1
                        % cosa - (z + z^{-1})/2
                        %   I           II

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        ca * vec( kron( MList{ idx }( 1 : nn, : ), E ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...  
                            vec( kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ) )';   
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2   

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            switch deg        
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S0
                                    % -cosb + (z + z^{-1})/2
                                    %   I           II                                                     
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    -cb * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                                    0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                                    0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    -cb * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) + ... 
                                    0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )' + ...
                                    0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )';
                            end
                            switch deg
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S1
                                    % cosa - (z + z^{-1})/2
                                    %   I           II
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                                    -0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * vec( kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end  

                            Line = Line + 1;
                        end
                    end
                case 7
                    % [ -pi,  a ] U [ b, pi ] trig. pol. complex coef.

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % two tangents
                    ta = tan( a / 2 );
                    tb = tan( b / 2 );  

                    % coeficients of Dab
                    d0 = -( ta * tb + 1 ) / 2;
                    d1 =  ( 1 - ta * tb ) / 4 + SR * ( ta + tb ) / 4;

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % -( d1'*z + d0 + d1*z^{-1} )

                         % d0
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -d0 * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % d1'+d1
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        d1' * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )' - ...
                         d1 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )';

                        Line = Line + 1;
                    end                            

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                           BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % -( d1'*z + d0 + d1*z^{-1} )

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d1 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % d0
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -d0 * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % d1'+d1
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    d1' * vec( kron( MList{ idx1 }(        deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    d1  * vec( kron( MList{ idx1 }( ( deg - 2 )* nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end   

                            Line = Line + 1;
                        end
                    end
                case 8
                    % [ -pi, -a ] U [ a, pi ] trig. pol. complex coef.

                    % the upper bound of the
                    % positivity interval 
                    a = ptype.int( 3 );

                    % cosine
                    ca = cos( a ); 

                    % S0: n+1, S1:n 

                    % size for the second
                    % Gram matrix
                    nn = n;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( cosa - (z+z^{-1})/2 )
                        %     I          II 

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        ca * vec( kron( MList{ idx1 }( 1 : nn, : ), E ) )';

                        % II
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : )', E ) )' - ...
                        0.5 * vec( kron( MList{ idx1 }( nn + 1 : 2 * nn, : ) , E ) )';

                        Line = Line + 1;
                    end                            

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2 

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( cosa - (z+z^{-1})/2 )
                            %     I          II 

                            % coefficients according
                            % to the degrees
                            switch deg
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    -0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ca * vec( kron( MList{ idx1 }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    0.5 * vec( kron( MList{ idx1 }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' - ...
                                    0.5 * vec( kron( MList{ idx1 }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end   

                            Line = Line + 1;
                        end
                    end
                case 9
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. even degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b );  

                    % S0: n+1, S1:n-1 

                    % size for the second
                    % Gram matrix
                    n1 = n - 1;

                    % indexes for the degrees in
                    % the list of unique degrees
                    idx0 = MVecUnq == n    ;
                    idx1 = MVecUnq == n - 2; 

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 );

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0

                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( 1 : ni1, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        %---------------------------------------------------------------------------------------
                        % coefficients from S1
                        % ( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                        %            I                        II                       III

                         % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                        if nOld >= 4
                            % II
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                            ( ca + cb ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                            % III
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...
                            0.5 * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2        

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            %---------------------------------------------------------------------------------------
                            % coefficient from S0

                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                            vec( mextend( '2D', 'zpd', kron( MList{ idx0 }( ( deg - 1 ) * ni1 + 1 : deg * ni1, : ), E ), ...
                                          BRLK2i, [ 'r' 'd' ] ) )';

                            %---------------------------------------------------------------------------------------
                            % coefficients from S1
                            % ( cosa*cosb + 1/2 ) + (z+z^{-1})/2 * ( cosa + cosb ) - (z^4+z^{-4})/4
                            %            I                        II                       III

                            % coefficients according
                            % to the degrees
                            switch deg
                                case 2
                                    if nOld >= 4 
                                        % I
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                        ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';

                                        % II
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                        0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 2 * n1 + 1 : 3 * n1, : ), E ) )';                                                   
                                    end

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...                                                 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( 1 : n1, : ), E ) )';

                                    % III
                                    if nOld >= 4
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...                                                      
                                        0.25 * vec( kron( MList{ idx1 }( n1 + 1 : 2 * n1, : ), E ) )';
                                    end
                                    if nOld >= 6
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                        D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                        0.25 * vec( kron( MList{ idx1 }( 3 * n1 + 1 : 4 * n1, : ), E ) )';                                                    
                                    end                                               
                                case QOsi - 3
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' - ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 2
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi - 1
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    -0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                case QOsi                                        
                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    ( ca * cb + 0.5 ) * vec( kron( MList{ idx1 }( ( deg - 1 ) * n1 + 1 : deg * n1, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) - ... 
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }(         deg * n1 + 1 : ( deg + 1 ) * n1, : ), E ) )' - ...
                                    0.5 * ( ca + cb ) * vec( kron( MList{ idx1 }( ( deg - 2 ) * n1 + 1 : ( deg - 1 ) * n1, : ), E ) )';

                                    % III
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...  
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                                    0.25 * vec( kron( MList{ idx1 }( ( deg + 1 ) * n1 + 1 : ( deg + 2 ) * n1, : ), E ) )' + ...
                                    0.25 * vec( kron( MList{ idx1 }( ( deg - 3 ) * n1 + 1 : ( deg - 2 ) * n1, : ), E ) )';
                            end

                            Line = Line + 1;
                        end
                    end
                case 10
                    % [ 0, a ] U [ b, pi ] trig. pol. real coef. odd degree

                    % the lower & upper bounds
                    % of the positivity interval 
                    a = ptype.int( 2 );
                    b = ptype.int( 3 );

                    % two cosines
                    ca = cos( a );
                    cb = cos( b ); 

                    % S0: n, S1:n

                    % size for Gram 
                    % matrices
                    nn = n;

                    % index for the degree in
                    % the list of unique degrees
                    idx = MVecUnq == n - 1;

                    % deg = 1
                    for elem = 1 : hK                        

                        [ row, col ] = sind2sub( K, elem );

                        % selector matrix     
                        E = sparse( col, row, 1, K, K, 1 ); 

                        %---------------------------------------------------------------------------------------
                        % coefficient from S0
                        % cosb - (z + z^{-1})/2
                        %  I           II

                        % I
                        D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                        cb * vec( mextend( '2D', 'zpd', kron( MList{ idx }( 1 : nn, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 1 ) : LimD( 2 ) ) = ...      
                            D( Line, LimD( 1 ) : LimD( 2 ) ) - ...  
                            vec( mextend( '2D', 'zpd', kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ), BRLK2i, [ 'r' 'd' ] ) )';  
                        end

                        %---------------------------------------------------------------------------------------
                        % coefficient from S1
                        % -cosa + (z + z^{-1})/2
                        %    I           II

                        % I
                        D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                        -ca * vec( kron( MList{ idx }( 1 : nn, : ), E ) )';

                        % II
                        if nOld >= 3
                            D( Line, LimD( 3 ) : LimD( 4 ) ) = ...      
                            D( Line, LimD( 3 ) : LimD( 4 ) ) + ...  
                            vec( kron( MList{ idx }( nn + 1 : 2 * nn, : ), E ) )';   
                        end

                        Line = Line + 1;
                    end

                    % for every degree of
                    % the polynomial
                    for deg = 2 : QOsi
                        for elem = 1 : K2 

                            [ row, col ] = ind2sub( K, elem );

                            % selector matrix     
                            E = sparse( col, row, 1, K, K, 1 );

                            switch deg        
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S0
                                    % cosb - (z + z^{-1})/2
                                    %  I           II                                       
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    cb * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ), ...
                                                       BRLK2i, [ 'r' 'd' ] ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                                    0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...       
                                    -0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ), ...
                                                         BRLK2i, [ 'r' 'd' ] ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...                                                    
                                    cb * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ), ...
                                                       BRLK2i, [ 'r' 'd' ] ) )';

                                    % II
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) = ...    
                                    D( Line, LimD( 1 ) : LimD( 2 ) ) - ... 
                                    0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )' - ...
                                    0.5 * vec( mextend( '2D', 'zpd', kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ), ...
                                                        BRLK2i, [ 'r' 'd' ] ) )';
                            end
                            switch deg
                                    %---------------------------------------------------------------------------------------
                                    % coefficient from S1
                                    % -cosa + (z + z^{-1})/2
                                    %    I           II
                                case QOsi - 1
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                case QOsi
                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...       
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                                otherwise
                                    % I
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...                                                    
                                    -ca * vec( kron( MList{ idx }( ( deg - 1 ) * nn + 1 : deg * nn, : ), E ) )';

                                    % II
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) = ...    
                                    D( Line, LimD( 3 ) : LimD( 4 ) ) + ... 
                                    0.5 * vec( kron( MList{ idx }(         deg * nn + 1 : ( deg + 1 ) * nn, : ), E ) )' + ...
                                    0.5 * vec( kron( MList{ idx }( ( deg - 2 ) * nn + 1 : ( deg - 1 ) * nn, : ), E ) )';
                            end   

                            Line = Line + 1;
                        end
                    end                        
            end
        end               
    end
end